/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci_cm@deepin.com>
 *
 * Maintainer: caimengci <caimengci_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "antiavdatastatusmodel.h"
#include <QDebug>

AntiavDataStatusModel *AntiavDataStatusModel::m_pInstance = nullptr;
AntiavDataStatusModel::AntiavDataStatusModel(QObject *parent) :
    QObject(parent)
{
}

void AntiavDataStatusModel::setCModule(CModule_Antiav *mm)
{
    m_cModule_Antiav = mm;
}

void AntiavDataStatusModel::setInstance()
{
    m_pInstance = new AntiavDataStatusModel(this);
}

AntiavDataStatusModel *AntiavDataStatusModel::getInstance()
{
    return m_pInstance;
}

/****************扫描******************/
void AntiavDataStatusModel::setDissectionStatus(State state)
{
    m_pInstance->m_cModule_Antiav->setdissectionStatus(state, m_pInstance->setDissectionStatusCallBack);
}

void AntiavDataStatusModel::setDissectionStatusCallBack(std::string path, std::string message)
{
    printf("path = %s, message = %s\n", path.c_str(), message.c_str());
    // 获取扫描信息
    QString sPath = QString::fromStdString(path);
    QString sMessage = QString::fromStdString(message);

    Q_EMIT m_pInstance->sendDissectionStatusResult(sPath, sMessage);
}

/*************问题文件处理（修复/信任/隔离）**************/
void AntiavDataStatusModel::repairProblemFile(QString sPath)
{
    // 修复问题文件接口
    m_pInstance->m_cModule_Antiav->repairProblemFile(sPath.toStdString(), m_pInstance->repairProblemFileCallBack);
}

void AntiavDataStatusModel::repairProblemFileCallBack(std::string path, std::string message)
{
    // 获取修复文件信息
    QString sPath = QString::fromStdString(path);
    QString sMessage = QString::fromStdString(message);

    // 发送信号
    Q_EMIT m_pInstance->sendRepairProblemFileResult(sPath, sMessage);
}

void AntiavDataStatusModel::reposeProblemFile(bool bFlag,QString sPath)
{
    // 信任问题文件接口
    bool flag =m_pInstance->m_cModule_Antiav->reposeProblemFile(bFlag, sPath.toStdString(), m_pInstance->reposeProblemFileCallBack);
    qDebug()<<flag;
}

void AntiavDataStatusModel::reposeProblemFileCallBack(std::string path, std::string message)
{
    // 获取信任文件信息
    QString sPath = QString::fromStdString(path);
    QString sMessage = QString::fromStdString(message);

    // 发送信号
    Q_EMIT m_pInstance->sendReposeProblemFileResult(sPath, sMessage);
}

void AntiavDataStatusModel::setProblemFile(bool bFlag,QString sPath)
{
    // 隔离问题文件接口
    bool flag = m_pInstance->m_cModule_Antiav->sepProblemFile(bFlag, sPath.toStdString(), m_pInstance->sepProblemFileCallBack);
    qDebug()<<flag;
}

void AntiavDataStatusModel::sepProblemFileCallBack(std::string path, std::string message)
{
    // 获取隔离文件信息
    QString sPath = QString::fromStdString(path);
    QString sMessage = QString::fromStdString(message);

    // 发送信号
    Q_EMIT m_pInstance->sendSepProblemFileResult(sPath, sMessage);
}

/*************查询问题文件（信任/隔离）**************/
void AntiavDataStatusModel::queryReposeFile()
{
    std::vector<FileInfo> m_sReposePath;
    // 查询信任文件
    if (m_pInstance->m_cModule_Antiav->queryAllReposeProblemFileInfo(m_sReposePath)){
        // 发送信号
        Q_EMIT m_pInstance->sendQueryReposeFileInfo(m_sReposePath);
    }
}

// 查询隔离文件
void AntiavDataStatusModel::querySepFile(){
    // 查询隔离文件
    std::vector<FileInfo> m_sSepPath;
    bool flag=m_pInstance->m_cModule_Antiav->queryAllSepProblemFileInfo(m_sSepPath);
    qDebug()<<"查询隔离文件 flag==="<<flag;
    if (flag){
        // 发送信号
        Q_EMIT m_pInstance->sendQuerySepFileInfo(m_sSepPath);
    }
}

void AntiavDataStatusModel::getVersion()
{
    std::string sAddr = "https://defender.chinauos.com/vdb";
    bool flag = m_pInstance->m_cModule_Antiav->get_vdb_version(sAddr, m_version);
    if(flag){
        // 获取版本信息
        QString version = QString::fromStdString(m_version);
        Q_EMIT m_pInstance->sendQueryVersion(version);
    }
}

void AntiavDataStatusModel::updateVersion()
{
    std::string sAddr = "https://defender.chinauos.com/vdb";
    bool flag = m_pInstance->m_cModule_Antiav->malware_update(sAddr);
    qDebug() << "flag" << flag;
    //获得更新结果
    Q_EMIT m_pInstance->sendUpdateVersion(flag);
}

// 是否在扫描中
bool AntiavDataStatusModel::isScanning()
{
    bool flag = m_pInstance->m_cModule_Antiav->is_scanning();
    return flag;
}

QString AntiavDataStatusModel::getVdbVersion()
{
    std::string sAddr = "https://defender.chinauos.com/vdb";
    QString version;
    bool flag = m_pInstance->m_cModule_Antiav->get_vdb_version(sAddr, m_version);
    qDebug() << "flag" << flag;
    if(flag){
        // 获取版本信息
        version = QString::fromStdString(m_version);
    }else{
        //获得失败
        version = "-1";
    }
    return version;
}








