/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci_cm@deepin.com>
 *
 * Maintainer: caimengci <caimengci_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "Antiav.h"
#include "Antiav_common.h"

#include <QObject>

class AntiavDataStatusModel : public QObject
{
    Q_OBJECT
public:
    explicit AntiavDataStatusModel(QObject *parent = nullptr);

    void setCModule(CModule_Antiav *mm);
    // 获取单例
    void setInstance();
    static AntiavDataStatusModel *getInstance();

    // 设置扫描状态
    void setDissectionStatus(State state);
    static void setDissectionStatusCallBack(std::string path, std::string message);

    // 修复问题文件
    void repairProblemFile(QString sPath);
    static void repairProblemFileCallBack(std::string path, std::string message);

    // 信任问题文件
    void reposeProblemFile(bool bFlag, QString sPath);
    static void reposeProblemFileCallBack(std::string path, std::string message);

    // 隔离信任文件
    void setProblemFile(bool bFlag, QString sPath);
    static void sepProblemFileCallBack(std::string path, std::string message);

    // 查询信任文件
    void queryReposeFile();
    // 查询隔离文件
    void querySepFile();

    void getVersion();   //获得版本
    void updateVersion();    //更新版本

    // 是否在扫描中
    bool isScanning();

    // 获得病毒库版本
    QString getVdbVersion();
Q_SIGNALS:
    // 发送扫描信号
    void sendDissectionStatusResult(QString sPath, QString sMessage);
    void sendRepairProblemFileResult(QString sPath, QString sMessage);
    void sendReposeProblemFileResult(QString sPath, QString sMessage);
    void sendSepProblemFileResult(QString sPath, QString sMessage);
    void sendQueryReposeFileInfo(std::vector<FileInfo> sReposePath);
    void sendQuerySepFileInfo(std::vector<FileInfo> sSepPath);

    void sendQueryVersion(QString version);
    void sendUpdateVersion(bool flag);   //更新结果
private:
    static AntiavDataStatusModel *m_pInstance;                 // 单例
    CModule_Antiav* m_cModule_Antiav;

    //std::vector<FileInfo> m_sReposePath;
    //std::vector<FileInfo> m_sSepPath;
    std::string m_version;
    std::string m_result;
};
