/*
 * Copyright (C) 2020 ~ 2020 Deepin Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci@uniontech.com>
 *
 * Maintainer: caimengci <caimengci@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "operatedbusdata.h"
#include "common.h"

#include <QDebug>
#include <QTimer>

// 安全中心配置目录
const QString SConfigDir = "/usr/bin/config/deepin-defender/";
// 本地病毒库版本文件
const QString SLocalVdbVerCfgFileName = "version";
// 本地病毒库默认版本
const QString SLocalVdbDefaultVer = "00.0.00.01";

OperateDBusData::OperateDBusData(QObject *parent)
    : QObject(parent)
    , m_adaptpr(nullptr)
    , m_antiavDataThread(nullptr)                   // 扫描单独线程
    , m_antiavDataStatusModel(nullptr)        // 病毒查杀 设置状态/处理问题文件/查询 单例
    , m_nTotal(0)
    , m_nTotalReal(0)
    , m_isbInVirusScanPage(false)
{
    m_adaptpr = new AntiavAdaptor(this);
    if (!QDBusConnection::systemBus().registerService(service) ||
            !QDBusConnection::systemBus().registerObject(path, this)) {
        qDebug() << "dbus service already registered! " << service;

        exit(0);
    }

    initialize();
}

void OperateDBusData::StartApp()
{

}

void OperateDBusData::ExitApp()
{
    qApp->quit();
}

void OperateDBusData::initialize()
{
    // 读取本地病毒库版本
    readLocalVdbVersion();

    // 初始化设置扫描文件的最大体量
    // caitao/2020.6.22 修改最大扫描文件大小为5GB
    m_cModule_Antiav.setFileSize(5120);

    // 扫描初始化
    m_antiavDataThread = new AntiavDataThread(&m_cModule_Antiav);
    connect(m_antiavDataThread, SIGNAL(sendDissectionResult(QString, double, Type, QString)), this, SLOT(doDissectionResult(QString, double, Type, QString)), Qt::DirectConnection);
    connect(m_antiavDataThread, SIGNAL(sendDissectionFinished()), this, SLOT(slot_DissectionFinished()));

    // 设置扫描状态初始化
    m_antiavDataStatusModel = new AntiavDataStatusModel;
    m_antiavDataStatusModel->setCModule(&m_cModule_Antiav);
    m_antiavDataStatusModel->setInstance();
    AntiavDataStatusModel *hand_status = AntiavDataStatusModel::getInstance();
    connect(hand_status, SIGNAL(sendDissectionStatusResult(QString, QString)), this, SLOT(doSetDissectionStateResult(QString, QString)));
    connect(hand_status, SIGNAL(sendRepairProblemFileResult(QString, QString)), this, SLOT(doRepairProblemFileResult(QString, QString)));
    connect(hand_status, SIGNAL(sendReposeProblemFileResult(QString, QString)), this, SLOT(doReposeProblemFileResult(QString, QString)));
    connect(hand_status, SIGNAL(sendSepProblemFileResult(QString, QString)), this, SLOT(doSepProblemFileResult(QString, QString)));

    connect(hand_status, SIGNAL(sendQueryReposeFileInfo(std::vector<FileInfo>)), this, SLOT(doQueryReposeFileResult(std::vector<FileInfo>)));
    connect(hand_status, SIGNAL(sendQuerySepFileInfo(std::vector<FileInfo>)), this, SLOT(doQuerySepFileResult(std::vector<FileInfo>)));

    connect(hand_status, SIGNAL(sendQueryVersion(QString)), this, SLOT(doQueryVersion(QString)));
    connect(hand_status, SIGNAL(sendUpdateVersion(bool)), this, SLOT(doUpdateVersion(bool)));

    qDebug() << "Antiav service started!";
    // 测试 --- 扫描
    //doDissection(false, "/home/uos0812/Downloads/send");

    // 测试 -----取消
    //QTimer::singleShot(3000, this, SLOT(stopTest()));

    //测试版本
    // queryVersion();
    // getVdbVersion();

    //测试更新
    // updateVersion();
}

void OperateDBusData::preInitialize()
{
}

/*********************扫描**********************/
void OperateDBusData::doDissection(bool bAllDissection, QString sDissectionPath)
{
    m_nTotal = 0;
    m_nTotalReal = 0;
    qDebug() << "bAllDissection = " << bAllDissection;
    qDebug() << "sDissectionPath = " << sDissectionPath;
    //QStringList sList = sDissectionPath.split(",");
    // 如果路径名称中有　","　就会照成扫描的个数不正确
    QStringList sList = sDissectionPath.split("*");
    std::vector<std::string> sPath;
    for (int i = 0; i < sList.count(); ++i) {
        sPath.push_back(sList.at(i).toStdString());
    }

    m_antiavDataThread->setDissectionPath(bAllDissection, sPath);
    m_antiavDataThread->start();
}

void OperateDBusData::doDissectionResult(QString sPath, double dProgress, Type eType, QString sDetailInfo)
{
    qDebug() << QString("caimnegci---path = %1, progress = %2, type = %3, info = %4 ").arg(sPath).arg(dProgress).arg(eType).arg(sDetailInfo);
    int a = int(eType);
    Q_EMIT onDissectionResult(sPath, dProgress, a, sDetailInfo);
}

void OperateDBusData::slot_DissectionFinished()
{
    emit signal_doDissectionFinished();
}

void OperateDBusData::stopTest()
{
    setDissection(State::Stop);
}

// 读取本地病毒库版本
void OperateDBusData::readLocalVdbVersion()
{
    QDir dir;
    //文件路径 文件夹
    QString dir_path = SConfigDir;
    if (!dir.exists(dir_path)) {
        dir.mkpath(dir_path);
    }
    //文件名
    QString m_fileName = SConfigDir + SLocalVdbVerCfgFileName;
    QFile f(m_fileName);
    if (!f.exists()) {
        // 如果文件不存在  写入初始版本
        if (!f.open(QIODevice::ReadOnly | QIODevice::WriteOnly | QIODevice::Text)) {
            return;
        }
        QTextStream outstream(&f);
        outstream.seek(0);
        // 写入数据
        outstream << SLocalVdbDefaultVer << "\n";
        f.close();
    }
    // 打开文件
    if (!f.open(QIODevice::ReadOnly | QIODevice::Text)) {
        return;
    }
    //输入流
    QTextStream txtInput(&f);
    QString sLocalVdbVer = txtInput.readLine(0);

    // 格式校验
    if (4 == sLocalVdbVer.split(".").count()) {
        m_sLocalVdbVer = sLocalVdbVer;
    }
    f.close();
}

// 保存本地病毒库版本
void OperateDBusData::writeLocalVdbVersion()
{
    QDir dir;
    //文件路径 文件夹
    QString dir_path = SConfigDir;
    if (!dir.exists(dir_path)) {
        dir.mkpath(dir_path);
    }
    //文件名
    QString m_fileName = SConfigDir + SLocalVdbVerCfgFileName;
    QFile f(m_fileName);
    if (!f.exists()) {
        // 如果文件不存在  写入初始版本
        if (!f.open(QIODevice::ReadOnly | QIODevice::WriteOnly | QIODevice::Text)) {
            return;
        }
        QTextStream outstream(&f);
        // 写入数据
        outstream << SLocalVdbDefaultVer << "\n";
        f.close();
    }
    // 打开文件
    if (!f.open(QIODevice::ReadOnly | QIODevice::WriteOnly | QIODevice::Text)) {
        return;
    }

    QTextStream outstream(&f);
    // 写入数据
    outstream << m_sLocalVdbVer << "\n";
    f.close();
}

/*************设置扫描状态（暂停/继续/终止）**************/
void OperateDBusData::setDissection(int eState)
{
    State state ;
    state = State(eState);
    m_antiavDataStatusModel->setDissectionStatus(state);
}

void OperateDBusData::doSetDissectionStateResult(QString sPath, QString sMessage)
{
    qDebug() << "path:" << sPath << "--message: " << sMessage;

    emit onSetDissectionStateResult(sPath, sMessage);
}

/*************问题文件处理（修复/信任/隔离）**************/
// 修复问题文件
void OperateDBusData::repairProblemFile(QString sPath)
{
    m_antiavDataStatusModel->repairProblemFile(sPath);
}

// 修复问题文件返回槽
void OperateDBusData::doRepairProblemFileResult(QString sPath, QString sMessage)
{
    qDebug() << "path:" << sPath << "--message: " << sMessage;

    emit onRepairProblemFileResult(sPath, sMessage);
}

// 信任问题文件
void OperateDBusData::reposeProblemFile(bool bFlag, QString sPath)
{
    m_antiavDataStatusModel->reposeProblemFile(bFlag, sPath);
}

// 信任问题文件返回槽
void OperateDBusData::doReposeProblemFileResult(QString sPath, QString sMessage)
{
    qDebug() << "path:" << sPath << "--message: " << sMessage;

    emit onReposeProblemFileResult(sPath, sMessage);
}

// 隔离问题文件
void OperateDBusData::sepProblemFile(bool bFlag, QString sPath)
{
    qDebug() << bFlag << "spath================" << sPath;
    m_antiavDataStatusModel->setProblemFile(bFlag, sPath);
}

// 隔离问题文件返回槽
void OperateDBusData::doSepProblemFileResult(QString sPath, QString sMessage)
{
    qDebug() << "path:" << sPath << "--message: " << sMessage;

    emit onSepProblemFileResult(sPath, sMessage);
}

/*************查询文件（信任/隔离）**************/
// 查询信任文件
void OperateDBusData::queryReposeFile()
{
    m_antiavDataStatusModel->queryReposeFile();
}

void OperateDBusData::doQueryReposeFileResult(std::vector<FileInfo> sReposePath)
{
    size_t len = sReposePath.size();
    qDebug() << "信任文件数量。。。lsize====" << len;
    QString str = "";
    for (size_t i = 0; i < len; i ++) {
        QString strpath = QString::fromStdString(sReposePath[i].path);
        QString time = QString::fromStdString(sReposePath[i].time);
        str = str + strpath + "*" + time + "[*]";
    }
    //   qDebug()<<"信任文件信息===="<<str;
    emit signal_onQueryReposeFile(str);

}

// 查询隔离文件
void OperateDBusData::querySepFile()
{
    qDebug() << "开始查询隔离";
    m_antiavDataStatusModel->querySepFile();
}

void OperateDBusData::doQuerySepFileResult(std::vector<FileInfo> sSepPath)
{
    size_t len = sSepPath.size();
    int length = int(sSepPath.size());
    qDebug() << "lsize====" << len << ",length=====" << length;
    QString str = "";
    for (size_t i = 0; i < len ; i ++) {
        QString strpath = QString::fromStdString(sSepPath[i].path);
        QString time = QString::fromStdString(sSepPath[i].time);
        str = str + strpath + "*" + time + "[*]";
    }
    // qDebug()<<"隔离文件信息===="<<str;
    emit signal_onQuerySepFile(str);
}

void OperateDBusData::queryVersion()
{
    m_antiavDataStatusModel->getVersion();
}

void OperateDBusData::doQueryVersion(QString version)
{
    qDebug() << "version" << version;
    emit signal_doQueryVersion(version);
}

void OperateDBusData::updateVersion()
{
    qDebug() << "start update";
    // 先获取服务器版本
    getVdbVersion();
    m_antiavDataStatusModel->updateVersion();
}

void OperateDBusData::doUpdateVersion(bool flag)
{
    qDebug() << "update virus database finished, result:" << flag;

    // 安恒引擎在无网络情况下，更新病毒库，也会返回成功信号
    // 因此具体更新成功还是失败要作进一步判断
    QString sSrvVdbVer = getVdbVersion();
    // 如果获取服务器版本号失败，则说明更新病毒库失败
    if ("-1" == sSrvVdbVer) {
        emit signal_UpdateVersionResult(false);
        return;
    }

    // 如果成功，则更新并存储本地病毒库版本号
    if (flag) {
        m_sLocalVdbVer = m_sSrvVdbVer;
        writeLocalVdbVersion();
    }

    emit signal_UpdateVersionResult(flag);
}

bool OperateDBusData::isScanning()
{
    bool flag = m_antiavDataStatusModel->isScanning();
    // 如果主界面已打开，则将界面书否处于扫描界面加入判断
    return  flag || m_isbInVirusScanPage;
}

bool OperateDBusData::isTrueScanning()
{
    return m_antiavDataStatusModel->isScanning();
}

// 获得服务器病毒库版本
QString OperateDBusData::getVdbVersion()
{
    qDebug() << "query version start";
    QString version = m_antiavDataStatusModel->getVdbVersion();
    qDebug() << "version" << version;
    // 更新服务器病毒库版本号
    if ("-1" != version) {
        m_sSrvVdbVer = version;
    }
    return version;
}

// 后台更新版本
void OperateDBusData::backgroundUpdate()
{
    emit onBackgroundUpdate();
}

void OperateDBusData::scanThreatsFile()
{
    emit onRequstScanThreatsFile();
}

void OperateDBusData::SetIsbInVirusScanningPage(bool isbOn)
{
    m_isbInVirusScanPage = isbOn;
    qDebug() << "SetIsbInVirusScanningPage" << isbOn;
}

void OperateDBusData::setScanStart()
{
    emit onSendScanStart();
}

