/*
 * Copyright (C) 2020 ~ 200 Deepin Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci@uniontech.com>
 *
 * Maintainer: caimengci <caimengci@uniontech.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once

#include "antiav_adaptor.h"
#include <QObject>

#include "Antiav.h"
#include "Antiav_common.h"

#include "antiavdatathread.h"
#include "antiavdatastatusmodel.h"

class OperateDBusData : public QObject
{
    Q_OBJECT
public:
    explicit OperateDBusData(QObject *parent = nullptr);

public Q_SLOTS:
    // DBus服务对外扫描接口
    void StartApp();
    void ExitApp();

    void initialize();
    void preInitialize();

    // 扫描 （全盘true/自定义false）
    void doDissection(bool bAllDissection, QString sDissectionPath);
    // 扫描返回槽
    void doDissectionResult(QString sID, double dProgress, Type eType, QString sDetailInfo);
    void slot_DissectionFinished();    //扫描完成

    // 设置扫描状态 （暂停/继续/终止
    void setDissection(int eState);
    // 设置扫描状态返回槽
    void doSetDissectionStateResult(QString sPath, QString sMessage);

    // 修复问题文件
    void repairProblemFile(QString sPath);
    // 修复问题文件返回槽
    void doRepairProblemFileResult(QString sPath, QString sMessage);

    // 信任问题文件 (添加/移除)
    void reposeProblemFile(bool bFlag, QString sPath);
    // 信任问题文件返回槽
    void doReposeProblemFileResult(QString sPath, QString sMessage);

    // 隔离问题文件 (添加/移除)
    void sepProblemFile(bool bFlag, QString sPath);
    // 隔离问题文件返回槽
    void doSepProblemFileResult(QString sPath, QString sMessage);

    // 查询信任文件
    void queryReposeFile();
    void doQueryReposeFileResult(std::vector<FileInfo>);

    // 查询隔离文件
    void querySepFile();
    void doQuerySepFileResult(std::vector<FileInfo>);

    //版本查询
    void queryVersion();
    void doQueryVersion(QString version);

    //版本后台更新
    void updateVersion();
    void doUpdateVersion(bool flag);

    // 是否在扫描中
    bool isScanning();
    bool isTrueScanning();

    // 获得病毒库版本
    QString getVdbVersion();

    // 后台更新版本
    void backgroundUpdate();

    // 查杀为处理异常文件
    void scanThreatsFile();
    // 设置主程序是否处于病毒扫描页面
    void SetIsbInVirusScanningPage(bool isbOn);

    // 设置扫描开始
    void setScanStart();

Q_SIGNALS:
    // 扫描回调函数对外信号
    void onDissectionResult(QString sPath, double dProgress, int eType, QString sDetailInfo);
    // 设置扫描状态回调函数对外信号
    void onSetDissectionStateResult(QString sPath, QString sMessage);
    // 修复问题文件函数对外信号
    void onRepairProblemFileResult(QString sPath, QString sMessage);
    // 信任问题文件函数对外信号
    void onReposeProblemFileResult(QString sPath, QString sMessage);
    // 信任问题文件函数对外信号
    void onSepProblemFileResult(QString sPath, QString sMessage);
    void onQueryReposeFile(std::vector<FileInfo>);    //查询信任文件对外
    void onQuerySepFile(std::vector<FileInfo>);      //查询隔离文件对外
    void signal_onQueryReposeFile(QString fileinfo);   //查询信任文件对外
    void signal_onQuerySepFile(QString fileinfo);   //查询隔离文件对外
    void signal_doDissectionFinished();   //扫描完成信号

    void signal_doQueryVersion(QString version);   //版本信号
    void signal_UpdateVersionResult(bool flag);    //版本更新结果

    void onBackgroundUpdate();   //后台更新
    void onRequstScanThreatsFile();   // 查杀未处理异常文件

    // 发送扫描开始
    void onSendScanStart();
private:
    void stopTest();
    // 读取本地病毒库版本
    void readLocalVdbVersion();
    // 保存本地病毒库版本
    void writeLocalVdbVersion();

private:
    AntiavAdaptor *m_adaptpr;
    CModule_Antiav m_cModule_Antiav;

    AntiavDataThread *m_antiavDataThread;                   // 扫描单独线程
    AntiavDataStatusModel *m_antiavDataStatusModel;         // 病毒查杀 设置状态/处理问题文件/查询 单例
    // 服务器病毒库版本
    QString m_sSrvVdbVer;
    // 本地病毒库版本
    QString m_sLocalVdbVer;
    int m_nTotal = 0;
    int m_nTotalReal = 0;
    // 是否处于扫描界面
    bool m_isbInVirusScanPage;
};





























