/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     wubowen <wubowen_cm@deepin.com>
 *
 * Maintainer: wubowen <wubowen_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "servicedbus.h"
#include "common/common.h"
#include <pwd.h>
#include <unistd.h>

const int timeInter = 3;

const QString service = "com.deepin.defender.daemonservice";
const QString path = "/com/deepin/defender/daemonservice";

ServiceDBus::ServiceDBus(QObject *parent)
    : QObject(parent)
    , m_monitornetflow(new MonitorNetFlow("com.deepin.defender.MonitorNetFlow", "/com/deepin/defender/MonitorNetFlow",
                                          QDBusConnection::systemBus(), this))
    , m_antiavInterServer(new AntiavInterServer("com.deepin.defender.antiav", "/com/deepin/defender/antiav",
                                          QDBusConnection::systemBus(), this))
    , m_DataInterFaceServer(new DataInterFaceServer("com.deepin.defender.datainterface", "/com/deepin/defender/datainterface",
                                                    QDBusConnection::sessionBus(), this))
    , m_managerInter(new ManagerInter("com.deepin.lastore", "/com/deepin/lastore",
                                      QDBusConnection::systemBus(), this))
    , m_updaterInter(new UpdaterInter("com.deepin.lastore", "/com/deepin/lastore",
                                      QDBusConnection::systemBus(), this))
    , m_accounts(new accounts("com.deepin.daemon.Accounts", "/com/deepin/daemon/Accounts", QDBusConnection::systemBus(), this))
    , m_contorl(new controlcenter("com.deepin.dde.ControlCenter", "/com/deepin/dde/ControlCenter", QDBusConnection::sessionBus(), this))
    , m_freedesktopSessDBusInter(nullptr)
{
    m_gset = new QGSettings("com.deepin.defender.deamonservice", QByteArray(), this);
    m_adaptor = new DaemonserviceAdaptor(this);
    if (!QDBusConnection::sessionBus().registerService(service) ||
            !QDBusConnection::sessionBus().registerObject(path, this)) {
        //qDebug() << "dbus service already registered! " << service;

        exit(0);
    }

    // org.freedesktop.DBus Inter
    m_freedesktopSessDBusInter = new QDBusInterface("org.freedesktop.DBus",
                                                  "/org/freedesktop/DBus",
                                                  "org.freedesktop.DBus",
                                                  QDBusConnection::sessionBus());

    QTimer *systemupdate = new QTimer;
    connect(systemupdate, &QTimer::timeout, [this] {
        //获取上次检测时间
        QString timeStr = m_gset->get(systemUpdatetime).toString();
        QDateTime lastCheckTime = QDateTime::fromString(timeStr, "yyyy-MM-dd-hh-mm-ss");;
        qint64 deltaTime = lastCheckTime.secsTo(QDateTime::currentDateTime());

        float minutes = deltaTime / 60;
        float hours = minutes / 60;
        float days = hours / 24;
        if (days > 1)
        {
            checkForUpdates();
            QString timeStr = QDateTime::currentDateTime().toString("yyyy-MM-dd-hh-mm-ss");
            m_gset->set(systemUpdatetime, timeStr);
        }
    });
    systemupdate->start(1000 * 60 * 10);

    //每隔3秒调启动一次各应用
    QTimer *timer = new QTimer;
    connect(timer, &QTimer::timeout, this, &ServiceDBus::StartAllService);
    timer->start(1000 * timeInter);
    StartAllService();

    int loginsafety = m_gset->get(loginSafety).toInt();
    m_adaptor->setSafetygrade(loginsafety);
    qDebug() << " caimengci for loginsagety date error log: - loginsafety" << loginsafety;

    int passwordchangeday = getPasswordChangeDay();//获取修改密码天数
    qDebug() << " caimengci for loginsagety date error log: - passwordchangeday" << passwordchangeday;

    int maturityday = m_gset->get(passwordexpiration).toInt();//获取密码到期天数
    m_adaptor->setMaturityDay(maturityday);
    qDebug() << " caimengci for loginsagety date error log: - maturityday" << maturityday;

    caseday(maturityday, passwordchangeday);
}

void ServiceDBus::caseday(int maturityday, int passwordchangeday)
{
    int value = 0;
    switch (maturityday) {
    case 0: {
        value = 30;
        if (passwordchangeday >= value) {
            showsystem(30);
            //qDebug() << " passwordchangeday>30";
        }
        break;
    }
    case 1: {

        value = 60;
        if (passwordchangeday >= value) {
            showsystem(60);
            //qDebug() << " passwordchangeday>60";
        }
        break;
    }
    case 2: {
        value = 90;
        if (passwordchangeday >= value) {
            showsystem(90);
            //qDebug() << " passwordchangeday>90";
        }
        break;
    }
    case 3: {
        value = 120;
        if (passwordchangeday >= value) {
            showsystem(120);
            //qDebug() << " passwordchangeday>120";
        }
        break;
    }
    case 4: {
        value = 150;
        if (passwordchangeday >= value) {
            showsystem(150);
            //qDebug() << " passwordchangeday>150";
        }
        break;
    }
    case 5: {
        value = 180;
        if (passwordchangeday >= value) {
            showsystem(180);
            //qDebug() << " passwordchangeday>180";
        }
        break;
    }
    default:
        //qDebug() << "nerver";
        break;
    }
}

int ServiceDBus::Charactertypes(QString password)
{
    int i = 0;
    int Number_flag = 0;
    int Capital_flag = 0;
    int Small_flag = 0;
    int Symbol_flag = 0;
    QByteArray ba = password.toLatin1();//QString 转换为 char*
    const char *s = ba.data();

    while (*s) {
        //qDebug() << "s = " << *s;
        //qDebug() << int(ba.at(i));//输出
        if ('0' <= *s && '9' >= *s) {
            Number_flag = 1 ;
        } else if ('A' <= *s && 'Z' >= *s) {
            Capital_flag = 1;
        } else if ('a' <= *s && 'z' >= *s) {
            Small_flag = 1;
        } else {
            Symbol_flag = 1;
        }
        s++;
    }
    i = Number_flag + Capital_flag + Small_flag + Symbol_flag;
    return i;
}

void ServiceDBus::ExitApp()
{
    qDebug() << Q_FUNC_INFO;
    qApp->quit();
}

void ServiceDBus::StartApp()
{
    qDebug() << Q_FUNC_INFO;
}

void ServiceDBus::StartAllService()
{
    //应用启动需要注意时序，服务直接彼此也会调用
    m_monitornetflow->StartApp();
    m_DataInterFaceServer->StartApp();
    m_antiavInterServer->StartApp();
    // 更新病毒扫描服务 "是否处于病毒查杀界面" 属性
    this->updateIsbInVirusScanningPage();
    if (m_DataInterFaceServer->lastError().type() != 0 || m_monitornetflow->lastError().type() != 0 || m_antiavInterServer->lastError().type() != 0)
    {
        qDebug() << " data interface server error : " << m_DataInterFaceServer->lastError().name() << m_DataInterFaceServer->lastError().type()
                    << " monitornetflow error : " << m_monitornetflow->lastError().name() << m_monitornetflow->lastError().type()
                    << " m_antiavInterServer error : " << m_antiavInterServer->lastError().name() << m_antiavInterServer->lastError().type();
    }
}

//退出任何一个服务进程，都需要将daemon退出，否则3秒后该服务进程又会重启
void ServiceDBus::ExitAllService()
{
    qDebug() << Q_FUNC_INFO;
    m_DataInterFaceServer->ExitApp();
    m_monitornetflow->ExitApp();
    m_antiavInterServer->ExitApp();
    ExitApp();
}

void ServiceDBus::ExitAnalysisService()
{
    qDebug() << Q_FUNC_INFO;
    m_DataInterFaceServer->ExitApp();
    ExitApp();
}

void ServiceDBus::ExitAutoStartService()
{
    qDebug() << Q_FUNC_INFO;
    m_DataInterFaceServer->ExitApp();
    ExitApp();
}

void ServiceDBus::ExitNetControlService()
{
    qDebug() << Q_FUNC_INFO;
    m_DataInterFaceServer->ExitApp();
    ExitApp();
}

void ServiceDBus::ExitTrafficDetailService()
{
    qDebug() << Q_FUNC_INFO;
    m_DataInterFaceServer->ExitApp();
    ExitApp();
}

void ServiceDBus::ExitMonitorNetFloService()
{
    qDebug() << Q_FUNC_INFO;
    m_monitornetflow->ExitApp();
    m_antiavInterServer->ExitApp();
    ExitApp();
}

void ServiceDBus::SetGsetting(int value)
{
    m_adaptor->setSafetygrade(value);
    m_gset->set(loginSafety, value);
}

void ServiceDBus::SetGesttingMaturityDay(int value)
{
    m_adaptor->setMaturityDay(value);
    m_gset->set(passwordexpiration, value);
}


QString ServiceDBus::notifySendPassword(const QString &password)
{
    Q_UNUSED(password);
#if 0
    int loginsafety = m_gset->get(loginSafety).toInt();
    QString ret = "";
    switch (static_cast<LoginSecurity>(loginsafety)) {
    case high : {
        if (password.size() >= 8 && Charactertypes(password) >= 3)
            ret = "true";
        else
            ret = getError(high);//QObject::tr("The password must have at least 8 characters, and contain at least 3 of the four available character types: lowercase letters, uppercase letters, numbers, and symbols");
        break;
    }
    case middle : {
        if (password.size() >= 6 && Charactertypes(password) >= 2)
            ret = "true";
        else
            ret = ret = getError(middle);//QObject::tr("The password must have at least 6 characters, and contain at least 2 of the four available character types: lowercase letters, uppercase letters, numbers, and symbols");
        break;
    }
    case low :
        ret = "true";
        break;
    }

    return ret;
#endif
    return "";
}

QString ServiceDBus::getError(LoginSecurity type)
{
    QString retsult = "false";
    QString language = QLocale::system().name();
    switch (type) {
    case high:
        if (language == "zh_CN") {
            retsult = "密码最少8位，至少同时包含小写字母、大写字母、数字、符号中的3种";
        } else {
            retsult = tr("The password must have at least 8 characters, and contain at least 3 of the four available character types: lowercase letters, uppercase letters, numbers, and symbols");
        }
        break;
    case middle:
        if (language == "zh_CN") {
            retsult = "密码最少6位，至少同时包含小写字母、大写字母、数字、符号中的2种";
        } else {
            retsult = tr("The password must have at least 6 characters, and contain at least 2 of the four available character types: lowercase letters, uppercase letters, numbers, and symbols");
        }
        break;
    default:
        break;
    }

    return retsult;
}

QString ServiceDBus::GetPwdError()
{
    int loginsafety = m_gset->get(loginSafety).toInt();
    QString retsult = "false";
    QString language = QLocale::system().name();
    switch (static_cast<LoginSecurity>(loginsafety)) {
    case high:
        if (language == "zh_CN") {
            retsult = "密码最少8位，至少同时包含小写字母、大写字母、数字、符号中的3种";
        } else {
            retsult = tr("The password must have at least 8 characters, and contain at least 3 of the four available character types: lowercase letters, uppercase letters, numbers, and symbols");
        }
        break;
    case middle:
        if (language == "zh_CN") {
            retsult = "密码最少6位，至少同时包含小写字母、大写字母、数字、符号中的2种";
        } else {
            retsult = tr("The password must have at least 6 characters, and contain at least 2 of the four available character types: lowercase letters, uppercase letters, numbers, and symbols");
        }
        break;
    default:
        break;
    }

    return retsult;
}

int ServiceDBus::GetPwdLen()
{
    int loginsafety = m_gset->get(loginSafety).toInt();
    int ret = 0;
    switch (static_cast<LoginSecurity>(loginsafety)) {
    case high : {
        ret = highlen_return;
        break;
    }
    case middle : {
        ret = middlelen_return;
        break;
    }
    default:
        break;
    }
    Q_EMIT UpdateUI();
    return ret;
}

int ServiceDBus::GetPwdTypeLen()
{
    int loginsafety = m_gset->get(loginSafety).toInt();
    int ret = 0;
    switch (static_cast<LoginSecurity>(loginsafety)) {
    case high : {
        ret = hightypelen_return;
        break;
    }
    case middle : {
        ret = middletypelen_return;
        break;
    }
    default:
        break;
    }
    return ret;
}

void ServiceDBus::JumpScreenSecurity()
{
    //qDebug() << "JumpScreenSecurity";
    Q_EMIT GotoScreenSecurity();
}

void ServiceDBus::JumpUpdatePolicy()
{
//    qDebug() << "JumpUpdatePolicy";
    Q_EMIT GotoUpdatePolicy();
}

int ServiceDBus::GetSystemUpdate()
{
    return m_gset->get(systemUpdate).toInt();
}

void ServiceDBus::checkForUpdates()
{
    if (checkDbusIsValid()) {
        //qDebug() << " checkDbusIsValid . do nothing";
        return;
    }

    m_gset->set(systemUpdate, 0);
    QDBusPendingCall call = m_managerInter->UpdateSource();
    QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(call, this);
    connect(watcher, &QDBusPendingCallWatcher::finished, [this, call, watcher] {
        if (!call.isError())
        {
            QDBusReply<QDBusObjectPath> reply = call.reply();
            const QString jobPath = reply.value().path();
            setCheckUpdatesJob(jobPath);
        } else
        {
            if (!m_checkUpdateJob.isNull()) {
                m_managerInter->CleanJob(m_checkUpdateJob->id());
            }
            qWarning() << "UpdateFailed, check for updates error: " << call.error().message();
        }
        watcher->deleteLater();
    });
}

bool ServiceDBus::checkDbusIsValid()
{
    bool ret = true;

    //如果检查任务已经存在，说明dbus有任务在执行
    if (!m_checkUpdateJob.isNull()) {
        if (m_checkUpdateJob->isValid() && ret) {
            return true;
        } else {
            //执行结束，删除任务
            m_checkUpdateJob->deleteLater();
        }
    }

    return false;
}

void ServiceDBus::setCheckUpdatesJob(const QString &jobPath)
{
    if (!m_checkUpdateJob.isNull())
        return;

    m_checkUpdateJob = new JobInter("com.deepin.lastore", jobPath, QDBusConnection::systemBus(), this);
    connect(m_checkUpdateJob, &__Job::StatusChanged, [this](const QString & status) {
        //qDebug() << "[setCheckUpdatesJob]status is: " << status;
        if (status == "failed") {
            qWarning() << "check for updates job failed";
            m_managerInter->CleanJob(m_checkUpdateJob->id());
            m_checkUpdateJob->deleteLater();
        } else if (status == "success" || status == "succeed") {
            m_checkUpdateJob->deleteLater();

            //apt update之后可以查看最新更新信息
            QDBusPendingCallWatcher *w = new QDBusPendingCallWatcher(m_updaterInter->ApplicationUpdateInfos(QLocale::system().name()), this);
            connect(w, &QDBusPendingCallWatcher::finished, this, &ServiceDBus::onAppUpdateInfoFinished);
        }
    });
}

void ServiceDBus::onAppUpdateInfoFinished(QDBusPendingCallWatcher *w)
{
    //qDebug() << "onAppUpdateInfoFinished";
    QDBusPendingReply<AppUpdateInfoList> reply = *w;

    if (reply.isError()) {
        const QJsonObject &obj = QJsonDocument::fromJson(reply.
                                                         error().
                                                         message().
                                                         toUtf8())
                                 .object();

        if (obj["Type"].toString().contains("dependenciesBroken")) {
        } else {
            qWarning() << Q_FUNC_INFO << "UpdateFailed, error msg : " << obj["Type"].toString();
        }

        w->deleteLater();
//        return;
    }

    //检查更新结束
    if (m_updaterInter->updatableApps().size() > 0
            || m_updaterInter->updatablePackages().size() > 0)
        m_gset->set(systemUpdate, 1);
    else {
        m_gset->set(systemUpdate, 0);
    }
    w->deleteLater();
}

qint64 ServiceDBus::getPasswordChangeDay()
{
    struct passwd *pws;
    pws = getpwuid(getuid());
    QString m_currentUserName = QString(pws->pw_name);
    //qDebug() << "m_currentUserName = " << m_currentUserName;
    user = m_accounts->FindUserByName(m_currentUserName);//获取当前用户名称
    //qDebug() << "user = " << user;
    m_usermanage = new DbususerManager(user);
    qint64 date =  m_usermanage->passwordLastChange();//获取上次设置密码距1970年1月1日的天数
    const QDateTime datetime(QDateTime::currentDateTime());
    QDateTime dt = QDateTime(QDate(1970, 1, 1));//获取当天距1970年1月1日的天数
    qint64 passwordchangeday = dt.daysTo(datetime) - date;
    return passwordchangeday;
}

void ServiceDBus::showsystem(int maturityday)
{
    QDBusMessage m = QDBusMessage::createMethodCall("com.deepin.dde.osd"
                                                    , "/com/deepin/dde/Notification"
                                                    , "com.deepin.dde.Notification"
                                                    , "Notify");
    QStringList m_action;
    m_action << "_open1" << tr("Change Password");  //添加按钮
    QVariantMap m_inform;//按钮的点击操作
//    m_inform.insert(QString("x-deepin-action-_open1"),QVariant(QString("/usr/bin/dde-control-center,--show,account")));
    m_inform.insert(QString("x-deepin-action-_open1"), QVariant(QString("/usr/bin/dde-control-center,--m,accounts")));
    m << QString("deepin-defender")
      << uint(0)
      << QString("deepin-defender")
      << QString("")
      << tr("You have not changed the password for %1 days, please change it regularly to improve system security").arg(maturityday)
      << m_action
      << m_inform
      << int(5000);
    QDBusMessage respone = QDBusConnection::sessionBus().call(m);
    if (respone.type() == QDBusMessage::ReplyMessage) {
        QStringList m_args; m_args << "";
        if (!QDBusConnection::sessionBus().connect("com.deepin.dde.osd", "/com/deepin/dde/Notification"
                                                   , "com.deepin.dde.Notification"
                                                   , "ActionInvoked",
                                                   this, SLOT(On_Click(uint, const QString)))) {
            //按钮点击信号未连接到本地;
            exit(-1);
        }
    }
}

void ServiceDBus::On_Click(uint in0, const QString &in1)
{
    Q_UNUSED(in0);
    Q_UNUSED(in1);
//    m_contorl->ShowModule("accounts");
}

bool ServiceDBus::isbDefenderMainWindowRunning()
{
    bool isbRunning = false;
    QStringList sServerList;
    QDBusReply<QStringList> reply = m_freedesktopSessDBusInter->call("ListNames");
    if (reply.isValid()) {
        sServerList = reply.value();
//        qDebug() << "all servers: " << sServerList;
    } else {
        qDebug() << "get ListActivatableNames failed" << reply.error();
    }

    // find com.deepin.defender.hmiscreen
    for (QString sServer : sServerList) {
        if (sServer.contains("com.deepin.defender.hmiscreen")) {
            isbRunning = true;
            break;
        }
    }

    return isbRunning;
}

void ServiceDBus::updateIsbInVirusScanningPage()
{
    // 若主窗口已经关闭，则设置病毒扫描服务的 "是否处于病毒查杀界面" 属性为false
    if (!isbDefenderMainWindowRunning()) {
        // 如果关闭了，停止扫描状态
        m_antiavInterServer->setDissection(2);
        m_antiavInterServer->SetIsbInVirusScanningPage(false);
    }
}
