/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     wubowen <wubowen_cm@deepin.com>
 *
 * Maintainer: wubowen <wubowen_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once

#include <QObject>
#include <QDBusConnection>
#include <QDBusInterface>

#include "daemonservice_adaptor.h"
#include "com_deepin_defender_daemonservice.h"
#include "com_deepin_defender_monitornetflow.h"
// 病毒查杀服务
#include "com_deepin_defender_antiav.h"
#include "com_deepin_defender_datainterface.h"
#include "com_deepin_lastore_jobmanager.h"
#include "com_deepin_lastore_updater.h"
#include "com_deepin_lastore_job.h"
#include "com_deepin_daemon_systeminfo.h"
#include "QGSettings/QGSettings"
#include "dbususermanager.h"
#include "com_deepin_daemon_accounts.h"
#include "com_deepin_dde_controlcenter.h"

using MonitorNetFlow = com::deepin::defender::MonitorNetFlow;
using AntiavInterServer = com::deepin::defender::antiav;
// 病毒查杀服务接口
using DaemonService = com::deepin::defender::daemonservice;
using ManagerInter = com::deepin::lastore::Manager;
using UpdaterInter = com::deepin::lastore::Updater;
using JobInter = com::deepin::lastore::Job;
using SystemInfoInter = com::deepin::daemon::SystemInfo;
using accounts = com::deepin::daemon::Accounts;
using controlcenter = com::deepin::dde::ControlCenter;

using DataInterFaceServer = com::deepin::defender::datainterface;

enum LoginSecurity {high = 0, middle, low};
const QString loginSafety = "login-safety";
const QString systemUpdate = "system-update";
const QString systemUpdatetime = "last-anti-virus-date";
const QString passwordexpiration = "password-expiration-days";

class ServiceDBus : public QObject
{
    Q_OBJECT
public:
    explicit ServiceDBus(QObject *parent = nullptr);
    int Charactertypes(QString password);
    void checkForUpdates();
    bool checkDbusIsValid();
    void setCheckUpdatesJob(const QString &jobPath);
    void onAppUpdateInfoFinished(QDBusPendingCallWatcher *w);
    qint64 getPasswordChangeDay();
    void caseday(int maturityday, int passwordchangeday);
    void showsystem(int maturityday);

Q_SIGNALS:
    void UpdateUI();
    void GotoScreenSecurity();
    void GotoUpdatePolicy();

public Q_SLOTS:
    void ExitApp();
    void StartApp();
    void StartAllService();
    void ExitAllService();
    void ExitAnalysisService();
    void ExitAutoStartService();
    void ExitNetControlService();
    void ExitTrafficDetailService();
    void ExitMonitorNetFloService();
    void SetGesttingMaturityDay(int value);
    void SetGsetting(int value);
    QString notifySendPassword(const QString &password);
    QString getError(LoginSecurity type);
    QString GetPwdError();
    int GetPwdLen();
    int GetPwdTypeLen();
    void JumpScreenSecurity();
    void JumpUpdatePolicy();
    int GetSystemUpdate();
    void On_Click(uint in0, const QString &in1);

private:
    // 判断主窗口是否启动
    bool isbDefenderMainWindowRunning();
    // 更新病毒扫描服务 "是否处于病毒查杀界面" 属性
    void updateIsbInVirusScanningPage();

private:
    DaemonserviceAdaptor *m_adaptor;
    MonitorNetFlow *m_monitornetflow;
    // 病毒查杀服务接口
    AntiavInterServer *m_antiavInterServer;
    DataInterFaceServer *m_DataInterFaceServer = nullptr;
    QGSettings *m_gset;
    DaemonService *m_daemonservice;
    ManagerInter *m_managerInter;
    UpdaterInter *m_updaterInter;
    QPointer<JobInter> m_checkUpdateJob;
    accounts *m_accounts;
    controlcenter *m_contorl;
    DbususerManager *m_usermanage;
    QString user;
    // org.freedesktop.DBus Inter
    QDBusInterface *m_freedesktopSessDBusInter;
};
