/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     wubowen <wubowen_cm@deepin.com>
 *
 * Maintainer: wubowen <wubowen_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "operatedbusdata.h"
#include "common.h"
#include "window/modules/antivirus/usbscanwork.h"

#include <DPinyin>
#include <QDebug>

#define Action_Flag_Disable     0                   // 状态标志 - 不允许
#define Action_Flag_Enable      1                   // 状态标志 - 允许

static const QString FIRE_WALL_ON_OFF = "fire-wall-on-off";
OperateDBusData::OperateDBusData(QObject *parent)
    : QObject(parent)
    , m_launcherInter(new Launcherd("com.deepin.dde.daemon.Launcher", "/com/deepin/dde/daemon/Launcher", QDBusConnection::sessionBus(), this))
    , m_startManagerInterface(new DBusStartManager)
    , m_threatData(nullptr)
    , m_gsFirewall(new QGSettings("com.deepin.dde.deepin-defender-datainterface", QByteArray(), this))
{
    m_adaptpr = new DatainterfaceAdaptor(this);
    if (!QDBusConnection::sessionBus().registerService(service) ||
            !QDBusConnection::sessionBus().registerObject(path, this)) {
        qDebug() << "dbus service already registered! " << service;

        exit(0);
    }

    // 初始化联网管控
    initialize();
    m_threatData = new ThreatData();

    // usb存储设备病毒扫描
    UsbScanWork *usbScanWork = new UsbScanWork(this);
    Q_UNUSED(usbScanWork);

    // 查询异常数据条数
//    connect(this,&OperateDBusData::notifySelectThreatsCount,m_threatData,&ThreatData::notifySelectThreatCount);
//    connect(m_threatData,&ThreatData::threadsCount,this,[=](int count){
//        // 发送异常数据条数
//        emit sendThreadsCount(count);
//    });
}

void OperateDBusData::StartApp()
{
}

void OperateDBusData::ExitApp()
{
    qApp->quit();
}

void OperateDBusData::preInitialize()
{
}

void OperateDBusData::initialize()
{
    // 流量详情
    registerDefenderProcInfoMetaType();
    registerDefenderProcInfoListMetaType();
    GetProcInfoInter::instance();

    m_trafficWorker = new TrafficDetailsWorker;
    m_monitorNetFlowInter = m_trafficWorker->getNetFlowMonitorInter();

    // all app
    connect(m_trafficWorker, &TrafficDetailsWorker::sendAllAppFlowData, this, &OperateDBusData::recAllAppFlowList);
    // app
    connect(m_trafficWorker, &TrafficDetailsWorker::sendAppFlowData, m_adaptpr, &DatainterfaceAdaptor::SendAppFlowList);

    // 初始化联网管控
    initNetControl();

    // 开机自启动管理
    m_startupMonitor = new StartupMonitor;
    m_startupMonitor->addWatchPath("~/.config/autostart");
    StartupMonitor *hand = StartupMonitor::getInstance();
    connect(hand, SIGNAL(doDataChage(bool, QString)), this, SLOT(refreshData(bool, QString)));

    // 初始化设置防火墙开关状态
    setFireWallSwitchStatus(m_gsFirewall->get(FIRE_WALL_ON_OFF).toBool());

    //usb管控
    connect(m_monitorNetFlowInter, &MonitorNetFlow::requestShowBlockAllSysNotify, this, &OperateDBusData::showBlockAllSysNotify);
    connect(m_monitorNetFlowInter, &MonitorNetFlow::requestShowOnlyWhiteListSysNotify, this, &OperateDBusData::showOnlyWhiteListSysNotify);
}

// 初始化联网管控
void OperateDBusData::initNetControl()
{
    // 新联网管控/远程访问
    m_netControlModel = new NetControlModel;
    m_remControlModel = new RemControlModel;
    m_netControlMonitor = new NetControlMonitor(m_netControlModel, m_remControlModel);
    connect(m_netControlModel, SIGNAL(sendNetAppsChange(QList<DefenderProcInfo>)), this, SLOT(setNetAppsChange(QList<DefenderProcInfo>)));
    connect(m_remControlModel, SIGNAL(sendRemAppsChange(QList<DefenderProcInfo>)), this, SLOT(setRemAppsChange(QList<DefenderProcInfo>)));

    connect(m_netControlMonitor, SIGNAL(sendNetControlSignal(ACLNetwork *, QString, QString, QString)),
            m_netControlModel, SLOT(doNetControlSlot(ACLNetwork *, QString, QString, QString)));
    connect(m_netControlMonitor, SIGNAL(sendRemControlSignal(ACLNetwork *, QString, QString, QString)),
            m_remControlModel, SLOT(doRemControlSlot(ACLNetwork *, QString, QString, QString)));

    // 远程访问设置为默认允许
    connect(m_remControlModel, SIGNAL(sendRemDefaultAllowStatus()), this, SLOT(setRemDefaultAllowStatus()));
}

/*---------------------新联网管控/远程访问---------------------*/
void OperateDBusData::setFireWallSwitchStatus(bool isbFireWall)
{
    m_netControlMonitor->setFireWallSwitchStatus(isbFireWall);
    // 开启/关闭进程流量监控功能
    m_monitorNetFlowInter->call("EnableNetFlowMonitor", QVariant(isbFireWall));
}

bool OperateDBusData::getFireWallSwitchStatus()
{
    return m_netControlMonitor->getFireWallWwitchStatus();
}

void OperateDBusData::getNetAppsInfo()
{
    m_netControlModel->refreshNetControlInfos();
}

void OperateDBusData::setNetAppsChange(QList<DefenderProcInfo> sNetAppsList)
{
    Q_EMIT m_adaptpr->requestNetAppsChange(sNetAppsList);
}

void OperateDBusData::setNetAppStatusChange(QString sDkgName, int nStatus)
{
    qDebug() << "sDkgName value: " << sDkgName << "nStatus value: " << nStatus;

    m_netControlModel->setNetControlStatusChange(sDkgName, nStatus);
}

void OperateDBusData::setNetRequestReply(QString sPkgName, QString sId, int nType)
{
    m_netControlModel->setNetRequestReply(sPkgName, sId, nType);
}

void OperateDBusData::setNetControlDefaultStatus(int nDefaultStatus)
{
    m_netControlModel->setNetControlDefaultStatus(nDefaultStatus);
}

int OperateDBusData::getNetControlDefaultStatus()
{
    return m_netControlModel->getNetControlDefaultStatus();
}

void OperateDBusData::getRemAppsInfo()
{
}

void OperateDBusData::setRemAppsChange(QList<DefenderProcInfo> sNetAppsList)
{
    Q_EMIT m_adaptpr->requestRemAppsChange(sNetAppsList);
}

void OperateDBusData::setRemAppStatusChange(QString sDkgName, int nStatus)
{
    Q_UNUSED(sDkgName);
    Q_UNUSED(nStatus);
}

void OperateDBusData::setRemRequestReply(QString sPkgName, QString sId, int nType)
{
    m_remControlModel->setRemRequestReply(sPkgName, sId, nType);
}

void OperateDBusData::setRemControlDefaultStatus(int nDefaultStatus)
{
    m_remControlModel->setRemControlDefaultStatus(nDefaultStatus);
}

int OperateDBusData::getRemControlDefaultStatus()
{
    return m_remControlModel->getRemControlDefaultStatus();
}

void OperateDBusData::setRemDefaultAllowStatus()
{
    Q_EMIT m_adaptpr->requestRemDefaultAllowStatus();
}


// 联网管控禁止弹框
void OperateDBusData::showNetControlDialog(const QString &sPkgName, const QString &sAppName)
{
    if (m_netDisableDialog == nullptr) {
        m_netDisableDialog = new NetDisableDialog(sPkgName, sAppName);
        connect(m_netDisableDialog, &NetDisableDialog::onDialogClose, [ = ]() {
            m_netDisableDialog = nullptr;
        });
        m_netDisableDialog->setAttribute(Qt::WA_DeleteOnClose);
        m_netDisableDialog->setOnButtonClickedClose(true);
        m_netDisableDialog->show();
    }
}

// 联网管控询问弹框
void OperateDBusData::showNetControlAskDialog(const QString &sPkgName, const QString &sAppName, const QString &sId)
{
    if (m_netAskDialog == nullptr) {
        m_netAskDialog = new NetAskDialog(sPkgName, sAppName, sId);
        connect(m_netAskDialog, &NetAskDialog::onDialogClose, [ = ]() {
            m_netAskDialog = nullptr;
        });
        m_netAskDialog->setAttribute(Qt::WA_DeleteOnClose);
        m_netAskDialog->setOnButtonClickedClose(true);
        m_netAskDialog->show();
    }
}

// 远程访问禁止弹框
void OperateDBusData::showRemControlDialog(const QString &sPkgName, const QString &sAppName)
{
    qDebug() << "sPkgName value: " << sPkgName << "sAppName value: " << sAppName;
    if (m_remDisableDialog == nullptr) {
        m_remDisableDialog = new RemDisableDialog(sPkgName, sAppName);
        connect(m_remDisableDialog, &RemDisableDialog::onDialogClose, [ = ]() {
            m_remDisableDialog = nullptr;
        });
        m_remDisableDialog->setAttribute(Qt::WA_DeleteOnClose);
        m_remDisableDialog->setOnButtonClickedClose(true);
        m_remDisableDialog->show();
    }
}

// 远程访问询问弹框
void OperateDBusData::showRemControlAskDialog(const QString &sPkgName, const QString &sAppName, const QString &sId)
{
    if (m_remAskDialog == nullptr) {
        m_remAskDialog = new RemAskDialog(sPkgName, sAppName, sId);
        connect(m_remAskDialog, &RemAskDialog::onDialogClose, [ = ]() {
            m_remAskDialog = nullptr;
        });
        m_remAskDialog->setAttribute(Qt::WA_DeleteOnClose);
        m_remAskDialog->setOnButtonClickedClose(true);
        m_remAskDialog->show();
    }
}

/*---------------------开机自启动管理---------------------*/
bool OperateDBusData::startLauncherManage()
{
    m_EnableAppsInfo.clear();
    m_DisableAppsInfo.clear();
    m_EnableData.clear();
    m_DisableData.clear();

    m_reply = m_launcherInter->GetAllItemInfos();
    m_reply.waitForFinished();
    if (m_reply.isError()) {
        qWarning() << "data is empty, quit!!";
        qWarning() << m_reply.error();
    }

    // 添加数据到QMap容器
    m_mapEnable.clear();
    m_mapDisable.clear();
    const LauncherItemInfoList &datas = m_reply.value();
    for (const auto &it : datas) {
        QString sInfo = it.Path
                        + "," + it.Icon
                        + "," + it.ID
                        + "," + it.Name;

        // 判断该应用状态（是否是自启动）
        bool bStatus = isAutostart(it.Path);

        QString sName = DTK_CORE_NAMESPACE::Chinese2Pinyin(it.Name);
        for (auto iterEnable = m_mapEnable.begin(); iterEnable != m_mapEnable.end(); ++iterEnable) {
            if (sName == iterEnable.key())
                sName = sName + "_deepin";
        }

        for (auto iterDisable = m_mapDisable.begin(); iterDisable != m_mapDisable.end(); ++iterDisable) {
            if (sName == iterDisable.key())
                sName = sName + "_deepin";
        }
        sInfo = sInfo + "," + sName;

        if (bStatus) {
            m_EnableData.append(sInfo);
            m_mapEnable[sName] = sInfo;
        } else {
            m_DisableData.append(sInfo);
            m_mapDisable[sName] = sInfo;
        }
    }

    for (int i = 0; i < m_EnableData.count(); ++i) {
        if (i == 0)
            m_EnableAppsInfo = m_EnableData.at(0);
        else
            m_EnableAppsInfo = m_EnableAppsInfo + "|" + m_EnableData.at(i);
    }

    for (int i = 0; i < m_DisableData.count(); ++i) {
        if (i == 0)
            m_DisableAppsInfo = m_DisableData.at(0);
        else
            m_DisableAppsInfo = m_DisableAppsInfo + "|" + m_DisableData.at(i);
    }

    return true;
}
QString OperateDBusData::getAppsInfoEnable()
{
    return m_EnableAppsInfo;
}
QString OperateDBusData::getAppsInfoDisable()
{
    return m_DisableAppsInfo;
}

// 判断是否是开机自启
bool OperateDBusData::isAutostart(QString sPath)
{
    return m_startManagerInterface->IsAutostart(sPath);
}

// 执行开机自启动管理 开启/禁止操作
bool OperateDBusData::exeAutostart(int nStatus, QString sPath)
{
    bool flag = false;
    // 执行自启动关闭操作
    if (Action_Flag_Disable == nStatus) {
        QDBusPendingReply<bool> reply = m_startManagerInterface->RemoveAutostart(sPath);
        reply.waitForFinished();
        if (!reply.isError()) {
            flag = true;
        }
    } else {
        // 执行自启动开启操作
        QDBusPendingReply<bool> reply =  m_startManagerInterface->AddAutostart(sPath);
        reply.waitForFinished();
        if (!reply.isError()) {
            flag = true;
        }
    }

    return flag;
}
// 外部调用刷新表格数据
void OperateDBusData::refreshData(bool bAdd, QString sID)
{
    Q_EMIT AccessRefreshData(bAdd, sID);
}

DefenderProcInfoList OperateDBusData::GetAllProcInfos()
{
    return  m_trafficWorker->getFlowInfoList();
}

void OperateDBusData::recAllAppFlowList(DefenderProcInfoList allAppFlowList, FlowRangeStyle range)
{
    QString sFlowRange;
    // 根据时间跨度枚举 得到 时间跨度字符串
    switch (range) {
    case FlowRangeStyle::None:
        sFlowRange = "None";
        break;
    case FlowRangeStyle::Today:
        sFlowRange = "Today";
        break;
    case FlowRangeStyle::Yesterday:
        sFlowRange = "Yesterday";
        break;
    case FlowRangeStyle::ThisMonth:
        sFlowRange = "ThisMonth";
        break;
    case FlowRangeStyle::LastMonth:
        sFlowRange = "LastMonth";
        break;
    }
    Q_EMIT m_adaptpr->SendAllAppFlowList(allAppFlowList, sFlowRange);//每秒发送系统使用过流量的应用的信息
}

void OperateDBusData::notifyGetYesterdayFlowData()
{
    m_trafficWorker->notifyGetYesterdayFlowData();
}

void OperateDBusData::notifyGetTodayFlowData()
{
    m_trafficWorker->notifyGetTodayFlowData();
}

void OperateDBusData::notifyGetLastMonthFlowData()
{
    m_trafficWorker->notifyGetLastMonthFlowData();
}

// 通知停止获取所有应用流量数据
void OperateDBusData::notifyGetThisMonthFlowData()
{
    m_trafficWorker->notifyGetThisMonthFlowData();
}

// 通知停止获取所有应用流量数据
void OperateDBusData::NotifyDontGetFlowData()
{
    m_trafficWorker->NotifyDontGetFlowData();
}

void OperateDBusData::notifyGetAppYesterdayFlowData(QString appname)
{
    m_trafficWorker->notifyGetAppYesterdayFlowData(appname);
}

void OperateDBusData::notifyGetAppTodayFlowData(QString appname)
{
    m_trafficWorker->notifyGetAppTodayFlowData(appname);
}

void OperateDBusData::notifyGetAppLastMonthFlowData(QString appname)
{
    m_trafficWorker->notifyGetAppLastMonthFlowData(appname);
}

void OperateDBusData::notifyGetAppThisMonthFlowData(QString appname)
{
    m_trafficWorker->notifyGetAppThisMonthFlowData(appname);
}

void OperateDBusData::showBlockAllSysNotify()
{
    qDebug() << "showBlockAllSysNotify";
    QDBusMessage m = QDBusMessage::createMethodCall("com.deepin.dde.Notification"
                                                    , "/com/deepin/dde/Notification"
                                                    , "com.deepin.dde.Notification"
                                                    , "Notify");
    QStringList m_action;
    m_action << "_open1" << tr("Settings"); //添加按钮
    QVariantMap m_inform;//按钮的点击操作
    QString openSysSafetyPageCmd = "qdbus,com.deepin.defender.hmiscreen,"
                                   "/com/deepin/defender/hmiscreen,"
                                   "com.deepin.defender.hmiscreen.ShowPage,"
                                   "systemsafety,UsbConnection";

    qDebug() << openSysSafetyPageCmd;
    m_inform.insert(QString("x-deepin-action-_open1"), openSysSafetyPageCmd);
    m << QString("deepin-defender")
      << uint(0)
      << QString("deepin-defender")
      << QString("") << tr("Your system is set to block all USB storage devices")
      << m_action
      << m_inform
      << int(5000);
    QDBusMessage respone = QDBusConnection::sessionBus().call(m);
}


void OperateDBusData::showOnlyWhiteListSysNotify()
{
    qDebug() << "showOnlyWhiteListSysNotify";
    QDBusMessage m = QDBusMessage::createMethodCall("com.deepin.dde.Notification"
                                                    , "/com/deepin/dde/Notification"
                                                    , "com.deepin.dde.Notification"
                                                    , "Notify");
    QStringList m_action;
    m_action << "_open1" << tr("Settings"); //添加按钮
    QVariantMap m_inform;//按钮的点击操作
    QString openSysSafetyPageCmd = "qdbus,com.deepin.defender.hmiscreen,"
                                   "/com/deepin/defender/hmiscreen,"
                                   "com.deepin.defender.hmiscreen.ShowPage,"
                                   "systemsafety,UsbConnection";

    qDebug() << openSysSafetyPageCmd;
    m_inform.insert(QString("x-deepin-action-_open1"), openSysSafetyPageCmd);
    m << QString("deepin-defender")
      << uint(0)
      << QString("deepin-defender")
      << QString("") << tr("Your USB device is not in the whitelist and has been blocked")
      << m_action
      << m_inform
      << int(5000);
    QDBusMessage respone = QDBusConnection::sessionBus().call(m);
}

// 添加异常数据
bool OperateDBusData::notifyAppendThreatDataByPath(QString sfilePath)
{
    bool flag = m_threatData->appendThreatDataByPath(sfilePath);
    return flag;
}

// 删除异常数据
bool OperateDBusData::notifyDeleteThreatDataByPath(QString sfilePath)
{
    bool flag = m_threatData->deleteThreatDataByPath(sfilePath);
    return flag;
}

// 查询异常数据
bool OperateDBusData::notifySelectThreatDataByPath(QString sfilePath)
{
    bool flag = m_threatData->selectThreatDataByPath(sfilePath);
    return flag;
}

int OperateDBusData::getSelectThreatsCount()
{
    int count = m_threatData->notifySelectThreatCount();
    return count;
   // emit notifySelectThreatsCount();
}

// 查询总的数据
QString OperateDBusData::notifySelectThreatCount()
{
    QStringList list = m_threatData->selectThreatCount();
    QString spath;
    for (int i = 0; i < list.size(); i++) {
        QString str ;
        str = list.at(i);
        spath = spath + str + "[*]" ;
    }
    qDebug() << "查询异常list======" << list << "拼接以后=======" << spath;
    return spath;
}
