/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     wubowen <wubowen_cm@deepin.com>
 *
 * Maintainer: wubowen <wubowen_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once

#include "datainterface_adaptor.h"
#include "com_deepin_dde_daemon_launcherd.h"
// 联网管控/远程访问
#include "window/modules/netcontrol/netcontrolmodel.h"
#include "window/modules/remcontrol/remcontrolmodel.h"
#include "window/modules/networkmonitor/netcontrolmonitor.h"

#include "window/modules/netcontrol/netdisabledialog.h"
#include "window/modules/netcontrol/netaskdialog.h"
#include "window/modules/remcontrol/remdisabledialog.h"
#include "window/modules/remcontrol/remaskdialog.h"

// 开机自启动
#include "window/modules/startup/startupmonitor.h"
#include "window/modules/startup/dbustartmanager.h"

// 流量详情
#include "window/modules/trafficdetails/trafficdetailsworker.h"
#include "window/modules/trafficdetails/usedata.h"

#include "window/modules/antivirus/threatdata.h"

// 流量详情名称解析
#include "window/modules/common/getprocinfointer.h"

#include <QObject>
#include <QGSettings>

using Launcherd = com::deepin::dde::daemon::Launcher;

class OperateDBusData : public QObject
{
    Q_OBJECT
public:
    explicit OperateDBusData(QObject *parent = nullptr);

Q_SIGNALS:

public Q_SLOTS:
    // 接口
    void StartApp();
    void ExitApp();
    void preInitialize();

    // 初始化
    void initialize();
    void initNetControl();

    // 新联网管控/远程访问
    void setFireWallSwitchStatus(bool);
    bool getFireWallSwitchStatus();
    void setNetAppsChange(QList<DefenderProcInfo>);
    void getNetAppsInfo();
    void setNetAppStatusChange(QString, int);
    void setNetControlDefaultStatus(int);
    int getNetControlDefaultStatus();
    void setNetRequestReply(QString, QString, int);

    void setRemAppsChange(QList<DefenderProcInfo>);
    void getRemAppsInfo();
    void setRemAppStatusChange(QString, int);
    void setRemControlDefaultStatus(int);
    int getRemControlDefaultStatus();
    void setRemRequestReply(QString, QString, int);
    void setRemDefaultAllowStatus();

    // 联网管控禁止/询问弹框
    void showNetControlDialog(const QString &sPkgName, const QString &sAppName);
    void showNetControlAskDialog(const QString &sPkgName, const QString &sAppName, const QString &sId);

    // 远程访问禁止/询问弹框
    void showRemControlDialog(const QString &sPkgName, const QString &sAppName);
    void showRemControlAskDialog(const QString &sPkgName, const QString &sAppName, const QString &sId);


    // 开机自启动管理
    bool startLauncherManage();
    QString getAppsInfoEnable();
    QString getAppsInfoDisable();
    bool isAutostart(QString sPath);
    bool exeAutostart(int nStatus, QString sPath);
    void refreshData(bool bAdd, QString sID);

    // 流量详情
    DefenderProcInfoList GetAllProcInfos();
    void recAllAppFlowList(DefenderProcInfoList, FlowRangeStyle range);
    //通知获取所有应用当前时间跨度下流量数据总和
    void notifyGetYesterdayFlowData();
    void notifyGetTodayFlowData();
    void notifyGetLastMonthFlowData();
    void notifyGetThisMonthFlowData();
    // 通知停止获取所有应用流量数据
    void NotifyDontGetFlowData();
    //通知获取应用当前时间跨度下流量数据
    void notifyGetAppYesterdayFlowData(QString appname);
    void notifyGetAppTodayFlowData(QString appname);
    void notifyGetAppLastMonthFlowData(QString appname);
    void notifyGetAppThisMonthFlowData(QString appname);

    //usb管控：显示限制所有设备系统通知
    void showBlockAllSysNotify();
    //usb管控：显示限制非白名单设备系统通知
    void showOnlyWhiteListSysNotify();

    // 添加异常数据
    bool notifyAppendThreatDataByPath(QString sfilePath);

    // 删除异常数据
    bool notifyDeleteThreatDataByPath(QString sfilePath);

    // 查询异常数据
    bool notifySelectThreatDataByPath(QString sfilePath);

    //查询总的数据
    int getSelectThreatsCount();

    // 查询总的数据　返回的是列表
    QString notifySelectThreatCount();
Q_SIGNALS:
    // 联网管控/远程访问
    void netChangeAsk();
    void remChangeAsk();

    // 开机自启动管理
    void AccessRefreshData(bool bAdd, QString sID);

    // 请求查询异常数据的总数
    //void notifySelectThreatsCount();
    //void sendThreadsCount(int ncount);

private:
    DatainterfaceAdaptor *m_adaptpr;
    // 联网管控/远程访问
    NetControlModel         *m_netControlModel              = nullptr;          // 联网管控数据类
    RemControlModel         *m_remControlModel              = nullptr;          // 联网管控数据类
    NetControlMonitor       *m_netControlMonitor            = nullptr;          // 联网管控监控类

    // 联网管控、远程访问弹框
    NetDisableDialog *m_netDisableDialog = nullptr;
    NetAskDialog *m_netAskDialog = nullptr;
    RemDisableDialog *m_remDisableDialog = nullptr;
    RemAskDialog *m_remAskDialog = nullptr;

    // 开机自启动管理
    StartupMonitor *m_startupMonitor = nullptr;
    QMap<QString, QString> m_mapEnable;                     // 不允许自启动数据容器
    QMap<QString, QString> m_mapDisable;                    // 允许自启动数据容器
    QStringList m_EnableData;
    QStringList m_DisableData;
    QString m_EnableAppsInfo;
    QString m_DisableAppsInfo;
    Launcherd          *m_launcherInter     = nullptr;          // 取应用数据接口
    QDBusPendingReply<LauncherItemInfoList> m_reply;            // 应用信息接口
    DBusStartManager   *m_startManagerInterface = nullptr;      // 自启动接口

    // 流量详情
    TrafficDetailsWorker *m_trafficWorker;
    MonitorNetFlow *m_monitorNetFlowInter;

    //使用数据库
    UseData *m_useData;
    ThreatData *m_threatData = nullptr;

    QGSettings          *m_gsFirewall;
};
