/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "getprocinfointer.h"
#include <proc/readproc.h>
#include <fstream>

#include <QDebug>
#include <QThread>
#include <QTimer>
#include <QSettings>
#include <QSqlDatabase>
#include <QSqlQuery>

#include <sstream>
#include <iostream>
#include <string.h>     //strrchr(), strncpy()
#include <stdlib.h>     //realloc()
#include <unistd.h>     //readlink()

#define MAXBUFSIZE 1024

std::string checkSymbolic(const std::string &szPackageName)
{
    if(szPackageName == "")
        return szPackageName;

    char shellCmd[260] = {0};
    strcpy( shellCmd, "file " );
    strcat( shellCmd, szPackageName.c_str());

    FILE *pp = nullptr;
    if((pp = popen(shellCmd, "r")) == nullptr )
    {
        perror("popen() error!");
        return szPackageName;
    }

    char buf[1024] = {0};
    std::string szDestLink = "";

    if(fgets(buf, sizeof buf, pp))
    {
        szDestLink = buf;
        szDestLink.erase(std::remove(szDestLink.begin(), szDestLink.end(), '\n'), szDestLink.end());

        size_t nPos = szDestLink.find(" symbolic link to ");
        if (nPos != std::string::npos)
        {
            size_t nDestPos = nPos + sizeof " symbolic link to ";
            szDestLink = szDestLink.substr(nDestPos - 1);
        }
        else
        {
            szDestLink = szPackageName;
        }
    }

    pclose(pp);

    return szDestLink;
}

//通过链接文件名获取被目标文件绝对路径
//为了防止buffer不够大，包装一下readlink()
char* getFilePathByLink(const char *filename)
{
    unsigned int size = 100;
    char *buffer = nullptr;

    while(1) {
        buffer = new char[size];
        //readlink()返回的路径名并没有以'\0'结尾
        long nchars = readlink(filename, buffer, size - 1);
        if (nchars < 0)
        {
          free(buffer);
          return nullptr;
        }
        if (nchars <= size - 1) {
        buffer[nchars] = '\0';        //让路径名以'\0'结尾
        return buffer;
        }
        size *= 2;    //buffer空间不足，扩大为2倍
    }
}

//通过链接文件名获取被目标文件所在目录
char* getFileDirByLink(const char *linkfile)
{
    char* pathbuf = nullptr;
    pathbuf = getFilePathByLink(linkfile);
    if (nullptr == pathbuf) {
       printf("getFilePathByLink() Failed!\n");
       return(nullptr);
    }

    char* last_slash = nullptr;
    //找到最后一个'/',last_slash保存其地址
      last_slash = strrchr(pathbuf, '/');
    //如果是空，则未找到'/'
    if (last_slash == nullptr) {
        printf("can not find '/'!\n");
        return(nullptr);
    }

    long dir_len = 0;
    char* dir = nullptr;
    dir_len = last_slash - pathbuf + 1;        //包括末尾的'/'
    dir = new char[dir_len];

    strncpy(dir, pathbuf, size_t(dir_len));
    dir[dir_len] = '\0';

    return dir;
}

//const QString defaultWineName = "wineserver.real";
//const int updateIntervel = 2;

GetProcInfoInter *GetProcInfoInter::m_myInstance = nullptr;

GetProcInfoInter::GetProcInfoInter()
    : m_launcherInter(nullptr)
{
    // m_launcherInter
    m_launcherInter = new LauncherInter("com.deepin.dde.daemon.Launcher",
                                        "/com/deepin/dde/daemon/Launcher",
                                        QDBusConnection::sessionBus(), this);

    // 更新启动器应用信息列表
    registerLauncherItemInfoMetaType();
    registerLauncherItemInfoListMetaType();
    updateLauncherItemInfoList();
    connect(m_launcherInter, &LauncherInter::ItemChanged, this, [ = ]() {
        // 更新启动器应用信息列表
        updateLauncherItemInfoList();
        // 发送启动器应用列表改变信号
        Q_EMIT this->launcherItemsChanged();
        qDebug() << "LauncherInter::ItemChanged";
    });

    // 初始化时，更新所有进程信息
    updateProInfoList();
    // 进程信息更新定时器
    QTimer *updateProInfoTimer = new QTimer(this);
    updateProInfoTimer->setInterval(1000 * 2);
    connect(updateProInfoTimer, &QTimer::timeout, this, &GetProcInfoInter::updateProInfoList);
    updateProInfoTimer->start();
}

GetProcInfoInter::~GetProcInfoInter()
{

}

GetProcInfoInter *GetProcInfoInter::instance()
{
    if (m_myInstance == nullptr) {
        // 创建实例
        m_myInstance = new GetProcInfoInter;
    }
    return m_myInstance;
}

DefenderProcInfo GetProcInfoInter::getProcInfoByPid(int pid)
{
    // 待返回的进程信息
    DefenderProcInfo retProcInfo;
    retProcInfo.nPid = 0;
    // 从进程信息列表中搜寻pid对应的进程信息
    for (DefenderProcInfo info : m_procInfoList) {
        if (info.nPid == pid) {
            retProcInfo = info;
            break;
        }
    }

    // debug
    if (retProcInfo.sAppName.isEmpty())
        qDebug() << "can't find pid's infos of " << pid;

    return retProcInfo;
}

QString GetProcInfoInter::getDesktopPathByPid(int pid)
{
    return getProcInfoByExecPath(getProcExecPath(pid)).sDesktopPath;
}

QString GetProcInfoInter::getPkgNameByExecPath(QString sExecPath)
{
    // 初始包名为空
    QString sPkgName = "";

    QString sRealExecPath = getFilePathByLink(sExecPath.toStdString().c_str());
    if (sRealExecPath.isEmpty()) {
        sRealExecPath = sExecPath;
    }

    sPkgName = RequestPackageName(sRealExecPath);
    return sPkgName;
}

DefenderProcInfo GetProcInfoInter::getProcInfoByExecPath(QString sExecPath)
{
    DefenderProcInfo retInfo;
    QString sPkgName = getPkgNameByExecPath(sExecPath);

    // 由包名从启动器服务中获取进程信息（应用名称、图标、包名）
    retInfo = getProcInfoByPkgName(sPkgName);

    // 判断是否是系统应用
    retInfo.isbSysApp = sExecPath.contains("/opt/") ? false : true;

    return retInfo;
}

void GetProcInfoInter::updateLauncherItemInfoList()
{
    // 从启动器dbus服务中获取所有应用信息
    QDBusPendingReply<LauncherItemInfoList> reply = m_launcherInter->GetAllItemInfos();
    reply.waitForFinished();
    if (reply.isError()) {
        qDebug() << "data is empty, quit!!";
        qDebug() << reply.error();
    }

    m_installedAppInfoList.clear();
    for (LauncherItemInfo launcherItemInfo : reply.value()) {
        DefenderProcInfo procInfo;
        procInfo.nPid = 0;

        // 获取执行路径
        QSettings readIniSettingMethod(launcherItemInfo.Path, QSettings::Format::IniFormat);
        readIniSettingMethod.beginGroup("Desktop Entry");
        procInfo.sExecPath = readIniSettingMethod.value("Exec").toString();
        readIniSettingMethod.endGroup();

        // 判断是否是系统应用
        procInfo.isbSysApp = true;
        if (procInfo.sExecPath.contains("/opt/"))
            procInfo.isbSysApp = false;

        // 获取进程名
        procInfo.sProcName = "";
        // 获取desktop路径
        procInfo.sDesktopPath = launcherItemInfo.Path;

        // 获取包名
        procInfo.sPkgName = getPkgNameByExecPath(procInfo.sDesktopPath);

        // 获取 应用名称、图标
        procInfo.sAppName = launcherItemInfo.Name;
        procInfo.sThemeIcon = launcherItemInfo.Icon;
        // 获取应用启动器id
        procInfo.sID = launcherItemInfo.ID;
        // 获取应用启动器分类ID
        procInfo.nCategoryID = launcherItemInfo.CategoryID;
        // 获取应用安装时间
        procInfo.nTimeInstalled = launcherItemInfo.TimeInstalled;

        // 初始化流量信息
        procInfo.dDownloadSpeed = 0.0;
        procInfo.dUploadSpeed = 0.0;
        procInfo.dDownloads = 0.0;
        procInfo.dUploads = 0.0;
        m_installedAppInfoList.append(procInfo);
    }

}

// 更新进程信息列表
void GetProcInfoInter::updateProInfoList()
{
#if log_switch
    qDebug() << "updateProInfoList" << "===============================";
#endif
    QList<DefenderProcInfo> lastProcInfoList = m_procInfoList;
    QList<DefenderProcInfo> deletedProcInfoList;           // 关闭的进程信息列表
    QList<DefenderProcInfo> closedAppList;                 // 关闭的应用信息列表
    QList<DefenderProcInfo> newStartingAppList;            // 新加的进程信息列表
    m_procInfoList.clear();

    // 获取系统所有进程信息
    PROCTAB *proc = openproc(
                        PROC_FILLMEM |                  // memory status: read information from /proc/#pid/statm
                        PROC_FILLSTAT |                 // cpu status: read information from /proc/#pid/stat
                        PROC_FILLUSR                    // user status: resolve user ids to names via /etc/passwd
                    );
    static proc_t proc_info;
    memset(&proc_info, 0, sizeof(proc_t));

    // 遍历所有进程
    while (readproc(proc, &proc_info) != nullptr) {
        // 如果是已存在进程，则保持信息不变
        bool isbNewProc = true;
        for (int i = 0; i < lastProcInfoList.size(); i++) {
            DefenderProcInfo info = lastProcInfoList.at(i);
            if (proc_info.tid == info.nPid) {
                isbNewProc = false;
                m_procInfoList.append(info);
                lastProcInfoList.removeAt(i);
                break;
            }
        }

        if (!isbNewProc)
            continue;

        // 只更新新打开的进程
        DefenderProcInfo procInfo;
        procInfo.nPid = proc_info.tid;

        // 获取执行路径
        QString sExecPathTmp = getProcExecPath(proc_info.tid);
        procInfo.sExecPath = sExecPathTmp;
        if (sExecPathTmp.isEmpty())
            procInfo.sExecPath = proc_info.cmd;

        // 判断是否是系统应用
        procInfo.isbSysApp = true;
        if (procInfo.sExecPath.contains("/opt/"))
            procInfo.isbSysApp = false;

        // 获取进程名
        procInfo.sProcName = sExecPathTmp.split("/").last();
        if (sExecPathTmp.isEmpty())
            procInfo.sProcName = proc_info.cmd;

        // 根据进程名从启动器应用信息列表中获取 桌面文件名、应用名称、图标
        DefenderProcInfo procInfoFromLauncher = getProcInfoByExecPath(procInfo.sExecPath);
        procInfo.sDesktopPath = procInfoFromLauncher.sDesktopPath;
        procInfo.sPkgName = procInfoFromLauncher.sPkgName;
        procInfo.sAppName = procInfoFromLauncher.sAppName;
        procInfo.sThemeIcon = procInfoFromLauncher.sThemeIcon;

        // 初始化流量信息
        procInfo.dDownloadSpeed = 0.0;
        procInfo.dUploadSpeed = 0.0;
        procInfo.dDownloads = 0.0;
        procInfo.dUploads = 0.0;

        QString sDesktopPath = getProcEnvProperties(procInfo.nPid, "GIO_LAUNCHED_DESKTOP_FILE");
        // 从已安装应用信息列表中获取wine程序的信息
        for (DefenderProcInfo appInfo : m_installedAppInfoList) {
            if (sDesktopPath == appInfo.sDesktopPath && !sDesktopPath.isEmpty()) {
                procInfo.isbSysApp = appInfo.isbSysApp;
                procInfo.sDesktopPath = appInfo.sDesktopPath;
                procInfo.sPkgName = appInfo.sPkgName;
                procInfo.sAppName = appInfo.sAppName;
                procInfo.sThemeIcon = appInfo.sThemeIcon;
                break;
            }
        }

        if (procInfo.sPkgName.isEmpty())
            procInfo.sPkgName = getProcEnvProperties(procInfo.nPid, "DEB_PACKAGE_NAME");

        m_procInfoList.append(procInfo);

        // 将 新添加的、存在包名的、不存在于新启动应用列表的
        // 进程信息加入到 新启动应用列表
        if (!procInfo.sPkgName.isEmpty()) {
            bool isbInNewStartingAppList = false;
            for (DefenderProcInfo newApp : newStartingAppList) {
                if (procInfo.sPkgName == newApp.sPkgName) {
                    isbInNewStartingAppList = true;
                    break;
                }
            }
            if (!isbInNewStartingAppList)
                newStartingAppList.append(procInfo);
        }
    }
    closeproc(proc);

    // 将 已关闭的、存在包名的、不存在于已关闭应用列表的
    // 进程信息加入到 已关闭应用列表
    for (DefenderProcInfo info : lastProcInfoList) {
        // 如果包名为空，则不处理
        if (info.sPkgName.isEmpty())
            continue;

        // 如果当前进程已在关闭应用列表，则不处理
        bool isbInClosedAppList = false;
        for (DefenderProcInfo closedApp : closedAppList) {
            if (info.sPkgName == closedApp.sPkgName) {
                isbInClosedAppList = true;
                break;
            }
        }
        if (isbInClosedAppList)
            continue;

        // 是否是关闭的应用
        bool isbCloseApp = true;
        for (DefenderProcInfo currInfo : m_procInfoList) {
            if (info.sPkgName == currInfo.sPkgName) {
                isbCloseApp = false;
                break;
            }
        }
        if (isbCloseApp) {
            closedAppList.append(info);
        }
    }

    // 判断是否有新添加或关闭的进程，并发送信号
    if (!newStartingAppList.isEmpty()) {
        Q_EMIT sendNewStartingApps(newStartingAppList);
    }
    if (!closedAppList.isEmpty())
        Q_EMIT sendClosedApps(closedAppList);

#if log_switch
    for (DefenderProcInfo info : m_procInfoList) {
        qDebug() << "m_procInfoList" << info.nPid<< info.sExecPath << info.sAppName << info.sDesktopPath << info.sPkgName;
    }
    qDebug() << "m_procInfoList ====================";
#endif

}

QString GetProcInfoInter::getProcExecPath(pid_t pid)
{
    // 待返回的进程名
    QString retStr = "";

    // 获取执行路径
    QString sExeLinkPath = "/proc/" + QString::number(pid) + "/exe";
    retStr = getFilePathByLink(sExeLinkPath.toStdString().c_str());
    if (!retStr.isEmpty())
        return retStr;

    // 如无法获取执行路径，则使用cmdline代替
    // 获取/prc/路径下对应pid目录下cmdline文件中内容
    std::string temp;
    try {
        std::fstream fs;
        fs.open("/proc/"+std::to_string(long(pid))+"/cmdline", std::fstream::in);
        std::getline(fs,temp);
        fs.close();
    } catch(std::ifstream::failure e) {
        return "FAILED TO READ PROC";
    }

    // 如果获取cmdline文件内容为空，则返回空名称
    if (temp.size() < 1) {
        return retStr;
    }

    // change \0 to ' '
    std::replace(temp.begin(),temp.end(),'\0',' ');

    QString sTmpStr = QString::fromStdString(temp);
    // 按空格分割进程命令行
    QStringList retStrList = sTmpStr.split(" ");
    // 取字符串中"/"后的进程名
    if (0 < retStrList.count()) {
        retStr = retStrList.first();
    }

    return retStr;
}

// 通过pid获取进程的包名
QString GetProcInfoInter::getPkgNameByPid(int pid)
{
    // 初始包名为空
    QString sPkgName = "";
    QString sExePath = getProcExecPath(pid);
    if (sExePath.isEmpty()) {
        return "";
    }
    // 获取包名的第一种方式 直接通过执行路径从数据库获取
    sPkgName = RequestPackageName(sExePath);

    // 获取包名的第二种方式 通过Launcher容器数据中获取
    if (sPkgName.isEmpty()) {
        QString sDesktopPath = getProcEnvProperties(pid, "GIO_LAUNCHED_DESKTOP_FILE");
        // 从已安装应用信息列表中获取wine程序的信息
        for (DefenderProcInfo appInfo : m_installedAppInfoList) {
            if (sDesktopPath == appInfo.sDesktopPath && !sDesktopPath.isEmpty()) {
                sPkgName = appInfo.sPkgName;
            }
        }
    }

    // 获取包名的第三种方式 获取环境变量
    if (sPkgName.isEmpty())
        sPkgName = getProcEnvProperties(pid, "DEB_PACKAGE_NAME");

    return sPkgName;
}

// 通过包名，从已安装列表中获取进程信息
DefenderProcInfo GetProcInfoInter::getProcInfoByPkgName(QString sPkgName)
{
    DefenderProcInfo retProcInfo = {};
    retProcInfo.sExecPath = "";
    retProcInfo.sDesktopPath = "";
    retProcInfo.sProcName = "";
    retProcInfo.sPkgName = "";
    retProcInfo.sAppName = "";
    retProcInfo.sThemeIcon = "";

    if (sPkgName.isEmpty())
        return retProcInfo;

    // 匹配包名
    for (DefenderProcInfo info : m_installedAppInfoList) {
        QString sPkgNameTmp = info.sPkgName;
        if (sPkgNameTmp.contains(sPkgName)) {
            retProcInfo.isbSysApp = info.isbSysApp;
            retProcInfo.sProcName = info.sProcName;
            retProcInfo.sDesktopPath = info.sDesktopPath;
            retProcInfo.sPkgName = info.sPkgName;
            retProcInfo.sAppName = info.sAppName;
            retProcInfo.sThemeIcon = info.sThemeIcon;
            break;
        }
    }

#if log_switch
    if (retProcInfo.sAppName.isEmpty())
        qDebug() << "can't find infos of " << sPkgName;
#endif

    return retProcInfo;
}

QString GetProcInfoInter::getProcEnvProperties(pid_t pid, QString sEvnKey)
{
    std::string temp;
    try {
        std::fstream fs;
        fs.open("/proc/"+std::to_string(pid)+"/environ", std::fstream::in);
        std::getline(fs,temp);
        fs.close();
    } catch(std::ios_base::failure e) {
        return "FAILED TO READ PROC";
    }

    // change \0 to ' '
//    std::replace(temp.begin(),temp.end(),'\0','\n');

    if (temp.size()<1) {
        return "";
    }

    QStringList sEnvs = QString::fromStdString(temp).trimmed().split('\0');
    foreach (auto envValue, sEnvs) {
        if (envValue.startsWith(QString("%1=").arg(sEvnKey))) {
            return envValue.remove(0, sEvnKey.length() + 1);
        }
    }

    return "";
}

QString GetProcInfoInter::RequestPackageName(QString path)
{
    QSqlDatabase db;

    if (QSqlDatabase::contains("PkgCacheManager")) {
        db = QSqlDatabase::database("PkgCacheManager");
    } else {
        db = QSqlDatabase::addDatabase("QSQLITE", "PkgCacheManager");
    }

    db.setDatabaseName("/usr/share/deepin-app-store/cache.db");

    db.open();

    QString strPackageName = "";

    std::ostringstream oss;
    oss <<  "select * from package_binary_file where binaryPath = '" << path.toStdString() << "'";
    QString sqlCmd = oss.str().c_str();

    QSqlQuery query(db);
    bool bRet = query.exec(sqlCmd);
    if(bRet)
    {
        while(query.next())
        {
            strPackageName = query.value("packageName").toString();
        }
    }

    db.close();

    return strPackageName;
}
