/*
 * Copyright (C) 2019 ~ 2020 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once

#define log_switch 0

#include "com_deepin_dde_daemon_launcherd.h"
#include "window/modules/common/defenderprocinfo.h"

#include <QObject>

//struct ProcInfo {
//    int pid;
//    QString procName;
//    QString desktop;
//    QString appName;
//    QString themeIcon;
//};

using LauncherInter = com::deepin::dde::daemon::Launcher;

class GetProcInfoInter : public QObject
{
    Q_OBJECT
public:
    explicit GetProcInfoInter();
    ~GetProcInfoInter();
    // 获取实例
    static GetProcInfoInter *instance();
    // 获取进程信息列表
    inline QList<DefenderProcInfo> getProcInfoList() {return m_procInfoList;}
    // 获取已安装应用信息列表
    inline QList<DefenderProcInfo> getInstalledAppInfoList() {return m_installedAppInfoList;}
    // 通过pid获取进程信息
    DefenderProcInfo getProcInfoByPid(int pid);
    // 通过pid获取进程desktop路径
    QString getDesktopPathByPid(int pid);

    // 通过执行路径获取包名
    QString getPkgNameByExecPath(QString sExecPath);
    // 通过执行路径获取进程信息
    DefenderProcInfo getProcInfoByExecPath(QString sExecPath);

    // 通过pid获取进程的包名
    QString getPkgNameByPid(int pid);

    // 通过包名，从已安装列表中获取进程信息
    DefenderProcInfo getProcInfoByPkgName(QString sPkgName);

private:
    // 更新启动器应用信息列表
    void updateLauncherItemInfoList();
    // 更新进程信息列表
    void updateProInfoList();
    // 通过pid获取进程执行路径
    QString getProcExecPath(pid_t pid);

    // 从进程环境变量中获取环境信息
    QString getProcEnvProperties(pid_t pid, QString sEvnKey);

    // 从/usr/share/deepin-app-store/cache.db数据库里直接读取包名
    QString RequestPackageName(QString path);

Q_SIGNALS:
    // 启动器应用列表改变信号
    void launcherItemsChanged();
    // 应用启动关闭改变信号
    void sendNewStartingApps(QList<DefenderProcInfo> list);
    void sendClosedApps(QList<DefenderProcInfo> list);

    void sendNewProcInfos(QList<DefenderProcInfo> list);
    void sendDeletedProcInfos(QList<DefenderProcInfo> list);

private:
    // 实例
    static GetProcInfoInter *m_myInstance;
    // 进程信息列表
    QList<DefenderProcInfo> m_procInfoList;
    // 启动器接口
    LauncherInter *m_launcherInter;
    // 已安装应用信息列表
    QList<DefenderProcInfo> m_installedAppInfoList;

    QMap<QString, QString> m_procNameToDesktop;
};
