#include "netaskdialog.h"

#include <QDesktopWidget>
#include <QApplication>
#include <QWindow>
#include <DTitlebar>
#include <QHBoxLayout>

#define         NET_ALLOW_ONCE                  0          // 弹框第一个按钮
#define         NET_ALLOW_ALWAYS                1          // 弹框第二个按钮
#define         NET_DISABLE                     2          // 弹框第三个按钮

NetAskDialog::NetAskDialog(const QString &sPkgName, const QString &sAppName, const QString &sId, DDialog *parent)
    : DDialog(parent)
    , m_DataInterFaceServer(new DataInterFaceServer("com.deepin.defender.datainterface", "/com/deepin/defender/datainterface", QDBusConnection::sessionBus(), this))
    , m_sAppName(sAppName)
    , m_sPkgName(sPkgName)
    , m_sId(sId)
{
    // 初始化界面
    initUi();
}

// 初始化界面
void NetAskDialog::initUi()
{
    // 弹框样式
    setCloseButtonVisible(true);
    this->setWindowFlag(Qt::WindowStaysOnTopHint);
    this->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    this->setModal(true);

    setIcon(QIcon::fromTheme("deepin-defender"));

    // 添加布局
    DLabel *txt = nullptr;
    txt = new DLabel(tr("%1 wants to connect to the Internet").arg(m_sAppName.isEmpty() ? m_sPkgName : m_sAppName));
    txt->setWordWrap(true);
    // 字体剧中
    txt->setAlignment(Qt::AlignCenter);
    addContent(txt);
    addButton(tr("Allow Once"));
    addButton(tr("Allow Always"));
    addButton(tr("Disable (%1)").arg(m_elapsedSeconds), true, DDialog::ButtonWarning);

    // 添加信号槽
    connect(this, &DDialog::buttonClicked, this, [ = ](int index, const QString & text) {
        Q_UNUSED(text)

        if (index == NET_ALLOW_ONCE) {
            m_DataInterFaceServer->setNetRequestReply(m_sPkgName, m_sId, NET_ALLOW_ONCE);
        } else if (index == NET_ALLOW_ALWAYS) {
            m_DataInterFaceServer->setNetRequestReply(m_sPkgName, m_sId, NET_ALLOW_ALWAYS);
        } else {
            m_DataInterFaceServer->setNetRequestReply(m_sPkgName, m_sId, NET_DISABLE);
        }

        // 停止定时器、关闭弹框
        m_timer->stop();
        close();
    });

    // 安装定时器
    m_timer = new QTimer();
    connect(m_timer, &QTimer::timeout, [ = ]() {
        m_elapsedSeconds--;

        // 实时变化按钮的时间
        if (m_elapsedSeconds > 0) {
            setButtonText(NET_DISABLE, tr("Disable (%1)").arg(m_elapsedSeconds));
            return;
        } else {
            m_DataInterFaceServer->setNetRequestReply(m_sPkgName, m_sId, NET_DISABLE);
            // 关闭弹框
            close();
        }
    });
    m_timer->start(1000);
}

NetAskDialog::~NetAskDialog()
{
    Q_EMIT onDialogClose();

    m_timer->stop();
    m_timer->deleteLater();
}
