#include "netcontrolmodel.h"

#define         NET_CONTROL_OPTION                  "netcontroloption"          // 联网管控联网选项
#define         NET_CONTROL_OPTION_APPS             "netcontroloptionapps"      // 联网管控所有APP联网选项
#define         SYS_APP                             "sysapp"                    // 系统标志
#define         OPT_APP                             "optapp"                    // 第三方标志
#define         SPLIT_FLAG_1                        "|"                         // 分割符 |
#define         SPLIT_FLAG_2                        "*"                         // 分割符 *
#define         NET_ALLOW_ONCE                       0                          // 弹框第一个按钮
#define         NET_ALLOW_ALWAYS                     1                          // 弹框第二个按钮
#define         NET_DISABLE                          2                          // 弹框第三个按钮

NetControlModel::NetControlModel(QObject *parent)
    : QObject(parent)
    , m_gsNetControl(new QGSettings("com.deepin.dde.deepin-defender-datainterface", QByteArray(), this))
    , m_DataInterFace(new DataInterFace("com.deepin.defender.datainterface", "/com/deepin/defender/datainterface", QDBusConnection::sessionBus(), this))
{
    // 初始化公共数据接口
    m_getProcInfoInter = GetProcInfoInter::instance();
    connect(m_getProcInfoInter, SIGNAL(launcherItemsChanged()), this, SLOT(doAppsChange()));
    connect(m_getProcInfoInter, SIGNAL(sendNewStartingApps(QList<DefenderProcInfo>)), this, SLOT(doAppStartSlot(QList<DefenderProcInfo>)));
    connect(m_getProcInfoInter, SIGNAL(sendClosedApps(QList<DefenderProcInfo>)), this, SLOT(doAppEndSlot(QList<DefenderProcInfo>)));

    // 初始化数据
    initData();
}

// 应用程序开启关闭槽
void NetControlModel::doAppStartSlot(QList<DefenderProcInfo> sAppList)
{
    for (DefenderProcInfo info : sAppList) {
        QString sPkgName        = info.sPkgName;
        qDebug() << "App Start value: " << sPkgName;
    }
}

// 应用程序开启关闭槽
void NetControlModel::doAppEndSlot(QList<DefenderProcInfo> sAppList)
{
    for (DefenderProcInfo info : sAppList) {
        QString sPkgName        = info.sPkgName;
        qDebug() << "App End value: " << sPkgName;
        m_askOneAllowList.removeOne(sPkgName);
    }
}

// 数据初始化
void NetControlModel::initData()
{
    // 获取所有APP联网选项
    QString sNetAppsOption      = m_gsNetControl->get(NET_CONTROL_OPTION_APPS).toString();

    // 初始化时拆分本地保存项，用QMap接收
    m_mapNetControl.clear();
    QStringList sNetAppsOptionList = sNetAppsOption.split(SPLIT_FLAG_1);
    for (int i = 0; i < sNetAppsOptionList.count(); ++i) {
        QStringList sNetAppsList = sNetAppsOptionList.at(i).split(SPLIT_FLAG_2);

        // 保证拆分的总数必须为2,否则崩溃
        if (sNetAppsList.count() >= 2) {
            m_mapNetControl[sNetAppsList.at(0)].append(sNetAppsList.at(1));
        }
    }

    // 更新APPS应用程序后，匹配对应的本地保存项
    doAppsChange();
}

// 更新APPS应用程序后，匹配对应的本地保存项
void NetControlModel::doAppsChange()
{
    // 获取本次的联网管控选项
    QString sNetOption          = m_gsNetControl->get(NET_CONTROL_OPTION).toString();

    // 获取所有数据
    QList<DefenderProcInfo> sAppsInfoList;
    sAppsInfoList = m_getProcInfoInter->getInstalledAppInfoList();

    m_mapNetControlStatus.clear();
    m_mapNetControlAppName.clear();
    // 检查是否有卸载的用用程序， 如果有删除
    if (m_mapNetControl.count() > 0) {
        QMapIterator<QString, QString> iter(m_mapNetControl);
        while (iter.hasNext()) {
            iter.next();
            for (DefenderProcInfo info : sAppsInfoList) {
                if (info.sPkgName == iter.key()) {
                    m_mapNetControlStatus[iter.key()] = iter.value();
                    m_mapNetControlAppName[iter.key()] = info.sAppName;
                }
            }
        }
    }

    // 获取所有启动项APP详细信息，并且将新增的数据倒入到QMap
    for (DefenderProcInfo info : sAppsInfoList) {
        QString sPkgName        = info.sPkgName;

        // 新增的应用程序 如果是系统应用默认设为允许， 如果是第三方应用默认设为联网管控默认选项。
        if (!m_mapNetControlStatus.contains(sPkgName) && !sPkgName.isEmpty()) {
            m_mapNetControlStatus[sPkgName].append(info.isbSysApp ? QString(NET_STATUS_ItemAllow) : sNetOption);
            m_mapNetControlAppName[sPkgName].append(info.sAppName);
        }
    }

    // 将数据集中数据保存到本地
    setGsLocalData(m_mapNetControlStatus);

    // 刷新APP数据信息
    refreshNetControlInfos();
}

// 将数据集中数据保存到本地
void NetControlModel::setGsLocalData(QMap<QString, QString> mapNetControl)
{
    // 清除本地配置
    reset();

    //加载数据
    QString str;
    QMapIterator<QString, QString> iter(mapNetControl);
    while (iter.hasNext()) {
        iter.next();
        str += QString("%1%2%3%4").arg(iter.key()).arg(SPLIT_FLAG_2).arg(iter.value()).arg(SPLIT_FLAG_1);
    }
    m_gsNetControl->set(NET_CONTROL_OPTION_APPS, str);
}

// 清除本地配置
void NetControlModel::reset()
{
    m_gsNetControl->reset(NET_CONTROL_OPTION_APPS);
}

// 获取联网管控弹框所有信息
void NetControlModel::refreshNetControlInfos()
{
    // 获取本次的联网管控选项
    QString sNetOption          = m_gsNetControl->get(NET_CONTROL_OPTION).toString();

    // 初始化结构体
    DefenderProcInfo retProcInfo;
    retProcInfo.sExecPath = "";
    retProcInfo.sProcName = "";
    retProcInfo.sPkgName = "";
    retProcInfo.sAppName = "";
    retProcInfo.sThemeIcon = "";

    // 用于接收APP的保存状态及其他所有信息
    QList<DefenderProcInfo> sNetAppInfoList;

    QList<DefenderProcInfo> sAppsInfoList = m_getProcInfoInter->getInstalledAppInfoList();
    // 获取所有启动项APP详细信息，并且将新增的数据倒入到QMap
    for (DefenderProcInfo info : sAppsInfoList) {
        // 插入保存状态
        info.sStatus        = m_mapNetControlStatus.value(info.sPkgName);
        info.sDefaultStatus = sNetOption;
        sNetAppInfoList.append(info);
    }

    Q_EMIT sendNetAppsChange(sNetAppInfoList);
}

// 联网管控列表 联网状态改变
void NetControlModel::setNetControlStatusChange(QString sPkgName, int nStatus)
{
    m_mapNetControl[sPkgName]           = QString::number(nStatus);
    m_mapNetControlStatus[sPkgName]     = QString::number(nStatus);
    // 将数据集中数据保存到本地
    setGsLocalData(m_mapNetControlStatus);

    // 刷新APP数据信息
    refreshNetControlInfos();
}

// 联网请求槽
void NetControlModel::doNetControlSlot(ACLNetwork *aclNetWork, QString sPkgName, QString sPid, QString sId)
{
    // 获取acl对象
    m_aclInter = aclNetWork;

    // 获取pid对应应用的状态
    QString sStatus = m_mapNetControlStatus.value(sPkgName);

    // 如果是询问 或者是 默认询问，则弹框
    if (NET_STATUS_ITEMASK == sStatus || NET_STATUS_ItemDefaultAsk == sStatus) {
        // 联网管控询问点击禁止后 15秒不再弹框
        if (m_appNetAskDisableTime.contains(sPkgName)) {
            qint64 ago = m_appNetAskDisableTime.value(sPkgName);
            if (QDateTime::currentSecsSinceEpoch() - ago <= 15) {
                m_aclInter->Enable(sId, false);
                return;
            }
            else {
                m_appNetAskDisableTime.remove(sPkgName);
            }
        }

        // 询问弹框
        m_DataInterFace->showNetControlAskDialog(sPkgName, m_mapNetControlAppName.value(sPkgName), sId);
    }
    // 如果是允许 或者是 默认允许 以及其他情况，则将请求置为true，放行
    else {
        m_aclInter->Enable(sId, true);
        return;
    }
}

// 联网管控列表 联网状态改变
void NetControlModel::setNetRequestReply(QString sPkgName, QString sId, int nType)
{
    if (nType == NET_DISABLE) {
        // 联网管控询问点击禁止后保存到容器
        if (!m_appNetAskDisableTime.contains(sPkgName)) {
            m_appNetAskDisableTime.insert(sPkgName, QDateTime::currentSecsSinceEpoch());
        }
        m_aclInter->Enable(sId, false);
    } else {
        m_aclInter->Enable(sId, true);

        if (nType == NET_ALLOW_ONCE) {
            m_askOneAllowList.append(sPkgName);
        } else if (nType == NET_ALLOW_ALWAYS) {
            // 刷新应用程序总是允许状态
            m_mapNetControl[sPkgName] = NET_STATUS_ItemAllow;
            m_mapNetControlStatus[sPkgName] = NET_STATUS_ItemAllow;
            refreshNetControlInfos();

            // 应用程序总是允许添加到本地
            setGsLocalData(m_mapNetControlStatus);
        }
    }
}

// 设置/获取联网管控默认状态
void NetControlModel::setNetControlDefaultStatus(int nDefaultStatus)
{
    // 设置联网管控默认状态
    m_gsNetControl->set(NET_CONTROL_OPTION, nDefaultStatus);

    QMapIterator<QString, QString> iter(m_mapNetControl);
    while (iter.hasNext()) {
        iter.next();
        if (iter.value() == NET_STATUS_ItemDefaultAsk
                || iter.value() == NET_STATUS_ItemDefaultAllow
                || iter.value() == NET_STATUS_ItemDefaultDisallow) {
            m_mapNetControl[iter.key()] = QString::number(nDefaultStatus);
        }
    }

    // 刷新联网管控状态
    doAppsChange();
}

// 设置/获取联网管控默认状态
int NetControlModel::getNetControlDefaultStatus()
{
    return m_gsNetControl->get(NET_CONTROL_OPTION).toInt();
}

// 判断报名是否是允许
bool NetControlModel::getPkgAllowStatus(QString sPkgName)
{
    // 保证数据集里有数据
    if (m_mapNetControlStatus.count() <= 0) {
        return true;
    }

    // 仅允许一次数据容器，直接返回true
    for (int i = 0; i < m_askOneAllowList.count(); ++i) {
        if (m_askOneAllowList.at(i) == sPkgName) {
            return true;
        }
    }

    // 获取pid对应应用的状态
    QString sStatus = m_mapNetControlStatus.value(sPkgName);
    if (NET_STATUS_ItemAllow == sStatus || NET_STATUS_ItemDefaultAllow == sStatus) {
        return true;
    }
    return false;
}

// 判断包名是否是禁止
bool NetControlModel::getPkgDisableStatus(QString sPkgName)
{
    // 获取pid对应应用的状态
    QString sStatus = m_mapNetControlStatus.value(sPkgName);
    if (NET_STATUS_ItemDisallow == sStatus || NET_STATUS_ItemDefaultDisallow == sStatus) {
        return true;
    }
    return false;
}

// 获取APP名称
QString NetControlModel::getAppNameByPkgName(QString sPkgName)
{
    return m_mapNetControlAppName.value(sPkgName);
}
