#ifndef NETCONTROLMODEL_H
#define NETCONTROLMODEL_H

#include "../common/getprocinfointer.h"
#include "../common/defenderprocinfo.h"
#include "com_deepin_daemon_aclnetwork.h"
#include "com_deepin_defender_datainterface.h"

#include <QObject>
#include <QGSettings>
#include <QMap>

#define         NET_STATUS_ITEMASK                  "0"          //询问
#define         NET_STATUS_ItemAllow                "1"          //允许
#define         NET_STATUS_ItemDisallow             "2"          //禁止
#define         NET_STATUS_ItemDefaultAsk           "3"          //默认询问
#define         NET_STATUS_ItemDefaultAllow         "4"          //默认允许
#define         NET_STATUS_ItemDefaultDisallow      "5"          //默认禁止

using ACLNetwork = com::deepin::daemon::acl::Network;
using DataInterFace = com::deepin::defender::datainterface;

class GetProcInfoInter;
class NetControlModel : public QObject
{
    Q_OBJECT
public:
    explicit NetControlModel(QObject *parent = nullptr);

    // 刷新联网管控弹框所有信息
    void refreshNetControlInfos();

    // 联网管控列表 联网状态改变
    void setNetControlStatusChange(QString, int);

    // 联网管控列表 联网状态改变
    void setNetRequestReply(QString, QString, int);

    // 设置/获取联网管控默认状态
    void setNetControlDefaultStatus(int);
    int getNetControlDefaultStatus();

    // 判断包名是否是允许
    bool getPkgAllowStatus(QString);

    // 判断包名是否是禁止
    bool getPkgDisableStatus(QString);
    // 获取APP名称
    QString getAppNameByPkgName(QString);

private:
    // 初始化出局
    void initData();

    // 将数据集中数据保存到本地
    void setGsLocalData(QMap<QString, QString>);

    // 清除本地配置
    void reset();

public Q_SLOTS:
    // 更新APPS应用程序后，匹配对应的本地保存项
    void doAppsChange();

    // 联网请求槽
    void doNetControlSlot(ACLNetwork *, QString, QString, QString);

    // 应用程序开启关闭槽
    void doAppStartSlot(QList<DefenderProcInfo> sAppList);
    void doAppEndSlot(QList<DefenderProcInfo> sAppList);

Q_SIGNALS:
    void sendNetAppsChange(QList<DefenderProcInfo>);


private:
    GetProcInfoInter            *m_getProcInfoInter = nullptr;              // 安全中心公共数据接口
    QGSettings                  *m_gsNetControl     = nullptr;              // 联网管控本地保存项
    QMap<QString, QString>       m_mapNetControl;                           // 联网管控App容器
    QMap<QString, QString>       m_mapNetControlStatus;                     // 联网管控Apps状态容器
    QMap<QString, QString>       m_mapNetControlAppName;                    // 联网管控Apps名称容器
    QList<QString>               m_askOneAllowList;                         // 联网管控仅允许一次容器
    QMap<QString, qint64>        m_appNetAskDisableTime;                    // 联网管控询问点击禁止后保存容器

    ACLNetwork                  *m_aclInter         = nullptr;
    DataInterFace               *m_DataInterFace    = nullptr;
};

#endif // NETCONTROLMODEL_H
