#include "netcontrolmonitor.h"

static const QString MINOTOR_FLAG_NET = "91";
static const QString MINOTOR_FLAG_REM = "92";
static const QString FIRE_WALL_ON_OFF = "fire-wall-on-off";

#define      SPLIT_FLAG_3                "-"            // 分割符 横杠
#define      SPLIT_Item_0                0              // 字符串第一项 请求类型
#define      SPLIT_Item_2                2              // 分隔符第三项 Pid

NetControlMonitor::NetControlMonitor(NetControlModel *netModel, RemControlModel *remModel, QObject *parent)
    : QObject(parent)
    , m_aclInter(new ACLNetwork("com.deepin.daemon.ACL", "/com/deepin/daemon/ACL/Network", QDBusConnection::systemBus(), this))
    , m_loginManager(new LoginManager("org.freedesktop.login1", "/org/freedesktop/login/Manager", QDBusConnection::systemBus(), this))
    , m_netControlModel(netModel)
    , m_remControlModel(remModel)
    , m_gsFirewall(new QGSettings("com.deepin.dde.deepin-defender-datainterface", QByteArray(), this))
    , m_DataInterFace(new DataInterFace("com.deepin.defender.datainterface", "/com/deepin/defender/datainterface", QDBusConnection::sessionBus(), this))
{
    // 获取防火墙总开关
    m_bFireWallSwitch = m_gsFirewall->get(FIRE_WALL_ON_OFF).toBool();

    // 初始化公共数据接口
    m_getProcInfoInter = GetProcInfoInter::instance();
}

// 设置防火墙总开关状态
void NetControlMonitor::setFireWallSwitchStatus(bool isbFireWall)
{
    m_bFireWallSwitch = isbFireWall;
    if (isbFireWall) {
        m_gsFirewall->set(FIRE_WALL_ON_OFF, 1);

        // ACL信号接收
        connect(m_aclInter, &ACLNetwork::AccessRequest, this, &NetControlMonitor::onAppNetworkConnect);
        connect(this, &NetControlMonitor::registerReply, this, [ = ]() {
            QDBusPendingReply<bool> reply = m_aclInter->Register(QCoreApplication::applicationFilePath(), QString::number(QCoreApplication::applicationPid()).toInt(), 20);
            reply.waitForFinished();
            qDebug() << "NetworkConnectMonitor register result : " << reply.value() << QCoreApplication::applicationFilePath() << QCoreApplication::applicationPid();
        });
        Q_EMIT registerReply();

    } else {
        m_gsFirewall->set(FIRE_WALL_ON_OFF, 0);
        m_aclInter->Unregister(QCoreApplication::applicationFilePath(),QString::number(QCoreApplication::applicationPid()).toInt());
    }
}

//获取防火墙总开关状态
bool NetControlMonitor::getFireWallWwitchStatus()
{
    return m_bFireWallSwitch;
}

// 连接ACL请求
void NetControlMonitor::onAppNetworkConnect(const QString &sId)
{
    // 防火墙总开关关闭，直接放行联网请求
    if (!m_bFireWallSwitch) {
        m_aclInter->Enable(sId, true);
        return;
    }

    // 取pid 进程号 / 请求类型
    QStringList sList_Request = sId.split(SPLIT_FLAG_3);
    QString sRequestType, sPid;

    // 判断是否符合规范，不符合规范直接放行
    if (sList_Request.count() < 3) {
        m_aclInter->Enable(sId, true);
        return;
    } else {
        // 获取联网管控标志以及Pid
        sRequestType    = sList_Request.at(SPLIT_Item_0);
        sPid            = sList_Request.at(sList_Request.count() - 1);
    }

    // 获取sPid对应的应用名,如果为空则放行
    QString sPkgName = m_getProcInfoInter->getPkgNameByPid(sPid.toInt());
    if (sPkgName.isEmpty()) {
        m_aclInter->Enable(sId, true);
        return;
    }

    // 判断类型分别给联网管控 和 远程访问发送 信号， 否则直接通过
    if (sRequestType == MINOTOR_FLAG_NET) {
        // 判断该报名对应状态是否是允许，如果是直接放行
        if (m_netControlModel->getPkgAllowStatus(sPkgName)) {
            m_aclInter->Enable(sId, true);
            return;
        }
        else if (m_netControlModel->getPkgDisableStatus(sPkgName)) {
            // 15秒内不再禁止弹框
            if (!m_appNetDisableTime.contains(sPkgName)) {
                m_appNetDisableTime.insert(sPkgName, QDateTime::currentSecsSinceEpoch());
            }
            else {
                qint64 ago = m_appNetDisableTime.value(sPkgName);
                if (QDateTime::currentSecsSinceEpoch() - ago <= 15) {
                    m_aclInter->Enable(sId, false);
                    return;
                }
                else {
                    m_appNetDisableTime.remove(sPkgName);
                }
            }

            // 判断该报名对应状态是否是禁止，如果是直接禁止
            m_aclInter->Enable(sId, false);
            m_DataInterFace->showNetControlDialog(sPkgName, m_netControlModel->getAppNameByPkgName(sPkgName));
            return;
        }
        else {
            Q_EMIT sendNetControlSignal(m_aclInter, sPkgName, sPid, sId);
        }
    } else if (sRequestType == MINOTOR_FLAG_REM) {

        // 判断该报名对应状态是否是允许，如果是直接放行
        if (m_remControlModel->getPkgAllowStatus(sPkgName)) {
            m_aclInter->Enable(sId, true);
            return;
        }
        else if (m_remControlModel->getPkgDisableStatus(sPkgName)) {
            // 15秒内不再禁止弹框
            if (!m_appRemDisableTime.contains(sPkgName)) {
                m_appRemDisableTime.insert(sPkgName, QDateTime::currentSecsSinceEpoch());
            }
            else {
                qint64 ago = m_appRemDisableTime.value(sPkgName);
                if (QDateTime::currentSecsSinceEpoch() - ago <= 15) {
                    m_aclInter->Enable(sId, false);
                    return;
                }
                else {
                    m_appRemDisableTime.remove(sPkgName);
                }
            }

            // 判断该报名对应状态是否是禁止，如果是直接禁止
            m_aclInter->Enable(sId, false);
            m_DataInterFace->showRemControlDialog(sPkgName, m_remControlModel->getAppNameByPkgName(sPkgName));
            return;
        }
        else {
            Q_EMIT sendRemControlSignal(m_aclInter, sPkgName, sPid, sId);
        }
    } else {
        m_aclInter->Enable(sId, true);
        return;
    }
}

// 断开ACL请求
void NetControlMonitor::onAppNetworkDisConnect(const QString &in0, const QDBusObjectPath &in1)
{
    Q_UNUSED(in0);
    Q_UNUSED(in1);
    //m_aclInter->Unregister(QCoreApplication::applicationFilePath(), QCoreApplication::applicationPid());
}
