#ifndef NETCONTROLMONITOR_H
#define NETCONTROLMONITOR_H

#include "com_deepin_daemon_aclnetwork.h"
#include "org_freedesktop_login1.h"
#include "../common/getprocinfointer.h"
#include "../common/defenderprocinfo.h"
#include "window/modules/netcontrol/netcontrolmodel.h"
#include "window/modules/remcontrol/remcontrolmodel.h"
#include "com_deepin_defender_datainterface.h"

#include <QObject>
#include <QThread>
#include <QGSettings>

using ACLNetwork = com::deepin::daemon::acl::Network;
using LoginManager = org::freedesktop::login1::Manager;
using DataInterFace = com::deepin::defender::datainterface;

class NetControlMonitor : public QObject
{
    Q_OBJECT
public:
    explicit NetControlMonitor(NetControlModel *netModel, RemControlModel *remModel, QObject *parent = nullptr);

    // 设置防火墙总开关状态
    void setFireWallSwitchStatus(bool isbFireWall);

    //获取防火墙总开关状态
    bool getFireWallWwitchStatus();

private Q_SLOTS:
    // 连接ACL请求
    void onAppNetworkConnect(const QString &sId);
    // 断开ACL请求
    void onAppNetworkDisConnect(const QString &in0, const QDBusObjectPath &in1);

Q_SIGNALS:
    // 发送注册信号
    void registerReply();

    // 发送联网请求信号
    void sendNetControlSignal(ACLNetwork *, QString, QString, QString);

    // 发送远程请求信号
    void sendRemControlSignal(ACLNetwork *, QString, QString, QString);

private:
    ACLNetwork          *m_aclInter;
    LoginManager        *m_loginManager;
    GetProcInfoInter    *m_getProcInfoInter = nullptr;              // 安全中心公共数据接口
    NetControlModel     *m_netControlModel  = nullptr;              // 联网管控数据类
    RemControlModel     *m_remControlModel  = nullptr;              // 联网管控数据类
    QGSettings          *m_gsFirewall;
    bool                 m_bFireWallSwitch  = false;
    DataInterFace       *m_DataInterFace    = nullptr;

    QMap<QString, qint64> m_appNetDisableTime;
    QMap<QString, qint64> m_appRemDisableTime;
};

#endif // NETCONTROLMONITOR_H
