#include "remcontrolmodel.h"

#define         REM_CONTROL_OPTION                  "remcontroloption"          // 联网管控联网选项
#define         REM_CONTROL_OPTION_APPS             "remcontroloptionapps"      // 联网管控所有APP联网选项
#define         SYS_APP                             "sysapp"                    // 系统标志
#define         OPT_APP                             "optapp"                    // 第三方标志
#define         SPLIT_FLAG_1                        "|"                         // 分割符 |
#define         SPLIT_FLAG_2                        "*"                         // 分割符 *
#define         REM_ALLOW_ONCE                       0                          // 弹框第一个按钮
#define         REM_ALLOW_ALWAYS                     1                          // 弹框第二个按钮
#define         REM_DISABLE                          2                          // 弹框第三个按钮

RemControlModel::RemControlModel(QObject *parent)
    : QObject(parent)
    , m_gsRemControl(new QGSettings("com.deepin.dde.deepin-defender-datainterface", QByteArray(), this))
    , m_DataInterFace(new DataInterFace("com.deepin.defender.datainterface", "/com/deepin/defender/datainterface", QDBusConnection::sessionBus(), this))
{
    // 初始化公共数据接口
    m_getProcInfoInter = GetProcInfoInter::instance();
    connect(m_getProcInfoInter, SIGNAL(launcherItemsChanged), this, SLOT(doAppsChange));
    connect(m_getProcInfoInter, SIGNAL(sendNewStartingApps(QList<DefenderProcInfo>)), this, SLOT(doAppStartSlot(QList<DefenderProcInfo>)));
    connect(m_getProcInfoInter, SIGNAL(sendClosedApps(QList<DefenderProcInfo>)), this, SLOT(doAppEndSlot(QList<DefenderProcInfo>)));


    // 初始化数据
    initData();
}

// 应用程序开启关闭槽
void RemControlModel::doAppStartSlot(QList<DefenderProcInfo> sAppList)
{
    for (DefenderProcInfo info : sAppList) {
        QString sPkgName        = info.sPkgName;
        qDebug() << "App Start value: " << sPkgName;
    }
}

// 应用程序开启关闭槽
void RemControlModel::doAppEndSlot(QList<DefenderProcInfo> sAppList)
{
    for (DefenderProcInfo info : sAppList) {
        QString sPkgName        = info.sPkgName;
        qDebug() << "App End value: " << sPkgName;
        m_askOneAllowList.removeOne(sPkgName);
    }
}

void RemControlModel::initData()
{
    // 更新APPS应用程序后，匹配对应的本地保存项
    doAppsChange();
}

// 更新APPS应用程序后，匹配对应的本地保存项
void RemControlModel::doAppsChange()
{
    // 获取本次的联网管控选项
    QString sRemOption          = m_gsRemControl->get(REM_CONTROL_OPTION).toString();

    // 获取所有数据
    QList<DefenderProcInfo> sAppsInfoList;
    sAppsInfoList = m_getProcInfoInter->getInstalledAppInfoList();

    m_mapRemControlAppName.clear();
    m_mapRemControlAppSystem.clear();

    // 获取所有启动项APP详细信息，并且将新增的数据倒入到QMap
    for (DefenderProcInfo info : sAppsInfoList) {
        QString sPkgName        = info.sPkgName;
        m_mapRemControlAppName[sPkgName] = info.sAppName;
        m_mapRemControlAppSystem[sPkgName] = info.isbSysApp;
    }
}

// 清除本地配置
void RemControlModel::reset()
{
    m_gsRemControl->reset(REM_CONTROL_OPTION_APPS);
}

// 联网请求槽
void RemControlModel::doRemControlSlot(ACLNetwork *aclNetWork, QString sPkgName, QString sPid, QString sId)
{
    // 获取acl对象
    m_aclInter = aclNetWork;

    // 获取本次的联网管控选项
    QString sRemOption          = m_gsRemControl->get(REM_CONTROL_OPTION).toString();

    // 如果是询问 或者是 默认询问，则弹框
    if (REM_STATUS_ItemDefaultAsk == sRemOption) {
        // 联网管控询问点击禁止后 15秒不再弹框
        if (m_appNetAskDisableTime.contains(sPkgName)) {
            qint64 ago = m_appNetAskDisableTime.value(sPkgName);
            if (QDateTime::currentSecsSinceEpoch() - ago <= 15) {
                m_aclInter->Enable(sId, false);
                return;
            }
            else {
                m_appNetAskDisableTime.remove(sPkgName);
            }
        }

        // 询问弹框
        m_DataInterFace->showRemControlAskDialog(sPkgName, m_mapRemControlAppName.value(sPkgName), sId);
    }
    // 如果是允许 或者是 默认允许 以及其他情况，则将请求置为true，放行
    else {
        m_aclInter->Enable(sId, true);
        return;
    }
}

// 联网管控列表 联网状态改变
void RemControlModel::setRemRequestReply(QString sPkgName, QString sId, int nType)
{
    if (nType == REM_DISABLE) {
        // 联网管控询问点击禁止后保存到容器
        if (!m_appNetAskDisableTime.contains(sPkgName)) {
            m_appNetAskDisableTime.insert(sPkgName, QDateTime::currentSecsSinceEpoch());
        }
        m_aclInter->Enable(sId, false);
    } else {
        m_aclInter->Enable(sId, true);

        if (nType == REM_ALLOW_ONCE) {
            m_askOneAllowList.append(sPkgName);
        } else if (nType == REM_ALLOW_ALWAYS) {
            // 设置联网管控默认状态
            m_gsRemControl->set(REM_CONTROL_OPTION, REM_STATUS_ItemDefaultAllow);
            Q_EMIT sendRemDefaultAllowStatus();
        }
    }
}

// 设置/获取联网管控默认状态
void RemControlModel::setRemControlDefaultStatus(int nDefaultStatus)
{
    // 设置联网管控默认状态
    m_gsRemControl->set(REM_CONTROL_OPTION, nDefaultStatus);
}

// 设置/获取联网管控默认状态
int RemControlModel::getRemControlDefaultStatus()
{
    return m_gsRemControl->get(REM_CONTROL_OPTION).toInt();
}

// 判断报名是否是允许
bool RemControlModel::getPkgAllowStatus(QString sPkgName)
{
    qDebug() << "sPkgName value ===" << sPkgName;
    // 系统应用,直接返回true
    if (m_mapRemControlAppSystem.value(sPkgName)) {
        qDebug() << "sPkgName is system app ===";
        return true;
    }

    // 如果获取应用名称为空,返回true
    if (m_mapRemControlAppName.value(sPkgName).isEmpty()) {
        qDebug() << "appName is null  ===";
        return true;
    }

    // 仅允许一次数据容器，直接返回true
    for (int i = 0; i < m_askOneAllowList.count(); ++i) {
        if (m_askOneAllowList.at(i) == sPkgName) {
            return true;
        }
    }

    // 获取本次的联网管控选项
    QString sRemOption          = m_gsRemControl->get(REM_CONTROL_OPTION).toString();
    if (REM_STATUS_ItemDefaultAllow == sRemOption) {
        return true;
    }
    return false;
}

// 判断包名是否是禁止
bool RemControlModel::getPkgDisableStatus(QString sPkgName)
{
    Q_UNUSED(sPkgName);
    // 获取本次的联网管控选项
    QString sRemOption          = m_gsRemControl->get(REM_CONTROL_OPTION).toString();
    if (REM_STATUS_ItemDefaultDisallow == sRemOption) {
        return true;
    }
    return false;
}

// 获取APP名称
QString RemControlModel::getAppNameByPkgName(QString sPkgName)
{
    return m_mapRemControlAppName.value(sPkgName);
}
