#include "remdisabledialog.h"

#define         REM_SETTING                     0          // 弹框第一个按钮
#define         REM_OK                          1          // 弹框第二个按钮

DWIDGET_USE_NAMESPACE
RemDisableDialog::RemDisableDialog(const QString &sPkgName, const QString &sAppName, DDialog *parent)
    : DDialog(parent)
    , m_sAppName(sAppName)
    , m_sPkgName(sPkgName)
    , m_hmiScreen(new HmiScreen("com.deepin.defender.hmiscreen", "/com/deepin/defender/hmiscreen", QDBusConnection::sessionBus(), this))
{
    // 初始化界面
    initUi();
}

// 初始化界面
void RemDisableDialog::initUi()
{
    // 弹框样式
    setCloseButtonVisible(true);
    this->setWindowFlag(Qt::WindowStaysOnTopHint);
    this->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    this->setModal(true);

    // 清除布局
    clearContents();
    clearButtons();
    setIcon(QIcon::fromTheme("deepin-defender"));
    // 添加布局
    DLabel *txt = nullptr;
    txt = new DLabel(tr("The firewall has blocked %1 from remote connection").arg(m_sAppName.isEmpty() ? m_sPkgName : m_sAppName));
    // 字体剧中
    txt->setAlignment(Qt::AlignCenter);
    txt->setWordWrap(true);
    addContent(txt);
    addButton(tr("Go to Settings"));
    addButton(tr("OK (%1)").arg(m_elapsedSeconds), true, DDialog::ButtonWarning);

    // 添加信号槽
    connect(this, &DDialog::buttonClicked, this, [ = ](int index, const QString & text) {
        Q_UNUSED(text)

        if (index == REM_SETTING) {
            m_hmiScreen->showFuncConnectRemControl(m_sPkgName);
        }
        // 停止定时器、关闭弹框
        m_timer->stop();
        close();
    });

    // 安装定时器
    m_timer = new QTimer();
    connect(m_timer, &QTimer::timeout, [ = ]() {
        m_elapsedSeconds--;

        // 实时变化按钮的时间
        if (m_elapsedSeconds > 0) {
            setButtonText(REM_OK, tr("OK (%1)").arg(m_elapsedSeconds));
            return;
        } else {
            // 关闭弹框
            close();
        }
    });
    m_timer->start(1000);
}

RemDisableDialog::~RemDisableDialog()
{
    Q_EMIT onDialogClose();

    m_timer->stop();
    m_timer->deleteLater();
}
