/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci_cm@deepin.com>
 *
 * Maintainer: caimengci <caimengci_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <QObject>
#include <QMap>
#include <QFileSystemWatcher>

#include "startupmonitordialog.h"

class StartupMonitor : public QObject
{
    Q_OBJECT
public:
    explicit StartupMonitor(QObject *parent = nullptr);

    // 外部接口-添加监控路径
    void addWatchPath(QString path);

    static StartupMonitor *getInstance();
public Q_SLOTS:
    // 目录更新时调用
    void directoryUpdated(const QString &dirPath);
    // 文件被修改时调用
    void fileUpdated(const QString &filePath);
    // 延迟调用响应函数
    void delayAction();
    // 表格数据刷新
    void doDataRefresh();
    // 弹框关闭槽
    void doDialogClose();

private:
    void checkActionSource(QString sFilePath, QString sAppsNmae);
    void doDataChange(bool isStartupD);

Q_SIGNALS:
    void doDataChage(bool bAdd, QString sID);

private:
    static StartupMonitor *m_pInstance;                 // 单例
    QFileSystemWatcher *m_pWatcherDir;                  // 文件系统监视器 -- 监控文件夹
    QFileSystemWatcher *m_pWatcherFile;                 // 文件系统监视器 -- 监控文件
    QMap<QString, QStringList> m_currentContentsMap;    // 当前每个监控的内容目录列表

    StartupMonitorDialog *m_dialog;                     // 开机自启动 -- 拦截弹框
    QString m_sFilePath;
    QString m_sFileName;
    QString m_sLastFileName;
};
