/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci_cm@deepin.com>
 *
 * Maintainer: caimengci <caimengci_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "startupmonitordialog.h"

#include <DLabel>

#include <QFile>
#include <QDebug>
#include <QAbstractButton>

#define Monitor_Dialog_Times                30

StartupMonitorDialog::StartupMonitorDialog(DDialog *parent)
    : DDialog(parent)
    , m_startManagerInterface(new DBusStartManager)
    , m_launcherInter(new Launcherd("com.deepin.dde.daemon.Launcher", "/com/deepin/dde/daemon/Launcher", QDBusConnection::sessionBus(), this))
{
    setOnButtonClickedClose(true);

    this->setWindowFlag(Qt::WindowStaysOnTopHint);
    this->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

    // 安装定时器
    m_showTimer = new QTimer(this);
    connect(m_showTimer, SIGNAL(timeout()), this, SLOT(showButtonTime()));
    m_showTimer->setInterval(1000);
    m_closeTimer = new QTimer(this);
    connect(m_closeTimer, SIGNAL(timeout()), this, SLOT(closeDialog()));
    m_closeTimer->setInterval(Monitor_Dialog_Times * 1000);
}

StartupMonitorDialog::~StartupMonitorDialog()
{
    // modified by caimc at 2020/06/28 弹框关闭发送信号到外部清除弹框保存值
    Q_EMIT onDialogClose();

    if (m_startManagerInterface)
        m_startManagerInterface->deleteLater();
    m_startManagerInterface = nullptr;
}

void StartupMonitorDialog::setAppsName(QString sAppsName)
{
    m_sAppsName = sAppsName;
    qDebug() << "dialog monitor name:" << m_sAppsName;
}

void StartupMonitorDialog::setAppsPath(QString sAppsPath)
{
    m_sAppsPath = sAppsPath;
    qDebug() << "dialog monitor path:" << m_sAppsPath;
    QDBusPendingReply<LauncherItemInfoList> reply = m_launcherInter->GetAllItemInfos();
    reply.waitForFinished();
    if (reply.isError()) {
        qDebug() << "data is empty, quit!!";
        qDebug() << reply.error();
    }

    // 添加数据到QMap容器
    const LauncherItemInfoList &datas = reply.value();
    for (const auto &it : datas) {
        QString sPath = it.Path;
        QStringList sPathList = sPath.split("/");
        if (sPathList.at(sPathList.count() - 1) == m_sAppsName) {
            QIcon icon = QIcon::fromTheme(it.Icon);
            setIcon(icon);
            m_sAppsName = it.Name;
        }
    }
    // 初始化弹框界面
    initUI();
}

void StartupMonitorDialog::initUI()
{
    // 初始化弹框
    DLabel *txt = new DLabel(tr("%1 is adding to system startup").arg(m_sAppsName));
    txt->setWordWrap(true);
    addContent(txt);
    addButton(tr("Allow"), true, DDialog::ButtonRecommend);
    addButton(tr("Disable") + QString("  (%1s)").arg(Monitor_Dialog_Times), true, DDialog::ButtonRecommend);

    connect(this, &DDialog::buttonClicked, this, [ = ](int index, const QString & text) {
        Q_UNUSED(text)
        qDebug() << index << text;
        if (index == 1) {
            deleteFile();
        }
    });

    // 启动定时器
    m_showTimer->start();
    m_closeTimer->start();
}

void StartupMonitorDialog::showButtonTime()
{
    m_nTimeCout ++;
    getButton(1)->setText(tr("Disable") + QString("  (%1s)").arg(30 - m_nTimeCout));
}

void StartupMonitorDialog::closeDialog()
{
    deleteFile();
    reject();
}

// 检测第三方软件修改开机自启动 并且关闭弹框 移除开机自启切删除文件
void StartupMonitorDialog::deleteFile()
{
    m_showTimer->stop();
    m_closeTimer->stop();
    if (m_startManagerInterface->IsAutostart(m_sAppsPath)) {
        QDBusPendingReply<bool> reply = m_startManagerInterface->RemoveAutostart(m_sAppsPath);
        reply.waitForFinished();
        if (!reply.isError())
            qDebug() << m_sAppsName << " close finished";
    }

    // 删除自启动项目录 的文件
    QFile file(m_sAppsPath);
    qDebug() << m_sAppsPath;

    if (file.remove())
        qDebug() << m_sAppsName << " delete finished";

    Q_EMIT monitorChange();
}

void StartupMonitorDialog::closeEvent(QCloseEvent *event)
{
    Q_UNUSED(event);

    deleteFile();
}
