/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     luokai <luokai_cm@deepin.com>
 *
 * Maintainer: luokai <luokai_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "trafficdetailsworker.h"

//流量数据缓冲存储间隔，单位s，最大设置为60s，间隔影响流量跨时统计误差
const int SaveFlowDateInterval = 5;

TrafficDetailsWorker::TrafficDetailsWorker(QObject *parent)
    : QObject(parent)
    , m_monitornetflow(new MonitorNetFlow("com.deepin.defender.MonitorNetFlow", "/com/deepin/defender/MonitorNetFlow", QDBusConnection::systemBus(), this))
    , m_useData(new UseData)
    , m_saveFlowDateTimer(new QTimer)
{
    //获取数据
    connect(m_monitornetflow, &MonitorNetFlow::NotifyDetailData, this, &TrafficDetailsWorker::onAppMonitorNetFlowDetailData);
    m_monitornetflow->ExitApp();
    QThread::currentThread()->msleep(500);
    for (int i = 0; i < 10; i++)
        m_monitornetflow->StartApp();

    //初始化默认显示的时间跨度
    m_flowRangeStyle = FlowRangeStyle::None;

    //save flow date
    QThread *m_useDateThread = new QThread;
    m_useData->moveToThread(m_useDateThread);
    m_useDateThread->start();

    m_saveFlowDateTimer->setInterval(1000 * SaveFlowDateInterval);
    m_saveFlowDateTimer->start();
    connect(m_saveFlowDateTimer, &QTimer::timeout, this, &TrafficDetailsWorker::saveFlowData);

    connect(m_useData, &UseData::sendTodayFlowData, this, [ = ](QList<AppFlowDateParameter> list) {
        onReceiveAllAppFlowDate(list, FlowRangeStyle::Today);
    });
    connect(m_useData, &UseData::sendYesterdayFlowData, this, [ = ](QList<AppFlowDateParameter> list) {
        onReceiveAllAppFlowDate(list, FlowRangeStyle::Yesterday);
    });
    connect(m_useData, &UseData::sendThisMonthFlowData, this, [ = ](QList<AppFlowDateParameter> list) {
        onReceiveAllAppFlowDate(list, FlowRangeStyle::ThisMonth);
    });
    connect(m_useData, &UseData::sendLastMonthFlowData, this, [ = ](QList<AppFlowDateParameter> list) {
        onReceiveAllAppFlowDate(list, FlowRangeStyle::LastMonth);
    });

    connect(m_useData, &UseData::sendAppTodayFlowData, this, &TrafficDetailsWorker::onReceiveAppFlowDate);
    connect(m_useData, &UseData::sendAppYesterdayFlowData, this, &TrafficDetailsWorker::onReceiveAppFlowDate);
    connect(m_useData, &UseData::sendAppThisMonthFlowData, this, &TrafficDetailsWorker::onReceiveAppFlowDate);
    connect(m_useData, &UseData::sendAppLastMonthFlowData, this, &TrafficDetailsWorker::onReceiveAppFlowDate);

    // 获取当前所有进程流量信息列表
    m_flowInfoList = GetProcInfoInter::instance()->getProcInfoList();
    // 定时更新进程流量数据
    QTimer *flowInfoUpdateTimer = new QTimer(this);
    flowInfoUpdateTimer->setInterval(1000 * 2);
    connect(flowInfoUpdateTimer, &QTimer::timeout,this, &TrafficDetailsWorker::updateFlowInfoList);
    flowInfoUpdateTimer->start();
}

QList<DefenderProcInfo> TrafficDetailsWorker::getFlowInfoList()
{
    QList<DefenderProcInfo> retProcInfos;
    // 合并无包名的进程信息为 "其它应用"
    DefenderProcInfo otherAppInfo;
    otherAppInfo.nPid = 0;
    otherAppInfo.sExecPath = "";
    otherAppInfo.isbSysApp = true;
    otherAppInfo.sProcName = "other apps";
    otherAppInfo.sDesktopPath = "";
    otherAppInfo.sPkgName = "";
    otherAppInfo.sThemeIcon = "";
    otherAppInfo.dDownloadSpeed = 0.0;
    otherAppInfo.dUploadSpeed = 0.0;
    otherAppInfo.dDownloads = 0.0;
    otherAppInfo.dUploads = 0.0;
    // 网速信息以上一段时间内统计的为准
    for (DefenderProcInfo procInfo : m_lastPeriodFlowInfos) {
        if (procInfo.sPkgName.isEmpty()) {
            otherAppInfo.dDownloadSpeed += procInfo.dDownloadSpeed;
            otherAppInfo.dUploadSpeed += procInfo.dUploadSpeed;
            otherAppInfo.dDownloads += procInfo.dDownloads;
            otherAppInfo.dUploads += procInfo.dUploads;
        } else {
            // 合并相同包名项
            bool isbSamePkgInfo = false;
            for (int i = 0; i < retProcInfos.size(); i++) {
                DefenderProcInfo sameProcInfo = retProcInfos.at(i);
                if (procInfo.sPkgName == sameProcInfo.sPkgName) {
                    sameProcInfo.dDownloadSpeed += procInfo.dDownloadSpeed;
                    sameProcInfo.dUploadSpeed += procInfo.dUploadSpeed;
                    sameProcInfo.dDownloads += procInfo.dDownloads;
                    sameProcInfo.dUploads += procInfo.dUploads;

                    isbSamePkgInfo = true;
                    retProcInfos.replace(i, sameProcInfo);
                    break;
                }
            }
            if (!isbSamePkgInfo) {
                retProcInfos.append(procInfo);
            }
        }
    }

    retProcInfos.append(otherAppInfo);
    return retProcInfos;
}

//获取网速/流量并写入对应list
void TrafficDetailsWorker::onAppMonitorNetFlowDetailData(const QString &name, double recv, double send,
                                                         int pid, const QString &device, const QString &type)
{
    Q_UNUSED(name);
    Q_UNUSED(device);

#if Work_Debug_Flag
    // 打印进程名信息
    qDebug() << "|pid" << pid << "|name" << name.split(" ").first().split("/").last()
             // 打印 接受、发送和类型 信息
             << "|recv" << recv << "|send" << send << "|type" << type;
#endif

    if((recv == 0.0) && (send == 0.0)){
        return;
    }

    if (pid == 0)
        return;

    if(recv >0 && recv <0.01){
        recv = 0.01;
    }
    if(send >0 && send <0.01){
        send = 0.01;
    }

    // 更新进程流量信息列表
    for (int i = 0; i < m_flowInfoList.size(); i++) {
        DefenderProcInfo flowInfo = m_flowInfoList.at(i);
        // 找到相同进程
        if (pid == flowInfo.nPid) {
            if ("kb/s" == type) {
                flowInfo.dDownloadSpeed = recv;
                flowInfo.dUploadSpeed = send;
                m_flowInfoList.replace(i, flowInfo);
            } else if ("total_kb" == type) {
                flowInfo.dDownloads = recv;
                flowInfo.dUploads = send;
                m_flowInfoList.replace(i, flowInfo);
            }
            // 更新数据后退出循环
            break;
        }
    }

    // 更新上段时间进程流量信息列表
    for (int i = 0; i < m_lastPeriodFlowInfos.size(); i++) {
        DefenderProcInfo flowInfo = m_lastPeriodFlowInfos.at(i);
        // 找到相同进程
        if (pid == flowInfo.nPid) {
            if ("kb/s" == type) {
                // 若为网速数据
                flowInfo.dDownloadSpeed = recv;
                flowInfo.dUploadSpeed = send;
                // 更新列表中相同进程的数据
                m_lastPeriodFlowInfos.replace(i, flowInfo);
            } else if ("total_kb" == type) {
                // 若为累计流量数据
                flowInfo.dDownloads = recv;
                flowInfo.dUploads = send;
                // 更新列表中相同进程的数据
                m_lastPeriodFlowInfos.replace(i, flowInfo);
            }
            // 更新数据后退出循环
            break;
        }
    }

}

void TrafficDetailsWorker::notifyGetYesterdayFlowData()
{
    m_flowRangeStyle = FlowRangeStyle::Yesterday;
    Q_EMIT m_useData->notifyGetYesterdayFlowData();
}

void TrafficDetailsWorker::notifyGetTodayFlowData()
{
    m_flowRangeStyle = FlowRangeStyle::Today;
    Q_EMIT m_useData->notifyGetTodayFlowData();
}

void TrafficDetailsWorker::notifyGetLastMonthFlowData()
{
    m_flowRangeStyle = FlowRangeStyle::LastMonth;
    Q_EMIT m_useData->notifyGetLastMonthFlowData();
}

void TrafficDetailsWorker::notifyGetThisMonthFlowData()
{
    m_flowRangeStyle = FlowRangeStyle::ThisMonth;
    Q_EMIT m_useData->notifyGetThisMonthFlowData();
}

// 通知停止获取所有应用流量数据
void TrafficDetailsWorker::NotifyDontGetFlowData()
{
    m_flowRangeStyle = FlowRangeStyle::None;
}

void TrafficDetailsWorker::notifyGetAppYesterdayFlowData(QString sPkgName)
{
    Q_EMIT m_useData->notifyGetAppYesterdayFlowData(sPkgName);
}

void TrafficDetailsWorker::notifyGetAppTodayFlowData(QString sPkgName)
{
    Q_EMIT m_useData->notifyGetAppTodayFlowData(sPkgName);
}

void TrafficDetailsWorker::notifyGetAppLastMonthFlowData(QString sPkgName)
{
    Q_EMIT m_useData->notifyGetAppLastMonthFlowData(sPkgName);
}

void TrafficDetailsWorker::notifyGetAppThisMonthFlowData(QString sPkgName)
{
    Q_EMIT m_useData->notifyGetAppThisMonthFlowData(sPkgName);
}

void TrafficDetailsWorker::updateFlowInfoList()
{
    // 保存当前时间段的网速和流量数据
    m_lastPeriodFlowInfos = m_flowInfoList;
    // 获得最新的进程信息列表，同时也清空了所有进程的网速和流量数据
    m_flowInfoList = GetProcInfoInter::instance()->getProcInfoList();

    // 各进程累计流量数据不需要清空，需要继承前一段时间的各进程累计流量数据
    for (int i = 0; i < m_flowInfoList.size(); i++) {
        // 待更新的进程信息
        DefenderProcInfo updatingProInfo = m_flowInfoList.at(i);
        for (int j = 0; j < m_lastPeriodFlowInfos.size(); j++) {
            DefenderProcInfo procInfoTmp = m_lastPeriodFlowInfos.at(j);
            // 找到相同进程
            if (updatingProInfo.nPid == procInfoTmp.nPid) {
                // 更新累计下载流量数据
                updatingProInfo.dDownloads = procInfoTmp.dDownloads;
                // 更新累计上传流量数据
                updatingProInfo.dUploads = procInfoTmp.dUploads;
                m_flowInfoList.replace(i, updatingProInfo);
                break;
            }
        }
    }
}

void TrafficDetailsWorker::saveFlowData()
{
//    DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
    //如果修改了系统时间
    if (QDateTime::currentDateTime() < m_lastSavedTime) {
        m_monitornetflow->ExitApp();
        QThread::currentThread()->msleep(500);
        for (int i = 0; i < 5; i++)
            m_monitornetflow->StartApp();
    }

    QList<AppFlowDateParameter> appFlowCacheList = {};
    for (DefenderProcInfo procInfo : m_flowInfoList) {
        AppFlowDateParameter appFlowCache = {};
        // 当前进程有流量数据时，向数据库中储存
        if (0.001 < procInfo.dDownloads || 0.001 < procInfo.dUploads) {
            appFlowCache.sPkgName = procInfo.sPkgName;
            appFlowCache.sExecPath = procInfo.sExecPath;
            // 优先使用desktop路径代替执行路径
            if (!procInfo.sDesktopPath.isEmpty()) {
                appFlowCache.sExecPath = procInfo.sDesktopPath;
            }
            appFlowCache.dDownloaded = procInfo.dDownloads;
            appFlowCache.dUploaded = procInfo.dUploads;
            appFlowCacheList.append(appFlowCache);
        }
    }
    Q_EMIT m_useData->notifySaveFlowCacheDataToDB(appFlowCacheList);

    //根据此时刻更新详情列表显示的数据
    switch (m_flowRangeStyle) {
    case FlowRangeStyle::Today:
        notifyGetTodayFlowData();
        break;
    case FlowRangeStyle::Yesterday:
        notifyGetYesterdayFlowData();
        break;
    case FlowRangeStyle::ThisMonth:
        notifyGetThisMonthFlowData();
        break;
    case FlowRangeStyle::LastMonth:
        notifyGetLastMonthFlowData();
        break;
    default:
        break;
    }

    m_lastSavedTime = QDateTime::currentDateTime();
}

void TrafficDetailsWorker::onReceiveAllAppFlowDate(QList<AppFlowDateParameter> appLst, FlowRangeStyle range)
{
//    DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);

    m_onShowFlowList.clear();
    for (AppFlowDateParameter app : appLst) {
        DefenderProcInfo procInfoTmp = {};
        procInfoTmp.sExecPath = app.sExecPath;

        // 从进程信息解析接口中得到 应用名称、图标、包名、是否是系统应用
        QString sPkgName = GetProcInfoInter::instance()->getPkgNameByExecPath(procInfoTmp.sExecPath);
        DefenderProcInfo installedAppInfo = GetProcInfoInter::instance()->getProcInfoByPkgName(sPkgName);

        procInfoTmp.sProcName = procInfoTmp.sExecPath.split("/").last();
        procInfoTmp.isbSysApp = installedAppInfo.isbSysApp;
        procInfoTmp.sDesktopPath = installedAppInfo.sDesktopPath;
        procInfoTmp.sPkgName = installedAppInfo.sPkgName;
        // 如果包名为空，则将进程名和执行路径设为 "other apps"
        if (procInfoTmp.sPkgName.isEmpty()) {
            procInfoTmp.sProcName = "other apps";
            procInfoTmp.sExecPath = "other apps";
            procInfoTmp.isbSysApp = true;
        }

        procInfoTmp.sAppName = installedAppInfo.sAppName;
        procInfoTmp.sThemeIcon = installedAppInfo.sThemeIcon;
        // 更新下载量
        procInfoTmp.dDownloads = app.dDownloaded;
        // 更新上传量
        procInfoTmp.dUploads = app.dUploaded;

        // 在显示列表中找到相同应用项
        int iSameIndex = -1;
        for (int i = 0; i < m_onShowFlowList.size(); i++) {
            DefenderProcInfo sameInfoTmp = m_onShowFlowList.at(i);
            if (procInfoTmp.sPkgName == sameInfoTmp.sPkgName) {
                iSameIndex = i;
                break;
            }
        }
        // 合并相同应用的数据
        if (-1 != iSameIndex) {
            DefenderProcInfo sameInfo = m_onShowFlowList.at(iSameIndex);
            sameInfo.dDownloads += procInfoTmp.dDownloads;
            sameInfo.dUploads += procInfoTmp.dUploads;
            m_onShowFlowList.replace(iSameIndex, sameInfo);
        } else {
            m_onShowFlowList.append(procInfoTmp);
        }
    }
    Q_EMIT sendAllAppFlowData(m_onShowFlowList, range);
#if Work_Debug_Flag
    qDebug() << m_onShowFlowList.size();
#endif
}

void TrafficDetailsWorker::onReceiveAppFlowDate(QList<AppFlowDateParameter> appLst)
{
    DefenderProcInfoList onShowAppFlowList;
    for (AppFlowDateParameter app : appLst) {
        DefenderProcInfo procInfoTmp;
        procInfoTmp.sExecPath = app.sExecPath;

        // 初始化 应用名称、图标、包名、是否是系统应用
        procInfoTmp.sProcName = "";
        procInfoTmp.isbSysApp = true;
        procInfoTmp.sDesktopPath = "";
        procInfoTmp.sPkgName = app.sPkgName;
        procInfoTmp.sAppName = "";
        procInfoTmp.sThemeIcon = "";
        // 更新日期时间
        procInfoTmp.sTimeyMdh = app.sTimeyMdh;
        // 更新下载量
        procInfoTmp.dDownloads = app.dDownloaded;
        // 更新上传量
        procInfoTmp.dUploads = app.dUploaded;

        onShowAppFlowList.append(procInfoTmp);
    }
    Q_EMIT sendAppFlowData(onShowAppFlowList);
}
