/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     luokai <luokai_cm@deepin.com>
 *
 * Maintainer: luokai <luokai_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once

#include "com_deepin_defender_monitornetflow.h"
#include "csingleteon.h"
#include "usedata.h"
#include "common.h"
#include "window/modules/common/getprocinfointer.h"
#include "window/modules/common/defenderprocinfo.h"
#include "window/modules/common/defenderprocinfolist.h"

#include <QThread>
#include <QObject>

#define Work_Debug_Flag 0

//时间跨度
enum FlowRangeStyle {None, Today, Yesterday, ThisMonth, LastMonth};

using Launcherd = com::deepin::dde::daemon::Launcher;
using MonitorNetFlow = com::deepin::defender::MonitorNetFlow;

class TrafficDetailsWorker : public QObject
{
    Q_OBJECT
public:
    TrafficDetailsWorker(QObject *parent = nullptr);
    //获取worker中的使用数据库类实例
    inline UseData *getUseData() {return m_useData;}
    //获取流量监控接口
    inline MonitorNetFlow *getNetFlowMonitorInter() {return m_monitornetflow;}

    // 获取当前进程流量信息列表
    QList<DefenderProcInfo> getFlowInfoList();

    void notifyGetYesterdayFlowData();
    void notifyGetTodayFlowData();
    void notifyGetLastMonthFlowData();
    void notifyGetThisMonthFlowData();
    // 通知停止获取所有应用流量数据
    void NotifyDontGetFlowData();

    void notifyGetAppYesterdayFlowData(QString sPkgName);
    void notifyGetAppTodayFlowData(QString sPkgName);
    void notifyGetAppLastMonthFlowData(QString sPkgName);
    void notifyGetAppThisMonthFlowData(QString sPkgName);

Q_SIGNALS:
    void sendAllAppFlowData(QList<DefenderProcInfo> allAppFlowList, FlowRangeStyle range);
     void sendAppFlowData(QList<DefenderProcInfo> allAppFlowList);

public Q_SLOTS:
    void onAppMonitorNetFlowDetailData(const QString &name, double recv, double send, int pid, const QString &device, const QString &type);

    // 更新进程流量信息列表
    void updateFlowInfoList();
    //存储应用流量数据
    void saveFlowData();
    //当从UseData中接收到流量数据
    void onReceiveAllAppFlowDate(QList<AppFlowDateParameter> appLst, FlowRangeStyle range);
    //当从UseData中接收到流量数据
    void onReceiveAppFlowDate(QList<AppFlowDateParameter> appLst);

private:
    MonitorNetFlow *m_monitornetflow;
    //当前显示的时间段
    FlowRangeStyle m_flowRangeStyle;
    //当前显示时间段下的应用流量统计列表
    QList<DefenderProcInfo> m_onShowFlowList;
    //使用数据库
    UseData *m_useData;
    //应用数据存储定时
    QTimer *m_saveFlowDateTimer;
    //上次存储流量数据的时间
    QDateTime m_lastSavedTime;

    QList<DefenderProcInfo> m_flowInfoList;
    // 前一段时间进程流量信息
    QList<DefenderProcInfo> m_lastPeriodFlowInfos;
};
