/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     wubowen <wubowen_cm@deepin.com>
 *
 * Maintainer: wubowen <wubowen_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once

#define USE_DEBUG 1
#define FlowLogFlag 0

#if USE_DEBUG
#include "monitornetflow_adaptor.h"
#endif

#include "usbconnection/usbmanagemonitor.h"

#include <QObject>
#include <QThread>

// 流量监控服务类
class CatchNetFlow;

const QString service = "com.deepin.defender.MonitorNetFlow";
const QString path = "/com/deepin/defender/MonitorNetFlow";
const int timeout = 3;

class WriteDBusData : public QObject
{
    Q_OBJECT
public:
    explicit WriteDBusData(int argc, char *argv[], QObject *parent = nullptr);
    ~WriteDBusData();

Q_SIGNALS:
    void notifyDBusKbData(QString, double, double, int, QString, QString);
    void notifyGlobalData(double, double);
    void notifyWorking();
    //usb设备接入的信号，入参bflag 为false,表示限制状态
    void NotifyUsbConnectionInfo(bool bFlag, QString sUsbInfo);
    void SendUsbConnectionInfo(QString sUsbInfos);
    //通知显示限制所有usb设备的系统通知
    void requestShowBlockAllSysNotify();
    //通知显示限制非白名单usb设备的系统通知
    void requestShowOnlyWhiteListSysNotify();

public Q_SLOTS:
    void onNotifyDBusData(QString name, double recv_value, double send_value, int pid, QString device, QString type);
    void onNotifyGlobalData(double recv_value, double send_value);
    void StartApp();
    void ExitApp();
    void onNotifyHeartbeat();
    QString getProExePath(QString sId);
    // 开启/关闭进程流量监控功能
    void EnableNetFlowMonitor(bool enable);
    // 将待发送设备信息整合成字符串
    inline QString MergeDevSavingInfoToStr(QString sName, QString sStatus, QString sSerial, QString sVendorID,
                                           QString sProductID, QString sParentSize, QString sSize) {
        return  m_usbManageMonitor->mergeDevSavingInfoToStr(sName, sStatus, sSerial, sVendorID,
                                                    sProductID, sParentSize, sSize);
    }
    inline void WriteConnectionRecord(QString sName, QString sDatetime, QString sStatus,
                                      QString sSerial, QString sVendorID, QString sProductID,
                                      QString sParentSize, QString sSize) {
        m_usbManageMonitor->writeConnectionRecord(sName, sDatetime, sStatus, sSerial,
                                                  sVendorID, sProductID, sParentSize, sSize);
    }

    inline void ClearUsbConnectionLog() {
        m_usbManageMonitor->clearUsbConnectionLog();
    }

    inline void WriteConnectionWhiteLst(QString sName, QString sDatetime, QString sStatus,
                                        QString sSerial, QString sVendorID, QString sProductID,
                                        QString sParentSize, QString sSize) {
        m_usbManageMonitor->writeConnectionWhiteLst(sName, sDatetime, sStatus, sSerial,
                                                    sVendorID, sProductID, sParentSize, sSize);
    }

    inline void RemoveFromWhiteLst(QString sName, QString sSerial, QString sVendorID, QString sProductID) {
        // 暂不使用设备名称
        Q_UNUSED(sName);
        m_usbManageMonitor->removeFromWhiteLst(sSerial, sVendorID, sProductID);
    }

    inline void ChangeIsbSaveRecord(bool change) {m_usbManageMonitor->changeIsbSaveRecord(change);}
    inline void SelectLimitationModel(QString modelStr) {m_usbManageMonitor->selectLimitationModel(modelStr);}
    inline bool GetRecordSaveSwitch() {return m_usbManageMonitor->getRecordSaveSwitch();}
    inline QString GetLimitModel() {return m_usbManageMonitor->getLimitModel();}
    inline QString GetUsbConnectionRecords() {return m_usbManageMonitor->getUsbConnectionRecords();}
    inline QString GetWhiteList() {return m_usbManageMonitor->getWhiteList();}

private:
    // 流量监控服务
    CatchNetFlow *m_catchNetFlow;
    double m_recv_value;
    double m_send_value;
    double global_recv_value;
    double global_send_value;
    int m_pid;
    QString m_device;

    UsbManageMonitor *m_usbManageMonitor;
    QThread *m_usbManageMonitorThread;
#if USE_DEBUG
    MonitorNetFlowAdaptor *m_adaptor;
#endif
};
