/*
 * Copyright (C) 2019 ~ 2020 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "defendertable.h"

#include <DFrame>
#include <DLabel>
#include <DApplicationHelper>

#include <QtDebug>

DWIDGET_USE_NAMESPACE

namespace def {
namespace widgets {

DefenderTable::DefenderTable(QWidget *parent)
    : DFrame(parent)
    , m_tableModel(nullptr)
    , m_table(nullptr)
{
    m_rowHeight = 30;
    m_rowCount = 5;

    initUI();
}

DefenderTable::~DefenderTable()
{

}

void DefenderTable::initUI()
{
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->setContentsMargins(2, 2, 2, 2);
//    mainLayout->setContentsMargins(0, 0, 0, 0);
    setLayout(mainLayout);

    //创建主题风格背景
    m_mainBackgroundWidgt = new DFrame(this);
    m_mainBackgroundWidgt->lower();
    m_mainBackgroundWidgt->resize(size());

    // 设置表头
    QStandardItem *standaritem0 = new QStandardItem("head1.");
    standaritem0->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *standaritem1 = new QStandardItem("head2");
    standaritem1->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    m_tableModel = new QStandardItemModel;;
    m_tableModel->clear();
    m_tableModel->setHorizontalHeaderItem(0, standaritem0);
    m_tableModel->setHorizontalHeaderItem(1, standaritem1);
    m_tableModel->setRowCount(m_rowCount);

    m_table = new DTableView();

    m_table->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_table->setSelectionMode(QAbstractItemView::NoSelection);
    m_table->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_table->setFrameShape(QTableView::NoFrame);
    m_table->setFocusPolicy(Qt::NoFocus);
    //自动调整最后一列的宽度使它和表格的右边界对齐
    m_table->horizontalHeader()->setStretchLastSection(true);
    m_table->verticalHeader()->setHidden(true);
    m_table->setShowGrid(false);
    m_table->setSortingEnabled(true);
    m_table->verticalHeader()->setDefaultSectionSize(48);
    //设置内容
    m_table->setModel(m_tableModel);

    //设置每一行的高度
    for (int i = 0; i < m_rowCount; i++)
        m_table->setRowHeight(i, m_rowHeight);

    //设置表格内容部分背景颜色为透明
    QPalette pa = m_table->palette();
    pa.setBrush(QPalette::Base, Qt::transparent);
    m_table->viewport()->setPalette(pa);

    //为表格添加背景
    for (int i = 0; i < m_rowCount; i++) {
        DFrame *bgmTmp = new DFrame(m_table);
        bgmTmp->setFixedHeight(m_rowHeight);
        bgmTmp->setLineWidth(0);
        bgmTmp->lower();
        if (0 == i % 2) {
            bgmTmp->setBackgroundRole(DPalette::ItemBackground);
        }
        m_backgroundLst.append(bgmTmp);
    }

    mainLayout->addWidget(m_table);

    //connect
    connect(m_tableModel, &QStandardItemModel::rowsRemoved, this, [ = ] {
        m_backgroundLst.last()->deleteLater();
        m_backgroundLst.removeLast();
        m_rowCount--;
    });
    connect(m_tableModel, &QStandardItemModel::rowsInserted, this, [ = ] {
        m_rowCount++;
        DFrame *bgmTmp = new DFrame(m_table);
        bgmTmp->setFixedHeight(m_rowHeight);
        bgmTmp->setLineWidth(0);
        bgmTmp->lower();
        if (0 == m_rowCount % 2)
        {
            bgmTmp->setBackgroundRole(DPalette::ItemBackground);
        }
        m_backgroundLst.append(bgmTmp);
    });
}

void DefenderTable::setTableAndModel(DTableView *table, QStandardItemModel *model)
{
    disconnect(m_table, nullptr, nullptr, nullptr);
    m_table->deleteLater();
    m_tableModel->deleteLater();
    //清除表格背景
    for (int i = 0; i < m_rowCount; i++) {
        m_backgroundLst.last()->deleteLater();
        m_backgroundLst.removeLast();
    }

    //设置table and model
    m_table = table;
    m_tableModel = model;
    m_rowCount = m_tableModel->rowCount();

    m_table->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_table->setSelectionMode(QAbstractItemView::NoSelection);
    m_table->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_table->setFrameShape(QTableView::NoFrame);
    m_table->setFocusPolicy(Qt::NoFocus);
    m_table->setWordWrap(false);
    //自动调整最后一列的宽度使它和表格的右边界对齐
    m_table->horizontalHeader()->setStretchLastSection(true);
    m_table->verticalHeader()->setHidden(true);
    m_table->setShowGrid(false);
    m_table->setSortingEnabled(true);
    //设置内容
    m_table->setModel(m_tableModel);

    //设置每一行的高度
    for (int i = 0; i < m_rowCount; i++)
        m_table->setRowHeight(i, m_rowHeight);

    //设置表格内容部分背景颜色为透明
    QPalette pa = m_table->palette();
    pa.setBrush(QPalette::Base, Qt::transparent);
    m_table->viewport()->setPalette(pa);

    //为表格每行添加背景
    for (int i = 0; i < m_rowCount; i++) {
        DFrame *bgmTmp = new DFrame(m_table);
        bgmTmp->setFixedHeight(m_rowHeight);
        bgmTmp->setLineWidth(0);
        bgmTmp->lower();
        if (0 == i % 2) {
            bgmTmp->setBackgroundRole(DPalette::ItemBackground);
        }
        m_backgroundLst.append(bgmTmp);
    }

    //添加到主布局
    layout()->addWidget(m_table);

    //connect
    connect(m_tableModel, &QStandardItemModel::rowsRemoved, this, [ = ] {
        m_backgroundLst.last()->deleteLater();
        m_backgroundLst.removeLast();
        m_rowCount--;
    });
    connect(m_tableModel, &QStandardItemModel::rowsInserted, this, [ = ] {
        m_rowCount++;
        DFrame *bgmTmp = new DFrame(m_table);
        bgmTmp->setFixedHeight(m_rowHeight);
        bgmTmp->setLineWidth(0);
        bgmTmp->lower();
        bgmTmp->show();
        if (0 == m_rowCount % 2)
        {
            bgmTmp->setBackgroundRole(DPalette::ItemBackground);
        }
        m_backgroundLst.append(bgmTmp);
    });

}

//废弃于2020.4.30
void DefenderTable::setRowHeight(int height)
{
    m_rowHeight = height;
    //设置每一行的高度
    for (int i = 0; i < m_rowCount; i++)
        m_table->setRowHeight(i, m_rowHeight);
}

void DefenderTable::resizeEvent(QResizeEvent *event)
{
    Q_UNUSED(event);

    m_mainBackgroundWidgt->resize(size());

    int initY = m_table->viewport()->y();
    for (int i = 0; i < m_rowCount; i++) {
        int rowHeight = m_table->rowHeight(i);
        m_table->setRowHeight(i, rowHeight);
        m_backgroundLst[i]->setFixedWidth(m_table->width() + m_table->horizontalScrollBar()->maximum() - 1);
        m_backgroundLst[i]->setFixedHeight(rowHeight);
        m_backgroundLst[i]->move(- m_table->horizontalScrollBar()->value(),
                                 initY + i * rowHeight - m_table->verticalScrollBar()->value());
    }
}

void DefenderTable::paintEvent(QPaintEvent *event)
{
    Q_UNUSED(event);

    int initY = m_table->viewport()->y();
    for (int i = 0; i < m_rowCount; i++) {
        int rowHeight = m_table->rowHeight(i);
        m_table->setRowHeight(i, rowHeight);
        m_backgroundLst[i]->setFixedWidth(m_table->width() + m_table->horizontalScrollBar()->maximum() - 1);
        m_backgroundLst[i]->setFixedHeight(rowHeight);
        m_backgroundLst[i]->move(- m_table->horizontalScrollBar()->value(),
                                 initY + i * rowHeight - m_table->verticalScrollBar()->value());
    }

}

}
}
