/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "scoreprogressbar.h"

#include <QPainter>
#include <QHBoxLayout>
#include <QDebug>

DWIDGET_USE_NAMESPACE

//ScoreProgressBarBg setting
const int SingleRectangleWidth = 65;
const int SingleRectangleHeight = 12;
const int MiniWidth = SingleRectangleWidth * 3 + 20;

//ScoreProgressBar setting
const QColor BackgroundColor(229, 229, 229);

ScoreProgressBar::ScoreProgressBar(QWidget *parent)
    : QWidget(parent)
    , m_timer(nullptr)
{
    setContentsMargins(0, 0, 0, 0);
    setBackgroundRole(QPalette::Window);

    m_currentValue = 0;
    m_value = 100;
    m_moveDir = 1;
    m_color = BackgroundColor;

    m_timer = new QTimer(this);
    m_timer->setInterval(10);

    //connect
    connect(m_timer, &QTimer::timeout, this, &ScoreProgressBar::updateValue);
}

ScoreProgressBar::~ScoreProgressBar()
{
    if (m_timer->isActive()) m_timer->stop();
}

void ScoreProgressBar::setCurrentValue(int value)
{
    m_currentValue = value;
}

void ScoreProgressBar::setValue(int value)
{
    m_value = value;

    if (m_value == m_currentValue) return;
    else if (m_value > m_currentValue) m_moveDir = 1;
    else if(m_value < m_currentValue) m_moveDir = -1;

    m_timer->start();
}

void ScoreProgressBar::updateValue()
{
    m_currentValue += m_moveDir;
    if (m_value == m_currentValue) m_timer->stop();

    update();
}

void ScoreProgressBar::resizeEvent(QResizeEvent *event)
{
    QWidget::resizeEvent(event);
    setFixedSize(size());
}

QSize ScoreProgressBar::minimumSizeHint() const
{
    return QSize(MiniWidth, SingleRectangleHeight);
}

void ScoreProgressBar::paintEvent(QPaintEvent *)
{
    int scaleWidth = this->width();
    int scaleHeight = this->height();
    if (MiniWidth > scaleWidth) scaleWidth = MiniWidth; //放大比例不能小于1
    if (SingleRectangleHeight > scaleHeight) scaleHeight = SingleRectangleHeight;

    QPainter painter(this);
    painter.setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing); //设置渲染特性：反锯齿
    painter.scale(scaleWidth / MiniWidth, scaleHeight / SingleRectangleHeight);
    painter.save();

    //三个圆角矩形尺寸
    QRect rect1(1, 1, SingleRectangleWidth - 1, SingleRectangleHeight - 1);
    QRect rect2(1 + SingleRectangleWidth + 10, 1, SingleRectangleWidth - 1, SingleRectangleHeight - 1);
    //QRect rect3(1 + SingleRectangleWidth * 2 + 10 * 2, 1, SingleRectangleWidth - 1, SingleRectangleHeight - 1);

    //绘制当前进度
    painter.setBrush(m_color);
    painter.setPen(Qt::NoPen);

    double percent = m_currentValue / 100.0;
    if (0.33 >= percent) {
        int width = int(SingleRectangleWidth * 3 * percent);
        QRect rect(1, 1, width, SingleRectangleHeight - 1);
        painter.drawRoundedRect(rect, 2, 2);
    } else if (0.67 >= percent) {
        painter.drawRoundedRect(rect1, 2, 2);
        int width = int(SingleRectangleWidth * 3 * (percent - 0.33) - 1);
        QRect rect(1 + SingleRectangleWidth + 10, 1, width, SingleRectangleHeight - 1);
        painter.drawRoundedRect(rect, 2, 2);
    } else {
        painter.drawRoundedRect(rect1, 2, 2);
        painter.drawRoundedRect(rect2, 2, 2);
        int width = int(SingleRectangleWidth * 3 * (percent - 0.66) - 1);
        QRect rect(1 + SingleRectangleWidth * 2 + 10 * 2, 1, width, SingleRectangleHeight - 1);
        painter.drawRoundedRect(rect, 2, 2);
    }

    painter.restore();
}
