/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     zccrs <zccrs@live.com>
 *
 * Maintainer: zccrs <zhangjide@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "mainwindow.h"

#include <DBackgroundGroup>
#include <DGuiApplicationHelper>
#include <DListView>
#include <DStandardItem>
#include <DApplicationHelper>
#include <DTitlebar>
#include <DDialog>
#include <DRadioButton>
#include <DCheckBox>
#include <DSettingsDialog>
#include <QDateTime>

#include <QHBoxLayout>
#include <QMetaEnum>
#include <QDebug>
#include <QStandardItemModel>
#include <QPushButton>
#include <QLocale>
#include <QLinearGradient>
#include <QWidget>
#include <QResizeEvent>
#include <qsettingbackend.h>
#include <QGSettings>
#include <QDBusInterface>

DEF_USING_NAMESPACE
DTK_USE_NAMESPACE

const int WidgetMinimumWidget = 950;
const int WidgetMinimumHeight = 640;

//此处为带边距的宽度
const int first_widget_min_width = 200;

const QMargins navItemMargin(5, 3, 5, 3);
const QVariant NavItemMargin = QVariant::fromValue(navItemMargin);
const QString GSettingsCloseReminder = "close-type";
const QString GSettingsAskType = "ask-type";
const QString dmanInterface = "com.deepin.Manual.Open";

#define DIALOG_WIDTH                380                     //dialog  宽度
#define DIALOG_HEIGHT               145                     //dialog  高度
#define DIALOG_PB_WIDTH             140                     //dialog  按钮宽度
#define DIALOG_PB_HEIGHT            36                      //dialog  按钮高度

#define INSTALL_TIME             "defender-install-time"      // 安装时间

MainWindow::MainWindow(QWidget *parent)
    : DMainWindow(parent)
    , m_contentLayout(nullptr)
    , m_rightContentLayout(nullptr)
    , m_navView(nullptr)
    , m_rightView(nullptr)
    , m_navModel(nullptr)
    , m_homepageModule(nullptr)
    , m_antiVirusModule(nullptr)
    , m_antiVirusWork(nullptr)
    , m_pSystemTray(nullptr)
    , m_trayMenu(nullptr)
    , m_trayDialag(nullptr)
    , m_gsetting(nullptr)
    , m_DataInterFaceServer(new DataInterFaceServer("com.deepin.defender.datainterface", "/com/deepin/defender/datainterface", QDBusConnection::sessionBus(), this))
    , m_daemonservice(new DaemonService("com.deepin.defender.daemonservice", "/com/deepin/defender/daemonservice",
                                        QDBusConnection::sessionBus(), this))
    , m_AntivirusServer(new AntivirusServer("com.deepin.defender.antiav",
                                            "/com/deepin/defender/antiav",
                                            QDBusConnection::systemBus(), this))
{
    // 设置主窗口属性
    setFixedSize(QSize(WidgetMinimumWidget, WidgetMinimumHeight));
    setWindowFlags(Qt::FramelessWindowHint | Qt::WindowMinimizeButtonHint | Qt::WindowCloseButtonHint);

    // 初始化安全中心gsetting配置
    m_gsetting = new QGSettings("com.deepin.dde.deepin-defender", QByteArray(), this);
    // 设置安装时间
    setInstallTime();
     //启动守护进程，确保守护进程每次都能正常工作
    initDaemonService();
    // 主题变换
    m_guiHelper = DGuiApplicationHelper::instance();
    connect(m_guiHelper, SIGNAL(themeTypeChanged(ColorType)), this, SLOT(themeTypeChange(ColorType)));
    // 构建扫描主页
    //m_antiavWid = new NatiavWidget();

    //Initialize view and layout structure
    QWidget *content = new QWidget(this);
    setCentralWidget(content);

    m_contentLayout = new QHBoxLayout(content);
    m_contentLayout->setContentsMargins(0, 0, 0, 0);
    m_contentLayout->setMargin(0);
    m_contentLayout->setSpacing(0);
    //左边list
    m_navView = new DListView(this);
    m_navView->setItemSize(QSize(180, 48));
    m_navView->setItemSpacing(0);
    m_navView->setSpacing(20);
    m_navView->setIconSize(QSize(32, 32));
    m_navView->setFrameShape(QFrame::Shape::NoFrame);
    m_navView->setEditTriggers(QListView::NoEditTriggers);
    m_navView->setAutoScroll(true);//滚动条
    m_navView->setTabKeyNavigation(true);//tab按键是否有效
    m_navView->setViewportMargins(QMargins(10, 10, 10, 0));//滚动区域边框大小
    m_navView->setFixedWidth(first_widget_min_width);//固定窗口值
    m_navView->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding);
    m_navView->setSpacing(0);
    m_navView->setSelectionMode(QAbstractItemView::SingleSelection);
    m_navView->setAutoFillBackground(true);
    m_navView->setBackgroundType(DStyledItemDelegate::BackgroundType(DStyledItemDelegate::RoundedBackground | DStyledItemDelegate::NoNormalState));
    m_contentLayout->addWidget(m_navView);

    // 右侧布局
    m_rightContentLayout = new QHBoxLayout;

    m_rightView = new DBackgroundGroup(m_rightContentLayout);
    m_rightView->setItemSpacing(2);
    m_rightView->setItemMargins(QMargins(0, 0, 0, 0));
    m_rightView->setContentsMargins(0, 0, 0, 0);
    m_rightView->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    m_contentLayout->addWidget(m_rightView);

    //标题栏
    DTitlebar *titlebar = this->titlebar();
    titlebar->setIcon(QIcon::fromTheme("deepin-defender"));

    auto menu = titlebar->menu();
    if (!menu) {
        qDebug() << "menu is nullptr, create menu!";
        menu = new QMenu;
    } else {
        qDebug() << "get title bar menu :" << menu;
    }
    titlebar->setMenu(menu);

    // 设置
    auto action = new QAction(tr("Settings"));
    menu->addAction(action);
    connect(action, &QAction::triggered, this, &MainWindow::showDefaultSettingDialog);

    //add help button
//    auto helpAction = new QAction(tr("Help"));
//    //menu->addAction(helpAction);
//    connect(helpAction, &QAction::triggered, this, [] {
//        QDBusInterface *inter = new QDBusInterface(dmanInterface,
//                                                   "/com/deepin/Manual/Open",
//                                                   dmanInterface,
//                                                   QDBusConnection::sessionBus());
//        inter->call("ShowManual", "dde");
//        inter->deleteLater();
//    });

    m_backwardBtn = new DIconButton(this);    //上一步按钮
    m_backwardBtn->setEnabled(false);
    m_backwardBtn->setIcon(QStyle::SP_ArrowBack);
    titlebar->addWidget(m_backwardBtn, Qt::AlignLeft | Qt::AlignVCenter);
    connect(m_backwardBtn, &DIconButton::clicked, [this] {
        qDebug() << "m_contentStack.count()" << m_contentStack.count() << m_contentStack.top().first->name();   //stack 数量  其第一个名称

//        if (2 > m_contentStack.count()) {
//            return;
//        }

        // 只针对 首页 和 病毒查杀页 进行后退判断和处理
        QString sModuleName = m_contentStack.top().first->name();
        if ("homepage" == sModuleName) {
            popWidget();
            m_homepageModule->deleteExamDetailWidget();
        } else if ("VirusScan" == sModuleName) {
            popWidget();
            m_antiVirusModule->deleteScanWidget();
        } else if ("firewall" == sModuleName) {
            this->popAndDelWidget();
        } else {
            // 其他模块直接返回首页
            this->onFirstItemClick(m_navModel->index(MODULE_HOMEPAGE_INDEX, 0));
        }

        // 若显示栈为空, 则返回首页
        if (this->m_contentStack.empty()) {
            qDebug() << "content stack is empty, turn to homepage";
            this->onFirstItemClick(m_navModel->index(MODULE_HOMEPAGE_INDEX, 0));
            return;
        }

        QWidget *topWidget = this->m_contentStack.top().second;
        if (topWidget) {
            topWidget->show();
        }

        // 更新后退按钮
        this->updateBackwardBtn();
    });

    //Initialize top page view and model
    m_navModel = new QStandardItemModel(m_navView);
    m_navView->setModel(m_navModel);
    // 连接按下信号与窗口切换槽函数
    connect(m_navView, &DListView::pressed, this, &MainWindow::onFirstItemClick);

    QTimer::singleShot(0, this, &MainWindow::initAllModule);    //初始化所有模块
    QTimer::singleShot(0, this, &MainWindow::modulePreInitialize);   //模块初始化前准备

    // 初始化托盘
    initTray();
}

MainWindow::~MainWindow()
{

}

void MainWindow::popWidget(ModuleInterface *const inter)
{
    Q_UNUSED(inter)
    // 将最后一层窗口出栈
    popWidget();

    // 每次移出窗口之后, 判断是否可以返回
    // 更新后退按钮
    this->updateBackwardBtn();
}

void MainWindow::popAndDelWidget(ModuleInterface * const inter)
{
    Q_UNUSED(inter)
    // 将最后一层窗口出栈并删除
    popAndDelWidget();
}

void MainWindow::pushWidget(ModuleInterface *const inter, QWidget *const w)
{
    if (!inter)  {
        qWarning() << Q_FUNC_INFO << " inter is nullptr";
        return;
    }

    if (!w)  {
        qWarning() << Q_FUNC_INFO << " widget is nullptr";
        return;
    }

    pushNormalWidget(inter, w);

    // 更新后退按钮
    this->updateBackwardBtn();
}

void MainWindow::setModuleVisible(ModuleInterface *const inter, const bool visible)
{
    auto find_it = std::find_if(m_modules.cbegin(),
                                m_modules.cend(),
    [ = ](const QPair<ModuleInterface *, QString> &pair) {
        return pair.first == inter;
    });

    if (find_it != m_modules.cend()) {
        m_navView->setRowHidden(find_it - m_modules.cbegin(), !visible);
        Q_EMIT moduleVisibleChanged(find_it->first->name(), visible);
    }
}

void MainWindow::setCurrentModule(int iModuleIdx, int iPageIdx)
{
    // 社区版功能裁剪
    if (DSysInfo::uosEditionType() == DSysInfo::UosEdition::UosCommunity) {
        if (iModuleIdx > 1) {
            iModuleIdx = iModuleIdx - 1;
        }
    }

    // 获取QModelIndex
    QModelIndex modelIndex = m_navModel->index(iModuleIdx, 0);
    m_navView->setCurrentIndex(m_navModel->index(iModuleIdx, 0));
    // 将显示栈清空
    popAllWidgets(0);

    // 激活模块
    ModuleInterface *inter = m_modules[modelIndex.row()].first;
    inter->active(iPageIdx);

    // 更新后退按钮
    this->updateBackwardBtn();
}

ModuleInterface *MainWindow::getCurrentModule()
{
    return m_contentStack.top().first;
}

int MainWindow::getModuleIndex(const QString &name)
{
    int index = 0;
    for (QPair<ModuleInterface *, QString> p : m_modules) {
        if (p.first->name() == name)
            return index;
        index++;
    }
    return -1;
}

// 显示默认设置弹框
void MainWindow::showDefaultSettingDialog()
{
    showSettingDialog("");
}

// 显示谈设置弹窗，并跳转到相应组位置
void MainWindow::showSettingDialog(const QString &groupKey)
{
    QTemporaryFile *tmpFile = new QTemporaryFile;
    tmpFile->open();
    auto backend = new Dtk::Core::QSettingBackend(tmpFile->fileName());

    // 只有是社区版才是配置社区版设置配置文件
    QString sSettingJson;
    if (DSysInfo::uosEditionType() == DSysInfo::UosEdition::UosCommunity) {
        sSettingJson = ":/dt-settings-community.json";
    }
    else {
        sSettingJson = ":/dt-settings.json";
    }

    auto settings = Dtk::Core::DSettings::fromJsonFile(sSettingJson);
    settings->setBackend(backend);

    //根据病毒查杀gsetting配置，更新设置界面选项
    if (DSysInfo::uosEditionType() != DSysInfo::UosEdition::UosCommunity) {
        settings->setOption("base.virus_scan.auto_scan_usb", m_gsetting->get("auto-scan-usb"));
    }

    //根据关闭窗口属性gsetting配置，更新设置界面选项
    if (0 == this->getAskType()) //优先显示每次询问按钮是否选中
        settings->setOption("base.close_window.close_window_type", getCloseType());
    else if (1 == this->getAskType())
        settings->setOption("base.close_window.close_window_type", 2);

    // 连接设置信号
    connect(settings, &Dtk::Core::DSettings::valueChanged, this, [ = ](const QString & key, const QVariant & value) {
        qDebug() << "[QSettings] key : " << key << " , value : " << value << value.toInt();
        if (key.contains("auto_scan_downloaded_file")) {
            // 设置自动查杀下载文件
            m_gsetting->set("auto-scan-downloaded-file", value);
        } else if (key.contains("auto_scan_usb")) {
            // 设置自动查杀USB存储设备
            m_gsetting->set("auto-scan-usb", value);
        } else if (key.contains("close_window_type")) {
            int type = value.toInt();
            if (type >= Tray && type <= Exit) {
                this->setCloseType(type);  //设置关闭类型 托盘还是推出
                this->setAskType(0);       //设置访问类型
            } else if (type == Ask) {
                this->setAskType(1);     //设置访问类型是每次访问
            }
        }
    });

    // 设置弹窗
    DSettingsDialog *dsd = new DSettingsDialog(this);
    dsd->updateSettings(settings);
    dsd->setModal(true);
    this->setWindowState(Qt::WindowState::WindowActive);
    this->show();
    dsd->show();
    dsd->scrollToGroup(groupKey);
    connect(dsd, &DSettingsDialog::finished, [ = ] {
        qDebug() << "setting dialog finished";
        settings->deleteLater();
        tmpFile->autoRemove();
        tmpFile->deleteLater();
        dsd->deleteLater();
    });
}

void MainWindow::toggle()
{
    raise();
    if (isMinimized() || !isVisible()) {
        // 显示并恢复窗口为正常状态
        showNormal();
    }

    activateWindow();
}

void MainWindow::popAndDelWidget()
{
    if (!m_contentStack.size())
        return;

    QWidget *w = m_contentStack.pop().second;     //栈的顶栈的widget
    m_rightContentLayout->removeWidget(w);
    w->hide();
    w->setParent(nullptr);
    w->deleteLater();
}

void MainWindow::popWidget()
{
    if (!m_contentStack.size())
        return;

    QWidget *w = m_contentStack.pop().second;     //栈的顶栈的widget
    w->hide();
    m_rightContentLayout->removeWidget(w);

    qDebug() << "popped up widget, current m_contentStack.count: " << m_contentStack.count();
}

//确保“安全中心”启动后，所有的服务都能正常工作
void MainWindow::initDaemonService()
{
    m_daemonservice->StartApp();
}


//初始化模块
void MainWindow::initAllModule()
{
    DEF_USING_ANTIVIRUS_NAMESPACE
    m_homepageModule = new HomePageModule(this);
    m_antiVirusModule = new NatiavModule(this, this);

    if (DSysInfo::uosEditionType() == DSysInfo::UosEdition::UosCommunity) {
        m_modules = {
            { m_homepageModule, tr("Home")},
            { new NetworkProtectionModule(this), tr("Firewall")},
            { new StartupModule(this), tr("Startup Programs")},
            { new SystemSafetyModule(this), tr("System Security")},
        };
    }
    else {
        m_modules = {
            { m_homepageModule, tr("Home")},
            { m_antiVirusModule, tr("Virus Scan")},
            { new NetworkProtectionModule(this), tr("Firewall")},
            { new StartupModule(this), tr("Startup Programs")},
            { new SystemSafetyModule(this), tr("System Security")},
        };
    }

    // 浅色主题图标
    m_modulesLightIconList.clear();
    m_modulesLightIconList << "dcc_home" << "dcc_virus_scan" << "dcc_network_protection" << "dcc_application_security" << "dcc_system_security"
                           << "dcc_system_security";
    // 深色主题图标
    m_modulesDarakIconList.clear();
    m_modulesDarakIconList << "dcc_home_dark" << "dcc_Virus_Scan_dark" << "dcc_network_protection_dark"
                           << "dcc_application_security_dark" << "dcc_system_security_dark"
                           << "dcc_system_security";
    // 获得主题类型
    int type = m_guiHelper->themeType();
    qDebug()<<"theme type " << type;
    for (auto it = m_modules.cbegin(); it != m_modules.cend(); ++it) {
        DStandardItem *item = new DStandardItem;
        item->setText(it->second);
        QString iconPath ;
        if(type == 1){
            iconPath = m_modulesLightIconList.at(it - m_modules.cbegin());
        }else{
            iconPath = m_modulesDarakIconList.at(it - m_modules.cbegin());
        }
        QIcon icon = QIcon::fromTheme(iconPath);
        item->setIcon(icon);

        item->setData(NavItemMargin, Dtk::MarginsRole);

        m_navModel->appendRow(item);
    }
}

void MainWindow::showModulePage(const QString &module, const QString &page)
{
    qDebug() << Q_FUNC_INFO << module << page;

    int moduleIndex = 0;
    int pageIndex = 0;
    ModuleInterface *inter = nullptr;
    for (int i = 0; i < m_modules.count(); i++) {
        inter = m_modules[i].first;
        qDebug() << inter->name() << module;
        if (module == inter->name()) {
            moduleIndex = i;
            break;
        }
    }

    if ("UsbConnection" == page) {
        // 若显示系统安全-usb安全子页
        pageIndex = USB_CONNECTION_INDEX;
    } else {
        pageIndex = 0;
    }

    if (DSysInfo::uosEditionType() == DSysInfo::UosEdition::UosCommunity) {
        moduleIndex = moduleIndex +  1;
    }

    setCurrentModule(moduleIndex, pageIndex);
    // 显示并恢复窗口为正常状态
    showNormal();
    activateWindow();
}

int MainWindow::getModuleCount()
{
    return m_modules.count();
}

// 右键展现扫描页面
void MainWindow::showRightScan(const QString &path)
{
    // 若没有病毒扫描页, 直接退出软件
    if (m_antiVirusModule->isbInScanningPage()) {
        // 若扫描页处于打开状态(包括后台打开)
        if(!m_rightScanDialogFlag){
            //　判断右键扫描提示框是不是存在
            showScanTipDialog();
        }
        return;
    }

    // 如果服务正在扫描
    if (m_AntivirusServer->isScanning()) {
        //　判断首页体检提示框是不是存在
        if(!m_homePageScanDialogFlag){
            showHomepageScanTip();
        }
        return;
    }

    // 如果界面和服务都没在扫描
    setCurrentModule(MODULE_NATIAV_INDEX);
    this->showNormal();
    this->activateWindow();
    m_antiVirusModule->scanVirusByPath(path);
}

// 有扫描任务弹窗
void MainWindow::showScanTipDialog()
{
    // mxz scanFlagTip 弹窗标志 https://pms.uniontech.com/zentao/bug-view-41009.html
    DDialog *dialog = new DDialog;
    dialog->setMinimumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    dialog->setMaximumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    dialog->setIcon(QIcon::fromTheme("dcc_login_tip"));
    dialog->setMessage(tr("A scanning task is running"));
    dialog->setWordWrapMessage(true);
    dialog->addButton(tr("Cancel"), false);
    // 查看   推荐
    dialog->addButton(tr("View"), true, DDialog::ButtonType::ButtonRecommend);
    connect(dialog, &DDialog::buttonClicked, [ = ](int index, QString) {
        switch (index) {
        case 0:
            // 取消
            dialog->close();
            // 右键扫描的提示框为状态　false
            m_rightScanDialogFlag = false ;
            break;
        case 1:
            // 1.先跳转到扫描页面
            this->showNormal();
            this->activateWindow();
            this->setCurrentModule(MODULE_NATIAV_INDEX);
            dialog->close();
            // 右键扫描的提示框为false
            m_rightScanDialogFlag = false ;
            break;
        default:
            break;
        }
    });
    // 关闭时候设置右键扫描的按钮状态
    connect(dialog, &DDialog::closed, [ = ](){
        m_rightScanDialogFlag = false ;
    });
    //　开启右键扫描的提示框为ｔｒｕｅ
    m_rightScanDialogFlag = true ;
    dialog->exec();
}

//首页病毒查杀中的提示弹框
void MainWindow::showHomepageScanTip()
{
    DDialog *dialog = new DDialog;
    dialog->setMinimumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    dialog->setMaximumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    dialog->setIcon(QIcon::fromTheme("dcc_login_tip"));
    //首页体检正在执行快速查杀，请稍候
    dialog->setMessage(tr("Running quick scan in the Home page, please scan later"));
    dialog->setWordWrapMessage(true);
    dialog->addButton(tr("Cancel"), false);
    // 查看   推荐
    dialog->addButton(tr("View"), true, DDialog::ButtonType::ButtonRecommend);
    connect(dialog, &DDialog::buttonClicked, [ = ](int index, QString) {
        switch (index) {
        case 0:
            // 取消
            dialog->close();
            // 首页体检的提示框为状态　false
            m_homePageScanDialogFlag = false;
            break;
        case 1:
            // 1.先跳转到首页
            this->showNormal();
            this->activateWindow();
            this->setCurrentModule(MODULE_HOMEPAGE_INDEX);
            dialog->close();
            // 首页体检的提示框为状态　false
            m_homePageScanDialogFlag = false;
            break;
        default:
            break;
        }
    });
    // 关闭时候设置首页病毒查杀的提示框状态
    connect(dialog, &DDialog::closed, [ = ](){
        m_homePageScanDialogFlag = false ;
    });
    // 开启首页病毒查杀的提示框为ｔｒｕｅ
    m_homePageScanDialogFlag = true;
    dialog->exec();
}

// mxz 退出确认框  如果实在扫描中
void MainWindow::showScaningCloseTipDialog()
{
    DDialog *dialog = new DDialog;
    dialog->setMinimumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    dialog->setMaximumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    dialog->setIcon(QIcon::fromTheme("dcc_login_tip"));
    // 扫描进行中，您确定要退出吗？
    dialog->setMessage(tr("A scan is in progress. Are you sure you want to exit?"));
    dialog->setWordWrapMessage(true);
    dialog->addButton(tr("Cancel"), false);
    //推出    推荐
    dialog->addButton(tr("Exit"), true, DDialog::ButtonType::ButtonWarning);
    connect(dialog, &DDialog::buttonClicked, [ = ](int index, QString) {
        switch (index) {
        case 0:
            dialog->close();
            // 扫描中退出的提示框为状态　false
            m_scaningQuitDialogFlag = false ;
            break;
        case 1:
            //扫描停止
            m_AntivirusServer->setDissection(2);
            dialog->close();
            // 扫描中退出的提示框为状态　false
            m_scaningQuitDialogFlag = false ;
            qApp->quit();
            break;
        default:
            break;
        }
    });
    // 关闭时候设置扫描中退出的提示框状态
    connect(dialog,&DDialog::closed,[=](){
        m_scaningQuitDialogFlag = false ;
    });
    // 扫描中退出的提示框为ｔｒｕｅ
    m_scaningQuitDialogFlag = true ;
    // 弹窗为阻态
    dialog->exec();
}

// 没有处理的异常提示
void MainWindow::showThreatsTipDialog()
{
    DDialog *dialog = new DDialog;
    dialog->setMinimumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    dialog->setMaximumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    dialog->setIcon(QIcon::fromTheme("dcc_login_tip"));
    // 存在未处理的风险项，确定退出吗？
    dialog->setMessage(tr("There are unprocessed threats. Are you sure you want to exit?"));
    dialog->setWordWrapMessage(true);
    dialog->addButton(tr("Cancel"), false);
    //推出    推荐
    dialog->addButton(tr("Exit"), true, DDialog::ButtonType::ButtonWarning);
    connect(dialog, &DDialog::buttonClicked, [ = ](int index, QString) {
        switch (index) {
        case 0:
            dialog->close();
            // 有未处理异常退出的提示框为状态　false
            m_threatsQuitDialogFlag = false ;
            break;
        case 1:
            m_antiVirusModule->backOrQuitThreatsOPeration();
//            this->threatOperation();
            dialog->close();
            // 有未处理异常提示退出的提示框为状态　false
            m_threatsQuitDialogFlag = false ;
            qApp->quit();
            break;
        default:
            break;
        }
    });
     // 关闭时候设有未处理异常退出状态
    connect(dialog,&DDialog::closed,[=](){
        m_threatsQuitDialogFlag = false ;
    });
    //　开启有未处理异常退出的提示框为ｔｒｕｅ
    m_threatsQuitDialogFlag = true;
    // 弹窗为阻态
    dialog->exec();
}

// 首页立即查杀 在病毒查杀中的提示框
void MainWindow::showThreatScanTipDialog()
{
    DDialog *dialog = new DDialog;
    dialog->setMinimumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    dialog->setMaximumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    dialog->setIcon(QIcon::fromTheme("dcc_login_tip"));
    //请先结束当前进行中的查杀任务
    dialog->setMessage(tr("A task is in process, please stop it first"));
    dialog->setWordWrapMessage(true);

    dialog->addButton(tr("Cancel"), false);
    // 查看   推荐
    dialog->addButton(tr("View"), true, DDialog::ButtonType::ButtonRecommend);
    connect(dialog, &DDialog::buttonClicked, [ = ](int index, QString) {
        switch (index) {
        case 0:
            // 取消
            dialog->close();
            break;
        case 1:
            // 查看
            this->setCurrentModule(MODULE_NATIAV_INDEX);
            dialog->close();
            break;
        default:
            break;
        }
    });
    dialog->exec();
}

// 联网管控弹框
void MainWindow::showFuncConnectNetControl(const QString &sPkgName)
{
    if (m_netControlWidget == nullptr) {
        m_netControlWidget = new NetControlWidget(sPkgName, this);
        connect(m_netControlWidget, &NetControlWidget::onDialogClose, [ = ]() {
            m_netControlWidget = nullptr;
        });
        m_netControlWidget->setAttribute(Qt::WA_DeleteOnClose);
        m_netControlWidget->setWindowFlags(m_netControlWidget->windowFlags());
        m_netControlWidget->setModal(true);
        m_netControlWidget->setVisible(true);
        m_netControlWidget->setEnabled(true);
    }
    else {
        // 当联网管控弹框存在，并且被其他应用程序遮挡，需要再次显示弹框设置联网状态时，重新激活安全中心窗口，使安全中心窗口置顶让用户可以选择
        this->showNormal();
        this->activateWindow();
        // 定位联网管控选择项
        if (!sPkgName.isEmpty()) {
            m_netControlWidget->setNetControlIndex(sPkgName);
        }
    }
}

// 远程访问弹框
void MainWindow::showFuncConnectRemControl(const QString &sPkgName)
{
    Q_UNUSED(sPkgName);

    // 远程访问设置跳转
    showNormal();
    this->activateWindow();
    // 显示防火墙页
    showModulePage("firewall","");
}


void MainWindow::closeWindow()
{
    // 若没有病毒扫描页, 直接退出软件
    if (!m_antiVirusModule->isbInScanningPage()) {
        qApp->quit();
        return;
    }
    // 若扫描页处于打开状态(包括后台打开)
    AntiVirusWork::ScanningStatus scanningStatus = m_antiVirusModule->scanningStatus();
    switch (scanningStatus) {
    case AntiVirusWork::ScanningStatus::None:
        break;
    case AntiVirusWork::ScanningStatus::Scanning:
        //　判断扫描中退出提示框是不是存在
        if(!m_scaningQuitDialogFlag){
            showScaningCloseTipDialog();
        }
        break;
    case AntiVirusWork::ScanningStatus::Result:
        //　判断有未处理异常退出提示框是不是存在
        if(!m_threatsQuitDialogFlag){
            showThreatsTipDialog();
        }
        break;
    case AntiVirusWork::ScanningStatus::ResultNoThreats:
        // 在结果页面但是没有异常数据
        qApp->quit();
        break;
    }
}

// 设置安装时间
void MainWindow::setInstallTime()
{
    // 获得安装时间  如果是默认值-1 设置当前时间为安装时间
    QString lasttime = m_gsetting->get(INSTALL_TIME).toString();
    QString defaultTime = "-1";
    if(lasttime == defaultTime){
        //当前时间
        QDateTime date = QDateTime::currentDateTime();
        QString installTime = date.toString("yyyy-MM-dd hh:mm:ss");
        m_gsetting->set(INSTALL_TIME, installTime);
    }
}

// 关闭窗口事件
void MainWindow::closeEvent(QCloseEvent *event)
{
    // 获得访问类型  0 不访问  1是展现访问关闭窗口
    if (!getAskType()) {
        // 不访问 获得关闭类型 0最小化 1退出
        if (getCloseType()) {
            closeWindow();
            event->ignore();
        } else {
            hide();
            event->ignore();
        }
    } else {
        initCloseDialog(event);
    }
}

void MainWindow::hideEvent(QHideEvent *event)
{
    Q_UNUSED(event);
}

//左侧列表的点击
void MainWindow::onFirstItemClick(const QModelIndex &index)
{
    m_navView->setCurrentIndex(index);
    ModuleInterface *inter = m_modules[index.row()].first;
    qDebug() << "pressed inter:" << inter->name();

    //有数据  并且最后一个和现在点击的一致
    if (!m_contentStack.isEmpty() && m_contentStack.last().first == inter) {
        return;
    }
    popAllWidgets(0);

    inter->active(0);

    // 更新后退按钮
    this->updateBackwardBtn();
}


void MainWindow::onTrayActivated(QSystemTrayIcon::ActivationReason state)
{
    qDebug() << Q_FUNC_INFO << state;

    if (state == QSystemTrayIcon::ActivationReason::Trigger) {
        if (!this->isVisible()) {
            //页面隐藏，点击托盘重新show
            this->show();
        } else {
            //页面能显示的时候，点击托盘会＂正常/最小化＂切换窗口
            if (this->isMinimized()) {
                this->setWindowState(Qt::WindowState::WindowActive);
            } else {
                this->setWindowState(Qt::WindowState::WindowMinimized);
            }
        }
    }
}

void MainWindow::setSystemsafety(const QModelIndex &index, int sonindex)
{
    ModuleInterface *inter = m_modules[index.row()].first;
//    if (m_contentStack.isEmpty() && m_contentStack.last().first == inter) {
//        return;
//    }
    popAllWidgets();
    inter->active(sonindex);
}

// 背景主题变化
void MainWindow::themeTypeChange(ColorType themeType)
{
    int type;
    switch (themeType) {
    case UnknownType:
        type = 0;
        break;
    case LightType:
        type = 1;
        break;
    case DarkType:
        type = 2 ;
        break;
    }

    // 获得所有行
    int count = m_navModel->rowCount();
    for (int i = 0; i < count; i++) {
         // 每行的item
          QStandardItem *item =  m_navModel->takeItem(i,0);
          QString iconPath ;
          if(type == 1){
              iconPath = m_modulesLightIconList.at(i);
          }else{
              iconPath = m_modulesDarakIconList.at(i);
          }
          // 每行item的图标设置
          QIcon icon = QIcon::fromTheme(iconPath);
          item->setIcon(icon);
          item->setData(NavItemMargin, Dtk::MarginsRole);
          m_navModel->setItem(i,0,item);
    }
}

// 当扫描结束时
void MainWindow::onScanningFinished()
{
    qDebug() << "ui's virus scanning finished!";
    // 更新后退按钮
    this->updateBackwardBtn();
}

// 更新后退按钮
void MainWindow::updateBackwardBtn()
{
    m_backwardBtn->setEnabled(false);

    // 只对首页 病毒查杀页 和 防火墙页 作后退按钮判断
    QString sModuleName = m_contentStack.top().first->name();
    if ("homepage" == sModuleName) {
        if (1 < m_contentStack.count()) {
            m_backwardBtn->setEnabled(true);
        } else {
            m_backwardBtn->setEnabled(false);
        }
        return;
    }

    if ("VirusScan" == sModuleName) {
        if (0 < m_contentStack.count()) {
            m_backwardBtn->setEnabled(true);
        } else {
            m_backwardBtn->setEnabled(false);
        }

        // 若扫描页处于打开状态(包括后台打开)
        AntiVirusWork::ScanningStatus scanningStatus = m_antiVirusModule->scanningStatus();
        if (AntiVirusWork::ScanningStatus::Scanning == scanningStatus) {
            m_backwardBtn->setEnabled(false);
        }
        return;
    }

    // 其他模块下, 后退按钮状态判断
    if (0 < m_contentStack.count()) {
        m_backwardBtn->setEnabled(true);
    } else {
        m_backwardBtn->setEnabled(false);
    }


/*
    m_backwardBtn->setEnabled(false);
    // 若当前显示栈中只有一页, 则不可后退
    if (2 > m_contentStack.count()) {
        m_backwardBtn->setEnabled(false);
        return;
    }

    // 只对首页 病毒查杀页 和 防火墙页 作后退按钮判断
    QString sModuleName = m_contentStack.top().first->name();
    if ("homepage" == sModuleName ||
        "firewall" == sModuleName) {
        if (1 < m_contentStack.count()) {
            m_backwardBtn->setEnabled(true);
        } else {
            m_backwardBtn->setEnabled(false);
        }
        return;
    }

    if ("VirusScan" == sModuleName) {
        // 若扫描页处于打开状态(包括后台打开)
        AntiVirusWork::ScanningStatus scanningStatus = m_antiVirusModule->scanningStatus();
        switch (scanningStatus) {
        // 如果没有扫描, 则可后退
        case AntiVirusWork::ScanningStatus::None:
            m_backwardBtn->setEnabled(true);
            break;
        // 如果正在扫描, 则不可后退
        case AntiVirusWork::ScanningStatus::Scanning:
            m_backwardBtn->setEnabled(false);
            break;
        // 如果处于扫描结果页, 则可后退
        case AntiVirusWork::ScanningStatus::Result:
            m_backwardBtn->setEnabled(true);
            break;
        }
        return;
    }
    */
}

// 初始化托盘
void MainWindow::initTray()
{
    m_pSystemTray = new QSystemTrayIcon(this);
    m_trayMenu = new QMenu(this);

    // 安全中心
    QAction *show = new QAction;
    show->setText(tr("Security Center"));
    m_trayMenu->addAction(show);
    // 点击托盘中的安全中心  激活该窗口并且展现出来
    connect(show, &QAction::triggered, [this] {
        this->show();
        this->setWindowState(Qt::WindowState::WindowActive);
    });

    // 托盘退出
    QAction *exit = new QAction;
    exit->setText(tr("Exit"));
    m_trayMenu->addAction(exit);
    connect(exit, &QAction::triggered, [ = ] {
        closeWindow();
    });

    // 设置系统托盘的上下文菜单
    m_pSystemTray->setContextMenu(m_trayMenu);
    // 设置系统托盘提示信息、托盘图标
    QString name = tr("Security Center");
    m_pSystemTray->setToolTip(name);
    m_pSystemTray->setIcon(QIcon::fromTheme("deepin-defender"));
    connect(m_pSystemTray, &QSystemTrayIcon::activated, this, &MainWindow::onTrayActivated);

    m_pSystemTray->show();
}

// 关闭弹窗  只有在设置是访问才有这个
void MainWindow::initCloseDialog(QCloseEvent *event)
{
    m_trayDialag = new DDialog(this);
    m_trayDialag->setModal(true);
    m_trayDialag->setIcon(QIcon::fromTheme("deepin-defender"));
    DRadioButton *radioExit = new DRadioButton(tr("Exit"));
    DRadioButton *radioMin = new DRadioButton(tr("Minimize to system tray"));
    DCheckBox *radioAsk = new DCheckBox(tr("Do not ask again"));
    closeType nChoose = static_cast<closeType>(getCloseType());
    //根据GSettings设置settings页面的默认值
    switch (nChoose) {
    case Exit:
        radioExit->setChecked(true);
        break;
    case Tray:
    default:
        radioMin->setChecked(true);
        break;
    }
    radioAsk->setCheckState(Qt::CheckState::Unchecked);

    // 选择你的操作
    m_trayDialag->setMessage(tr("Please choose your action"));
    m_trayDialag->addContent(radioMin);
    m_trayDialag->addContent(radioExit);
    m_trayDialag->addContent(radioAsk);
    m_trayDialag->addButton(tr("Cancel"));
    m_trayDialag->addButton(tr("Confirm"), true, DDialog::ButtonRecommend);

    int a = m_trayDialag->exec();
    if (1 != a) {
        event->ignore();
        return;
    } else {
        // 确认按钮点击
        if (radioMin->isChecked()) {  //最小花点击
            this->hide();
            event->ignore();
        } else if (radioExit->isChecked()) {  //退出点击
            closeWindow();
            event->ignore();
        }

        // 根据选择的方式  设置关闭方式
        if (radioMin->isChecked()) {
            setCloseType(Tray);
        } else if (radioExit->isChecked()) {
            setCloseType(Exit);
        }

        //只有当选择radioAsk后才会设置不再询问
        if (radioAsk->isChecked()) {
            setAskType(0);
        }

        m_trayDialag->deleteLater();
        m_trayDialag = nullptr;
    }
}
//模块初始化前准备
void MainWindow::modulePreInitialize()
{
    for (auto it = m_modules.cbegin(); it != m_modules.cend(); ++it) {
        QElapsedTimer et;
        et.start();
        it->first->preInitialize();
        qDebug() << QString("initalize %1 module using time: %2ms")
                 .arg(it->first->name())
                 .arg(et.elapsed());
    }

    QModelIndex index0 = m_navModel->index(0, 0);
    if (index0.isValid()) {
        onFirstItemClick(index0);    //默认点击首页
    }

    // 获取病毒查杀work数据对象
    m_antiVirusWork = m_antiVirusModule->work();

    // 信号链接
    // 首页评分 未处理异常 立即查杀
    connect(m_AntivirusServer, &AntivirusServer::onRequstScanThreatsFile, this, &MainWindow::threatsFileOperation);
    // 开始扫描时更新后退按钮状态
    connect(m_antiVirusWork, &AntiVirusWork::startScanning,this, &MainWindow::updateBackwardBtn);
    // 扫描结束时更新后退按钮状态
    connect(m_antiVirusWork, &AntiVirusWork::scanningFinished,this, &MainWindow::updateBackwardBtn);
}

//Only used to from third page to top page can use it
void MainWindow::popAllWidgets(int place)
{
    qDebug() << "m_contentStack.count()" << m_contentStack.count();
    if (!m_contentStack.count())
        return;

    for (int pageCount = m_contentStack.count(); pageCount > place; pageCount--) {
        QString sModuleName = m_contentStack.top().first->name();
        if ("homepage" == sModuleName) {
            popWidget();
        } else if ("VirusScan" == sModuleName) {
            popWidget();
        } else {
            popAndDelWidget();
        }
    }
}

void MainWindow::pushNormalWidget(ModuleInterface *const inter, QWidget *const w)
{
    //Set the newly added page to fill the blank area
    w->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    w->show();

    m_contentStack.push({inter, w});
    m_rightContentLayout->addWidget(w, m_contentStack.size() == 1 ? 3 : 7);
    // debug
    qDebug() << QString("push widget of inter(%1)").arg(inter->name());
    qDebug() << "m_contentStack.count()" << m_contentStack.count();
}

// 获得关闭类型 0托盘最小化 1推出
int MainWindow::getCloseType()
{
    return m_gsetting ? m_gsetting->get(GSettingsCloseReminder).toInt() : 0;
}

//0 : Not Ask  1 : Ask      0 托盘最小化  1 退出
void MainWindow::setCloseType(int value)
{
    if (m_gsetting) {
        m_gsetting->set(GSettingsCloseReminder, value);
    }
}

// 获得访问类型 0不访问 1访问
int MainWindow::getAskType()
{
    return m_gsetting ? m_gsetting->get(GSettingsAskType).toInt() : 0;
}

// 设置访问  0 不访问 1访问
void MainWindow::setAskType(int value)
{
    if (m_gsetting) {
        m_gsetting->set(GSettingsAskType, value);
    }
}

// 首页评分未处理异常 点击立即查杀
void MainWindow::threatsFileOperation()
{
    // debug
    qDebug() << "threatsFileOperation";

    if (m_antiVirusModule->isbInScanningPage()) {
        showThreatScanTipDialog();
        return;
    }

    // 跳转到病毒查杀页
    setCurrentModule(MODULE_NATIAV_INDEX);
    // 处理未处理的危险文件
    m_antiVirusModule->scanThreatsFiles();
}
