/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     zccrs <zccrs@live.com>
 *
 * Maintainer: zccrs <zhangjide@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once

#include "utils.h"
#include "window/modules/common/common.h"
#include "window/interface/frameproxyinterface.h"
#include "window/interface/moduleinterface.h"
#include "com_deepin_defender_datainterface.h"
#include "com_deepin_defender_daemonservice.h"
#include "com_deepin_defender_antiav.h"
// 首页模块
#include "window/modules/homepage/homepagemodule.h"
// 病毒查杀模块
#include "window/modules/antivirus/natiavmodule.h"
#include "window/modules/antivirus/natiavparentwidget.h"
#include "window/modules/antivirus/natiavwidget.h"
// 安全等级模块
#include "window/modules/safetylevel/safetylevelmodule.h"
// 网络防护模块
#include "window/modules/networkprotection/networkprotectionmodule.h"
#include "src/window/modules/netcontrol/netcontrolwidget.h"
#include "src/window/modules/remcontrol/remcontrolwidget.h"
// 自启动模块
#include "window/modules/startup/startupmodule.h"
// 系统安全模块
#include "window/modules/systemsafety/systemsafetymodule.h"
// 系统清理模块
#include "window/modules/cleaner/cleanermodule.h"

#include <DMainWindow>
#include <DGuiApplicationHelper>
#include <DIconButton>
#include <QCloseEvent>
#include <QGSettings>
#include <DDialog>
#include <QStack>
#include <QPair>
#include <QDBusContext>
#include <QSystemTrayIcon>
#include <QStackedWidget>

DWIDGET_BEGIN_NAMESPACE
class DListView;
class DBackgroundGroup;
class DDialog;
DWIDGET_END_NAMESPACE

QT_BEGIN_NAMESPACE
class QHBoxLayout;
class QStandardItemModel;
class QGSettings;
QT_END_NAMESPACE

DWIDGET_USE_NAMESPACE

enum closeType {
    Tray,
    Exit,
    Ask,
    Count
};


using DataInterFaceServer = com::deepin::defender::datainterface;
using DaemonService = com::deepin::defender::daemonservice;
// 病毒查杀 dbus接口 服务名称
using AntivirusServer = com::deepin::defender::antiav;


DEF_NAMESPACE_BEGIN
class MainWindow : public DTK_WIDGET_NAMESPACE::DMainWindow, public FrameProxyInterface, protected QDBusContext
{
    Q_OBJECT
public:
    explicit MainWindow(QWidget *parent = nullptr);
    ~MainWindow() override;

    // 模板函数,供各模块调用
    void popWidget(ModuleInterface *const inter) override;   //弹栈
    void popAndDelWidget(ModuleInterface *const inter) override;
    void pushWidget(ModuleInterface *const inter, QWidget *const w) override;   //压栈
    void setModuleVisible(ModuleInterface *const inter, const bool visible) override;   //模块可视化
    void setCurrentModule(int iModuleIdx, int iPageIdx = 0) override;   //设置当前模块
    ModuleInterface *getCurrentModule() override;
    int getModuleIndex(const QString &name) override;           //获得模块下标

    // 显示默认设置弹框
    void showDefaultSettingDialog();
    // 显示谈设置弹窗，并跳转到相应组位置
    void showSettingDialog(const QString &groupKey);
    void toggle();
    // 将窗口出显示栈, 并删除
    void popAndDelWidget();
    // 将窗口出显示栈
    void popWidget();
    void initDaemonService();
    void initAllModule();
    void showModulePage(const QString &module, const QString &page);
    int getModuleCount();
    // 右键展现扫描页面
    void showRightScan(const QString &path);
    // 扫描提示框
    void showScanTipDialog();
    //首页病毒查杀中的提示弹框
    void showHomepageScanTip();

    // 退出提示框
    void showScaningCloseTipDialog();
    // 没有处理的异常提示
    void showThreatsTipDialog();
    // 首页立即查杀 在病毒查杀中的提示框
    void showThreatScanTipDialog();

    // 联网管控弹框
    void showFuncConnectNetControl(const QString &sPkgName);
    // 远程访问弹框
    void showFuncConnectRemControl(const QString &sPkgName);

    void closeWindow();
    // 设置安装时间
    void setInstallTime();
protected:
    void closeEvent(QCloseEvent *event) override;
    void hideEvent(QHideEvent *event) override;

Q_SIGNALS:
    void moduleVisibleChanged(const QString &module, bool visible);

private Q_SLOTS:
    void onFirstItemClick(const QModelIndex &index);
    void onTrayActivated(QSystemTrayIcon::ActivationReason state);
    void setSystemsafety(const QModelIndex &index, int sonindex);

    // 主题类型改变
    void themeTypeChange(ColorType themeType);

    // 当扫描结束时
    void onScanningFinished();
    // 更新后退按钮
    void updateBackwardBtn();

private:
    void initTray();
    void initCloseDialog(QCloseEvent *event);
    void modulePreInitialize();
    void popAllWidgets(int place = 0);//place is Remain count
    void pushNormalWidget(ModuleInterface *const inter, QWidget *const w);  //exchange third widget : push new widget
    int getCloseType();
    void setCloseType(int value);
    int getAskType();
    void setAskType(int value);

    // 首页评分未处理异常 点击立即查杀
    void threatsFileOperation();

private:
    QHBoxLayout *m_contentLayout;
    QHBoxLayout *m_rightContentLayout;
    DTK_WIDGET_NAMESPACE::DListView *m_navView;
    DTK_WIDGET_NAMESPACE::DBackgroundGroup *m_rightView;
    QStandardItemModel *m_navModel;
    QStack<QPair<ModuleInterface *, QWidget *>> m_contentStack;

    HomePageModule *m_homepageModule;
    NatiavModule *m_antiVirusModule;
    AntiVirusWork *m_antiVirusWork;
    QList<QPair<ModuleInterface *, QString>> m_modules;
    DIconButton *m_backwardBtn;     //上一步
    QSystemTrayIcon *m_pSystemTray;   //系统托盘图标
    QMenu *m_trayMenu;     //托盘目录
    DDialog *m_trayDialag;
    QGSettings *m_gsetting; //安全中心gsetting配置
    DataInterFaceServer  *m_DataInterFaceServer = nullptr; // 联网管控接口
    DaemonService *m_daemonservice;
    AntivirusServer *m_AntivirusServer = nullptr;             //病毒查杀服务接口

    NetControlWidget *m_netControlWidget = nullptr;     // 联网管控弹框
    RemControlWidget *m_remControlWidget = nullptr;     // 联网管控弹框

    DGuiApplicationHelper       *m_guiHelper = nullptr;               // 方便得到系统主题
    QStringList                 m_modulesDarakIconList ;              // 深色主题图标列表
    QStringList                 m_modulesLightIconList ;              // 浅色主题图标列表

    bool                     m_rightScanDialogFlag = false;            // 右键扫描弹窗
    bool                     m_homePageScanDialogFlag = false;         // 首页扫描弹窗
    bool                     m_scaningQuitDialogFlag = false;          // 扫描中退出弹窗
    bool                     m_threatsQuitDialogFlag = false;          // 有未处理异常退出弹窗

};
DEF_NAMESPACE_END
