#include "natiavmodule.h"
#include "updatedialog.h"
#include "src/window/mainwindow.h"

#include <DMainWindow>

NatiavModule::NatiavModule(FrameProxyInterface *frame, QObject *parent)
    : QObject(parent)
    , ModuleInterface(frame)
    , m_work(nullptr)
    , m_antiVirusPwidget(nullptr)
    , m_antiVirusHomeWidget(nullptr)
    , m_scanWidget(nullptr)

{

}

void NatiavModule::initialize()
{

}

void NatiavModule::preInitialize()
{
    m_work = new AntiVirusWork(this);
    // 连接请求打开更新病毒库弹窗信号
    connect(m_work, &AntiVirusWork::notifyOpenVdbUpdateDialog, this, &NatiavModule::openVdbUpdateDialog);
}

const QString NatiavModule::name() const
{
    return "VirusScan";
}

//模块第一次被点击进入时
void NatiavModule::active(int index)
{
    Q_UNUSED(index);
    // 病毒查杀页始终保留
    if (nullptr == m_antiVirusPwidget) {
        m_antiVirusPwidget = new NatiavParentWidget(m_work);
        m_antiVirusPwidget->setCurrentPage(0);
        connect(m_antiVirusPwidget, &NatiavParentWidget::notifyOpenAntiVirusHomePage, [ = ]() {
            if (this != m_frameProxy->getCurrentModule()) {
                // 如果主框架显示的不是当前模块, 则不进行推栈
                qDebug() << "主框架显示的不是当前模块, 不进行推栈";
                return;
            }
            m_antiVirusHomeWidget = m_antiVirusPwidget->antiVirusHomeWidget();
            m_scanWidget = m_antiVirusPwidget->scanWidget();
            m_frameProxy->popWidget(this);
            this->deleteScanWidget();
            m_antiVirusHomeWidget->show();
        });
        connect(m_antiVirusPwidget, &NatiavParentWidget::notifyOpenScanVirusPage, [ = ]() {
            if (this != m_frameProxy->getCurrentModule()) {
                // 如果主框架显示的不是当前模块, 则不进行推栈
                qDebug() << "主框架显示的不是当前模块, 不进行推栈";
                return;
            }
            m_antiVirusHomeWidget = m_antiVirusPwidget->antiVirusHomeWidget();
            m_scanWidget = m_antiVirusPwidget->scanWidget();
            m_antiVirusHomeWidget->hide();
            m_scanWidget->show();
            m_frameProxy->pushWidget(this, m_scanWidget);
        });
    }

    m_antiVirusHomeWidget = m_antiVirusPwidget->antiVirusHomeWidget();
    m_scanWidget = m_antiVirusPwidget->scanWidget();

    // 优先显示扫描页
    if (nullptr != m_scanWidget) {
        m_frameProxy->pushWidget(this, m_antiVirusHomeWidget);
        m_antiVirusHomeWidget->hide();
        m_scanWidget = m_antiVirusPwidget->scanWidget();
        m_scanWidget->show();
        m_frameProxy->pushWidget(this, m_scanWidget);
    } else {
        m_antiVirusHomeWidget->show();
        m_frameProxy->pushWidget(this, m_antiVirusHomeWidget);
    }
}

void NatiavModule::deactive()
{
    m_antiVirusPwidget->scanThreatsFiles();
}

void NatiavModule::deleteScanWidget()
{
    m_scanWidget = m_antiVirusPwidget->scanWidget();
    if (nullptr == m_scanWidget) {
        return;
    }
    m_antiVirusPwidget->deleteScanWidget();
    m_scanWidget = nullptr;
    m_work->setIsbInVirusScanningPage(false);
}

bool NatiavModule::isbInScanningPage()
{
    return (nullptr != m_scanWidget);
}

AntiVirusWork::ScanningStatus NatiavModule::scanningStatus()
{
    return m_work->scanningStatus();
}

void NatiavModule::scanThreatsFiles()
{
    m_antiVirusPwidget->scanThreatsFiles();
}

void NatiavModule::scanVirusByPath(QString sPath)
{
    m_antiVirusPwidget->scanVirusByPath(sPath);
}

void NatiavModule::backOrQuitThreatsOPeration()
{
    m_antiVirusPwidget->threatOperation();
}

// 打开病毒库更新弹窗
void NatiavModule::openVdbUpdateDialog()
{
    // 指定弹窗父对象
    MainWindow *mainWindow = qobject_cast<MainWindow *>(this->parent());
    UpdateDialog *dialog = new UpdateDialog(mainWindow);
    dialog->show();
    // 连接弹窗中请求更新病毒库信号
    connect(dialog, &UpdateDialog::notifyUpdateVdb, m_work, &AntiVirusWork::requestUpdateVirusDb);
}
