﻿#include "natiavparentwidget.h"

#define LEFT_MARGINS                  0                     //左边距
#define TOP_MARGINS                   0                     //上边距
#define RIGHT_MARGINS                 0                     //右边距离
#define BUTTOM_MARGINS                0                     //下边距离
#define DIALOG_WIDTH                380                     //dialog  宽度
#define DIALOG_HEIGHT               145                     //dialog  高度
#define DIALOG_PB_WIDTH             140                     //dialog  按钮宽度
#define DIALOG_PB_HEIGHT            36                      //dialog  按钮高度



NatiavParentWidget::NatiavParentWidget(AntiVirusWork *work, QWidget *parent)
    : QWidget(parent)
    , m_isbUsbDissection(false)
    , m_antivirusWork(work)
    , m_isbInScanningPage(false)
{
    m_natiav = new NatiavWidget(work, this);
    // init stack widget
    initGUI();
    m_stackWidget->setCurrentIndex(0);
    m_antivirusWork->setIsbInVirusScanningPage(false);
    // 初始设置 不在扫描状态
    m_antivirusWork->setScanningStatus(AntiVirusWork::ScanningStatus::None);

    // mxz 对异常数据操作　单独开辟一个处理线程
    thread = new QThread();
    m_threatOperation = new ThreatsOPerationThread();
    m_threatOperation->moveToThread(thread);
    connect(thread, &QThread::finished, thread, &QThread::deleteLater);
    // 处理异常数据
    connect(this, &NatiavParentWidget::notifyGetThreatsFile, m_threatOperation, &ThreatsOPerationThread::getThreatFileList);
    //获得异常数据
    connect(m_threatOperation, &ThreatsOPerationThread::sendThreatsFileList, this, [ = ](QString sendpath) {
        bool flag = false;
        //扫描方法
        this->setCurrentPage(1);
        // 获取本地病毒库版本
        QString version = m_antivirusWork->localVdbVersion();
        m_scan->setType(2, sendpath, version);
        m_antivirusWork->doDissection(flag, sendpath);
    });

    connect(this, &NatiavParentWidget::notifyAddThreatsFile, m_threatOperation, &ThreatsOPerationThread::addThreatsFile);
    connect(this, &NatiavParentWidget::notifyDelteThreatsFile, m_threatOperation, &ThreatsOPerationThread::deleteThreatsFile);

    // 批量处理异常文件
    connect(this, &NatiavParentWidget::senBatchOperation, m_threatOperation, &ThreatsOPerationThread::batchOPeration);

    connect(m_threatOperation, &ThreatsOPerationThread::onAllRepairProblemFile, this, [ = ](QString path) {
        qDebug() << "修复路径====" << path;
        m_antivirusWork->requestRepairProblemFile(path);
    });
    connect(m_threatOperation, &ThreatsOPerationThread::onAllIsolationFile, m_antivirusWork, [ = ](bool flag, QString path) {
        qDebug() << "flag==" << flag << ",隔离路径====" << path;
        m_antivirusWork->requestChangeIsolationFile(flag, path);
    });
    connect(m_threatOperation, &ThreatsOPerationThread::onAllTruestFile, m_antivirusWork, [ = ](bool flag, QString path) {
        qDebug() << "flag==" << flag << ",信任路径====" << path;
        m_antivirusWork->requestChangeTrustFiles(flag, path);
    });

    thread->start();

    //跳转到扫描页面
    connect(m_natiav, &NatiavWidget::onNotifyShowScan, this, [ = ](int type, QString path, QString version) {
        this->setCurrentPage(1);
        m_scan->setType(type, path, version);
        if (type == 0) {
            bool flag = true;
            //扫描方法
            m_antivirusWork->doDissection(flag, path);
        } else {
            bool flag = false;
            //扫描方法
            m_antivirusWork->doDissection(flag, path);
        }
        m_antivirusWork->setScanStart();
    });
    //查询信任文件
    connect(m_natiav, &NatiavWidget::onRequestSelectTrustFile, m_antivirusWork, &AntiVirusWork::requestQueryTrustFiles);
    //查询隔离文件
    connect(m_natiav, &NatiavWidget::onRequstSelectIsolationFile, m_antivirusWork, &AntiVirusWork::requestQueryIsolationFile);
    //查询信任的结果
    connect(m_antivirusWork, &AntiVirusWork::sendTrustFilesByList, m_natiav, &NatiavWidget::doQueryTrustFile);
    //查询隔离文件的结果
    connect(m_antivirusWork, &AntiVirusWork::sendIsolationFilesByList, m_natiav, &NatiavWidget::doQueryIsolationFile);
    // 增加/删除 隔离文件
    connect(m_natiav, &NatiavWidget::onRemoveIsolationFile, m_antivirusWork, [ = ](bool flag, QString path) {
        qDebug() << "flag==" << flag << ",移除隔离路径====" << path;
        m_antivirusWork->requestChangeIsolationFile(flag, path);
    });
    // 增加/删除 信任文件
    connect(m_natiav, &NatiavWidget::onRemoveTrustFile, m_antivirusWork, [ = ](bool flag, QString path) {
        qDebug() << "flag==" << flag << ",移除信任路径====" << path;
        m_antivirusWork->requestChangeTrustFiles(flag, path);
    });
}

//初始化界面
void NatiavParentWidget::initGUI()
{
    QHBoxLayout *mainLayout = new QHBoxLayout(this);
    mainLayout->setContentsMargins(LEFT_MARGINS, TOP_MARGINS, RIGHT_MARGINS, BUTTOM_MARGINS);
    setLayout(mainLayout);

    m_stackWidget = new QStackedWidget;
    m_stackWidget->show();
    mainLayout->addWidget(m_stackWidget);

    m_stackWidget->addWidget(new QWidget);
    m_stackWidget->addWidget(new QWidget);
}

void NatiavParentWidget::initScanWidget()
{
    m_scan = new ScanWidget(m_antivirusWork, this);

    // 再次扫描
    connect(m_scan, &ScanWidget::scanAgain, this, [ = ](int nType, QString sPath, QString sVersion) {
        this->setCurrentPage(1);
        m_scan->setType(nType, sPath, sVersion);
        if (nType == 0) {
            bool flag = true;
            //扫描方法
            m_antivirusWork->doDissection(flag, sPath);
        } else {
            bool flag = false;
            //扫描方法
            m_antivirusWork->doDissection(flag, sPath);
        }
        m_antivirusWork->setScanStart();
    });

    // dubs扫瞄信号反馈数据
    connect(m_antivirusWork, &AntiVirusWork::sendDissectionResult, m_scan, &ScanWidget::doDissectionResult);
    // dubs设置状态反馈数据
    //connect(m_antivirusWork, &AntiVirusWork::sendSetDissectionStateResult, m_scan, &ScanWidget::doDissectionStateResult);
    // dbus 扫描完成信号
    connect(m_antivirusWork, &AntiVirusWork::sendDissectionFinishedSignal, m_scan, &ScanWidget::doScanFinsh);

    //dbus 修复信号
    connect(m_antivirusWork, &AntiVirusWork::sendRepairProblemFileResult, m_scan, &ScanWidget::doRepairResult);
    //dbus 信任文件信号
    connect(m_antivirusWork, &AntiVirusWork::sendChangeTrustFilesResult, m_scan, &ScanWidget::doTruestResult);
    //dbus 隔离文件信号
    connect(m_antivirusWork, &AntiVirusWork::sendChangeIsolationFilesResult, m_scan, &ScanWidget::doIsolationResult);

    //对文件的操作
    connect(m_scan, &ScanWidget::onAllRepairProblemFile, m_antivirusWork, [ = ](QString path) {
        qDebug() << "修复路径====" << path;
        m_antivirusWork->requestRepairProblemFile(path);
    });
    connect(m_scan, &ScanWidget::onAllIsolationFile, m_antivirusWork, [ = ](bool flag, QString path) {
        qDebug() << "flag==" << flag << ",隔离路径====" << path;
        m_antivirusWork->requestChangeIsolationFile(flag, path);
    });
    connect(m_scan, &ScanWidget::onAllTruestFile, m_antivirusWork, [ = ](bool flag, QString path) {
        qDebug() << "flag==" << flag << ",信任路径====" << path;
        m_antivirusWork->requestChangeTrustFiles(flag, path);
    });
    //扫描状态
    connect(m_scan, &ScanWidget::onScanStatus, m_antivirusWork, [ = ](DissectionState eState) {
        qDebug() << "状态" << eState;
        m_antivirusWork->requestSetDissectionStatus(eState);
    });

    //返回信号处理
    connect(m_scan, &ScanWidget::onBackpage, this, [this] {
        // 返回扫描主页同时  并更新上次扫描时间
        Q_EMIT onUpdateScanTime();
        this->setCurrentPage(0);
    });

    // 更新上次扫描时间
    connect(this, &NatiavParentWidget::onUpdateScanTime, m_antivirusWork, &AntiVirusWork::updateLastScanningTime);

    // 添加没有处理的异常数据
    connect(m_scan, &ScanWidget::onAddThreatsFile, [ = ](QString path) {
        emit notifyAddThreatsFile(path);
    });

    // 准备移除的异常数据
    connect(m_scan, &ScanWidget::readRemoveThreatsFile, [ = ](QString path) {
        emit notifyDelteThreatsFile(path);
    });

    // 批量处理文件  给线程处理
    connect(m_scan, &ScanWidget::senBatchOperation, this, [ = ](QList<QString> list, int nType) {
        emit senBatchOperation(list, nType);
    });
}

NatiavParentWidget::~NatiavParentWidget()
{
    thread->quit();
}

void NatiavParentWidget::deleteScanWidget()
{
    // 扫描页删除之前, 存储危险数据
    threatOperation();
    disconnect(m_scan);
    m_scan->hide();
    m_scan->setParent(nullptr);
    m_scan->deleteLater();
    m_scan = nullptr;
    m_stackWidget->setCurrentIndex(0);
}

// 查杀为处理的异常文件
void NatiavParentWidget::scanThreatsFiles()
{
    emit notifyGetThreatsFile();
}

void NatiavParentWidget::setWork(AntiVirusWork *work)
{
    m_antivirusWork = work;
}

void NatiavParentWidget::setCurrentPage(int nPage)
{
    qDebug() << "m_stackWidget->currentIndex: " << m_stackWidget->currentIndex();
    qDebug() << "change to page index: " << nPage;

    // 设置状态
    switch (nPage) {
    case 0:
        // 打开病毒查杀首页
        m_antivirusWork->setIsbInVirusScanningPage(false);
        m_antivirusWork->setScanningStatus(AntiVirusWork::ScanningStatus::None);
        m_isbInScanningPage = false;
        break;
    case 1:
        // 打开扫描页
        m_antivirusWork->setIsbInVirusScanningPage(true);
        m_antivirusWork->setScanningStatus(AntiVirusWork::ScanningStatus::Scanning);
        m_isbInScanningPage = true;
        break;
    default:
        // 打开病毒查杀首页
        m_antivirusWork->setIsbInVirusScanningPage(false);
        m_antivirusWork->setScanningStatus(AntiVirusWork::ScanningStatus::None);
        m_isbInScanningPage = false;
        break;
    }

    // 若已处于当页, 则不进行页面切换操作
    if (nPage == m_stackWidget->currentIndex()) {
        qDebug() << "已处于当页, 则不进行页面切换操作";
        return;
    }

    // 设置 和 通知切换页面
    switch (nPage) {
    case 0:
        // 打开病毒查杀首页
        m_stackWidget->setCurrentIndex(0);
        // 通知module: 打开病毒查杀首页,并删除扫描页
        Q_EMIT notifyOpenAntiVirusHomePage();
        break;
    case 1:
        // 打开扫描页
        m_stackWidget->setCurrentIndex(1);
        // 创建扫描页
        initScanWidget();
        // 通知module: 打开扫描页
        Q_EMIT notifyOpenScanVirusPage();
        break;
    default:
        // 打开病毒查杀首页
        m_stackWidget->setCurrentIndex(0);
        // 通知module: 打开病毒查杀首页,并删除扫描页
        Q_EMIT notifyOpenAntiVirusHomePage();
        break;
    }
}

void NatiavParentWidget::scanVirusByPath(QString sPath)
{
    //初始化扫描页并切换到该页
    this->setCurrentPage(1);
    // 获取本地病毒库版本
    QString version = m_antivirusWork->localVdbVersion();
    m_scan->setType(2, sPath, version);
    m_antivirusWork->doDissection(false, sPath);
}

// 获得病毒查杀当前的页面
int NatiavParentWidget::getCurrentPage()
{
    return m_stackWidget->currentIndex();
}

// 是否有没有处理的风险异常
bool NatiavParentWidget::isThreats()
{
    return m_scan->finishAllOperation();
}

void NatiavParentWidget::threatOperation()
{
    m_scan->addThreats();
    m_antivirusWork->setIsbInVirusScanningPage(false);
}
