#include "natiavscanlog.h"
#include "csingleteon.h"

#include <DFontSizeManager>
#include <DApplicationHelper>
#include <DListView>
#include <DTitlebar>

#include <QDebug>
#include <QTime>
#include <QHeaderView>
#include <QDir>
#include <QDesktopServices>
#include <QUrl>
#include <QFileDialog>
#include <DBlurEffectWidget>


DWIDGET_USE_NAMESPACE
#define TABLE_SPACE                   5          //表栏两边间距
#define DIALOG_WIDTH                  500        //dialog  宽度
#define DIALOG_HEIGHT                 500        //dialog  高度
#define TABLEVIEW_WIDTH               480        //tableview  宽度
#define TABLEVIEW_HEIGHT              320        //tableview  高度
#define TABLEVIEW_ROW_HEIGHT          40         //tableview 每列高度

#define TABLE_COL_TIME                0          //table表 第二列 时间
#define TABLE_COL_TYPE                1          //table表 第三列 扫描类型
#define TABLE_COL_RESULT              2          //table表 第四列 结果
#define TABLE_COL_ACTION              3          //table表 第五列 操作
#define TABLE_COL_CHECK_STATUS        4          //table表 第六列 选择状态
#define TABLE_COL_FILE_NAME           5          //table表 第七列 文件名
#define TABLE_COL_ADD_TIME            6          //table表 第八列 时间
#define TABLE_COL_STATUS              7          //table表 第九列 扫描状态
#define TABLE_COL_RES                 8          //table表 第十列 结果

#define TABLE_COL_1_WIDTH             180        //table表 第二列 时间宽度
#define TABLE_COL_2_WIDTH             125        //table表 第三列 类型宽度
#define TABLE_COL_3_WIDTH             120        //table表 第四列 结果宽度
#define TABLE_COL_4_WIDTH             55        //table表 第五列 操作宽度
#define SPINNER_WIDTH                 100        //spinner 宽度
#define SPINNER_HEIGHT                100        //spinner 高度
#define PAGE_TABLE                    0          //表格页面
#define PAGE_SPINNER                  1          //加载页面
#define PB_DELETE_WIDTH               140        //删除日志按钮宽度
#define PB_DELETE_HEIGHT              37         //删除日志按钮高度
#define LEFT_MARGINS                  0          //左边距
#define TOP_MARGINS                   0          //上边距
#define RIGHT_MARGINS                 0          //右边距离
#define BUTTOM_MARGINS                0          //下边距离
#define SPACING                       5          //布局间隙
#define ACTION_ICON_WIDTH             16         //操作图表 宽度
#define ACTION_ICON_HEIGHT            16         //操作图标 高度



NatiavScanLog::NatiavScanLog(QWidget *parent)
    : DFrame(parent)
{
    installEventFilter(this);
    m_guiHelper = DGuiApplicationHelper::instance();
    // 大小设置
    this->resize(DIALOG_WIDTH, DIALOG_HEIGHT);
    this->setMinimumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    this->setMaximumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    m_checkBoxAll = new DCheckBox(this);
    m_pBLog = new DPushButton(this);

    // 设置为Dialog
    setWindowFlags(Qt::Dialog);
    // 设置为阻态
    setWindowModality(Qt::ApplicationModal);
    // 背景透明化
    setAttribute(Qt::WA_TranslucentBackground);
    // 背景透明化
    DBlurEffectWidget *m_blurEffect = new DBlurEffectWidget(this);
    m_blurEffect->setBlendMode(DBlurEffectWidget::BehindWindowBlend);
    // 设置模糊大小
    m_blurEffect->resize(size());
    m_blurEffect->lower();

    //缓冲显示定时器
    m_buffTimer = new QTimer();
    connect(m_buffTimer, &QTimer::timeout, this, &NatiavScanLog::doAddBufferLog);
    // 界面初始化
    initGUI();

    // 主题变换 改变部分图标颜色
    connect(m_guiHelper, SIGNAL(themeTypeChanged(ColorType)), this, SLOT(themeTypeChange(ColorType)));
    // 全选按钮改变信号
    connect(m_checkBoxAll, &DCheckBox::stateChanged, this, &NatiavScanLog::checkAll);
    // 删除日志
    connect(m_pBLog, &DPushButton::clicked, this, &NatiavScanLog::deleteLog);
    // 表头改变宽度
    connect(m_tableView->horizontalHeader(), &QHeaderView::sectionResized, this, &NatiavScanLog::updateTableForWidth);

    connect(m_tableView, &DTableView::clicked, this, &NatiavScanLog::currentRowClicked);

    // 点击表头触发排序
    connect(m_tableView->horizontalHeader(), &QHeaderView::sectionClicked, this, [ = ](int index) {
        // 根据不同的点击列  对应的列进行排序
        if (index == TABLE_COL_TIME) {
            // 第二列排序
            m_tableView->sortByColumn(TABLE_COL_CHECK_STATUS);
        } else if (index == TABLE_COL_TYPE) {
            // 第三列 排序
            m_tableView->sortByColumn(TABLE_COL_STATUS);
        } else if (index == TABLE_COL_RESULT) {
            // 第四列排序
            m_tableView->sortByColumn(TABLE_COL_RES);
        } else {
            return ;
        }
    });
}

//布局
void NatiavScanLog::initGUI()
{
    // 主布局    installEventFilter(this);
    QVBoxLayout *mainlayout = new  QVBoxLayout(this);
    // 边距离
    mainlayout->setContentsMargins(LEFT_MARGINS, TOP_MARGINS, RIGHT_MARGINS, BUTTOM_MARGINS + 10);
    mainlayout->setSpacing(SPACING);
    QHBoxLayout *titleBarLayout = new QHBoxLayout(this);
    titleBarLayout->setContentsMargins(LEFT_MARGINS, TOP_MARGINS, RIGHT_MARGINS, BUTTOM_MARGINS);
    // 右上角关闭按钮
    DTitlebar *titleBar = new DTitlebar(this);
    titleBar->setBackgroundTransparent(true);
    titleBar->setTitle(" ");
    titleBar->setMenuVisible(false);
    titleBarLayout->addWidget(titleBar, 0, Qt::AlignRight | Qt::AlignTop);

    mainlayout->addLayout(titleBarLayout);
    // 标题  查杀日志
    DLabel *label_title = new DLabel(tr("Scan Logs"));
    label_title->setAlignment(Qt::AlignCenter);
    QFont ft = label_title->font();
    ft.setBold(true);
    label_title->setFont(ft);
    DFontSizeManager::instance()->bind(label_title, DFontSizeManager::T5);
    mainlayout->addWidget(label_title, 0, Qt::AlignHCenter | Qt::AlignVCenter);

    // 中间表格布局
    DWidget *CenterWidget = new DWidget;
    CenterWidget->setFixedHeight(TABLEVIEW_HEIGHT);
    CenterWidget->setMinimumWidth(TABLEVIEW_WIDTH);
    CenterWidget->setLayout(initTableWidget());
    mainlayout->addWidget(CenterWidget, 0, Qt::AlignHCenter | Qt::AlignVCenter);

    QHBoxLayout *hlayout = new QHBoxLayout();
    hlayout->setContentsMargins(15, TOP_MARGINS, 15, BUTTOM_MARGINS);

    // 全选
    m_checkBoxAll->setText(tr("Select All"));
    hlayout->addWidget(m_checkBoxAll, 0, Qt::AlignVCenter | Qt::AlignLeft);
    hlayout->addStretch(1);
    m_proLable = new QLabel;
    hlayout->addWidget(m_proLable, 0, Qt::AlignVCenter | Qt::AlignRight);
    mainlayout->addLayout(hlayout);
    // 删除按钮
    m_pBLog->setText(tr("Delete Log"));
    m_pBLog->setMinimumSize(QSize(PB_DELETE_WIDTH, PB_DELETE_HEIGHT));
    m_pBLog->setMaximumSize(QSize(PB_DELETE_WIDTH, PB_DELETE_HEIGHT));
    mainlayout->addWidget(m_pBLog, 0, Qt::AlignBottom | Qt::AlignHCenter);

    m_checkBoxAll->setDisabled(true);
    m_pBLog->setDisabled(true);
    this->setLayout(mainlayout);

    getlogData();
}

//初始化tableview
QHBoxLayout *NatiavScanLog::initTableWidget()
{
    QHBoxLayout *layout = new QHBoxLayout;
    layout->setMargin(0);
    layout->setSpacing(0);

    // 表格属性设置
    m_tableView = new QTableView;
    m_tableView->verticalHeader()->setDefaultSectionSize(TABLEVIEW_ROW_HEIGHT);
    m_tableView->verticalHeader()->setVisible(false);
    m_tableView->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_tableView->setSelectionMode(QAbstractItemView::SingleSelection);
    m_tableView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_tableView->setFrameShape(QTableView::NoFrame);
    m_tableView->setShowGrid(false);
    m_tableView->setSortingEnabled(true);

    // 连接表格与模型
    m_model = new QStandardItemModel(this);
    m_tableView->setModel(m_model);

    // 时间
    QStandardItem *standaritem1 = new QStandardItem(tr("Time"));
    standaritem1->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    // 扫描类型
    QStandardItem *standaritem2 = new QStandardItem(tr("Scan type"));
    standaritem2->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    // 结果
    QStandardItem *standaritem3 = new QStandardItem(tr("Results"));
    standaritem3->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    // 操作
    QStandardItem *standaritem4 = new QStandardItem(tr("Action"));
    standaritem4->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);

    m_model->setHorizontalHeaderItem(TABLE_COL_TIME, standaritem1);
    m_model->setHorizontalHeaderItem(TABLE_COL_TYPE, standaritem2);
    m_model->setHorizontalHeaderItem(TABLE_COL_RESULT, standaritem3);
    m_model->setHorizontalHeaderItem(TABLE_COL_ACTION, standaritem4);
    // 选择状态
    m_model->setHorizontalHeaderItem(TABLE_COL_CHECK_STATUS, new QStandardItem("status"));
    // 文件名
    m_model->setHorizontalHeaderItem(TABLE_COL_FILE_NAME, new QStandardItem("fileName"));
    // 添加时间
    m_model->setHorizontalHeaderItem(TABLE_COL_ADD_TIME, new QStandardItem("addTime"));

    // 表格每列设置
    m_tableView->setColumnWidth(TABLE_COL_TIME, TABLE_COL_1_WIDTH);
    m_tableView->setColumnWidth(TABLE_COL_TYPE, TABLE_COL_2_WIDTH);
    m_tableView->setColumnWidth(TABLE_COL_RESULT, TABLE_COL_3_WIDTH);
    m_tableView->setColumnWidth(TABLE_COL_ACTION, TABLE_COL_4_WIDTH);
    // 隐藏文件名列
    m_tableView->setColumnHidden(TABLE_COL_CHECK_STATUS, true);
    m_tableView->setColumnHidden(TABLE_COL_FILE_NAME, true);
    m_tableView->setColumnHidden(TABLE_COL_ADD_TIME, true);
    m_tableView->setColumnHidden(TABLE_COL_STATUS, true);
    m_tableView->setColumnHidden(TABLE_COL_RES, true);
    // 不设置高亮
    m_tableView->horizontalHeader()->setHighlightSections(false);

    // 设置表格样式
    DefenderTable *defender = new DefenderTable;
    defender->setTableAndModel(m_tableView, m_model);
    layout->addWidget(defender);
    return layout;
}

//获得加载的数据
void NatiavScanLog::getlogData()
{
    // 异常文件
    QStringList  errorFiles ;
    // log 列表清空
    m_logFile.clear();
    QString dir_path = QString("%1/.deepin-defender/natiav/").arg(QDir::homePath());
    // 当前路径
    m_currentDir = dir_path;
    QDir *dir = new QDir(dir_path);
    QStringList fileInfo;
    fileInfo = dir->entryList();
    for (int i = 0; i < fileInfo.size(); i++) {
        if (fileInfo.at(i).contains(".log")) {
            // 日志文件列表
            m_logFile.append(fileInfo.at(i));
        }
    }

    for (int i = 0; i < m_logFile.size(); i++) {
        // 文件路径
        QString filePath = dir_path + m_logFile.at(i);
        QFile f(filePath);
        if (!f.open(QIODevice::ReadOnly | QIODevice::Text)) {
            qDebug() << "Open loglist failed." << endl;
            continue;
        }

        // 输入流
        QTextStream inputStream(&f);
        QStringList linelist;
        // 起始行
        int j = 0;
        // 读取 0-11 行的有效信息
        while (!inputStream.atEnd() && j++ < 14) {
            // 读取每一行
            QString lineStr = inputStream.readLine();
            if (!lineStr.isNull()) {
                // 有效数据
                linelist.append(lineStr);
            }
        }
        f.close();

        // mxz 10/29 https://pms.uniontech.com/zentao/bug-view-52676.html
        // 必须读取14 行信息
        if (linelist.size() == 14) {
            // 校验文本内容
            bool checkContextFlage = checkContext(linelist);
            if (checkContextFlage) {
                // 展现的数据类型
                ScanLogInfo scanlogInfo;
                // 时间
                QString dateTime = "";
                // 类型
                QString scanType = "";
                // 异常数量
                QString threatCount = "" ;
                // 截取出要展现的数据   依据中文的冒号  和英文的冒号
                if (linelist.at(1).split("：").size() == 2) {
                    // 扫描时间
                    dateTime = linelist.at(2).split("：").at(1);
                    //scanlogInfo.scanType = linelist.at(3).split("：").at(1).trimmed();
                    scanType = linelist.at(13).split(">>").at(1).trimmed();
                    threatCount = linelist.at(8).split("：").at(1).trimmed();

                } else if (linelist.at(1).split(":").size() == 2) {
                    // 扫描时间 eg:Scan Time:2020-09-09 16:49:13 https://pms.uniontech.com/zentao/bug-view-47031.html
                    dateTime = linelist.at(2).split(":").at(1) + ":" + linelist.at(2).split(":").at(2) + ":" + linelist.at(2).split(":").last();
                    scanType = linelist.at(13).split(">>").at(1).trimmed();
                    threatCount = linelist.at(8).split(":").at(1).trimmed();
                }
                scanlogInfo.datetime = dateTime;
                if (scanType == "0") {
                    scanlogInfo.scanType = tr("Full Scan");
                } else if (scanType == "1") {
                    scanlogInfo.scanType = tr("Quick Scan");
                } else {
                    scanlogInfo.scanType = tr("Custom Scan");
                }
                scanlogInfo.result = threatCount ;
                // 文件名称
                scanlogInfo.filePath = m_logFile.at(i);
                m_scanloglist.append(scanlogInfo);
            }
        } else {
            // 讲不正确的文件保存下来  然后删除
            errorFiles.append(m_logFile.at(i));
        }
    }
    // 总数
    m_total = m_scanloglist.size();
    // 启动缓加载
    m_buffTimer->start(1);

    // 删除异常文件
    for (int i = 0; i < errorFiles.size(); i++) {
        deletelogFile(errorFiles.at(i));
    }
}

// 打开指定文件
void NatiavScanLog::openLogFile(QString sFilepath)
{
    QDesktopServices::openUrl(QUrl::fromLocalFile(QFileInfo(sFilepath).absoluteFilePath()));
}

// 删除指定文件
void NatiavScanLog::deletelogFile(QString sFilepath)
{
    QFile file(sFilepath);
    // 判断文件是否存在
    if (file.exists()) {
        file.remove();
    }
}

// 更改table 因改变表头宽度
void NatiavScanLog::updateTableForWidth(int nLogicalIndex, int nOldSize, int nNewSize)
{
    Q_UNUSED(nOldSize);
    // 行数
    int count = m_model->rowCount();
    for (int i = 0; i < count; i++) {
        // 判断是第几列改变
        if (nLogicalIndex == TABLE_COL_TIME) {
            DWidget *widget = m_tableView->indexWidget(m_model->index(i, nLogicalIndex));
            DLabel *lable  = widget->findChild<DLabel *>("datetime");
            // 得到每个字符的宽度
            QFontMetrics fontWidth(lable->font());
            QString elideNote = fontWidth.elidedText(m_scanloglist.at(i).datetime, Qt::ElideMiddle, nNewSize - (2 * TABLE_SPACE) - 20);
            // 显示省略好的字符串
            lable->setText(elideNote);
            // 设置tooltips
            lable->setToolTip(m_scanloglist.at(i).datetime);
        } else if (nLogicalIndex == TABLE_COL_TYPE) {
            DWidget *widget = m_tableView->indexWidget(m_model->index(i, nLogicalIndex));
            DLabel *lable = widget->findChild<DLabel *>("scanType");
            // 得到每个字符的宽度
            QFontMetrics fontWidth(lable->font());
            QString elideNote = fontWidth.elidedText(m_scanloglist.at(i).scanType, Qt::ElideMiddle, nNewSize - (2 * TABLE_SPACE));
            // 显示省略好的字符串
            lable->setText(elideNote);
            lable->setToolTip(m_scanloglist.at(i).datetime);
        } else if (nLogicalIndex == TABLE_COL_RESULT) {
            DWidget *widget = m_tableView->indexWidget(m_model->index(i, nLogicalIndex));
            DLabel *lable = widget->findChild<DLabel *>("result");
            // 得到每个字符的宽度
            QFontMetrics fontWidth(lable->font());
            QString reslut;
            if (m_scanloglist.at(i).result == "0") {
                // "未发现风险项";
                reslut = QString(tr("No threats found"));
            } else {
                // 发现%d个风险
                reslut = QString(tr("%1 threats found").arg(m_scanloglist.at(i).result));
            }
            QString elideNote = fontWidth.elidedText(reslut, Qt::ElideMiddle, nNewSize - (2 * TABLE_SPACE));
            // 显示省略好的字符串
            lable->setText(elideNote);
            lable->setToolTip(reslut);
        } else {
            return;
        }
    }
}

// 刷新table 因字体改变
void NatiavScanLog::refreshTableByFontChange()
{
    // 行数
    int count = m_model->rowCount();
    // 获得没一列的宽度
    int wcol1 = m_tableView->columnWidth(TABLE_COL_TIME);
    int wcol2 = m_tableView->columnWidth(TABLE_COL_TYPE);
    int wcol3 = m_tableView->columnWidth(TABLE_COL_RESULT);
    for (int i = 0; i < count; i++) {
        // 第一列修改
        DWidget *widget = m_tableView->indexWidget(m_model->index(i, TABLE_COL_TIME));
        DLabel *lable  = widget->findChild<DLabel *>("datetime");
        // 得到每个字符的宽度
        QFontMetrics fontWidth(lable->font());
        QString elideNote = fontWidth.elidedText(m_scanloglist.at(i).datetime, Qt::ElideMiddle, wcol1 - (2 * TABLE_SPACE) - 20);
        // 显示省略好的字符串
        lable->setText(elideNote);
        // 设置tooltips
        lable->setToolTip(m_scanloglist.at(i).datetime);

        // 第二列修改
        DWidget *widget2 = m_tableView->indexWidget(m_model->index(i, TABLE_COL_TYPE));
        DLabel *lable2 = widget2->findChild<DLabel *>("scanType");
        // 得到每个字符的宽度
        //QFontMetrics fontWidth(lable->font());
        QString elideNote2 = fontWidth.elidedText(m_scanloglist.at(i).scanType, Qt::ElideMiddle, wcol2 - (2 * TABLE_SPACE));
        // 显示省略好的字符串
        lable2->setText(elideNote2);
        lable2->setToolTip(m_scanloglist.at(i).datetime);

        // 第三列修改
        DWidget *widget3 = m_tableView->indexWidget(m_model->index(i, TABLE_COL_RESULT));
        DLabel *lable3 = widget3->findChild<DLabel *>("result");
        // 得到每个字符的宽度
        //QFontMetrics fontWidth(lable->font());
        QString reslut;
        if (m_scanloglist.at(i).result == "0") {
            // "未发现风险项";
            reslut = QString(tr("No threats found"));
        } else {
            // 发现%d个风险
            reslut = QString(tr("%1 threats found").arg(m_scanloglist.at(i).result));
        }
        QString elideNote3 = fontWidth.elidedText(reslut, Qt::ElideMiddle, wcol3 - (2 * TABLE_SPACE));
        // 显示省略好的字符串
        lable3->setText(elideNote3);
        lable3->setToolTip(reslut);

    }
}

// 当前行被选中
void NatiavScanLog::currentRowClicked(QModelIndex index)
{
    // 当前点击的位置坐标
    int row = index.row();
    int column = index.column();

    if (column != TABLE_COL_ACTION) {
        // 为了避免操作 不可以在操作列点击
        DWidget *widget = m_tableView->indexWidget(m_model->index(row, TABLE_COL_TIME));
        // 多选框对象
        DCheckBox *check = widget->findChild<DCheckBox *>("singleChecks");
        // 按钮的状态
        int status = check->checkState();
        // 改变按钮状态
        if (status == 0) {
            check->setCheckState(Qt::CheckState::Checked);
        } else {
            check->setCheckState(Qt::CheckState::Unchecked);
        }
        // 判断全选的状态
        checkBoxAllStatus();
    } else {
        return;
    }
}

// 判断全选按钮
void NatiavScanLog::checkBoxAllStatus()
{
    // 总行数
    int count = m_model->rowCount();
    // 选中状态的map
    QMap<int, int> rowMap;
    for (int i = 0; i < count; i++) {
        DWidget *widget = m_tableView->indexWidget(m_model->index(i, TABLE_COL_TIME));
        // 多选框对象
        DCheckBox *check = widget->findChild<DCheckBox *>("singleChecks");
        // 按钮的状态
        int status = check->checkState();
        switch (status) {
        case 0:
            rowMap.insert(i, 0);
            m_model->item(i, TABLE_COL_CHECK_STATUS)->setText("0");
            break;
        case 1:
            break;
        case 2:
            // 如果是选中状态移除
            rowMap.insert(i, 1);
            m_model->item(i, TABLE_COL_CHECK_STATUS)->setText("1");
            break;
        }
    }
    // 打沟的数量
    int tcount = 0;
    // 不打沟的数量
    int fcount = 0;
    QMapIterator<int, int> rowMapIterator(rowMap);
    rowMapIterator.toBack();
    while (rowMapIterator.hasPrevious()) {
        int value;
        rowMapIterator.previous();
        value = rowMapIterator.value();
        if (value == 0) {
            fcount++;
        } else if (value == 1) {
            tcount++;
        }
    }

    // 根据数量判断  全选状态
    if (tcount == count) {
        m_checkBoxAll->setCheckState(Qt::CheckState::Checked);
    } else if (fcount == count) {
        m_checkBoxAll->setCheckState(Qt::CheckState::Unchecked);
    } else {
        // 有一个没有选中  先把全选设为未选择状态   然后吧以前的选中都恢复
        m_checkBoxAll->setCheckState(Qt::CheckState::Unchecked);
        QMap<int, int>::iterator it;
        for (it = rowMap.begin(); it != rowMap.end(); ++it) {
            if (it.value() == 0) {
                int row = it.key();
                DWidget *widget = m_tableView->indexWidget(m_model->index(row, TABLE_COL_TIME));
                DCheckBox *check = widget->findChild<DCheckBox *>("singleChecks");
                check->setCheckState(Qt::CheckState::Unchecked);
            } else {
                int row = it.key();
                DWidget *widget = m_tableView->indexWidget(m_model->index(row, TABLE_COL_TIME));
                DCheckBox *check = widget->findChild<DCheckBox *>("singleChecks");
                check->setCheckState(Qt::CheckState::Checked);
            }
        }
    }
}

void NatiavScanLog::appendFileInfo(ScanLogInfo info)
{
    //锁
    DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
    //当前行
    int currentRow = m_model->rowCount();
    // 默认是没有选中
    m_model->setItem(currentRow, TABLE_COL_CHECK_STATUS, new QStandardItem(QString("%1").arg(0)));
    m_model->setItem(currentRow, TABLE_COL_FILE_NAME, new QStandardItem(QString("%1").arg(info.filePath)));
    m_model->setItem(currentRow, TABLE_COL_ADD_TIME, new QStandardItem(QString("%1").arg(info.datetime)));   //时间
    m_model->setItem(currentRow, TABLE_COL_STATUS, new QStandardItem(QString("%1").arg(info.scanType)));   //扫描出的类型
    m_model->setItem(currentRow, TABLE_COL_RES, new QStandardItem(QString("%1").arg(info.result)));   //扫描出数据结果

    DWidget *widget1 = new DWidget;
    QHBoxLayout *Layout1 = new QHBoxLayout;

    // 第一列
    Layout1->setContentsMargins(0, SPACING, TABLE_SPACE, SPACING);
    Layout1->setSpacing(TABLE_SPACE);
    DCheckBox *check = new DCheckBox();
    check->setFixedSize(QSize(20, 20));
    // 多选框对象
    check->setObjectName("singleChecks");
    check->setCheckState(Qt::CheckState::Unchecked);
    Layout1->addWidget(check, 0, Qt::AlignLeft | Qt::AlignVCenter);
    // 复选框
    connect(check, &DCheckBox::clicked, this, &NatiavScanLog::tableCheckItem);

    DLabel *lable1 = new DLabel();
    lable1->setObjectName("datetime");
    // 得到每个字符的宽度
    QFontMetrics fontWidth(lable1->font());
    // 最大宽度显示文字
    QString elideNote = fontWidth.elidedText(info.datetime, Qt::ElideMiddle, TABLE_COL_1_WIDTH - 10 - 25);
    // 显示省略好的字符串
    lable1->setText(elideNote);
    // 设置tooltips
    lable1->setToolTip(info.datetime);
    Layout1->addWidget(lable1, 0, Qt::AlignLeft | Qt::AlignVCenter);
    Layout1->addStretch(0);
    widget1->setLayout(Layout1);

    DWidget *widget2 = new DWidget;
    QHBoxLayout *Layout2 = new QHBoxLayout;
    // 第三列
    Layout2->setContentsMargins(0, SPACING, TABLE_SPACE, SPACING);
    DLabel *lable2 = new DLabel();
    lable2->setObjectName("scanType");
    // 最大宽度显示文字
    QString scanType = fontWidth.elidedText(info.scanType, Qt::ElideMiddle, TABLE_COL_2_WIDTH - 10);
    lable2->setText(scanType);
    lable2->setToolTip(scanType);
    Layout2->addWidget(lable2);
    widget2->setLayout(Layout2);

    DWidget *widget3 = new DWidget;
    QHBoxLayout *Layout3 = new QHBoxLayout;
    // 第四列
    Layout3->setContentsMargins(0, SPACING, TABLE_SPACE, SPACING);
    DLabel *lable3 = new DLabel();
    lable3->setObjectName("result");
    QString reslut;
    // 根据结果显示  相关文本
    if (info.result == "0") {
        // "未发现风险项";
        reslut = QString(tr("No threats found"));
    } else {
        // 发现%d个风险
        reslut = QString(tr("%1 threats found").arg(info.result));
    }
    // 最大宽度显示文字
    QString reslut1 = fontWidth.elidedText(reslut, Qt::ElideMiddle, TABLE_COL_3_WIDTH);
    lable3->setText(reslut1);
    lable3->setToolTip(reslut);
    Layout3->addWidget(lable3);
    widget3->setLayout(Layout3);

    DWidget *pwidget = new DWidget;
    // 第五列
    QHBoxLayout *hlayout = new QHBoxLayout;
    hlayout->setContentsMargins(0, SPACING, TABLE_SPACE, SPACING);
    DPushButton *button = new DPushButton;
    button->setObjectName("pbOperation");
    // 操作显示栏
    button->setToolTip(tr("Action"));
    // 获得主题类型
    int type = m_guiHelper->themeType();
    // 浅色
    if (type == 1) {
        button->setIcon(QIcon::fromTheme("dcc_natiav_log_details"));
    } else {
        button->setIcon(QIcon::fromTheme("icon_log_details_dark"));
    }

    // 操作图表大小设置
    button->setIconSize(QSize(ACTION_ICON_WIDTH, ACTION_ICON_HEIGHT));
    button->setFixedSize(ACTION_ICON_WIDTH, ACTION_ICON_HEIGHT);
    button->setFlat(true);
    button->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    hlayout->addWidget(button);
    pwidget->setLayout(hlayout);
    // 文件路径
    QString pathfile = m_currentDir + info.filePath;
    connect(button, &DPushButton::clicked, this, [ = ] {
        // 打开文件路径
        this->openLogFile(pathfile);
    });

    // 设置每列的控件
    m_tableView->setIndexWidget(m_model->index(currentRow, TABLE_COL_TIME), widget1);
    m_tableView->setIndexWidget(m_model->index(currentRow, TABLE_COL_TYPE), widget2);
    m_tableView->setIndexWidget(m_model->index(currentRow, TABLE_COL_RESULT), widget3);
    m_tableView->setIndexWidget(m_model->index(currentRow, TABLE_COL_ACTION), pwidget);
    // 缓冲作用
    m_model->setItem(currentRow + 1, 0, new QStandardItem);
    m_model->removeRow(currentRow + 1);
    // 设置每列的宽度
    m_tableView->setColumnWidth(TABLE_COL_TIME, TABLE_COL_1_WIDTH);
    m_tableView->setColumnWidth(TABLE_COL_TYPE, TABLE_COL_2_WIDTH);
    m_tableView->setColumnWidth(TABLE_COL_RESULT, TABLE_COL_3_WIDTH);
    m_tableView->setColumnWidth(TABLE_COL_ACTION, TABLE_COL_4_WIDTH);
    // 隐藏列
    m_tableView->setColumnHidden(TABLE_COL_CHECK_STATUS, true);
    m_tableView->setColumnHidden(TABLE_COL_FILE_NAME, true);
    m_tableView->setColumnHidden(TABLE_COL_ADD_TIME, true);
    m_tableView->setColumnHidden(TABLE_COL_STATUS, true);
    m_tableView->setColumnHidden(TABLE_COL_RES, true);
}

void NatiavScanLog::setCurrentProcess(int nCurrent)
{
    m_proLable->setText(QString("%1/%2").arg(nCurrent).arg(m_total));
}

// 校验日志文件是否正确 输入参数是每行拼接的字符
bool NatiavScanLog::checkContext(QStringList list)
{
    // 第二行 校验病毒版本 由于翻译中文的时候  是中文冒号 英文是英文冒号 比如：病毒库版本：20.9.1.19
    if ((list.at(1).split("：").size() == 2) || (list.at(1).split(":").size() == 2)) {
        // 第三行 校验扫描时间 例如 扫描时间：2020-09-05 18:41:55
        if ((list.at(2).split("：").size() == 2) || (list.at(2).split(":").size() == 4)) {
            // 第八行 校验病毒个数 例如 扫描文件数：525
            if ((list.at(7).split("：").size() == 2) || (list.at(7).split(":").size() == 2)) {
                // 第14行 扫描类型
                if (list.at(13).split(">>").size() == 2) {
                    return true;
                } else {
                    return false;
                }
            } else {
                return false;
            }
        } else {
            return false;
        }
    } else {
        return false;
    }
}

// 定时添加缓冲数据
void NatiavScanLog::doAddBufferLog()
{
    ++ m_curret ;
    int size = m_scanloglist.size();
    if (size == 0) {
        qDebug() << " longing finish";
        // 定时器停止  按钮展现
        m_buffTimer->stop();
        m_pBLog->setDisabled(false);
        // 全选状态判断
        m_checkBoxAll->setCheckState(Qt::CheckState::Unchecked);
        if (m_total == 0) {
            // 没有隔离数据为不可以操作
            m_checkBoxAll->setDisabled(true);
        } else {
            // 有数据可以操作
            m_checkBoxAll->setDisabled(false);
        }
        return ;
    } else {
        setCurrentProcess(m_curret);
        appendFileInfo(m_scanloglist.at(0));
        m_scanloglist.removeAt(0);
    }
}

// 时间过滤
bool NatiavScanLog::eventFilter(QObject *obj, QEvent *event)
{
    if (this == obj) {
        // 如果当前是字体改变的事件
        if (event->type() == QEvent::FontChange) {
            // 刷新列表
            refreshTableByFontChange();
        }
    }
    return QWidget::eventFilter(obj, event);
}

// 全选
void NatiavScanLog::checkAll(int nStatus)
{
    // 行数
    int count = m_model->rowCount();
    for (int i = 0; i < count; i++) {
        DWidget *widget = m_tableView->indexWidget(m_model->index(i, TABLE_COL_TIME));
        DCheckBox *check = widget->findChild<DCheckBox *>("singleChecks");
        switch (nStatus) {
        case 0:
            check->setCheckState(Qt::CheckState::Unchecked);
            break;
        case 1:
            // check->setCheckState(Qt::CheckState::PartiallyChecked);
            break;
        case 2:
            check->setCheckState(Qt::CheckState::Checked);
            break;
        }
    }

}

// 删除日志
void NatiavScanLog::deleteLog()
{
    // 总行数
    int count = m_model->rowCount();
    // 选中状态的map
    QMap<int, int> rowMap;
    for (int i = 0; i < count; i++) {
        DWidget *widget = m_tableView->indexWidget(m_model->index(i, TABLE_COL_TIME));
        // 多选框对象
        DCheckBox *check = widget->findChild<DCheckBox *>("singleChecks");
        // 按钮的状态
        int status = check->checkState();
        switch (status) {
        case 0:
            rowMap.insert(i, 0);
            break;
        case 1:
            break;
        case 2:
            // 1代表是选中状态
            rowMap.insert(i, 1);
            break;
        }
    }

    int key;
    int value;
    // map  迭代
    QMapIterator<int, int> rowMapIterator(rowMap);
    rowMapIterator.toBack();
    // 是否在最后一个
    while (rowMapIterator.hasPrevious()) {
        rowMapIterator.previous();
        key = rowMapIterator.key();
        value = rowMapIterator.value();
        if (value == 1) {
            // 文件名
            QString fileName =  m_model->index(key, TABLE_COL_FILE_NAME).data(Qt::DisplayRole).toString();
            // 文件路径
            QString filepath = m_currentDir + fileName;
            deletelogFile(filepath);
            m_model->removeRow(key);
        }
    }

    // 删除完后剩余的总行数
    int count1 = m_model->rowCount();

    m_total = count1;
    setCurrentProcess(count1);
    if (count1 == 0) {
        // 表格中没有数据了 去选不会选中不至于灰色
        m_checkBoxAll->setCheckState(Qt::CheckState::Unchecked);
        m_checkBoxAll->setDisabled(true);
    }
}

// table 单选对象
void NatiavScanLog::tableCheckItem()
{
    checkBoxAllStatus();
}

// 主题切换
void NatiavScanLog::themeTypeChange(ColorType themeType)
{
    int type;
    switch (themeType) {
    case UnknownType:
        type = 0;
        break;
    case LightType:
        type = 1;
        break;
    case DarkType:
        type = 2 ;
        break;
    }
    //总行数
    int count = m_model->rowCount();
    for (int i = 0; i < count; i++) {
        DWidget *widget = m_tableView->indexWidget(m_model->index(i, TABLE_COL_ACTION));
        DPushButton *button  = widget->findChild<DPushButton *>("pbOperation");
        // 浅色
        if (type == 1) {
            button->setIcon(QIcon::fromTheme("dcc_natiav_log_details"));
        } else {
            button->setIcon(QIcon::fromTheme("icon_log_details_dark"));
        }
    }
}


