#include "natiavtrustdialog.h"
#include "csingleteon.h"

#include <DFontSizeManager>
#include <DApplicationHelper>
#include <DListView>
#include <QTimer>

#include <QDebug>
#include <QTime>
#include <QHeaderView>
#include <QDebug>
#include <DBlurEffectWidget>
#include <DTitlebar>

DWIDGET_USE_NAMESPACE
#define TABLE_SPACE                   5          //表栏两边间距
#define DIALOG_WIDTH                  510        //dialog  宽度
#define DIALOG_HEIGHT                 500        //dialog  高度
#define TABLEVIEW_WIDTH               490        //tableview  宽度
#define TABLEVIEW_HEIGHT              320        //tableview  高度
#define TABLEVIEW_ROW_HEIGHT          50         //tableview 每列高度

#define TABLE_COL_QFILES              0          //table表 第二列 隔离文件
#define TABLE_COL_TIME                1          //table表 第三列 添加时间
#define TABLE_COL_CHECK_STATUS        2          //table表 第四列 选择状态
#define TABLE_COL_FILE_PATH           3          //table表 第五列 文件路径
#define TABLE_COL_FILE_NAME           4          //table表 第六列 文件名称
#define TABLE_COL_ADD_TIME            5          //table表 第七列 时间

#define TABLE_COL_1_WIDTH             330        //table表 第二列 宽度
#define TABLE_COL_2_WIDTH             160        //table表 第三列 宽度
#define SPINNER_WIDTH                 100        //spinner 宽度
#define SPINNER_HEIGHT                100        //spinner 高度
#define PB_DELETE_WIDTH               140        //移除隔离按钮宽度
#define PB_DELETE_HEIGHT              37         //移除隔离按钮高度
#define PAGE_TABLE                    0          //表格页面
#define PAGE_SPINNER                  1          //加载页面
#define LEFT_MARGINS                  0          //左边距
#define TOP_MARGINS                   0          //上边距
#define RIGHT_MARGINS                 0          //右边距离
#define BUTTOM_MARGINS                0          //下边距离
#define SPACING                       5          //布局间隙


NatiavTrustDialog::NatiavTrustDialog(QWidget *parent)
    : DFrame(parent)
{
    installEventFilter(this);
    this->resize(DIALOG_WIDTH, DIALOG_HEIGHT);
    this->setMinimumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    this->setMaximumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    //设置为Dialog
    setWindowFlags(Qt::Dialog);
    //设置为阻态
    setWindowModality(Qt::ApplicationModal);
    //背景透明化
    setAttribute(Qt::WA_TranslucentBackground);
    //模糊效果
    DBlurEffectWidget *m_blurEffect = new DBlurEffectWidget(this);
    m_blurEffect->setBlendMode(DBlurEffectWidget::BehindWindowBlend);

    //设置模糊大小
    m_blurEffect->resize(size());
    m_blurEffect->lower();
    //全选
    m_checkBoxAll = new DCheckBox;
    //移出信任确认
    m_PbTrust = new DPushButton;
    m_PbTrust->setDisabled(true);
    m_TrustfileInfo.clear();

    //界面初始化
    initGUI();

    //缓冲显示定时器
    m_buffTimer = new QTimer();
    connect(m_buffTimer, &QTimer::timeout, this, &NatiavTrustDialog::doAddBuffer);

    //全选按钮改变信号
    connect(m_checkBoxAll, &DCheckBox::stateChanged, this, &NatiavTrustDialog::checkAll);
    //移除按钮点击
    connect(m_PbTrust, &DPushButton::clicked, this, &NatiavTrustDialog::deleteTrust);
    //表头改变宽度
    connect(m_tableView->horizontalHeader(), &QHeaderView::sectionResized, this, &NatiavTrustDialog::updateTableForWidth);
    connect(m_tableView, &DTableView::clicked, this, &NatiavTrustDialog::currentRowClicked);
    //点击表头触发排序
    connect(m_tableView->horizontalHeader(), &QHeaderView::sectionClicked, this, [ = ](int index) {
        // 根据不同的点击列  对应的列进行排序
        if (index == TABLE_COL_QFILES) {
            // 第二列排序
            m_tableView->sortByColumn(TABLE_COL_CHECK_STATUS);

        } else {
            // 第三列排序
            m_tableView->sortByColumn(TABLE_COL_ADD_TIME);
        }
    });

}

NatiavTrustDialog::~NatiavTrustDialog()
{
    disconnect(this);
}

//布局
void NatiavTrustDialog::initGUI()
{
    //主布局
    QVBoxLayout *mainlayout = new  QVBoxLayout(this);
    //边距离
    mainlayout->setContentsMargins(LEFT_MARGINS, TOP_MARGINS, RIGHT_MARGINS, BUTTOM_MARGINS + 10);
    mainlayout->setSpacing(SPACING);
    QHBoxLayout *titleBarLayout = new QHBoxLayout(this);
    titleBarLayout->setContentsMargins(LEFT_MARGINS, TOP_MARGINS, RIGHT_MARGINS, BUTTOM_MARGINS);
    //关闭按钮
    DTitlebar *titleBar = new DTitlebar(this);
    titleBar->setBackgroundTransparent(true);
    titleBar->setTitle("");
    titleBar->setMenuVisible(false);
    titleBarLayout->addWidget(titleBar, 0, Qt::AlignRight | Qt::AlignTop);
    mainlayout->addLayout(titleBarLayout);
    //标题  信任区
    DLabel *label_title = new DLabel(tr("Whitelist"));
    label_title->setAlignment(Qt::AlignCenter);
    QFont ft = label_title->font();
    ft.setBold(true);
    //标题字体
    label_title->setFont(ft);
    DFontSizeManager::instance()->bind(label_title, DFontSizeManager::T5);
    mainlayout->addWidget(label_title);

    //tableview
    DWidget *CenterWidget = new DWidget;
    CenterWidget->setFixedHeight(TABLEVIEW_HEIGHT);
    CenterWidget->setMinimumWidth(TABLEVIEW_WIDTH);
    //中间表格
    CenterWidget->setLayout(initTableWidget());
    mainlayout->addWidget(CenterWidget, 0, Qt::AlignVCenter | Qt::AlignHCenter);
    //mainlayout->addSpacing(SPACING);
    QHBoxLayout *hlayout = new QHBoxLayout();
    hlayout->setContentsMargins(15, TOP_MARGINS, 15, BUTTOM_MARGINS);
    //全选
    m_checkBoxAll->setText(tr("Select All"));
    hlayout->addWidget(m_checkBoxAll, 0, Qt::AlignVCenter | Qt::AlignLeft);
    hlayout->addStretch(1);
    m_proLable = new QLabel;
    hlayout->addWidget(m_proLable, 0, Qt::AlignVCenter | Qt::AlignRight);
    mainlayout->addLayout(hlayout);

    // 按钮  移出信任   Remove from Whitelist
    m_PbTrust->setText(tr("Remove"));
    m_PbTrust->setMinimumSize(QSize(PB_DELETE_WIDTH, PB_DELETE_HEIGHT));
    m_PbTrust->setMaximumSize(QSize(PB_DELETE_WIDTH, PB_DELETE_HEIGHT));
    mainlayout->addWidget(m_PbTrust, 0, Qt::AlignBottom | Qt::AlignHCenter);

    m_checkBoxAll->setDisabled(true);
    m_PbTrust->setDisabled(true);
}

//初始化中间tableview布局
QHBoxLayout *NatiavTrustDialog::initTableWidget()
{
    QHBoxLayout *layout = new QHBoxLayout;
    layout->setMargin(0);
    layout->setSpacing(0);

    //行高   列头去掉
    m_tableView = new DTableView;
    m_tableView->verticalHeader()->setDefaultSectionSize(TABLEVIEW_ROW_HEIGHT);
    m_tableView->verticalHeader()->setVisible(false);
    m_tableView->setSelectionBehavior(QAbstractItemView::SelectRows);
    //单选
    m_tableView->setSelectionMode(QAbstractItemView::NoSelection);
    //不可以编辑
    m_tableView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    //形状
    m_tableView->setFrameShape(QTableView::NoFrame);
    //栅格
    m_tableView->setShowGrid(false);
    //排序
    m_tableView->setSortingEnabled(true);

    // 连接表格与模型
    m_model = new QStandardItemModel(this);
    m_tableView->setModel(m_model);

    //item项
    QStandardItem *standaritem0 = new QStandardItem(" ");
    standaritem0->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    // 信任文件
    QStandardItem *standaritem1 = new QStandardItem(tr("Whitelist files"));
    standaritem1->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    //添加时间
    QStandardItem *standaritem2 = new QStandardItem(tr("Time added"));
    standaritem2->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);

    m_model->setHorizontalHeaderItem(TABLE_COL_QFILES, standaritem1);
    m_model->setHorizontalHeaderItem(TABLE_COL_TIME, standaritem2);
    //选择状态
    m_model->setHorizontalHeaderItem(TABLE_COL_CHECK_STATUS, new QStandardItem("checkStatus"));
    //路径
    m_model->setHorizontalHeaderItem(TABLE_COL_FILE_PATH, new QStandardItem("filePath"));
    //文件名称
    m_model->setHorizontalHeaderItem(TABLE_COL_FILE_NAME, new QStandardItem("fileName"));
    //添加时间
    m_model->setHorizontalHeaderItem(TABLE_COL_ADD_TIME, new QStandardItem("addtime"));
    //设置每隔宽度
    m_tableView->setColumnWidth(TABLE_COL_QFILES, TABLE_COL_1_WIDTH);
    m_tableView->setColumnWidth(TABLE_COL_TIME, TABLE_COL_2_WIDTH);
    //隐藏列
    m_tableView->setColumnHidden(TABLE_COL_CHECK_STATUS, true);
    m_tableView->setColumnHidden(TABLE_COL_FILE_PATH, true);
    m_tableView->setColumnHidden(TABLE_COL_FILE_NAME, true);
    m_tableView->setColumnHidden(TABLE_COL_ADD_TIME, true);

    m_tableView->horizontalHeader()->setHighlightSections(false);

    //表格样式
    DefenderTable *defender = new DefenderTable;
    defender->setTableAndModel(m_tableView, m_model);
    layout->addWidget(defender);
    //获得数据
    return layout;

}

//更改table  因改变表头宽度
void NatiavTrustDialog::updateTableForWidth(int nLogicalIndex, int nOldSize, int nNewSize)
{
    Q_UNUSED(nOldSize);
    //行数
    int count = m_model->rowCount();
    for (int i = 0; i < count; i++) {
        if (nLogicalIndex == TABLE_COL_QFILES) {
            //文件路径
            QString filePath = m_model->index(i, TABLE_COL_FILE_PATH).data(Qt::DisplayRole).toString();
            //文件名称
            QString fileName = m_model->index(i, TABLE_COL_FILE_NAME).data(Qt::DisplayRole).toString();
            DWidget *widget = m_tableView->indexWidget(m_model->index(i, nLogicalIndex));
            DLabel *fname  = widget->findChild<DLabel *>("fileName");
            //得到每个字符的宽度
            QFontMetrics fontWidth(fname->font());
            QString elideNote = fontWidth.elidedText(fileName, Qt::ElideMiddle, nNewSize - (2 * TABLE_SPACE) - 20);
            //显示省略好的字符串
            fname->setText(elideNote);
            //设置tooltips
            fname->setToolTip(fileName);

            DLabel *fpath  = widget->findChild<DLabel *>("filePath");
            //得到每个字符的宽度
            QFontMetrics fontWidth1(fpath->font());
            QString elideNote1 = fontWidth1.elidedText(filePath, Qt::ElideMiddle, nNewSize - (2 * TABLE_SPACE) - 20);
            //显示省略好的字符串
            fpath->setText(elideNote1);
            //设置tooltips
            fpath->setToolTip(filePath);
        } else {
            return;
        }
    }
}

bool NatiavTrustDialog::eventFilter(QObject *obj, QEvent *event)
{
    if (this == obj) {
        // 如果当前是字体改变的事件
        if (event->type() == QEvent::FontChange) {
            // 刷新列表
            refreshTableByFontChange();
        }
    }
    return QWidget::eventFilter(obj, event);
}

// 刷新table 因字体改变
void NatiavTrustDialog::refreshTableByFontChange()
{
    //行数
    int count = m_model->rowCount();
    // 获得每一列的宽度
    int wcol1 = m_tableView->columnWidth(TABLE_COL_QFILES);
    int wcol2 = m_tableView->columnWidth(TABLE_COL_TIME);

    for (int i = 0; i < count; i++) {
        // 第一列
        //文件路径
        QString filePath = m_model->index(i, TABLE_COL_FILE_PATH).data(Qt::DisplayRole).toString();
        //文件名称
        QString fileName = m_model->index(i, TABLE_COL_FILE_NAME).data(Qt::DisplayRole).toString();
        DWidget *widget = m_tableView->indexWidget(m_model->index(i, TABLE_COL_QFILES));
        DLabel *fname  = widget->findChild<DLabel *>("fileName");
        //得到每个字符的宽度
        QFontMetrics fontWidth(fname->font());
        QString elideNote = fontWidth.elidedText(fileName, Qt::ElideMiddle, wcol1 - (2 * TABLE_SPACE) - 20);
        //显示省略好的字符串
        fname->setText(elideNote);
        //设置tooltips
        fname->setToolTip(fileName);

        DLabel *fpath  = widget->findChild<DLabel *>("filePath");
        //得到每个字符的宽度
        QFontMetrics fontWidth1(fpath->font());
        QString elideNote1 = fontWidth1.elidedText(filePath, Qt::ElideMiddle, wcol1 - (2 * TABLE_SPACE) - 20);
        //显示省略好的字符串
        fpath->setText(elideNote1);
        //设置tooltips
        fpath->setToolTip(filePath);

        //第二列
        QString addTime = m_model->index(i, TABLE_COL_ADD_TIME).data(Qt::DisplayRole).toString();    //时间
        DWidget *widget2 = m_tableView->indexWidget(m_model->index(i, TABLE_COL_TIME));
        DLabel *lable1 = widget2->findChild<DLabel *>("addTime");
        QFontMetrics fontWidth2(lable1->font());
        QString elideNote3 = fontWidth2.elidedText(addTime, Qt::ElideMiddle, wcol2 - (2 * TABLE_SPACE));
        lable1->setText(elideNote3);
        //设置tooltips
        lable1->setToolTip(addTime);

    }
}

void NatiavTrustDialog::closeEvent(QCloseEvent *event)
{
    Q_UNUSED(event);
    m_buffTimer->stop();
    m_TrustfileInfo.clear();
}

// 当前行被选中
void NatiavTrustDialog::currentRowClicked(QModelIndex index)
{
    // 当前点击的位置坐标
    int row = index.row();

    DWidget *widget = m_tableView->indexWidget(m_model->index(row, TABLE_COL_QFILES));
    // 多选框对象
    DCheckBox *check = widget->findChild<DCheckBox *>("singCheck");
    // 按钮的状态
    int status = check->checkState();
    // 改变按钮状态
    if (status == 0) {
        check->setCheckState(Qt::CheckState::Checked);
    } else {
        check->setCheckState(Qt::CheckState::Unchecked);
    }
    // 判断全选的状态
    checkBoxAllStatus();
}

// 判断全选按钮
void NatiavTrustDialog::checkBoxAllStatus()
{
    //总行数
    int count = m_model->rowCount();
    //选中状态的map
    QMap<int, int> rowMap;
    for (int i = 0; i < count; i++) {
        DWidget *widget = m_tableView->indexWidget(m_model->index(i, TABLE_COL_QFILES));
        DCheckBox *check = widget->findChild<DCheckBox *>("singCheck");
        //按钮的状态
        int status = check->checkState();
        switch (status) {
        case 0:
            rowMap.insert(i, 0);
            m_model->item(i, TABLE_COL_CHECK_STATUS)->setText("0");
            break;
        case 1:
            break;
        case 2:
            rowMap.insert(i, 1);   //如果是选中状态移除
            m_model->item(i, TABLE_COL_CHECK_STATUS)->setText("1");
            break;
        }
    }
    //打沟的数量
    int tcount = 0;
    //不打沟的数量
    int fcount = 0;
    QMapIterator<int, int> rowMapIterator(rowMap);
    rowMapIterator.toBack();
    while (rowMapIterator.hasPrevious()) {
        int value;
        rowMapIterator.previous();
        value = rowMapIterator.value();
        if (value == 0) {
            fcount++;
        } else if (value == 1) {
            tcount++;
        }
    }

    //根据数量判断  全选状态
    if (tcount == count) {
        m_checkBoxAll->setCheckState(Qt::CheckState::Checked);
    } else if (fcount == count) {
        m_checkBoxAll->setCheckState(Qt::CheckState::Unchecked);
    } else {
        //有一个没有选中  先把全选设为未选择状态   然后吧以前的选中都恢复
        m_checkBoxAll->setCheckState(Qt::CheckState::Unchecked);
        QMap<int, int>::iterator it;
        for (it = rowMap.begin(); it != rowMap.end(); ++it) {
            int row = it.key();
            DWidget *widget = m_tableView->indexWidget(m_model->index(row, TABLE_COL_QFILES));
            //多选框对象
            DCheckBox *check = widget->findChild<DCheckBox *>("singCheck");
            if (it.value() == 0) {
                check->setCheckState(Qt::CheckState::Unchecked);
            } else {
                check->setCheckState(Qt::CheckState::Checked);
            }
        }
    }
}

void NatiavTrustDialog::appendFileInfo(TrustFileInfo info)
{
    //锁
    DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
    //当前行
    int currentRow = m_model->rowCount();
    //文件路径
    QString filePath = info.sFilePath;
    QString fileName = info.sFileName;
    QString addtime = info.sAddTime;
    //设置model的值
    m_model->setItem(currentRow, TABLE_COL_CHECK_STATUS, new QStandardItem("0"));
    m_model->setItem(currentRow, TABLE_COL_FILE_PATH, new QStandardItem(QString("%1").arg(filePath)));
    m_model->setItem(currentRow, TABLE_COL_FILE_NAME, new QStandardItem(QString("%1").arg(fileName)));
    m_model->setItem(currentRow, TABLE_COL_ADD_TIME, new QStandardItem(QString("%1").arg(addtime)));
    //第一列
    DWidget *widget1 = new DWidget;
    QHBoxLayout *Layout = new QHBoxLayout;
    Layout->setContentsMargins(0, 5, 5, 5);
    Layout->setSpacing(3);
    DCheckBox *check = new DCheckBox();
    check->setObjectName("singCheck");
    check->setFixedSize(QSize(20, 20));
    check->setCheckState(Qt::CheckState::Unchecked);
    connect(check, &DCheckBox::clicked, this, &NatiavTrustDialog::tableCheckItem);
    Layout->addWidget(check, 0, Qt::AlignLeft | Qt::AlignVCenter);

    QVBoxLayout *Layout1 = new QVBoxLayout;
    Layout1->setContentsMargins(0, 0, 5, 0);
    Layout1->setSpacing(3);
    DLabel *lable1 = new DLabel(widget1);
    //文件名
    lable1->setObjectName("fileName");
    QFont ft = lable1->font();
    ft.setBold(false);
    lable1->setFont(ft);
    DFontSizeManager::instance()->bind(lable1, DFontSizeManager::T6);
    //得到每个字符的宽度
    QFontMetrics fontWidth(lable1->font());
    //最大宽度 显示的字符
    QString elideNote = fontWidth.elidedText(fileName, Qt::ElideMiddle, TABLE_COL_1_WIDTH - (2  * TABLE_SPACE) - 20);
    lable1->setText(elideNote);
    //设置tooltips
    lable1->setToolTip(fileName);
    Layout1->addWidget(lable1);

    DLabel *lablepath = new DLabel(widget1);
    //文件路径
    lablepath->setObjectName("filePath");
    QFont ft1 = lablepath->font();
    ft1.setBold(false);
    lablepath->setFont(ft1);
    DFontSizeManager::instance()->bind(lablepath, DFontSizeManager::T8);
    //得到每个字符的宽度
    QFontMetrics fontWidth1(lablepath->font());
    QString elideNote1 = fontWidth1.elidedText(filePath, Qt::ElideMiddle, TABLE_COL_1_WIDTH - (2  * TABLE_SPACE) - 20);
    //显示省略好的字符串
    lablepath->setText(elideNote1);
    //设置tooltips
    lablepath->setToolTip(filePath);
    Layout1->addWidget(lablepath);

    Layout->addLayout(Layout1, 0);
    Layout->addStretch(0);
    widget1->setLayout(Layout);

    //第三列
    DWidget *widget2 = new DWidget;
    QHBoxLayout *Layout2 = new QHBoxLayout;
    Layout2->setContentsMargins(TABLE_SPACE, TABLE_SPACE, TABLE_SPACE, TABLE_SPACE);
    Layout2->setSpacing(SPACING);
    DLabel *lable3 = new DLabel();
    //得到每个字符的宽度
    lable3->setObjectName("addTime");
    QFontMetrics fontWidth2(lable3->font());
    QString scanType = fontWidth2.elidedText(addtime, Qt::ElideMiddle, TABLE_COL_2_WIDTH - (2 * TABLE_SPACE));
    //显示省略好的字符串
    lable3->setText(scanType);
    lable3->setToolTip(addtime);
    Layout2->addWidget(lable3);
    widget2->setLayout(Layout2);
    //每列的控件
    m_tableView->setIndexWidget(m_model->index(currentRow, TABLE_COL_QFILES), widget1);
    m_tableView->setIndexWidget(m_model->index(currentRow, TABLE_COL_TIME), widget2);
    // 缓冲作用
    m_model->setItem(currentRow + 1, 0, new QStandardItem);
    m_model->removeRow(currentRow + 1);
    //设置每隔宽度
    m_tableView->setColumnWidth(TABLE_COL_QFILES, TABLE_COL_1_WIDTH);
    m_tableView->setColumnWidth(TABLE_COL_TIME, TABLE_COL_2_WIDTH);
    //隐藏列
    m_tableView->setColumnHidden(TABLE_COL_CHECK_STATUS, true);
    m_tableView->setColumnHidden(TABLE_COL_FILE_PATH, true);
    m_tableView->setColumnHidden(TABLE_COL_FILE_NAME, true);
    m_tableView->setColumnHidden(TABLE_COL_ADD_TIME, true);
}

void NatiavTrustDialog::setCurrentProcess(int nCurrent)
{
    m_proLable->setText(QString("%1/%2").arg(nCurrent).arg(m_total));
}

//自适应
void NatiavTrustDialog::resizeEvent(QResizeEvent *event)
{
    Q_UNUSED(event);
    if (m_model->rowCount() == 0) {
        return;
    } else {
        int height = m_tableView->indexWidget(m_model->index(0, 0))->size().height();
        for (int i = 0; i < m_model->rowCount(); i++) {
            m_tableView->setRowHeight(i, height);
        }
    }
}

//全选
void NatiavTrustDialog::checkAll(int status)
{
    int count = m_model->rowCount();
    for (int i = 0; i < count; i++) {
        DWidget *widget = m_tableView->indexWidget(m_model->index(i, TABLE_COL_QFILES));
        //多选框对象
        DCheckBox *check = widget->findChild<DCheckBox *>("singCheck");
        switch (status) {
        case 0:
            check->setCheckState(Qt::CheckState::Unchecked);
            break;
        case 1:
            //check->setCheckState(Qt::CheckState::PartiallyChecked);
            break;
        case 2:
            check->setCheckState(Qt::CheckState::Checked);
            break;
        }
    }
}

//删除信任
void NatiavTrustDialog::deleteTrust()
{
    //总行数
    int count = m_model->rowCount();
    //选中状态的map
    QMap<int, int> rowMap;
    for (int i = 0; i < count; i++) {
        DWidget *widget = m_tableView->indexWidget(m_model->index(i, TABLE_COL_QFILES));
        //多选框对象
        DCheckBox *check = widget->findChild<DCheckBox *>("singCheck");
        //按钮的状态
        int status = check->checkState();
        switch (status) {
        case 0:
            rowMap.insert(i, 0);
            break;
        case 1:
            break;
        case 2:
            //1是选中状态  要移除
            rowMap.insert(i, 1);
            //文件路径
            QString filepath = m_model->index(i, TABLE_COL_FILE_PATH).data(Qt::DisplayRole).toString();
            //发送  删除信任文件信号
            Q_EMIT requsetRemoveTrustFile(filepath);
            break;
        }
    }
    int key;
    int value;
    //遍历选中状态的map
    QMapIterator<int, int> rowMapIterator(rowMap);
    rowMapIterator.toBack();
    //是否是最后一个
    while (rowMapIterator.hasPrevious()) {
        rowMapIterator.previous();
        key = rowMapIterator.key();
        value = rowMapIterator.value();
        //如果是选中状态
        if (value == 1) {
            //缓存删除
            m_model->removeRow(key);
            m_TrustfileInfo.removeAt(key);
        }
    }

    // 删除完后剩余的总行数
    int count1 = m_model->rowCount();
    m_total = count1;
    setCurrentProcess(count1);
    if (count1 == 0) {
        // 如果没有数据了 全选不会勾选 并且至于灰色
        m_checkBoxAll->setCheckState(Qt::CheckState::Unchecked);
        m_checkBoxAll->setDisabled(true);
    }
}

//table 单选对象
void NatiavTrustDialog::tableCheckItem()
{
    checkBoxAllStatus();
}

//信任文件接受
void NatiavTrustDialog::receptionTrustFile(QList<TrustFileInfo> sInfos)
{
    //展现信任列表
    m_TrustfileInfo = sInfos;
    m_total = m_TrustfileInfo.size();
    m_buffTimer->start(1);
}

// 定时添加缓存
void NatiavTrustDialog::doAddBuffer()
{
    ++ m_curret ;
    int size = m_TrustfileInfo.size();
    if (size == 0) {
        qDebug() << " longing finish";
        // 定时器停止  按钮展现
        m_buffTimer->stop();
        m_PbTrust->setDisabled(false);
        // 全选状态判断
        m_checkBoxAll->setCheckState(Qt::CheckState::Unchecked);
        if (m_total == 0) {
            // 没有隔离数据为不可以操作
            m_checkBoxAll->setDisabled(true);
        } else {
            // 有数据可以操作
            m_checkBoxAll->setDisabled(false);
        }
        // 定时器停止  按钮展现
        return ;
    } else {
        setCurrentProcess(m_curret);
        appendFileInfo(m_TrustfileInfo.at(0));
        m_TrustfileInfo.removeAt(0);
    }
}
