#include "natiavwidget.h"

#include <DFontSizeManager>
#include <DApplicationHelper>
#include <DListView>

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QDebug>
#include <QMessageBox>

DWIDGET_USE_NAMESPACE

#define LAST_CHECK_TIME             "last-scan-time"      //上次检测时间
#define VERSION                     "current-version"      //当前版本
#define VERSION_CONFING             "version.conf"          //版本配置文件
#define MAIN_WIDTH                  650                    //页面宽度
#define TITLE_HEIDTH                130                    //上部分高度
#define CENTER_HEIDTH               300                     //中间高度
#define BUTTOM_HEIDTH               120                     //下面高度
#define DIALOG_WIDTH                380                     //dialog  宽度
#define DIALOG_HEIGHT               145                     //dialog  高度
#define DIALOG_PB_WIDTH             140                     //dialog  按钮宽度
#define DIALOG_PB_HEIGHT            36                      //dialog  按钮高度
#define VDB_INIT_VERSION            "20.8.18.14"           //病毒库初使版本
#define LEFT_MARGINS                30                      //左边距
#define TOP_MARGINS                 0                       //上边距
#define RIGHT_MARGINS               0                       //右边距离
#define BUTTOM_MARGINS              0                       //下边距离
#define TITLE_LEFT_MARGINS          0                       //左边距
#define TITLE_TOP_MARGINS           40                      //上边距
#define TITLE_RIGHT_MARGINS         0                       //右边距离
#define TITLE_BUTTOM_MARGINS        10                      //下边距离
#define SPACING                     5                       //布局间隙

NatiavWidget::NatiavWidget(AntiVirusWork *work, QWidget *parent)
    : DWidget(parent)
    , m_antivirusWork(work)
{
    m_titleLable = new DLabel(this);
    m_timeLable = new DLabel(this);
    //日志页面
    m_log = new NatiavScanLog;

    m_guiHelper = DGuiApplicationHelper::instance();
    //初始化布局
    initGUI();
    // 更新界面
    updateUI();

    // 主题变换
    connect(m_guiHelper, SIGNAL(themeTypeChanged(ColorType)), this, SLOT(themeTypeChange(ColorType)));
    // 连接请求更新病毒查杀首页数据信号
    connect(m_antivirusWork, &AntiVirusWork::notifyUpdateAntiVirusHomeUI, this, &NatiavWidget::updateUI);
    // 连接请求打开病毒库更新失败弹窗信号
    connect(m_antivirusWork, &AntiVirusWork::notifyOpenVdbUpdateFailedDialog, this, &NatiavWidget::showUpdateFail);
}

//展现日志页面
void NatiavWidget::showScanLog()
{
    if (m_log)
        m_log->deleteLater();
    m_log = new NatiavScanLog(this);
    m_log->raise();
    m_log->show();
}

//展现信任区域
void NatiavWidget::showScanTrust()
{
    //如果在版本更新中  提示更新提示
    if (m_isbVdbUpdating) {
        showUpdateTip();
    } else {
        m_trust = new NatiavTrustDialog(this);
        if (m_trustFlag == false) {
            //如果不再查询信任文件中   发送查询信任文件
            Q_EMIT onRequestSelectTrustFile();
            m_trustFlag = true;
        }
        //查询信任文件结果给 信任页面
        connect(this, &NatiavWidget::onNotifyQueryTrustFile, m_trust, &NatiavTrustDialog::receptionTrustFile);
        //删除信任文件
        connect(m_trust, &NatiavTrustDialog::requsetRemoveTrustFile, this, &NatiavWidget::doRemoveTrustFile);
        m_trust->raise();
        m_trust->show();
    }
}

//隔离区也 弹框
void NatiavWidget::showIsolation()
{
    if (m_isbVdbUpdating) {
        //如果在版本更新中  提示更新提示
        showUpdateTip();
    } else {
        m_isolation = new NatiavIsolationDilog(this);
        if (m_isolationFlag == false) {
            //如果不再查询隔离文件中   发送查询隔离文件
            Q_EMIT onRequstSelectIsolationFile();
            m_isolationFlag = true;
        }
        //查询隔离文件结果给 隔离页面显示
        connect(this, &NatiavWidget::onNotifyQueryIsolationFile, m_isolation, &NatiavIsolationDilog::receptionIsolationFile);
        //删除隔离文件
        connect(m_isolation, SIGNAL(requestRemoveIsolationFile(QString)), this, SLOT(doRemoveIsolationFile(QString)));
        m_isolation->raise();
        m_isolation->show();
    }
}

//开始扫描弹窗
void NatiavWidget::showCustomScan()
{
    //通过 fileDialog控件得到扫描路径
    QString filepath = QFileDialog::getExistingDirectory(this, "路径", "./");
    qDebug() << "filepath==" << filepath;
    //获取的文件路径不为空  或者不掉空格也不为空  才会触发
    if ((!filepath.isEmpty()) || (!filepath.trimmed().isNull())) {
        Q_EMIT onNotifyShowScan(2, filepath, m_sLocalVdbVersion);
    }
}

//设置标题
void NatiavWidget::setTitleName(QString sTitle)
{
    m_titleLable->setText(QString("%1").arg(sTitle));
}

//提示弹框
void NatiavWidget::showUpdateTip()
{
    DDialog *dialog = new DDialog;
    dialog->setMinimumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    dialog->setMaximumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    dialog->setIcon(QIcon::fromTheme("deepin-defender"));
    //病毒库更新中，请稍后操作
    dialog->setMessage(tr("Updating the virus database, please scan later"));
    dialog->setWordWrapMessage(true);

    // 确定 推荐   MXZ 按钮颜色不一样 37176 / https://pms.uniontech.com/zentao/bug-view-37167.html
    dialog->addButton(tr("OK"));
    connect(dialog, &DDialog::buttonClicked, [ = ](int index, QString) {
        switch (index) {
        case 0:
            dialog->close();
            break;
        default:
            break;
        }
    });
    dialog->exec();
}
//首页病毒查杀中的提示弹框
void NatiavWidget::showHomepageScanTip()
{
    DDialog *dialog = new DDialog;
    dialog->setMinimumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    dialog->setMaximumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    dialog->setIcon(QIcon::fromTheme("dcc_login_tip"));
    //首页体检正在执行快速查杀，请稍候
    dialog->setMessage(tr("A scanning task is running"));
    dialog->setWordWrapMessage(true);

    dialog->addButton(tr("OK"));
    connect(dialog, &DDialog::buttonClicked, [ = ](int index, QString) {
        switch (index) {
        case 0:
            dialog->close();
            break;
        default:
            break;
        }
    });
    dialog->exec();
}

//更新失败
void NatiavWidget::showUpdateFail()
{
    m_updateFialFlag = true;
    DDialog *dialog = new DDialog;
    dialog->setMinimumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    dialog->setMaximumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    dialog->setIcon(QIcon::fromTheme("dcc_login_tip"));
    //网络异常，请检测网络后重新更新
    dialog->setMessage(tr("Please check your network and update again"));
    dialog->setWordWrapMessage(true);

    // 确定 推荐 MXZ 按钮颜色不一样 37176 / https://pms.uniontech.com/zentao/bug-view-37167.html
    dialog->addButton(tr("OK"), true, DDialog::ButtonType::ButtonRecommend);
    connect(dialog, &DDialog::buttonClicked, [ = ](int index, QString) {
        switch (index) {
        case 0:
            m_updateFialFlag = false;
            dialog->close();
            break;
        default:
            break;
        }
    });
    connect(dialog, &DDialog::closed, [ = ] {
        m_updateFialFlag = false;
        dialog->close();
    });

    dialog->exec();
}

//初始化界面
void NatiavWidget::initGUI()
{
    //垂直布局
    QVBoxLayout *m_mainLayout = new QVBoxLayout;

    m_mainLayout->setSpacing(SPACING);
    //四周边距
    m_mainLayout->setContentsMargins(LEFT_MARGINS, TOP_MARGINS, RIGHT_MARGINS, BUTTOM_MARGINS);
    //m_mainLayout->addSpacing(34);  //空格
    //上布局
    DWidget *titleWidget = new DWidget;
    titleWidget->setFixedHeight(TITLE_HEIDTH);
    titleWidget->setMinimumWidth(MAIN_WIDTH);
    titleWidget->setLayout(initTitle());
    m_mainLayout->addWidget(titleWidget, 0, Qt::AlignLeft);

    //三种查杀  全盘  快速  自定义
    DWidget *ConterWidget = new DWidget;
    ConterWidget->setFixedHeight(CENTER_HEIDTH);
    ConterWidget->setMinimumWidth(MAIN_WIDTH);
    //中间布局
    ConterWidget->setLayout(initCenter());
    m_mainLayout->addStretch();
    m_mainLayout->addWidget(ConterWidget, 0, Qt::AlignCenter | Qt::AlignLeft);

    //日志 信任区 隔离区
    DWidget *bottomWidget = new DWidget;
    bottomWidget->setFixedHeight(BUTTOM_HEIDTH);
    bottomWidget->setMinimumWidth(MAIN_WIDTH);
    //下面布局
    bottomWidget->setLayout(initBottom());
    m_mainLayout->addStretch();
    m_mainLayout->addWidget(bottomWidget, 0, Qt::AlignBottom | Qt::AlignLeft);
    //设置布局
    setLayout(m_mainLayout);
}

//中间的布局
QHBoxLayout *NatiavWidget::initCenter()
{
    //按钮水平布局
    QHBoxLayout *m_blayout = new QHBoxLayout;
    //四周边距
    m_blayout->setContentsMargins(0, 50, 0, 30);
    m_blayout->setSpacing(SPACING);

    //listview
    DListView *CnterView = new DListView;
    QStandardItemModel *buttonModel = new QStandardItemModel(CnterView);
    CnterView->setModel(buttonModel);
    //形状
    CnterView->setFrameShape(QFrame::Shape::NoFrame);
    //不能编辑触发
    CnterView->setEditTriggers(QListView::NoEditTriggers);
    CnterView->setResizeMode(QListView::Adjust);
    CnterView->setViewMode(QListView::IconMode);
    //不可以拖拽
    CnterView->setDragEnabled(false);
    CnterView->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    //显示图表大小
    CnterView->setIconSize(QSize(96, 96));
    CnterView->setSpacing(10);
    //每个对象大小
    CnterView->setItemSize(QSize(170, 170));
    CnterView->clearSelection();
    CnterView->clearFocus();
    CnterView->clearMask();
    CnterView->setSelectionMode(QAbstractItemView::NoSelection);
    CnterView->setTextElideMode(Qt::ElideRight);
    CnterView->setBackgroundType(DStyledItemDelegate::BackgroundType(DStyledItemDelegate::RoundedBackground | DStyledItemDelegate::NoNormalState));
    //退出模态页面(子页面)后，清除选中效果
    connect(CnterView, &QAbstractItemView::clicked, [ = ] {
        //失去焦点
        CnterView->clearFocus();
    });
    //按钮  中间标题
    QStringList titles;
    titles << tr("Full Scan") << tr("Quick Scan") << tr("Custom Scan");
    //按钮图标
    QStringList icons;
    icons << "dcc_antiav_all1" << "dcc_antiav_quick1" << "dcc_antiav_customer1";

    for (int i =  0; i < titles.size(); ++i) {
        QStandardItem *item = new QStandardItem;
        item->setIcon(QIcon::fromTheme(icons[i]));
        item->setText(titles[i]);
        item->setToolTip(titles[i]);
        item->setSizeHint(QSize(60, 80));
        buttonModel->appendRow(item);
    }
    m_blayout->addWidget(CnterView);

    //日志  信任区域   隔离区域的点击信号绑定
    connect(CnterView, &QListView::clicked, this, &NatiavWidget::displayCenteridget);
    return m_blayout;
}

//下面按钮布局
QHBoxLayout *NatiavWidget::initBottom()
{
    //按钮水平布局
    QHBoxLayout *m_blayout = new QHBoxLayout;
    m_blayout->setContentsMargins(0, 0, 0, 10);
    m_blayout->setSpacing(0);

    //listview
    buttonView = new DListView;
    buttonModel = new QStandardItemModel(buttonView);
    buttonView->setModel(buttonModel);
    //形状
    buttonView->setFrameShape(QFrame::Shape::NoFrame);
    //不能编辑触发
    buttonView->setEditTriggers(QListView::NoEditTriggers);
    buttonView->setResizeMode(QListView::Adjust);
    buttonView->setViewMode(QListView::IconMode);
    //不可以拖拽
    buttonView->setDragEnabled(false);
    buttonView->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    buttonView->setIconSize(QSize(38, 38));
    buttonView->setContentsMargins(0, TOP_MARGINS, RIGHT_MARGINS, BUTTOM_MARGINS);
    buttonView->setSpacing(2);
    buttonView->setItemSize(QSize(100, 85));
    buttonView->clearSelection();
    buttonView->clearFocus();
    buttonView->clearMask();
    buttonView->setMinimumWidth(350);
    buttonView->setMaximumWidth(350);
    buttonView->setSelectionMode(QAbstractItemView::NoSelection);
    buttonView->setTextElideMode(Qt::ElideRight);
    buttonView->setBackgroundType(DStyledItemDelegate::BackgroundType(DStyledItemDelegate::RoundedBackground | DStyledItemDelegate::NoNormalState));
    //退出模态页面(子页面)后，清除选中效果
    connect(buttonView, &QAbstractItemView::clicked, [ = ] {
        //失去焦点
        buttonView->clearFocus();
    });
    //按钮标题
    QStringList titles;
    titles << tr("Scan Logs") << tr("Quarantine") << tr("Whitelist") ;
    //按钮图标
    QStringList icons;

    // 获得主题类型
    int type = m_guiHelper->themeType();
    if (type == 1) {
        icons << "dcc_antiav_searching" << "dcc_antiav_Quarantine" << "dcc_natiav_Trust";
    } else {
        icons << "dcc_natiav_searching_dark" << "dcc_antiav_Quarantine_dark" << "dcc_natiav_Trust_dark";
    }
    for (int i =  0; i < titles.size(); ++i) {
        QStandardItem *item = new QStandardItem;
        item->setIcon(QIcon::fromTheme(icons[i]));
        item->setText(titles[i]);
        item->setToolTip(titles[i]);
        item->setSizeHint(QSize(100, 85));
        buttonModel->appendRow(item);
    }
    m_blayout->addWidget(buttonView);

    //日志  信任区域   隔离区域的点击信号绑定
    connect(buttonView, &QListView::clicked, this, &NatiavWidget::displayButtomWidget);

    //右侧病毒库的版本
    DWidget *versionWidget = new DWidget;

    QVBoxLayout *vLayout1 = new QVBoxLayout(this);
    QSpacerItem *verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);
    vLayout1->addItem(verticalSpacer);

    QHBoxLayout *hLayout1 = new QHBoxLayout();
    hLayout1->addStretch(1);
    DLabel *lable = new DLabel(this);
    //病毒库
    lable->setText(tr("Virus database"));
    m_version = new DLabel(this);
    //版本
    m_version->setText("25.00.00.15");
    QFont ft = m_version->font();
    ft.setBold(false);
    m_version->setFont(ft);
    lable->setFont(ft);
    DFontSizeManager::instance()->bind(lable, DFontSizeManager::T8);
    DFontSizeManager::instance()->bind(m_version, DFontSizeManager::T8);
    lable->setAlignment(Qt::AlignRight | Qt::AlignTrailing | Qt::AlignVCenter);
    m_version->setAlignment(Qt::AlignRight | Qt::AlignTrailing | Qt::AlignVCenter);
    hLayout1->addWidget(lable, 0, Qt::AlignRight);
    hLayout1->addWidget(m_version, 0, Qt::AlignRight);
    //加载 控件
    m_spinner = new DSpinner(this);
    m_spinner->setFixedSize(QSize(15, 15));
    m_spinner->hide();
    hLayout1->addWidget(m_spinner, 0, Qt::AlignRight);

    m_versionStatus = new DLabel(this);
    //最新
    m_versionStatus->setText(tr("Up to date"));
    QFont ft1 = m_versionStatus->font();
    ft1.setBold(false);
    m_versionStatus->setFont(ft1);
    DFontSizeManager::instance()->bind(m_versionStatus, DFontSizeManager::T8);
    hLayout1->addWidget(m_versionStatus, 0, Qt::AlignRight);
    //有更新   链接按钮
    m_pbUpdate = new DCommandLinkButton(tr("Updates available"));
    hLayout1->addWidget(m_pbUpdate, 0, Qt::AlignRight);
    //展现更新弹框
//    connect(m_pbUpdate, &DCommandLinkButton::clicked, this, &NatiavWidget::showBackground);
    connect(m_pbUpdate, &DCommandLinkButton::clicked, m_antivirusWork, &AntiVirusWork::notifyOpenVdbUpdateDialog);
    vLayout1->addLayout(hLayout1);
    versionWidget->setLayout(vLayout1);

    m_blayout->addWidget(versionWidget);
    return m_blayout;
}

//标题布局
QVBoxLayout *NatiavWidget::initTitle()
{
    QVBoxLayout *m_blayout = new QVBoxLayout;
    m_blayout->setContentsMargins(TITLE_LEFT_MARGINS, TITLE_TOP_MARGINS, TITLE_RIGHT_MARGINS, TITLE_BUTTOM_MARGINS);
    m_blayout->setSpacing(SPACING);
    //标题   定期杀毒，保障安全
    m_titleLable->setText(tr("Scan virus regularly to remove any threats"));
    QFont ft = m_titleLable->font();
    ft.setBold(true);
    m_titleLable->setFont(ft);
    DFontSizeManager::instance()->bind(m_titleLable, DFontSizeManager::T5);
    //上次更新的时间
    QString stTime = "2020/03/24 14:10" ;
    m_timeLable->setText(tr("The virus database is up to date. Last update: %1").arg(stTime));
    QFont ft1 = m_timeLable->font();
    ft1.setBold(false);
    m_timeLable->setFont(ft1);
    DFontSizeManager::instance()->bind(m_timeLable, DFontSizeManager::T7);
    QFrame  *line = new QFrame;
    //横线
    line->setObjectName(QStringLiteral("line"));
    line->setGeometry(QRect(80, 100, 500, 3));
    line->setFrameShape(QFrame::HLine);
    line->setFrameShadow(QFrame::Sunken);

    m_blayout->addWidget(m_titleLable);
    m_blayout->addWidget(m_timeLable);
    m_blayout->addWidget(line);

    return m_blayout;
}

//下面布局按钮点击
void NatiavWidget::displayButtomWidget(QModelIndex nIndex)
{
    switch (static_cast<NatiavAreaType>(nIndex.row())) {
    case killinglog:
        //展现日志
        showScanLog();
        break;
    case IsolationArea:
        //展现隔离
        showIsolation();
        break;
    case TrustArea:
        //展现信任
        showScanTrust();
        break;
    }
}

//中间布局点击
void NatiavWidget::displayCenteridget(QModelIndex nIndex)
{
    //快速查询路径
    QString quick = "/bin*/sbin*/usr/bin*/usr/sbin*/usr/local*/tmp";
    if (m_isbVdbUpdating) {
        showUpdateTip();
    } else if (isHomePageScaning()) {
        showHomepageScanTip();
    } else {
        switch (static_cast<NatiavScanType>(nIndex.row())) {
        case AllKilling:
            //去moudule 跳转的动作
            Q_EMIT onNotifyShowScan(0, "/", m_sLocalVdbVersion);
            break;
        case QuickKilling:
            //去moudule 跳转的动作
            Q_EMIT onNotifyShowScan(1, quick, m_sLocalVdbVersion);
            break;
        case CustomKilling:
            showCustomScan();
            break;
        default:
            break;
        }
        // 更新上次扫描时间
        m_antivirusWork->updateLastScanningTime();
    }
}

bool NatiavWidget::isHomePageScaning()
{
    return m_antivirusWork->isScanning();
}

//查询出的信任文件信息    转发给信任弹窗页面
void NatiavWidget::doQueryTrustFile(QList<TrustFileInfo> sInfos)
{
    Q_EMIT onNotifyQueryTrustFile(sInfos);
    m_trustFlag = false;
}

//查询隔离文件
void NatiavWidget::doQueryIsolationFile(QList<TrustFileInfo> sPaths)
{
    Q_EMIT onNotifyQueryIsolationFile(sPaths);
    m_isolationFlag = false;
}

//移除隔离文件
void NatiavWidget::doRemoveIsolationFile(QString sPath)
{
    Q_EMIT onRemoveIsolationFile(false, sPath);
}

//移除信任文件
void NatiavWidget::doRemoveTrustFile(QString sPath)
{
    Q_EMIT onRemoveTrustFile(false, sPath);
}

// 主题的改变
void NatiavWidget::themeTypeChange(ColorType themeType)
{
    int type;
    switch (themeType) {
    case UnknownType:
        type = 0;
        break;
    case LightType:
        type = 1;
        break;
    case DarkType:
        type = 2 ;
        break;
    }
    QStringList icons;
    // 浅色背景
    if (type == 1) {
        icons << "dcc_antiav_searching" << "dcc_antiav_Quarantine" << "dcc_natiav_Trust";
    } else {
        icons << "dcc_natiav_searching_dark" << "dcc_antiav_Quarantine_dark" << "dcc_natiav_Trust_dark";
    }
    for (int i = 0; i < buttonModel->rowCount(); i++) {
        QStandardItem *item = buttonModel->item(i);
        item->setIcon(QIcon::fromTheme(icons[i]));
    }

}

void NatiavWidget::updateUI()
{
    // 获取是否处于病毒更新状态
    m_isbVdbUpdating = m_antivirusWork->vdbUpdating();
    // 获取服务器病毒库版本
    m_sSrvVdbVersion = m_antivirusWork->srvVdbVersion();
    // 获取本地病毒库版本
    m_sLocalVdbVersion = m_antivirusWork->localVdbVersion();
    // 判断本地病毒库是否最新
    bool isbLocalVdbLatest = m_sSrvVdbVersion == m_sLocalVdbVersion;
    // 获取上次扫描时间
    QString sLastScanningTime = m_antivirusWork->lastScanningTime();

    // 更新顶部扫描提示
    if (isbLocalVdbLatest) {
        if (sLastScanningTime == "-1") {
            m_timeLable->setText(tr("The virus database is up to date. Start a scan please."));
        } else {
            //如果是显示   病毒库已经最新  上次查杀时间
            m_timeLable->setText(tr("The virus database is up to date. Last scan: %1").arg(sLastScanningTime));
        }
    } else {
        if (sLastScanningTime == "-1") {
            m_timeLable->setText(tr("The virus database is not up to date. Update first and start a scan please."));
        } else {
            //病毒库不是最新，上次更新时间
            m_timeLable->setText(tr("The virus database is not up to date. Last scan: %1").arg(sLastScanningTime));
        }
    }

    // 更新底部扫描提示
    m_version->setText(m_sLocalVdbVersion);
    m_version->show();
    m_versionStatus->hide();
    m_pbUpdate->hide();
    m_spinner->stop();
    m_spinner->hide();

    // 根据病毒库是否最新，设置底部控件隐藏与显示
    if (isbLocalVdbLatest) {
        m_versionStatus->show();
        m_pbUpdate->hide();
    } else {
        m_versionStatus->hide();
        m_pbUpdate->show();
    }

    // 判断是否正在更新本地病毒库
    if (m_isbVdbUpdating) {
        //正在更新中
        m_versionStatus->hide();
        m_pbUpdate->hide();
        m_spinner->start();
        m_spinner->show();
    }
}
