#ifndef NATIAVWIDGET_H
#define NATIAVWIDGET_H

#include "scanwidget.h"
#include "natiavscanlog.h"
#include "natiavtrustdialog.h"
#include "natiavisolationdilog.h"
#include "updatedialog.h"
#include "src/window/modules/common/common.h"
#include "antiviruswork.h"

#include <QWidget>
#include <QLabel>
#include <QGSettings>
#include <QFileDialog>

#include <DFrame>
#include <DSpinner>
#include <DLabel>
#include <DCommandLinkButton>
#include <DGuiApplicationHelper>

QT_BEGIN_NAMESPACE
class QVBoxLayout;
class QHBoxLayout;
class QLabel;
class QPushButton;
QT_END_NAMESPACE
DWIDGET_USE_NAMESPACE


//const int leftSpace = 30;

//病毒查杀类型
enum NatiavScanType {
    Default = -1,
    AllKilling,                  //全盘查杀
    QuickKilling,               //快速查杀
    CustomKilling               //自定义查杀
};

//病毒查杀的按钮点击类型
enum NatiavAreaType {
    killinglog = 0,                 //查杀日志
    IsolationArea,                  //隔离区域
    TrustArea                       //信任区
};


/*
@classname:(NatiavWidget)
@author：(mxz)
@discptions:(病毒查杀主页面)
@hirts:(继承)
@datatime:(04-28)
*/
class NatiavWidget : public DWidget
{
    Q_OBJECT
public:
    explicit NatiavWidget(AntiVirusWork *work, QWidget *parent = nullptr);

    //展现日志页面
    void showScanLog();
    //展现信任
    void showScanTrust();
    //展现隔离
    void showIsolation();
    //设置标题
    void setTitleName(QString sTitle);
    //展现自定义扫描
    void showCustomScan();

    //提示弹框
    void showUpdateTip();
    //首页病毒查杀中的提示弹框
    void showHomepageScanTip();

    //更新失败
    void showUpdateFail();
private:
    //初始化界面
    void initGUI();
    //中间布局
    QHBoxLayout *initCenter();
    //下面按钮布局
    QHBoxLayout *initBottom();
    //标题布局
    QVBoxLayout *initTitle();
    //中间布局的三种模式触发
    void displayButtomWidget(QModelIndex nIndex);
    //下面  日志  信任  隔离 触发
    void displayCenteridget(QModelIndex nIndex);

    // 首页是否在扫描中
    bool isHomePageScaning();
Q_SIGNALS:
    //请求展现扫描子页面    类型 （0 全盘 ，1快速 ，2自定义 ）
    void onNotifyShowScan(int ntype, QString sPath, QString sVersion);
    //查询信任文件
    void onRequestSelectTrustFile();
    //查询隔离文件
    void onRequstSelectIsolationFile();
    //查询出的信任文件   文件内容会给信任区域展现
    void onNotifyQueryTrustFile(QList<TrustFileInfo> sInfos);
    //查询出隔离文件    文件内容会给隔离区页面展现
    void onNotifyQueryIsolationFile(QList<TrustFileInfo> sPaths);
    //  添加 /移除  隔离文件
    void onRemoveIsolationFile(bool isbFlag, QString sPath);
    //  添加 /移除  隔离文件
    void onRemoveTrustFile(bool isbFlag, QString sPath);

public slots:
    //信任文件查询的结果
    void doQueryTrustFile(QList<TrustFileInfo> sPaths);
    //隔离文件查询结果
    void doQueryIsolationFile(QList<TrustFileInfo> sPaths);
    //移除隔离文件
    void doRemoveIsolationFile(QString sPath);
    //移除信任文件
    void doRemoveTrustFile(QString sPath);
    // 主题类型改变
    void themeTypeChange(ColorType themeType);

    // 更新界面
    void updateUI();

private:
    ScanWidget                  *m_widget = nullptr;            //扫描页面
    NatiavScanLog               *m_log = nullptr;               //查询日志页面
    NatiavTrustDialog           *m_trust = nullptr;             //信任区域页面
    NatiavIsolationDilog        *m_isolation = nullptr;         //隔离区域页面
    DLabel                      *m_titleLable = nullptr;        //标题
    DLabel                      *m_timeLable = nullptr;         //更新时间
    DLabel                      *m_version = nullptr;           //病毒库版本
    DLabel                      *m_versionStatus = nullptr;     //病毒库版本状态

    DCommandLinkButton          *m_pbUpdate = nullptr;          //更新按钮
    bool                        m_trustFlag = false;            //是否在查询的标识
    bool                        m_isolationFlag = false;
    QTimer                      *m_t1 = nullptr;
    bool                        m_isbVdbUpdating = false;       //正在更新
    QString                     m_sLocalVdbVersion;             //本地病毒库版本
    QString                     m_sSrvVdbVersion;               //服务器病毒库版本
    DSpinner                    *m_spinner = nullptr;           //加载控件

    UpdateDialog                *m_update = nullptr;
    DListView                   *buttonView = nullptr;          // 下方三个的listview
    QStandardItemModel          *buttonModel = nullptr;
    DGuiApplicationHelper       *m_guiHelper = nullptr;         // 方便得到系统主题
    bool                        m_updateFialFlag = false;       // 更新是失败的标志

    AntiVirusWork               *m_antivirusWork = nullptr;

};

#endif // NATIAVWIDGET_H
