﻿#include "scanwidget.h"
#include "csingleteon.h"
#include <QDebug>

#include <DFontSizeManager>
#include <DApplicationHelper>
#include <DListView>
#include <QHeaderView>
#include <QDebug>
#include <QMenu>
#include <DSuggestButton>
#include <QHeaderView>
#include <QDir>
#include <QGraphicsOpacityEffect>
#include <DDialog>


DWIDGET_USE_NAMESPACE

#define TABLE_SPACE                   5          // 表栏两边间距
#define TITLE_ICON_WIDTH              96         // 左上角图标的款
#define TITLE_ICON_HEIGHT             96         // 左上角扫描类型图标 高度
#define TABLEVIEW_ROW_HEIGHT          50         // tableview 每列高度
#define BUTTOM_PB_WIDTH               140        // 下布局按钮宽度
#define BUTTOM_PB_HEIGHT              36         // 下布局按钮高度
#define OPERATION_ICON_WIDTH          20         // 信任  隔离 修复 图标 宽度
#define OPERATION_ICON_HEIGHT         20         // 信任修复隔离 图标高度
#define ACTION_ICON_WIDTH             12         // 处理方式图标 宽度
#define ACTION_ICON_HEIGHT            12         // 处理方式图标 高度

#define TABLE_COL_RISKS               0          // table表 第二列 风险项
#define TABLE_COL_STATUS              1          // table表 第三列 状态   处理方式
#define TABLE_COL_OPEARTION           2          // table表 处理方式

#define TABLE_COL_CHECK_STATUS        3          // table表 第七列 选择状态
#define TABLE_COL_INFO_PATH           4          // table表 第八列 回馈信息路径
#define TABLE_COL_INFO_DETAIL         5          // table表 第九列 回馈信息详情
#define TABLE_COL_ACTION_STATUS       6          // table表 第十列 处理状态
#define TABLE_COL_OPERATION_FLAG      7          // table表 第十一列 操作状态
#define TABLE_COL_OPERATION_TYPE      8          // table表 第十二列 操作类型
#define TABLE_COL_OPERATING           9          // TABLE   当前行是否在操作中

#define TABLE_COL_1_WIDTH             430        // table表 第二列 宽度
#define TABLE_COL_2_WIDTH             110        // table表 第三列 宽度
#define TABLE_COL_3_WIDTH             135        // table表 第四列 宽度

#define MAIN_LEFT_MARGINS             30         // 主左边距
#define MAIN_TOP_MARGINS              0          // 主上边距
#define MAIN_RIGHT_MARGINS            30         // 主右边距离
#define MAIN_BUTTOM_MARGINS           10         // 主下边距离
#define TOP_LEFT_MARGINS              0          // 标题左边距
#define TOP_TOP_MARGINS               15         // 标题上边距
#define TOP_RIGHT_MARGINS             0          // 标题右边距离
#define TOP_BUTTOM_MARGINS            0         // 标题下边距离
#define SPACING                       5          // 布局间隙

#define PAGE_EMPTY                    0          // 没有异常  中间展现无异常图片
#define PAGE_TABLE                    1          // 有异常  中间展现表格

#define DIALOG_WIDTH                380                     //dialog  宽度
#define DIALOG_HEIGHT               145                     //dialog  高度
#define DIALOG_PB_WIDTH             140                     //dialog  按钮宽度
#define DIALOG_PB_HEIGHT            36                      //dialog  按钮高度

ScanWidget::ScanWidget(AntiVirusWork *work, QWidget *parent)
    : DWidget(parent)
    , m_antiViruswork(work)
{
    installEventFilter(this);
    m_progressBar = new QProgressBar(this);
    m_checkAll = new DCheckBox(this);
    m_guiHelper = DGuiApplicationHelper::instance();
    initGUI();

    // 主题变换 改变部分图标颜色
    connect(m_guiHelper, SIGNAL(themeTypeChanged(ColorType)), this, SLOT(themeTypeChange(ColorType)));

    m_stackwidget->setCurrentIndex(PAGE_TABLE);
    m_checkAll->setCheckState(Qt::CheckState::Unchecked);

    //***********按钮的信号链接***********
    //暂停 继续 按钮点击
    connect(m_pbType, &DPushButton::clicked, this, &ScanWidget::doTypeButtonClick);

    // 完成按钮
    connect(m_pbDone, &DSuggestButton::clicked, this, &ScanWidget::doneButtonClick);

    // 结束  重新扫描
    connect(m_pbFinish, &DPushButton::clicked, this, &ScanWidget::doScanFinishClick);
    connect(m_pbRescan, &DPushButton::clicked, this, &ScanWidget::doReScan);

    //全选
    connect(m_checkAll, &DCheckBox::stateChanged, this, &ScanWidget::doCheckAllClick);

    //拖动表头的列
    connect(m_tableView->horizontalHeader(), &QHeaderView::sectionResized, this, &ScanWidget::updateTableForWidth);
    connect(m_tableView, &DTableView::clicked, this, &ScanWidget::currentRowClicked);

    // 执行按钮
    connect(m_exec, &DSuggestButton::clicked, this, &ScanWidget::execButtonClick);

    //点击表头的列
    connect(m_tableView->horizontalHeader(), &QHeaderView::sectionClicked, this, [ = ](int index) {
        // 根据不同的点击列  对应的列进行排序
        if (index == TABLE_COL_RISKS) {
            // 第二列排序
            m_tableView->sortByColumn(TABLE_COL_INFO_PATH);
        } else if (index == TABLE_COL_STATUS) {
            // 第三列排序
            m_tableView->sortByColumn(TABLE_COL_ACTION_STATUS);
        } else {
            return ;
        }
    });

    // 全选和重新扫描失去焦点
    m_checkAll->setFocusPolicy(Qt::NoFocus);
    m_pbRescan->setFocusPolicy(Qt::NoFocus);
}

ScanWidget::~ScanWidget()
{
    // 出发信号让扫描停止
    Q_EMIT onScanStatus(DissectionState::Stop);
}

//设置左上角的图标
void ScanWidget::setIconType(int nType)
{
    if (nType == 0) {
        m_iconType->setPixmap(QIcon::fromTheme("dcc_antiav_all1").pixmap(QSize(TITLE_ICON_WIDTH, TITLE_ICON_HEIGHT)));
    } else if (nType == 1) {
        m_iconType->setPixmap(QIcon::fromTheme("dcc_antiav_quick1").pixmap(QSize(TITLE_ICON_WIDTH, TITLE_ICON_HEIGHT)));
    } else if (nType == 2) {
        m_iconType->setPixmap(QIcon::fromTheme("dcc_antiav_customer1").pixmap(QSize(TITLE_ICON_WIDTH, TITLE_ICON_HEIGHT)));
    }
}

//设置类型  从而改变左上角图标和标题
void ScanWidget::setType(int nType, QString sPath, QString sVersion)
{
    switch (nType) {
    case 0:
        m_currentType = 0;
        //全盘扫描中
        m_titleLabel->setText(tr("Full scanning"));
        m_currentScanType = tr("Full Scan");
        break;
    case 1:
        m_currentType = 1;
        //快速扫描中
        m_titleLabel->setText(tr("Quick scanning"));
        m_currentScanType = tr("Quick Scan");
        break;
    case 2:
        //自定义扫描中
        m_currentType = 2;
        m_titleLabel->setText(tr("Custom scanning"));
        m_currentScanType = tr("Custom Scan");
        break;
    }
    //展现第一个页面
    m_stackwidget->setCurrentIndex(PAGE_TABLE);

    m_currentScanName = m_titleLabel->text();
    m_scanPathInfo = sPath;
    //总行数
    int count = m_model->rowCount();
    for (int i = count; i >= 0; i--) {
        //清空上次扫描的数据
        m_model->removeRow(i);
    }
    //病毒库版本
    m_currentVersion = sVersion;

    // 扫描前准备
    preScan();

    QString time1 = "00:00:00";
    m_Santime->setText(tr("Time elapsed %1").arg(time1));

    m_checkAll->setDisabled(true);
}

// 扫描前准备
void ScanWidget::preScan()
{
    // 修复的下拉框隐藏 不可以用 内容是修复
    m_combobox->hide();
    m_exec->hide();
    m_combobox->setDisabled(false);
    m_exec->setDisabled(false);
    m_combobox->setCurrentIndex(0);

    //暂停  继续按钮  展现
    m_pbType->show();
    // 为了每次点击进去都是 停止和暂停
    m_pbRescan->hide();
    m_pbFinish->show();

    m_flagPause = true;
    m_pbType->setText(tr("Pause"));
    // 完成按钮隐藏
    m_pbDone->hide();

    //扫描总数
    m_totalScan = 0;
    //异常总数
    m_totalError = 0;
    m_countTime = 0;
    //扫描路径清空
    m_scanPath->setText("");
    setScanNum(m_totalScan);
    setScanErrorNum(m_totalError);
    //开始扫描计时
    startTimer();
    //设置左上角图标
    setIconType(m_currentType);
    //精度条清空
    m_progressBar->setValue(0);
    m_progressBar->show();

    // 创建日志的头文件标题 https://pms.uniontech.com/zentao/bug-view-37299.html
    createLogTitle();
    writeScanDatalog();

}

//设置扫描路径
void ScanWidget::setScanPath(QString sPath)
{
    // 设置扫描路径  如果长了中间省略
    QFontMetrics fontWidth(m_scanPath->font());
    QString elideNote = fontWidth.elidedText(sPath, Qt::ElideMiddle, 550);
    // 最大宽度显示的文字
    m_scanPath->setText(QString("%1").arg(elideNote));
    m_scanPath->setToolTip(QString("%1").arg(sPath));
}

//设置扫描异常数据
void ScanWidget::setScanErrorNum(int nVal)
{
    //发现异常
    m_errNum->setText(tr("Threats: %1").arg(QString::number(nVal)));
}

//设置扫描个数
void ScanWidget::setScanNum(int nValue)
{
    //已经扫描项目
    m_scanNum->setText(tr("Scanned: %1").arg(QString::number(nValue)));
}

//设置进度条
void ScanWidget::setProgressValue(double dVal)
{
    double val1 = dVal * 1000000;
    int current = int(val1);
   // qDebug() << "计算前进度值 = " << dVal << "; 计算后的进度值= " << val1 << ", current=" << current;
    m_progressBar->setValue(current);
}

//扫描时间定时器
void ScanWidget::startTimer()
{
    m_t1 = new QTimer();
    connect(m_t1, SIGNAL(timeout()), this, SLOT(updateTime()));
    m_t1->start(1000);
}

//下拉框全选修复  隔离  信任的操作
void ScanWidget::allOPeration(Itemtype type)
{
    QList<QString> list;
    m_combobox->setDisabled(true);
    m_exec->setDisabled(true);
    //本次操作的数据
    m_operationCount = 0;
    m_operationTime = 0;
    //2.得到展现出来所有的路径形象
    //总行数
    int count = m_model->rowCount();
    for (int i = 0; i < count; i++) {
        //是否被操作
        QString operaFlag = m_model->index(i, TABLE_COL_OPERATION_FLAG).data(Qt::DisplayRole).toString();
        if (operaFlag == "false") {
            //没有操作
            DWidget *widget = m_tableView->indexWidget(m_model->index(i, TABLE_COL_RISKS));
            //复选框
            DCheckBox *check = widget->findChild<DCheckBox *>("singleCheck");
            //按钮的状态
            int status = check->checkState();
            //如果是选中状态  发送处理信号
            switch (status) {
            case 0:
                break;
            case 1:
                break;
            case 2:
                //文件路径
                QString  filePath = m_model->index(i, TABLE_COL_INFO_PATH).data(Qt::DisplayRole).toString();
                list.append(filePath);
                m_lastFileName = filePath;
            }

            if (status == 2) {
                // 第三列 状态图表隐藏 spinner 控件显示
                DWidget *widget3 = m_tableView->indexWidget(m_model->index(i, TABLE_COL_STATUS));
                DSpinner *spinner = widget3->findChild<DSpinner *>("spinnerWidget");
                DLabel *labelIcon = widget3->findChild<DLabel *>("m_statusIcon");
                m_model->setItem(i, TABLE_COL_OPERATING, new QStandardItem(QString("true")));
                // spinner 显示 开始转动
                spinner->show();
                spinner->start();
                //状态图表隐藏
                labelIcon->hide();
            }
        }
    }
    int nType = int(type);
    emit senBatchOperation(list,nType);

}

//设置操作后结果   type 0 修复
void ScanWidget::setOperationResult(QString sPath, QString sMessage, Itemtype type)
{
    Q_UNUSED(sMessage);
    //总行数
    int count = m_model->rowCount();
    //int operationCount = 0;  //操作的行数
    for (int i = 0; i < count; i++) {
        //文件路径
        QString  filePath = m_model->index(i, TABLE_COL_INFO_PATH).data(Qt::DisplayRole).toString();
        if (filePath == sPath) {
            //1. 如果路径相同  给改相应的图标
            DWidget *widget1 = m_tableView->indexWidget(m_model->index(i, TABLE_COL_STATUS));
            DLabel *statusIconItem = widget1->findChild<DLabel *>("m_statusIcon");
            DSpinner *spinner = widget1->findChild<DSpinner *>("spinnerWidget");
            m_model->setItem(i, TABLE_COL_OPERATING, new QStandardItem(QString("fasle")));
            //spinner 停止隐藏
            spinner->hide();
            spinner->stop();

            //状态图展现
            statusIconItem->show();
            statusIconItem->setMinimumSize(QSize(ACTION_ICON_WIDTH, ACTION_ICON_HEIGHT));
            statusIconItem->setMaximumSize(QSize(ACTION_ICON_WIDTH, ACTION_ICON_HEIGHT));
            DLabel *statusInfoItem = widget1->findChild<DLabel *>("statInfo");
            statusIconItem->setPixmap(QIcon::fromTheme("dcc_antiav_normal").pixmap(QSize(ACTION_ICON_WIDTH, ACTION_ICON_HEIGHT)));

            //操作后的状态
            QString status ;
            //操作类型   0未操作  1 修复  2 隔离  3信任
            QString actionType;
            switch (type) {
            case Itemtype::Repair:
                status = QString(tr("Repaired"));
                actionType = "1" ;
                break;
            case Itemtype::Delete:
                break;
            case Itemtype::isolation:
                actionType = "2" ;
                status = QString(tr("Quarantined"));
                break;
            case Itemtype::Trust:
                actionType = "3" ;
                status = QString(tr("Trusted"));
                break;
            }
            //2.设置修改后的内容 内容伴随宽度省略 和提示信息显显示
            //操作状态
            m_model->setItem(i, TABLE_COL_ACTION_STATUS, new QStandardItem(QString("%1").arg(status)));
            //设置该行已经操作
            m_model->setItem(i, TABLE_COL_OPERATION_FLAG, new QStandardItem(QString("true")));
            //操作类型
            m_model->setItem(i, TABLE_COL_OPERATION_TYPE, new QStandardItem(QString("%1").arg(actionType)));
            m_tableView->setColumnHidden(TABLE_COL_ACTION_STATUS, true);
            m_tableView->setColumnHidden(TABLE_COL_OPERATION_FLAG, true);
            m_tableView->setColumnHidden(TABLE_COL_OPERATION_TYPE, true);
            m_tableView->setColumnHidden(TABLE_COL_OPERATING, true);

            //显示省略好的字符串
            statusInfoItem->setText(status);
            //提示
            statusInfoItem->setToolTip(status);
            //3.设置当前关闭当前行操作
            closeRowOperation(i);

            //操作的项数增加
            ++m_operationCount;
        }
        //判断是不是在处理最后一个
        if (m_lastFileName == sPath) {
            //处理结束
            m_titleLabel->setText(tr("The threats are processed successfully"));
            // 获得操作的数据明细
            QList<int> list;
            list = getOperationinfo();

            // 信任文件 隔离文件  信任文件
            m_scanPath->setText(tr("Repaired: %1   Quarantined: %2   Trusted: %3").arg(list.at(1)).arg(list.at(2)).arg(list.at(3)));
            QString procsssInfo = m_scanPath->text();
            m_scanPath->setToolTip(procsssInfo);
            m_Santime->setText(" ");

            // 处理完本次最后一个文件 判断可以操作的按钮状态  true 可以操作  false不可以操作
            bool flag = checkOperationBtStatus();
            if (flag) {
                // 下拉框可以操作
                m_combobox->setDisabled(false);
                m_exec->setDisabled(false);
            } else {
                // 下拉框不可以操作
                m_combobox->setDisabled(true);
                m_exec->setDisabled(true);
            }

            //如果操作总数等于列表总数  即所有的都完成
            bool finishFlag;
            finishFlag = finishAllOperation();
            if (finishFlag == true) {
                //如果都操作了
                m_combobox->hide();
                m_exec->hide();

                // 状态按钮隐藏  完成按钮展现
                m_pbType->hide();
                m_pbDone->show();
                // 异常都操作了以后 对异常数据在数据库中的操作mxz https://pms.uniontech.com/zentao/bug-view-41197.html
                addThreats();
            }

            //写入扫描数据log
            writeScanDatalog();
        }
    }
}

//写扫描日志数据
void ScanWidget::writeScanDatalog()
{
    QList<int> list;
    //获得操作的数据明细
    list = getOperationinfo();
    QDateTime date  = QDateTime::currentDateTime();
    QFile f(m_logName);
    if (!f.open(QIODevice::ReadOnly | QIODevice::WriteOnly | QIODevice::Text)) {
        //qDebug() << "Open ScanDate.log failed." << endl;
        return;
    }
    //输出流
    QTextStream outstream(&f);
    QString info = QString(tr("【Scan Information】"));
    outstream << info << "\n";
    QString version = QString(tr("Virus Database:"));
    //版本
    outstream << version << m_currentVersion << "\n";
    //  扫描时间：
    QString scanTime = QString(tr("Scan Time:"));
    QString updateTime = date.toString("yyyy-MM-dd hh:mm:ss");
    outstream << scanTime << updateTime << "\n";
    //扫描类型：
    QString scanType = QString(tr("Scan Type:"));
    outstream << scanType << m_currentScanType << "\n";
    //扫描范围：
    QString scanScope = QString(tr("Scan Scope:"));
    outstream << scanScope << m_scanPathInfo << "\n";
    outstream << "\n";
    //【扫描结果】
    QString scanResult = QString(tr("【Scan Result】"));
    outstream << scanResult << "\n";
    //扫描文件数：
    QString scanFiles = QString(tr("Files Scanned:"));
    outstream << scanFiles << m_totalScan << "\n";
    //发现风险数：
    QString scanError = QString(tr("Threats Found:"));
    outstream << scanError << m_totalError << "\n";
    //已经操作风险数：
    QString scanDisposeError = QString(tr("Threats Resolved:"));
    outstream << scanDisposeError << m_totalError - list.at(0) << "\n";
    //已经修复数：
    QString repairError = QString(tr("Repaired:"));
    outstream << repairError << list.at(1) << "\n";
    //已经隔离数：
    QString isolationError = QString(tr("Quarantined:"));
    outstream << isolationError << list.at(2) << "\n";
    //已经信任数：
    QString trustError = QString(tr("Trusted:"));
    outstream << trustError << list.at(3) << "\n";
    //类型标识 0 全盘扫描 1 快速程序  2 自定义扫描
    outstream << "Type_Flag>>" << m_currentType << "\n";

    outstream << "\n";
    //【扫描详情】
    QString scanDetails = QString(tr("【Scan Details】"));
    outstream << scanDetails << "\n";
    outstream << "===========================" << "\n";
    int datelenth = m_model->rowCount(); //总行数
    for (int i = 0; i < datelenth; i++) {
        //文件路径
        QString  filePath = m_model->index(i, TABLE_COL_INFO_PATH).data(Qt::DisplayRole).toString();
        //文件类型
        QString  message = m_model->index(i, TABLE_COL_INFO_DETAIL).data(Qt::DisplayRole).toString();
        //处理的结果
        QString  workstatus = m_model->index(i, TABLE_COL_ACTION_STATUS).data(Qt::DisplayRole).toString();
        if (workstatus == "建议修复"  || workstatus == "To be repaired") {
            //未处理
            workstatus = QString(tr("Unsolved"));
        }
        outstream << filePath << "\n";
        outstream << message << "[" << workstatus << "]" << "\n";
    }
    f.close();
}

//更改全选状态
void ScanWidget::updateCheckAllStaus()
{
    //总行数
    int count = m_model->rowCount();
    //选中状态的map
    QMap<int, int> rowMap;
    for (int i = 0; i < count; i++) {
        DWidget *widget = m_tableView->indexWidget(m_model->index(i, TABLE_COL_RISKS));
        DCheckBox *check = widget->findChild<DCheckBox *>("singleCheck");
        //按钮的状态
        int status = check->checkState();
        switch (status) {
        case 0:
            rowMap.insert(i, 0);
            m_model->item(i, TABLE_COL_CHECK_STATUS)->setText("0");
            break;
        case 1:
            break;
        case 2:
            //1是选中状态
            rowMap.insert(i, 1);
            m_model->item(i, TABLE_COL_CHECK_STATUS)->setText("1");
            break;
        }
    }
    //打沟的数量
    int tcount = 0;
    //不打沟的数量
    int fcount = 0;
    //便利状态map
    QMapIterator<int, int> rowMapIterator(rowMap);
    rowMapIterator.toBack();
    while (rowMapIterator.hasPrevious()) {
        int value;
        rowMapIterator.previous();
        value = rowMapIterator.value();
        if (value == 0) {
            fcount++;
        } else if (value == 1) {
            tcount++;
        }
    }
    //根据数量判断  全选状态
    if (tcount == count) {
        m_checkAll->setCheckState(Qt::CheckState::Checked);
        // 全选 下拉框可以操作
        m_combobox->setDisabled(false);
        m_exec->setDisabled(false);
    } else if (fcount == count) {
        m_checkAll->setCheckState(Qt::CheckState::Unchecked);
        // 下拉框不可以操作
        m_combobox->setDisabled(true);
        m_exec->setDisabled(true);
    } else {
        //有一个没有选中  先把全选设为未选择状态   然后把以前的选中都恢复
        m_checkAll->setCheckState(Qt::CheckState::Unchecked);
        QMap<int, int>::iterator it;
        for (it = rowMap.begin(); it != rowMap.end(); ++it) {
            if (it.value() == 0) {
                int row = it.key();
                DWidget *widget = m_tableView->indexWidget(m_model->index(row, TABLE_COL_RISKS));
                DCheckBox *check = widget->findChild<DCheckBox *>("singleCheck");
                check->setCheckState(Qt::CheckState::Unchecked);
            } else {
                int row = it.key();
                DWidget *widget = m_tableView->indexWidget(m_model->index(row, TABLE_COL_RISKS));
                DCheckBox *check = widget->findChild<DCheckBox *>("singleCheck");
                check->setCheckState(Qt::CheckState::Checked);
            }
        }
    }

    // 判断可以操作的按钮状态  true 可以操作  false不可以操作
    bool flag = checkOperationBtStatus();
    if (flag) {
        // 下拉框可以操作
        m_combobox->setDisabled(false);
        m_exec->setDisabled(false);
    } else {
        // 下拉框不可以操作
        m_combobox->setDisabled(true);
        m_exec->setDisabled(true);
    }
}

//添加数据
void ScanWidget::appendFileInfo(FileInfo info)
{
    //锁
    DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
    //当前行
    int finalRow = m_model->rowCount();
    //设置当前没有操作
    m_model->setItem(finalRow, TABLE_COL_OPERATION_FLAG, new QStandardItem(QString("false")));
    //得到每个隔离文件的详情信息
    QString filepath = info.path;
    QString message = info.detail;
    //第一列
    DWidget *widgetFile = new DWidget;
    QHBoxLayout *Layout = new QHBoxLayout;
    Layout->setContentsMargins(0, TABLE_SPACE, TABLE_SPACE, TABLE_SPACE);
    //复选框
    DCheckBox *check = new DCheckBox();
    check->setObjectName("singleCheck");
    check->setFixedSize(QSize(20, 20));
    check->setCheckState(Qt::CheckState::Checked);
    // 复选框是勾选中的状态  但是不可以点击在扫描过程中
    check->setDisabled(true);
    m_model->setItem(finalRow, TABLE_COL_CHECK_STATUS, new QStandardItem(QString("%1").arg(1)));
    connect(check, &DCheckBox::clicked, this, &ScanWidget::doSingalCheckClick);
    Layout->addWidget(check, 0, Qt::AlignLeft | Qt::AlignVCenter);

    QVBoxLayout *Layout1 = new QVBoxLayout;
    Layout1->setContentsMargins(0, 0, TABLE_SPACE, 0);
    Layout1->setSpacing(3);
    DLabel *lable1 = new DLabel(widgetFile);
    //文件路径
    lable1->setObjectName("filepath");
    QFont ft = lable1->font();
    ft.setBold(false);
    lable1->setFont(ft);
    DFontSizeManager::instance()->bind(lable1, DFontSizeManager::T7);
    //得到每个字符的宽度
    QFontMetrics fontWidth(lable1->font());
    QString elideNote = fontWidth.elidedText(filepath, Qt::ElideMiddle, TABLE_COL_1_WIDTH - (2 * TABLE_SPACE) - 20);
    //显示省略好的字符串
    lable1->setText(elideNote);
    //设置tooltips
    lable1->setToolTip(filepath);
    Layout1->addWidget(lable1);
    m_model->setItem(finalRow, TABLE_COL_INFO_PATH, new QStandardItem(QString("%1").arg(filepath)));

    //异常描述
    DLabel *lablepath = new DLabel(widgetFile);
    lablepath->setObjectName("infoDetail");
    QFont ft1 = lablepath->font();
    ft1.setBold(false);
    lablepath->setFont(ft1);
    DFontSizeManager::instance()->bind(lablepath, DFontSizeManager::T8);
    //得到每个字符的宽度
    QFontMetrics fontWidth1(lablepath->font());
    QString elideNote1 = fontWidth1.elidedText(message, Qt::ElideMiddle, TABLE_COL_1_WIDTH - (2 * TABLE_SPACE) - 20);
    //显示省略好的字符串
    lablepath->setText(elideNote1);
    //设置tooltips
    lablepath->setToolTip(message);
    m_model->setItem(finalRow, TABLE_COL_INFO_DETAIL, new QStandardItem(QString("%1").arg(message)));
    Layout1->addWidget(lablepath);
    Layout->addLayout(Layout1, 0);
    Layout->addStretch(0);
    widgetFile->setLayout(Layout);

    //第三列
    DWidget *lableFrame = new DWidget;
    QHBoxLayout *Layout2 = new QHBoxLayout;
    Layout2->setContentsMargins(TABLE_SPACE, TABLE_SPACE, TABLE_SPACE, TABLE_SPACE);
    //操作 建议 图标
    DLabel *statusIcon = new  DLabel;
    statusIcon->setMinimumSize(QSize(ACTION_ICON_WIDTH, ACTION_ICON_HEIGHT));
    statusIcon->setMaximumSize(QSize(ACTION_ICON_WIDTH, ACTION_ICON_HEIGHT));
    statusIcon->setObjectName("m_statusIcon");
    statusIcon->setPixmap(QIcon::fromTheme("dcc_antiav_warning").pixmap(QSize(ACTION_ICON_WIDTH, ACTION_ICON_HEIGHT)));

    DLabel *lableStatus = new DLabel;
    //建议修复
    QString opStatus = QString(tr("To be repaired"));
    //没有操作状态
    m_model->setItem(finalRow, TABLE_COL_OPERATION_TYPE, new QStandardItem(QString("0")));
    lableStatus->setObjectName("statInfo");
    QFont ft3 = lableStatus->font();
    ft3.setBold(false);
    lableStatus->setFont(ft3);
    DFontSizeManager::instance()->bind(lableStatus, DFontSizeManager::T7);
    //得到每个字符的宽度
    QFontMetrics fontWidth2(lableStatus->font());
    QString elideNote2 = fontWidth1.elidedText(opStatus, Qt::ElideMiddle, TABLE_COL_2_WIDTH - 30);
    //显示省略好的字符串
    lableStatus->setText(elideNote2);
    lableStatus->setToolTip(opStatus);
    //操作状态
    m_model->setItem(finalRow, TABLE_COL_ACTION_STATUS, new QStandardItem(QString("%1").arg(opStatus)));

    // 加载的spinner
    DSpinner *spiner = new DSpinner(this);
    spiner->setFixedSize(QSize(15, 15));
    spiner->setObjectName("spinnerWidget");
    m_model->setItem(finalRow, TABLE_COL_OPERATING, new QStandardItem(QString("fasle")));
    spiner->hide();

    // 第三列的控件布局
    Layout2->addWidget(spiner);
    Layout2->addWidget(statusIcon);
    Layout2->addWidget(lableStatus);
    lableFrame->setLayout(Layout2);

    //第四列  处理方式  显示3个按钮
    DWidget *widget1 = new DWidget;
    QHBoxLayout *hlayout3 = new QHBoxLayout;
    hlayout3->setContentsMargins(TABLE_SPACE * 2, TABLE_SPACE, TABLE_SPACE, TABLE_SPACE * 2);
    hlayout3->setSpacing(15);

    //修复按钮
    DPushButton *pBRepair = new DPushButton;
    pBRepair->setObjectName("m_pBRepair");
    pBRepair->setToolTip(tr("Repair"));
    pBRepair->setFlat(true);
    pBRepair->setIconSize(QSize(OPERATION_ICON_WIDTH, OPERATION_ICON_HEIGHT));
    pBRepair->setMinimumSize(QSize(OPERATION_ICON_WIDTH, OPERATION_ICON_HEIGHT));
    pBRepair->setMaximumSize(QSize(OPERATION_ICON_WIDTH, OPERATION_ICON_HEIGHT));

    // 修复点击
    connect(pBRepair, &DPushButton::clicked, this, [ = ] {
        singlRowOperation(filepath, Itemtype::Repair);
    });

    //隔离按钮
    DPushButton *pBIsolation = new DPushButton;
    pBIsolation->setObjectName("m_pBIsolation");
    pBIsolation->setToolTip(tr("Quarantine"));
    pBIsolation->setFlat(true);
    pBIsolation->setIconSize(QSize(OPERATION_ICON_WIDTH, OPERATION_ICON_HEIGHT));
    pBIsolation->setMinimumSize(QSize(OPERATION_ICON_WIDTH, OPERATION_ICON_HEIGHT));
    pBIsolation->setMaximumSize(QSize(OPERATION_ICON_WIDTH, OPERATION_ICON_HEIGHT));

    // 隔离点击
    connect(pBIsolation, &DPushButton::clicked, this, [ = ] {
        singlRowOperation(filepath, Itemtype::isolation);
    });

    DPushButton *pBTrust = new DPushButton;
    pBTrust->setFlat(true);
    pBTrust->setIconSize(QSize(OPERATION_ICON_WIDTH, OPERATION_ICON_HEIGHT));
    pBTrust->setMinimumSize(QSize(OPERATION_ICON_WIDTH, OPERATION_ICON_HEIGHT));
    pBTrust->setMaximumSize(QSize(OPERATION_ICON_WIDTH, OPERATION_ICON_HEIGHT));
    pBTrust->setObjectName("m_pBTrust");
    pBTrust->setToolTip(tr("Trust"));

    // 信任点击
    connect(pBTrust, &DPushButton::clicked, this, [ = ] {
        singlRowOperation(filepath, Itemtype::Trust);
    });

    // 判断是否在扫描中  扫描中的三个按钮为不可以点击状态
    pBRepair->setDisabled(true);
    pBIsolation->setDisabled(true);
    pBTrust->setDisabled(true);

    // 获得主题类型
    int type = m_guiHelper->themeType();
    // 浅色
    if(type == 1){
        pBRepair->setIcon(QIcon::fromTheme("icon_repair"));
        pBIsolation->setIcon(QIcon::fromTheme("icon_isolation"));
        pBTrust->setIcon(QIcon::fromTheme("icon_trust"));
    }else {
        pBRepair->setIcon(QIcon::fromTheme("icon_repair_dark"));
        pBIsolation->setIcon(QIcon::fromTheme("icon_isolation_dark"));
        pBTrust->setIcon(QIcon::fromTheme("icon_trust_dark"));
    }

    // 操作列的按钮布局
    hlayout3->addWidget(pBRepair);
    hlayout3->addWidget(pBIsolation);
    hlayout3->addWidget(pBTrust);
    widget1->setLayout(hlayout3);

    //设置每列的类
    m_tableView->setIndexWidget(m_model->index(finalRow, TABLE_COL_RISKS), widgetFile);
    m_tableView->setIndexWidget(m_model->index(finalRow, TABLE_COL_STATUS), lableFrame);
    m_tableView->setIndexWidget(m_model->index(finalRow, TABLE_COL_OPEARTION), widget1);

    //设置每列的宽度
    m_tableView->setColumnWidth(TABLE_COL_RISKS, TABLE_COL_1_WIDTH);
    m_tableView->setColumnWidth(TABLE_COL_STATUS, TABLE_COL_2_WIDTH);
    m_tableView->setColumnWidth(TABLE_COL_OPEARTION, TABLE_COL_3_WIDTH);

    //隐藏列
    m_tableView->setColumnHidden(TABLE_COL_CHECK_STATUS, true);
    m_tableView->setColumnHidden(TABLE_COL_INFO_PATH, true);
    m_tableView->setColumnHidden(TABLE_COL_INFO_DETAIL, true);
    m_tableView->setColumnHidden(TABLE_COL_ACTION_STATUS, true);
    m_tableView->setColumnHidden(TABLE_COL_OPERATION_FLAG, true);
    m_tableView->setColumnHidden(TABLE_COL_OPERATION_TYPE, true);
}

//拖动表头更新宽度
void ScanWidget::updateTableForWidth(int nLogicalIndex, int nOldSize, int nNewSize)
{
    Q_UNUSED(nOldSize);
    //行数
    int count = m_model->rowCount();
    for (int i = 0; i < count; i++) {
        if (nLogicalIndex == TABLE_COL_RISKS) {
            //文件路径
            QString  filePath = m_model->index(i, TABLE_COL_INFO_PATH).data(Qt::DisplayRole).toString();
            //文件的病毒详情
            QString  fileDetail = m_model->index(i, TABLE_COL_INFO_DETAIL).data(Qt::DisplayRole).toString();
            DWidget *widget = m_tableView->indexWidget(m_model->index(i, nLogicalIndex));
            DLabel *fpath  = widget->findChild<DLabel *>("filepath");
            //得到每个字符的宽度
            QFontMetrics fontWidth(fpath->font());
            QString elideNote = fontWidth.elidedText(filePath, Qt::ElideMiddle, nNewSize - (TABLE_SPACE * 2) - 20);
            //显示省略好的字符串
            fpath->setText(elideNote);
            //设置tooltips
            fpath->setToolTip(filePath);

            DLabel *fdetail  = widget->findChild<DLabel *>("infoDetail");
            //得到每个字符的宽度
            QFontMetrics fontWidth1(fdetail->font());
            QString elideNote1 = fontWidth1.elidedText(fileDetail, Qt::ElideMiddle, nNewSize - (TABLE_SPACE * 2) - 20);
            //显示省略好的字符串
            fdetail->setText(elideNote1);
            fdetail->setToolTip(fileDetail);
        } else if (nLogicalIndex == TABLE_COL_STATUS) {
            //操作状态
            QString  actionStatus = m_model->index(i, TABLE_COL_ACTION_STATUS).data(Qt::DisplayRole).toString();
            DWidget *widget = m_tableView->indexWidget(m_model->index(i, nLogicalIndex));
            DLabel *lable = widget->findChild<DLabel *>("statInfo");
            //得到每个字符的宽度
            QFontMetrics fontWidth(lable->font());
            QString elideNote = fontWidth.elidedText(actionStatus, Qt::ElideMiddle, nNewSize - (TABLE_SPACE * 2) - 15);
            //显示省略好的字符串
            lable->setText(elideNote);
            lable->setToolTip(actionStatus);
        } else {
            return;
        }
    }
}

//扫描完成后允许操作每行
void ScanWidget::allowOperation()
{
    //行数
    int count = m_model->rowCount();
    for (int i = 0; i < count ; i++) {
        // 允许操作复选框 https://pms.uniontech.com/zentao/bug-view-37374.html
        DWidget *widget = m_tableView->indexWidget(m_model->index(i, TABLE_COL_RISKS));
        DCheckBox *check = widget->findChild<DCheckBox *>("singleCheck");
        check->setDisabled(false);

        // 通过第4列 得到3个按钮
        DWidget *widget1 = m_tableView->indexWidget(m_model->index(i, TABLE_COL_OPEARTION));
        DPushButton *pbRepair = widget1->findChild<DPushButton *>("m_pBRepair");
        DPushButton *pBIsolation = widget1->findChild<DPushButton *>("m_pBIsolation");
        DPushButton *pBTrust = widget1->findChild<DPushButton *>("m_pBTrust");

        // 恢复三个按钮可以操作状态
        pbRepair->setDisabled(false);
        pBIsolation->setDisabled(false);
        pBTrust->setDisabled(false);
    }
}

//是否结束全部操作
bool ScanWidget::finishAllOperation()
{
    bool flag;
    //行数
    int count = m_model->rowCount();
    int actionNum = 0;
    //只有是所有的第10项目为 true 才是结束
    for (int i = 0 ; i < count ; i++) {
        QString operaFlag = m_model->index(i, TABLE_COL_OPERATION_FLAG).data(Qt::DisplayRole).toString();
        if (operaFlag  == "true") {
            ++actionNum;
        }
    }
    //qDebug() << "总行数据====" << count << "，操作行数====" << actionNum;
    if (actionNum == count) {
        flag = true ;
        addThreats();
    } else {
        flag = false;
    }
    return flag;
}

//获得操作的信息
QList<int> ScanWidget::getOperationinfo()
{
    QList<int> list;
    //行数
    int count = m_model->rowCount();
    //已修复
    int repaired = 0;
    //没有操作
    int noAction = 0;
    //已隔离
    int isolationed = 0;
    //已信任
    int trusted = 0;
    //只有是所有的第10项目为 true 才是结束
    for (int i = 0 ; i < count ; i++) {
        //操作状态
        QString actionType = m_model->index(i, TABLE_COL_OPERATION_TYPE).data(Qt::DisplayRole).toString();
        if (actionType  == "0") {
            ++noAction;
        } else if (actionType  == "1") {
            ++repaired;
        } else if (actionType  == "2") {
            ++isolationed;
        } else {
            ++trusted;
        }
    }
    list << noAction << repaired << isolationed << trusted;
    return list;
}


void ScanWidget::resizeEvent(QResizeEvent *event)
{
    Q_UNUSED(event);
}

bool ScanWidget::eventFilter(QObject *obj, QEvent *event)
{
    if(this == obj){
        // 如果当前是字体改变的事件
        if(event->type() == QEvent::FontChange){
            // 刷新列表
            refreshTableByFontChange();
        }
    }
    return QWidget::eventFilter(obj, event);
}

// 刷新表格音改变字体的大小
void ScanWidget::refreshTableByFontChange()
{
    //行数
    int count = m_model->rowCount();
    // 获得每一列的宽度
    int wcol1 = m_tableView->columnWidth(TABLE_COL_RISKS);
    int wcol2 = m_tableView->columnWidth(TABLE_COL_STATUS);

    for (int i = 0; i < count; i++) {
        //文件路径
        QString  filePath = m_model->index(i, TABLE_COL_INFO_PATH).data(Qt::DisplayRole).toString();
        //文件的病毒详情
        QString  fileDetail = m_model->index(i, TABLE_COL_INFO_DETAIL).data(Qt::DisplayRole).toString();
        DWidget *widget = m_tableView->indexWidget(m_model->index(i, TABLE_COL_RISKS));
        DLabel *fpath  = widget->findChild<DLabel *>("filepath");
        //得到每个字符的宽度
        QFontMetrics fontWidth(fpath->font());
        QString elideNote = fontWidth.elidedText(filePath, Qt::ElideMiddle, (wcol1 - (TABLE_SPACE * 2))- 40);
        //显示省略好的字符串
        fpath->setText(elideNote);
        //设置tooltips
        fpath->setToolTip(filePath);

        DLabel *fdetail  = widget->findChild<DLabel *>("infoDetail");
        //得到每个字符的宽度
        QFontMetrics fontWidth1(fdetail->font());
        QString elideNote1 = fontWidth1.elidedText(fileDetail, Qt::ElideMiddle, (wcol1 - (TABLE_SPACE * 2)) - 40);
        //显示省略好的字符串
        fdetail->setText(elideNote1);
        fdetail->setToolTip(fileDetail);

        //操作状态
        QString  actionStatus = m_model->index(i, TABLE_COL_ACTION_STATUS).data(Qt::DisplayRole).toString();
        DWidget *widget3 = m_tableView->indexWidget(m_model->index(i, TABLE_COL_STATUS));
        DLabel *lable = widget3->findChild<DLabel *>("statInfo");
        //得到每个字符的宽度
        QFontMetrics fontWidth3(lable->font());
        QString elideNote3 = fontWidth3.elidedText(actionStatus, Qt::ElideMiddle, wcol2 - (TABLE_SPACE * 2) - 15);
        //显示省略好的字符串
        lable->setText(elideNote3);
        lable->setToolTip(actionStatus);
    }
}

//布局
void ScanWidget::initGUI()
{
    //主垂直布局
    QVBoxLayout *mainlayout = new QVBoxLayout;
    mainlayout->setSpacing(SPACING);
    //四周边距
    mainlayout->setContentsMargins(MAIN_LEFT_MARGINS, MAIN_TOP_MARGINS, MAIN_RIGHT_MARGINS, MAIN_BUTTOM_MARGINS);

    //图表和右侧扫描标题布局
    QHBoxLayout  *hLayout2 = new QHBoxLayout();
    //头部右侧垂直布局
    QVBoxLayout *vlayout1 = new QVBoxLayout;
    vlayout1->setSpacing(0);
    vlayout1->setContentsMargins(TOP_LEFT_MARGINS, TOP_TOP_MARGINS, TOP_RIGHT_MARGINS, TOP_BUTTOM_MARGINS);
    //上布局
    QVBoxLayout *vlayout2 = new QVBoxLayout;
    vlayout2->setSpacing(SPACING);
    QHBoxLayout *thlayout = new QHBoxLayout;
    //标题
    m_titleLabel = new DLabel(tr("Full scanning..."));
    QFont ft = m_titleLabel->font();
    ft.setBold(true);
    m_titleLabel->setFont(ft);
    DFontSizeManager::instance()->bind(m_titleLabel, DFontSizeManager::T4);
    thlayout->addWidget(m_titleLabel, 0, Qt::AlignLeft);
    //扫描时间
    QString time1 = "00:00:00";
    m_Santime = new DLabel(tr("Time elapsed %1").arg(time1));
    m_Santime->setAlignment(Qt::AlignRight | Qt::AlignTrailing | Qt::AlignVCenter);
    QFont ft2 = m_Santime->font();
    ft2.setBold(false);
    m_Santime->setFont(ft2);
    DFontSizeManager::instance()->bind(m_Santime, DFontSizeManager::T7);
    thlayout->addWidget(m_Santime, 0, Qt::AlignRight);
    vlayout1->addLayout(thlayout, 0);

    //进度条
    m_progressBar->setObjectName(QStringLiteral("progressBar"));
    m_progressBar->setMinimumSize(QSize(0, 8));
    m_progressBar->setMaximumSize(QSize(16777215, 8));
    m_progressBar->setRange(0, 1000000);
    //默认值
    m_progressBar->setValue(0);
    m_progressBar->setTextVisible(false);
    vlayout1->addWidget(m_progressBar);
    //扫描路径
    m_scanPath = new DLabel("Scan path...");
    QFont ft1 = m_scanPath->font();
    ft1.setBold(false);
    m_scanPath->setFont(ft1);
    DFontSizeManager::instance()->bind(m_scanPath, DFontSizeManager::T7);
    vlayout1->addWidget(m_scanPath);

    m_iconType = new DLabel(this);
    m_iconType->setMinimumSize(QSize(TITLE_ICON_WIDTH + 14, TITLE_ICON_HEIGHT + 14));
    m_iconType->setMaximumSize(QSize(TITLE_ICON_WIDTH + 14, TITLE_ICON_HEIGHT + 14));
    //左侧的图表
    hLayout2->addWidget(m_iconType);
    hLayout2->addLayout(vlayout1);
    vlayout2->addLayout(hLayout2);
    //空格
    mainlayout->addSpacing(5);

    //空格
    mainlayout->addSpacing(SPACING);
    mainlayout->addLayout(vlayout2);

    // 栈 stacke
    m_stackwidget = new QStackedWidget(this);
    m_stackwidget->setContentsMargins(0, 0, 0, 0);

    // 栈 第一页
    DWidget *widget1 = new DWidget;
    QHBoxLayout *tlayout = new QHBoxLayout;
    DLabel *m_empty = new DLabel(this);
    m_empty->setPixmap(QIcon::fromTheme("dcc_antiav_empty").pixmap(QSize(100, 84)));
    tlayout->addWidget(m_empty, 0, Qt::AlignVCenter | Qt::AlignHCenter);
    widget1->setLayout(tlayout);
    m_stackwidget->addWidget(widget1);

    // 栈 第二页 tableview
    DWidget *CenterWidget = new DWidget;
    CenterWidget->setLayout(initTableWidget());
    m_stackwidget->addWidget(CenterWidget);
    mainlayout->addWidget(m_stackwidget, Qt::AlignBottom | Qt::AlignHCenter);

    //下布局
    QVBoxLayout *vlayout3 = new QVBoxLayout;

    //扫描的项目   报警  和个数
    QHBoxLayout  *hLayout3 = new QHBoxLayout();

    //按钮
    QHBoxLayout  *hLayout4 = new QHBoxLayout();

    //下画线
    hLayout3->setContentsMargins(10, -1, 15, -1);
    hLayout3->setSpacing(SPACING);

    //全选
    m_checkAll->setText(tr("Select All"));
    hLayout3->addWidget(m_checkAll, 0, Qt::AlignBottom | Qt::AlignLeft);

    //异常
    m_errNum = new DLabel(tr("Threats:"));
    m_errNum->setAlignment(Qt::AlignLeft | Qt::AlignTrailing | Qt::AlignVCenter);
    hLayout3->addWidget(m_errNum, 0, Qt::AlignBottom | Qt::AlignLeft);

    //已经扫描项目
    m_scanNum = new DLabel(tr("Scanned:"));
    m_scanNum->setAlignment(Qt::AlignRight | Qt::AlignTrailing | Qt::AlignVCenter);
    hLayout3->addWidget(m_errNum, 0, Qt::AlignBottom | Qt::AlignLeft);
    hLayout3->addStretch(1);
    hLayout3->addWidget(m_scanNum);
    vlayout3->addLayout(hLayout3);

    hLayout4->setSpacing(20);
    hLayout4->setContentsMargins(10, 0, 0, 0);

    // 结束按钮
    m_pbFinish = new DPushButton;
    m_pbFinish->setText(tr("Finish"));
    m_pbFinish->setMinimumSize(QSize(BUTTOM_PB_WIDTH, BUTTOM_PB_HEIGHT));
    m_pbFinish->setMaximumSize(QSize(BUTTOM_PB_WIDTH, BUTTOM_PB_HEIGHT));

    // 重新扫描
    m_pbRescan = new DPushButton;
    m_pbRescan->setText(tr("Scan Again"));
    m_pbRescan->setMinimumSize(QSize(BUTTOM_PB_WIDTH, BUTTOM_PB_HEIGHT));
    m_pbRescan->setMaximumSize(QSize(BUTTOM_PB_WIDTH, BUTTOM_PB_HEIGHT));

    //暂停按钮
    m_pbType = new DPushButton;
    m_pbType->setText(tr("Pause"));
    m_pbType->setMinimumSize(QSize(BUTTOM_PB_WIDTH, BUTTOM_PB_HEIGHT));
    m_pbType->setMaximumSize(QSize(BUTTOM_PB_WIDTH, BUTTOM_PB_HEIGHT));
    hLayout4->addWidget(m_pbFinish, 0, Qt::AlignLeft);
    hLayout4->addWidget(m_pbRescan, 0, Qt::AlignLeft);
    hLayout4->addStretch(1);
    hLayout4->addWidget(m_pbType, 0, Qt::AlignRight);

    // 完成按钮
    m_pbDone = new DSuggestButton;
    m_pbDone->setText(tr("Done"));
    m_pbDone->setMinimumSize(QSize(BUTTOM_PB_WIDTH, BUTTOM_PB_HEIGHT));
    m_pbDone->setMaximumSize(QSize(BUTTOM_PB_WIDTH, BUTTOM_PB_HEIGHT));
    hLayout4->addWidget(m_pbDone, 0, Qt::AlignRight);

    // 下拉按钮 combobox
    m_combobox = new DComboBox;
    m_combobox->clear();
    QStringList comlist;
    comlist << tr("Repair Now") << tr("Quarantine Now") << tr("Add to Whitelist");
    m_combobox->addItems(comlist);
    m_combobox->setMinimumSize(QSize(BUTTOM_PB_WIDTH, BUTTOM_PB_HEIGHT));
    m_combobox->setMaximumSize(QSize(BUTTOM_PB_WIDTH, BUTTOM_PB_HEIGHT));
    hLayout4->addWidget(m_combobox, 0, Qt::AlignRight);

    // 执行按钮
    m_exec = new DSuggestButton;
    m_exec->setText(tr("Start"));
    m_exec->setMinimumSize(QSize(BUTTOM_PB_WIDTH, BUTTOM_PB_HEIGHT));
    m_exec->setMaximumSize(QSize(BUTTOM_PB_WIDTH, BUTTOM_PB_HEIGHT));
    hLayout4->addWidget(m_exec, 0, Qt::AlignRight);

    vlayout3->addLayout(hLayout4);
    mainlayout->addLayout(vlayout3);

    setLayout(mainlayout);
}

// 初始化表格
QHBoxLayout *ScanWidget::initTableWidget()
{
    QHBoxLayout *layout = new QHBoxLayout;
    layout->setMargin(0);
    layout->setSpacing(0);
    m_tableView = new  DTableView(this);
    // 连接表格与模型
    m_model = new QStandardItemModel;
    m_tableView->setModel(m_model);

    //tableview 表高度
    m_tableView->verticalHeader()->setDefaultSectionSize(TABLEVIEW_ROW_HEIGHT);
    m_tableView->verticalHeader()->setVisible(false);
    m_tableView->horizontalHeader()->setVisible(true);
    //单选
    m_tableView->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_tableView->setSelectionMode(QAbstractItemView::NoSelection);
    //不可以编辑
    m_tableView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    //形状
    m_tableView->setFrameShape(QTableView::NoFrame);
    // 失去焦点
    m_tableView->setFocusPolicy(Qt::NoFocus);
    //栅格
    m_tableView->setShowGrid(false);
    //排序
    m_tableView->setSortingEnabled(true);

    //风险项
    QStandardItem *standaritem1 = new QStandardItem(tr("Risks"));
    standaritem1->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);

    //状态   处理方式
    QStandardItem *standaritem2 = new QStandardItem(tr("Status"));
    standaritem2->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);

    //处理方式
    QStandardItem *standaritem3 = new QStandardItem(tr("Action"));
    standaritem3->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);

    //设置每列
    m_model->setHorizontalHeaderItem(TABLE_COL_RISKS, standaritem1);
    m_model->setHorizontalHeaderItem(TABLE_COL_STATUS, standaritem2);
    m_model->setHorizontalHeaderItem(TABLE_COL_OPEARTION, standaritem3);

    //选择状态
    m_model->setHorizontalHeaderItem(TABLE_COL_CHECK_STATUS, new QStandardItem("checkStatus"));
    //回馈信息路径
    m_model->setHorizontalHeaderItem(TABLE_COL_INFO_PATH, new QStandardItem("InfoPath"));
    //回馈信息内容
    m_model->setHorizontalHeaderItem(TABLE_COL_INFO_DETAIL, new QStandardItem("InfoDetail"));
    //操作状态
    m_model->setHorizontalHeaderItem(TABLE_COL_ACTION_STATUS, new QStandardItem("ActionStatus"));
    //是否操作了   //操作为True   为操作为 false
    m_model->setHorizontalHeaderItem(TABLE_COL_OPERATION_FLAG, new QStandardItem("flag"));
    //操作类型   0没有操作  1修复  2隔离  3信任   由于操作状态保存的是英文或中文判断比较麻烦
    m_model->setHorizontalHeaderItem(TABLE_COL_OPERATION_TYPE, new QStandardItem("type"));
    m_model->setHorizontalHeaderItem(TABLE_COL_OPERATING, new QStandardItem("type"));
    //设置每列的宽度
    m_tableView->setColumnWidth(TABLE_COL_RISKS, TABLE_COL_1_WIDTH);
    m_tableView->setColumnWidth(TABLE_COL_STATUS, TABLE_COL_2_WIDTH);
    m_tableView->setColumnWidth(TABLE_COL_OPEARTION, TABLE_COL_3_WIDTH);

    //隐藏列
    m_tableView->setColumnHidden(TABLE_COL_CHECK_STATUS, true);
    m_tableView->setColumnHidden(TABLE_COL_INFO_PATH, true);
    m_tableView->setColumnHidden(TABLE_COL_INFO_DETAIL, true);
    m_tableView->setColumnHidden(TABLE_COL_ACTION_STATUS, true);
    m_tableView->setColumnHidden(TABLE_COL_OPERATION_FLAG, true);
    m_tableView->setColumnHidden(TABLE_COL_OPERATION_TYPE, true);
    m_tableView->setColumnHidden(TABLE_COL_OPERATING, true);

    m_tableView->horizontalHeader()->setSortIndicator(TABLE_COL_CHECK_STATUS, Qt::DescendingOrder);  //排序
    m_tableView->horizontalHeader()->setSortIndicator(TABLE_COL_INFO_PATH, Qt::DescendingOrder);  //排序
    //设置表格样式
    DefenderTable *defender = new DefenderTable;
    defender->setTableAndModel(m_tableView, m_model);
    layout->addWidget(defender);

    return layout;
}


// 扫描结束按钮点击
void ScanWidget::doScanFinishClick()
{
    m_pbFinish->setEnabled(false);
    Q_EMIT onScanStatus(DissectionState::Stop);
    // 停止扫描
    stopScan();
}

// 重新扫描
void ScanWidget::doReScan()
{
    bool result = isCurrentOperating();
    if(result == true){
        showOperatingTip();
    }else {
        addThreats();
        m_t1->stop();
        Q_EMIT scanAgain(m_currentType, m_scanPathInfo, m_currentVersion);
    }
}

// 暂停和继续按钮点击
void ScanWidget::doTypeButtonClick()
{
    m_pbType->setEnabled(false);
    QTimer::singleShot(1000, this, [ = ]() {
        m_pbType->setEnabled(true);
    });

    if (m_flagPause) {
        //暂停
        Q_EMIT onScanStatus(DissectionState::Pause);
        //下个是继续
        m_pbType->setText(tr("Continue"));

        // 缓存定时器停止  https://pms.uniontech.com/zentao/bug-view-37217.html
        // 设置扫描标题已经暂停
        m_t1->stop();
        m_titleLabel->setText(tr("%1 paused").arg(m_currentScanType));
    } else {
        //继续
        Q_EMIT onScanStatus(DissectionState::Continue);
        //下个是暂停
        m_pbType->setText(tr("Pause"));

        // 缓存定时器继续  扫描计时器启动  https://pms.uniontech.com/zentao/bug-view-37217.html
        m_t1->start(1000);
    }
    m_flagPause = ! m_flagPause;

    // 暂停或者继续记录数据 https://pms.uniontech.com/zentao/bug-view-37299.html
    writeScanDatalog();
}

void ScanWidget::updateTime()
{
    m_countTime++;
    int yu = m_countTime % 6;

    QString point = "";
    if (yu == 0) {
        point = ".";
    } else if (yu == 1) {
        point = "..";
    } else if (yu == 2) {
        point = "...";
    } else if (yu == 3) {
        point = "....";
    } else if (yu == 4) {
        point = ".....";
    } else if (yu == 5) {
        point = "......";
    }
    m_titleLabel->setText(QString("%1 %2").arg(m_currentScanName).arg(point));
    //小时
    int h = m_countTime / 3600;
    //分钟
    int m = (m_countTime - h * 3600) / 60;
    //秒
    int s = (m_countTime - h * 3600 - m * 60) % 60;
    m_Santime->setText(QString(tr("Time elapsed %1:%2:%3")
                               .arg(h, 2, 10, QChar('0'))
                               .arg(m, 2, 10, QChar('0'))
                               .arg(s, 2, 10, QChar('0'))));
}

//全选
void ScanWidget::doCheckAllClick(int nStatus)
{
    int count = m_model->rowCount();
    for (int i = 0; i < count; i++) {
        DWidget *widget = m_tableView->indexWidget(m_model->index(i, TABLE_COL_RISKS));
        DCheckBox *check = widget->findChild<DCheckBox *>("singleCheck");
        //　对已经操作了的不会有全选操作　https://pms.uniontech.com/zentao/bug-view-45727.html
        QString operaFlag = m_model->index(i, TABLE_COL_OPERATION_FLAG).data(Qt::DisplayRole).toString();
        if(operaFlag == "false"){
            //　如果该行没有操作　　执行全选/反全选状态改变
            switch (nStatus) {
            case 0:
                m_model->item(i, TABLE_COL_CHECK_STATUS)->setText("0");
                check->setCheckState(Qt::CheckState::Unchecked);

                // 下拉框不可以操作
                m_combobox->setDisabled(true);
                m_exec->setDisabled(true);
                break;
            case 1:
                break;
            case 2:
                check->setCheckState(Qt::CheckState::Checked);
                m_model->item(i, TABLE_COL_CHECK_STATUS)->setText("1");

                // 全选 下拉框可以操作
                m_combobox->setDisabled(false);
                m_exec->setDisabled(false);
                break;
            }
        }
    }
}

//扫描结束展现出 全选框和修复等下拉框
void ScanWidget::doScanFinsh()
{
    qDebug() << "发送过来扫描结束信号===";
    m_t1->stop();
    // 结束后的页面
    showFinishPage();
}

//修复回馈信号
void ScanWidget::doRepairResult(QString sPath, QString sMessage)
{
    setOperationResult(sPath, sMessage, Itemtype::Repair);
}

//隔离回馈信号
void ScanWidget::doIsolationResult(QString sPath, QString sMessage)
{
    setOperationResult(sPath, sMessage, Itemtype::isolation);
}

//信任隔离信号
void ScanWidget::doTruestResult(QString sPaths, QString sMessage)
{
    setOperationResult(sPaths, sMessage, Itemtype::Trust);
}

//单选点击
void ScanWidget::doSingalCheckClick()
{
    updateCheckAllStaus();
}

// 完成按钮点击
void ScanWidget::doneButtonClick()
{
    addThreats();
    Q_EMIT onBackpage();
}

// 执行按钮点击
void ScanWidget::execButtonClick()
{
    // 根据当前的combox 下标
    int index = m_combobox->currentIndex();
    // 根据下表触发不同的条件
    if(index == 0){
        doRepairClick();
    }else if (index == 1) {
        doIsolationClick();
    }else {
        doTrustClick();
    }
}

// 主题类型改变
void ScanWidget::themeTypeChange(ColorType themeType)
{
    int type;
    switch (themeType) {
    case UnknownType:
        type = 0;
        break;
    case LightType:
        type = 1;
        break;
    case DarkType:
        type = 2 ;
        break;
    }

    //总行数
    int count = m_model->rowCount();

    for (int i = 0; i< count; i++) {
        // 通过第4列 得到3个按钮
        DWidget *widget1 = m_tableView->indexWidget(m_model->index(i, TABLE_COL_OPEARTION));
        DPushButton *pbRepair = widget1->findChild<DPushButton *>("m_pBRepair");
        DPushButton *pBIsolation = widget1->findChild<DPushButton *>("m_pBIsolation");
        DPushButton *pBTrust = widget1->findChild<DPushButton *>("m_pBTrust");
        // 浅色主题
        if(type == 1){
            pbRepair->setIcon(QIcon::fromTheme("icon_repair"));
            pBIsolation->setIcon(QIcon::fromTheme("icon_isolation"));
            pBTrust->setIcon(QIcon::fromTheme("icon_trust"));
        }else {
            pbRepair->setIcon(QIcon::fromTheme("icon_repair_dark"));
            pBIsolation->setIcon(QIcon::fromTheme("icon_isolation_dark"));
            pBTrust->setIcon(QIcon::fromTheme("icon_trust_dark"));
        }
    }
}

//全部修复
void ScanWidget::doRepairClick()
{
    allOPeration(Itemtype::Repair);
}

//隔离
void ScanWidget::doIsolationClick()
{
    allOPeration(Itemtype::isolation);
}

//信任
void ScanWidget::doTrustClick()
{
    allOPeration(Itemtype::Trust);
}

//接受扫描回调信号
void ScanWidget::doDissectionResult(QString sPath, double dProgress, int eType, QString sDetailInfo)
{
    FileInfo file;
    file.path = sPath;
    file.type = Type(eType);
    file.detail = sDetailInfo;
    file.dProgress = dProgress;

    if(file.type != Type::Normal){
        //有异常的文件
        setScanErrorNum(++m_totalError);
        appendFileInfo(file);
    };
    //扫描总数
    setScanNum(++m_totalScan);
    //扫描路径
    setScanPath(sPath);
    setProgressValue(dProgress);
}

//单个文件的操作
void ScanWidget::operationFile(Itemtype type, QString sFilepath)
{
    switch (type) {
    case Itemtype::Repair:
        Q_EMIT onAllRepairProblemFile(sFilepath);
        break;
    case Itemtype::Delete:
        break;
    case Itemtype::isolation:
        Q_EMIT onAllIsolationFile(true, sFilepath);
        break;
    case Itemtype::Trust:
        Q_EMIT onAllTruestFile(true, sFilepath);
        break;
    }
    qDebug() << "filepath=======" << sFilepath;
}

//关闭当前行操作
void ScanWidget::closeRowOperation(int nIndex)
{
    //第三列 的状态信息
    DWidget *widget1 = m_tableView->indexWidget(m_model->index(nIndex, TABLE_COL_STATUS));
    DLabel *statusInfoItem = widget1->findChild<DLabel *>("statInfo");

    //第四列 得到 修复 信任 隔离按钮
    DWidget *widget = m_tableView->indexWidget(m_model->index(nIndex, TABLE_COL_OPEARTION));
    DPushButton *pbRepair = widget->findChild<DPushButton *>("m_pBRepair");
    DPushButton *pBIsolation = widget->findChild<DPushButton *>("m_pBIsolation");
    DPushButton *pBTrust = widget->findChild<DPushButton *>("m_pBTrust");

    //修复  信任  隔离  不可以在选中
    pbRepair->setDisabled(true);
    pBIsolation->setDisabled(true);
    pBTrust->setDisabled(true);

    //第一列 悬着框
    DWidget *checkWidget = m_tableView->indexWidget(m_model->index(nIndex, TABLE_COL_RISKS));
    DCheckBox *check = checkWidget->findChild<DCheckBox *>("singleCheck");
    //check至于灰色  不可以选中
    check->setDisabled(true);

    //第二列 风险项
    DWidget *widget4 = m_tableView->indexWidget(m_model->index(nIndex, TABLE_COL_RISKS));
    DLabel *lpath = widget4->findChild<DLabel *>("filepath");
    DLabel *ldetail = widget4->findChild<DLabel *>("infoDetail");
    //透明效果
    QGraphicsOpacityEffect *opacityEffect[3];
    for (int i = 0; i < 3; i++) {
        opacityEffect[i] = new QGraphicsOpacityEffect;
        //透明度50%
        opacityEffect[i]->setOpacity(0.5);
    }
    //分别设置透明度
    lpath->setGraphicsEffect(opacityEffect[0]);
    ldetail->setGraphicsEffect(opacityEffect[1]);
    statusInfoItem->setGraphicsEffect(opacityEffect[2]);
}

// 判断可以操作的按钮状态  true 可以操作  false不可以操作
bool ScanWidget::checkOperationBtStatus()
{
    bool flag;
    //待操作项目
    int operation = 0;

    //总行数
    int count = m_model->rowCount();
    for (int i = 0; i < count; i++) {
        DWidget *widget = m_tableView->indexWidget(m_model->index(i, TABLE_COL_RISKS));
        DCheckBox *check = widget->findChild<DCheckBox *>("singleCheck");
        //按钮的状态
        int status = check->checkState();
        //是否被操作
        QString operaFlag = m_model->index(i, TABLE_COL_OPERATION_FLAG).data(Qt::DisplayRole).toString();

        // 如果没有操作并且按钮是选中状态
        if ((operaFlag == "false") && (status == 2)) {
            ++operation;
        }
    }

    // 只要有一个待操作项 就是可以操作的
    if (operation != 0) {
        flag = true ;
    } else {
        flag = false ;
    }
    return  flag;
}

// 一行的单独操作
void ScanWidget::singlRowOperation(QString filepath, Itemtype nType)
{
    //　点击的行号
    int nIndex = 0;
    //　ｍｘｚ　https://pms.uniontech.com/zentao/bug-view-43290.html
    //　由于以前输入参数是行号　但是排序以后行号会发生改变
    int count = m_model->rowCount();
    for (int i = 0; i < count; i++) {
        QString  fPath = m_model->index(i, TABLE_COL_INFO_PATH).data(Qt::DisplayRole).toString();
        //　对比带参路径和　表中的路径相比　从而或者真实的行号
        if(fPath == filepath){
            nIndex = i;
            break;
        }
    }
   //路径
   QString  filePath = m_model->index(nIndex, TABLE_COL_INFO_PATH).data(Qt::DisplayRole).toString();

    // 第一列
    DWidget *widget = m_tableView->indexWidget(m_model->index(nIndex, TABLE_COL_RISKS));
    DCheckBox *check = widget->findChild<DCheckBox *>("singleCheck");
    check->setCheckState(Qt::CheckState::Checked);
    switch (nType) {
    case Itemtype::Repair:
        Q_EMIT onAllRepairProblemFile(filePath);
        break;
    case Itemtype::Delete:
        break;
    case Itemtype::isolation:
        Q_EMIT onAllIsolationFile(true, filePath);
        break;
    case Itemtype::Trust:
        Q_EMIT onAllTruestFile(true, filePath);
        break;
    }

    // 更新选择状态
    updateCheckAllStaus();

    // 第三列 状态图表隐藏 spinner 控件显示
    DWidget *widget3 = m_tableView->indexWidget(m_model->index(nIndex, TABLE_COL_STATUS));
    DSpinner *spinner = widget3->findChild<DSpinner *>("spinnerWidget");
    DLabel *labelIcon = widget3->findChild<DLabel *>("m_statusIcon");
    m_model->setItem(nIndex, TABLE_COL_OPERATING, new QStandardItem(QString("true")));
    // spinner 显示 开始转动
    spinner->show();
    spinner->start();

    //状态图表隐藏
    labelIcon->hide();

    // 最后路径经
    m_lastFileName = filePath;
    // 处理条数
    m_operationCount = 0;

    //关闭当前行操作
    closeRowOperation(nIndex);

    // 下拉框不可以操作 https://pms.uniontech.com/zentao/bug-view-46938.html
    m_combobox->setDisabled(true);
    m_exec->setDisabled(true);
}

// 创建日志的头文件标题
void ScanWidget::createLogTitle()
{
    QDir dir;
    //文件夹路径
    QString dir_path = QString("%1/.deepin-defender/natiav/").arg(QDir::homePath());
    if (!dir.exists(dir_path)) {
        dir.mkpath(dir_path);
    }
    QDateTime date  = QDateTime::currentDateTime();
    QString time = date.toString("yyyyMMddhhmmss");
    //时间 .log 作为文件名
    m_logName = dir_path + time + ".log";
    QFile f(m_logName);
    if (!f.open(QIODevice::ReadOnly | QIODevice::WriteOnly | QIODevice::Text)) {
        qDebug() << "==Open ScanDate.log failed." << endl;
        return;
    }
}

// 当前行选中
void ScanWidget::currentRowClicked(QModelIndex index)
{
    // 当前点击的位置坐标
    int row = index.row();
    int column = index.column();
    QString operaFlag = m_model->index(row, TABLE_COL_OPERATION_FLAG).data(Qt::DisplayRole).toString();

    // 如果可以通过文字点击实现选中，　条件必须是　不在扫描中，当前行没有操作，并且操作栏不可以点击
    if ((column != TABLE_COL_OPEARTION) && (operaFlag == "false")) {
        // 为了避免操作 不可以在操作列点击
        DWidget *widget = m_tableView->indexWidget(m_model->index(row, TABLE_COL_RISKS));
        // 多选框对象
        DCheckBox *check = widget->findChild<DCheckBox *>("singleCheck");
        // 按钮的状态
        int status = check->checkState();
        // 改变按钮状态
        if (status == 0) {
            check->setCheckState(Qt::CheckState::Checked);
        } else {
            check->setCheckState(Qt::CheckState::Unchecked);
        }
        // 判断全选的状态
        updateCheckAllStaus();
    } else {
        return;
    }
}

// 停止扫描
void ScanWidget::stopScan()
{
    // 设置扫描标题已经暂停
    m_t1->stop();
    // 使能完成按钮
    m_pbFinish->setEnabled(true);
    showFinishPage();
}

void ScanWidget::showFinishPage()
{
    //只有在加载完成以后才可以现实出结果
    // 结果页面有两种 分为没有异常和有异常
    if (m_totalError == 0) {
        // 如果异常数据是0
        if (m_currentType == 0) {
            //全盘扫描结束  没有异常
            m_titleLabel->setText(tr("Full scan finished, no threats found"));
        } else if (m_currentType == 1) {
            m_titleLabel->setText(tr("Quick scan finished, no threats found"));
        } else {
            m_titleLabel->setText(tr("Custom scan finished, no threats found"));
        }
        // 提示为空
        m_scanPath->setText(" ");
        m_scanPath->setToolTip(" ");
    } else {
        // 如果异常数据不为 0
        if (m_currentType == 0) {
            //全盘扫描结束   发现 X处异常
            m_titleLabel->setText(tr("Full scan finished, %1 threats found").arg(m_totalError));
        } else if (m_currentType == 1) {
            m_titleLabel->setText(tr("Quick scan finished, %1 threats found").arg(m_totalError));
        } else {
            m_titleLabel->setText(tr("Custom scan finished, %1 threats found").arg(m_totalError));
        }
        //提示 建议处理已发现的异常
        m_scanPath->setText(tr("Take actions to process the threats please"));
        m_scanPath->setToolTip(tr("Take actions to process the threats please"));
    }


    // 根据扫描出来的异常结果  判断展现第几页
    if (m_totalError == 0) {
        // 如果是没有异常 展现第一页 全选是没有勾选 并且不可以操作
        m_stackwidget->setCurrentIndex(PAGE_EMPTY);
        m_checkAll->setCheckState(Qt::CheckState::Unchecked);
        m_checkAll->setDisabled(true);

        //没有异常 结束按钮出现
        m_combobox->hide();
        m_exec->hide();
        // 状态按钮隐藏 完成按钮出现
        m_pbType->hide();
        m_pbDone->show();

    } else {
        // 如果是有异常 展现第二页 全选是勾选 并且可以操作
        m_stackwidget->setCurrentIndex(PAGE_TABLE);
        m_checkAll->setCheckState(Qt::CheckState::Checked);
        m_checkAll->setDisabled(false);

        m_combobox->show();
        m_exec->show();
        // 需要操作的过程中 状态按钮和完成隐藏
        m_pbType->hide();
        m_pbDone->hide();
    }
    //精度条隐藏
    m_progressBar->hide();

    //写入扫描数据
    writeScanDatalog();

    // mxz/2020.6.22 扫描列表加载完成后，允许列表中所有异常项进行操作
    allowOperation();

    m_pbFinish->hide();

    // 重新扫描按钮显示，因为频繁操作会导致操作不便，设置500毫秒置灰
    m_pbRescan->show();
    m_pbRescan->setEnabled(false);
    QTimer::singleShot(500, this, [ = ]() {
        m_pbRescan->setEnabled(true);
    });

    // 扫描结束后，清空work扫描缓冲区
    m_antiViruswork->getScanResultList().clear();
    // 设置 界面扫描状态 为 结果展示中状态
    if (m_totalError == 0) {
        // 扫描的异常是零 设置状态是结果页面 但无异常
        m_antiViruswork->setScanningStatus(AntiVirusWork::ScanningStatus::ResultNoThreats);
    } else {
        // 设置状态是结果页面
        m_antiViruswork->setScanningStatus(AntiVirusWork::ScanningStatus::Result);
    }
}

// 添加没有没有操作的异常项目
void ScanWidget::addThreats()
{
    // 1. 获得表中说有的数据
    int count = m_model->rowCount();
    // 没有操作的列表
    QString sPathlist;
    //操作了的列表
    QString rmPathlist;
    // 2. 判断表中是不是没有操作
    for (int i = 0; i < count; i++) {
        //是否被操作
        QString operaFlag = m_model->index(i, TABLE_COL_OPERATION_FLAG).data(Qt::DisplayRole).toString();
        if (operaFlag == "false") {
            QString  filePath = m_model->index(i, TABLE_COL_INFO_PATH).data(Qt::DisplayRole).toString();
            // 没有处理的路径 进行拼接
            sPathlist = sPathlist + filePath + "[*]";
        } else {
            // 操作了的
            QString  filePath = m_model->index(i, TABLE_COL_INFO_PATH).data(Qt::DisplayRole).toString();
            rmPathlist = rmPathlist + filePath + "[*]";
        }
    }
    //qDebug() << "sPathlist" << sPathlist;
    // 如果有数据就发送添加信号
    if (!sPathlist.isEmpty()) {
        Q_EMIT onAddThreatsFile(sPathlist);
    }

    if (!rmPathlist.isEmpty()) {
        Q_EMIT readRemoveThreatsFile(rmPathlist);
    }
    //暂停
    Q_EMIT onScanStatus(DissectionState::Stop);
}

//　当前是否在操作中
bool ScanWidget::isCurrentOperating()
{
    bool flag = false;
    // 1. 获得表中说有的数据
    int count = m_model->rowCount();
    for (int i = 0; i < count; i++) {
        QString operaFlag = m_model->index(i, TABLE_COL_OPERATING).data(Qt::DisplayRole).toString();
        // 是否在操作中
        if (operaFlag  == "true") {
            flag = true;
            break ;
        }
    }
    return flag;
}

// 处理异常中提示
void ScanWidget::showOperatingTip()
{
    DDialog *dialog = new DDialog;
    dialog->setMinimumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    dialog->setMaximumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    dialog->setIcon(QIcon::fromTheme("dcc_login_tip"));
    //正在修复异常，请稍后，请稍候
    dialog->setMessage(tr("Removing threats, please wait..."));
    dialog->setWordWrapMessage(true);

    dialog->addButton(tr("OK"));
    connect(dialog, &DDialog::buttonClicked, [ = ](int index, QString) {
        switch (index) {
        case 0:
            dialog->close();
            break;
        default:
            break;
        }
    });
    dialog->exec();
}

