﻿#ifndef SCANWIDGET_H
#define SCANWIDGET_H

#include "../src/widgets/defendertable.h"
#include "cominfo.h"
#include "src/window/modules/common/common.h"
#include "antiviruswork.h"

#include <QWidget>
#include <QObject>
#include <QResizeEvent>
#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QListView>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QDateTime>
#include <QTimer>
#include <QVector>

#include <DCheckBox>
#include <DFontSizeManager>
#include <DApplicationHelper>
#include <DListView>
#include <DToolButton>
#include <DLabel>
#include <QStackedWidget>
#include <DSpinner>
#include <DComboBox>
#include <DGuiApplicationHelper>
#include <DIconButton>

DWIDGET_BEGIN_NAMESPACE
class DSuggestButton;
class DTipLabel;
DWIDGET_END_NAMESPACE

QT_BEGIN_NAMESPACE
class QVBoxLayout;
class QHBoxLayout;
class QLabel;
class QPushButton;

QT_END_NAMESPACE
DWIDGET_USE_NAMESPACE
using namespace def::widgets;

/*
@classname:(ScanWidget)
@author：(mxz)
@discptions:(扫描子页面  就算点击全盘/快速/自定义扫描后进入界面)
@datatime:(04-29)
*/
class ScanWidget : public DWidget
{
    Q_OBJECT
public:
    explicit ScanWidget(AntiVirusWork *work, QWidget *parent = nullptr);
    ~ScanWidget();

    // 设置图表类型
    void setIconType(int nType);
    // 标题类型设置
    void setType(int nType, QString sPath, QString sVersion);
    // 扫描前的准备
    void preScan();

    // 设置扫描路径
    void setScanPath(QString sPath);
    // 设置扫描异常数据
    void setScanErrorNum(int nVal);
    // 设置扫描个数
    void setScanNum(int nValue);
    // 设置进度条的进度
    void setProgressValue(double dVal);
    // 启动定时器
    void startTimer();
    // 全部修复  隔离 信任的操作
    void allOPeration(Itemtype type);
    // 设置操作结果
    void setOperationResult(QString sPath, QString sMessage, Itemtype type);
    // 写入扫描数据日志
    void writeScanDatalog();
    // 更新全选状态
    void updateCheckAllStaus();
    // 添加扫描出来的结果
    void appendFileInfo(FileInfo info);

    // 允许操作
    void allowOperation();
    // 是否全部操作
    bool finishAllOperation();
    // 获得操作列表信息
    QList<int> getOperationinfo();

    // 文件操作  类型路径
    void operationFile(Itemtype type, QString sFilepath);

    // 关闭当前行操作
    void closeRowOperation(int nIndex);

    // 判断可以操作的按钮状态
    bool checkOperationBtStatus();

    // 一行的单独操作
    void singlRowOperation(QString filepath, Itemtype nType);

    // 创建日志的头文件标题
    void createLogTitle();

    // 当前行选中
    void currentRowClicked(QModelIndex index);

    // 停止操作
    void stopScan();

    // 结束页面
    void showFinishPage();

    // 添加没有没有操作的异常项目
    void addThreats();

    //　当前是否在操作中
    bool isCurrentOperating();

    // 处理中提示框
    void showOperatingTip();

private:


protected:
    void resizeEvent(QResizeEvent *event);
    bool eventFilter(QObject *obj, QEvent *event);
    // 刷新table 因字体改变
    void refreshTableByFontChange();

    // 因表格宽度改变表格
    void updateTableForWidth(int nLogicalIndex, int nOldSize, int nNewSize);
private:
    //初始化界面
    void initGUI();
    //初始化中间tableview布局
    QHBoxLayout *initTableWidget();

Q_SIGNALS:
    //返回上一层信号   即返回到扫描主页面
    void onBackpage();
    // 扫描状态
    void onScanStatus(DissectionState nStatus);
    // 立即修复所有问题文件
    void onAllRepairProblemFile(QString sPath);
    // 全部添加隔离
    void onAllIsolationFile(bool isbFlag, QString sPath);
    // 全部添加信任
    void onAllTruestFile(bool bFlag, QString sPath);

    // 添加异常
    void onAddThreatsFile(QString spath);

    // 准备移除的异常
    void readRemoveThreatsFile(QString spath);
    // 再次扫描
    void scanAgain(int nType, QString sPath, QString sVersion);

    // 批量处理文件
    void senBatchOperation(QList<QString>, int nType);
public slots:
    // 扫描结束
    void doScanFinishClick();
    // 重新扫描
    void doReScan();

    // 暂停 继续 点击
    void doTypeButtonClick();
    // 更新扫描时间
    void updateTime();
    // 全选
    void doCheckAllClick(int nStatus);
    // 立即修复
    void doRepairClick();
    // 立即隔离
    void doIsolationClick();
    // 立即添加信任
    void doTrustClick();

    //dbus *****************回馈信号****************
    // 接受扫描回调信号
    void doDissectionResult(QString sPath, double dProgress, int eType, QString sDetailInfo);
    // 扫描结束
    void doScanFinsh();
    // 修复回馈信号
    void doRepairResult(QString sPath, QString sMessage);
    // 隔离回馈信号
    void doIsolationResult(QString sPath, QString sMessage);
    // 信任回馈信号
    void doTruestResult(QString sPaths, QString sMessage);
    // 单个check点击
    void doSingalCheckClick();

    // 完成按钮点击
    void doneButtonClick();

    // 执行按钮点击
    void execButtonClick();

    // 主题类型改变
    void themeTypeChange(ColorType themeType);

private:
    AntiVirusWork               *m_antiViruswork;
    DPushButton                 *m_pbFinish = nullptr;                          // 结束按钮
    DPushButton                 *m_pbRescan = nullptr;                          // 重新扫描按钮
    DPushButton                 *m_pbType = nullptr;                            // 暂停、继续、
    DLabel                      *m_titleLabel = nullptr;                        // 标题
    DLabel                      *m_scanPath = nullptr;                          // 扫描路径
    DLabel                      *m_Santime = nullptr;                           // 扫描时间
    DLabel                      *m_errNum = nullptr;                            // 发现异常
    DLabel                      *m_iconType = nullptr;                          // 左上角图标
    DLabel                      *m_scanNum = nullptr;                           // 扫描项目个数
    DTableView                  *m_tableView = nullptr;
    QProgressBar                *m_progressBar = nullptr;                       // 精度条
    QTimer                      *m_t1 = nullptr;                                // 扫描计算定时器

    DComboBox                   *m_combobox = nullptr;                          // 下拉框
    DSuggestButton              *m_exec = nullptr;                              // 执行按钮
    DPushButton                 *m_scanBtns = nullptr;
    QMenu                       *m_tipMenu = nullptr;
    QAction                     *m_repairAction = nullptr;                      // 修复
    QAction                     *m_deleteAction = nullptr;                      // 删除
    QAction                     *m_isolationAction = nullptr;                   // 隔离
    QAction                     *m_trustAction = nullptr;                       // 信任

    int                         m_currentType;                                  // 当前类型
    QString                     m_currentScanName;                              // 当前扫描名称
    QString                     m_currentScanType;                              // 当前扫描名称 类型
    int                         m_countTime;                                    // 开始计算时间
    int                         m_totalError = 0;                               // 异常总数
    int                         m_totalScan = 0;                                // 扫描总数
    int                         m_operationTime;                                // 修复等操作
    int                         m_operationCount = 0;                           // 每次操作数据

    DCheckBox                   *m_checkAll = nullptr;                          // 全选
    QStandardItemModel          *m_model = nullptr;

    QString                     m_logName;                                      // 日志名称
    QString                     m_scanPathInfo;                                 // 扫描路径
    QString                     m_lastFileName;                                 // 最后一个文件名陈
    bool                        m_flagPause = true;                             // 暂停标识
    QString                     m_currentVersion;                               // 病毒库版本

    QStackedWidget              *m_stackwidget = nullptr;                       // stack 栈有两个页面  第一页图  第二页表
    DSuggestButton              *m_pbDone = nullptr;                            // 完成按钮
    DGuiApplicationHelper       *m_guiHelper = nullptr;                         // 方便得到系统主题

};

#endif // SCANWIDGET_H
