﻿/*
 * Copyright (C) 2019 ~ 2020 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "cleanerwidget.h"
#include "widgets/checkboxdelegate.h"
#include "src/window/modules/common/common.h"

#include <DTreeWidget>
#include <DDesktopServices>
#include <DLabel>
#include <DPushButton>
#include <DListView>
#include <DSpinner>

#include <QMenu>
#include <QtConcurrent>
#include <QThread>
#include <QDesktopServices>
#include <QProcess>
#include <QDebug>

DWIDGET_USE_NAMESPACE

DLabel *m_supportTypeTip = nullptr;
DPushButton *m_scanBtn = nullptr;
DListView *m_typeListView = nullptr;
QTreeWidget *m_treeWidget = nullptr;
QTreeWidgetItem *m_appCacheRootItem = nullptr;
QTreeWidgetItem *m_appLogsRootItem = nullptr;
QTreeWidgetItem *m_crashRepsRootItem = nullptr;
QTreeWidgetItem *m_pkgsRootItem = nullptr;
QTreeWidgetItem *m_shellCacheRootItem = nullptr;
QTreeWidgetItem *m_trashRootItem = nullptr;
DSpinner *m_spinner = nullptr;
DPushButton *m_cleanBtn = nullptr;
quint64 m_totalSize = 0;

quint64 m_totalCleanedSize = 0;


CleanerWidget::CleanerWidget(QWidget *parent)
    : QWidget(parent)
{
    //init ui
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->setContentsMargins(10, 10, 10, 10);
    setLayout(mainLayout);

    mainLayout->addLayout(initCleanerHead());

    mainLayout->addLayout(initSupportTypeWidget());

    mainLayout->addLayout(initScanResult());
    m_treeWidget->setVisible(false);
    m_spinner->hide();
    m_cleanBtn->setVisible(false);


    connect(m_scanBtn, &DPushButton::clicked, [ = ] {
        m_typeListView->setVisible(false);
        m_treeWidget->setVisible(true);

        m_spinner->show();
        m_spinner->start();
        m_cleanBtn->setVisible(false);

        QtConcurrent::run(this, &CleanerWidget::scan);
    });

    connect(m_treeWidget, &QTreeWidget::itemClicked, this, &CleanerWidget::handleItemClicked);
    connect(m_treeWidget, &QTreeWidget::customContextMenuRequested, this, &CleanerWidget::prepareMenu);
    connect(m_cleanBtn, &DPushButton::clicked, this, &CleanerWidget::clean);

    connect(this, &CleanerWidget::sendScanFinished, [ = ] {
        m_spinner->setVisible(false);
        m_spinner->stop();
        m_cleanBtn->setVisible(true);

        m_supportTypeTip->setText(QString("Scan is successful, and a total of %1 files were found this time")
                                  .arg(formatBytes(m_totalSize, 1)));
    });

}

CleanerWidget::~CleanerWidget()
{
    //注销时，断开相关信号链接

}

QHBoxLayout *CleanerWidget::initCleanerHead()
{
    QHBoxLayout *scanLayout = new QHBoxLayout;
    scanLayout->setAlignment(Qt::AlignmentFlag::AlignLeft);

    DLabel *cleanerIconLabel = new DLabel;
    cleanerIconLabel->setPixmap(QIcon::fromTheme("ddc_cleaner").pixmap(80, 80));
    scanLayout->addWidget(cleanerIconLabel);

    QVBoxLayout *tipLayout = new QVBoxLayout;
    scanLayout->addLayout(tipLayout, 1);

    DLabel *cleanerTip = new DLabel;
    cleanerTip->setAlignment(Qt::AlignmentFlag::AlignLeft);
    cleanerTip->setText("Clean up disk space");
    tipLayout->addWidget(cleanerTip);

    m_supportTypeTip = new DLabel;
    m_supportTypeTip->setAlignment(Qt::AlignmentFlag::AlignLeft);
    m_supportTypeTip->setText("Support scanning the following items");
    tipLayout->addWidget(m_supportTypeTip);

    m_scanBtn = new DPushButton("Scan");
    m_scanBtn->setFixedSize(150, 60);
    scanLayout->addWidget(m_scanBtn, 0, Qt::AlignmentFlag::AlignRight);

    return scanLayout;
}

QHBoxLayout *CleanerWidget::initSupportTypeWidget()
{
    QHBoxLayout *layout = new QHBoxLayout;

    m_typeListView = new DListView;
    m_typeListView->setEditTriggers(QListView::NoEditTriggers);
    m_typeListView->setResizeMode(QListView::Adjust);
    m_typeListView->setSelectionMode(DListView::SelectionMode::NoSelection);
    m_typeListView->setTextElideMode(Qt::ElideRight);
    m_typeListView->setDragEnabled(false);
    m_typeListView->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    m_typeListView->setSizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::MinimumExpanding);
    m_typeListView->clearSelection();
    m_typeListView->clearFocus();
    m_typeListView->clearMask();
    m_typeListView->setSpacing(5);
    m_typeListView->setIconSize(QSize(100, 100));
    m_typeListView->setItemSize(QSize(160, 160));
    m_typeListView->setViewMode(DListView::ViewMode::IconMode);
    m_typeListView->setMinimumHeight(400);
//    m_typeListView->setStyleSheet("background-color: red");
    layout->addWidget(m_typeListView, 0, Qt::AlignmentFlag::AlignVCenter);

    QStandardItemModel *typeListViewModel = new QStandardItemModel;
    m_typeListView->setModel(typeListViewModel);

    QStandardItem *appCachesTypeItem = new QStandardItem;
    appCachesTypeItem->setIcon(QIcon::fromTheme("ddc_cleaner_app_cache"));
    appCachesTypeItem->setText("Application Caches");
    typeListViewModel->appendRow(appCachesTypeItem);

    QStandardItem *appLogsTypeItem = new QStandardItem;
    appLogsTypeItem->setIcon(QIcon::fromTheme("ddc_cleaner_app_logs"));
    appLogsTypeItem->setText("Application Logs");
    typeListViewModel->appendRow(appLogsTypeItem);

    QStandardItem *crashReportsTypeItem = new QStandardItem;
    crashReportsTypeItem->setIcon(QIcon::fromTheme("ddc_cleaner_crash"));
    crashReportsTypeItem->setText("Crash Reports");
    typeListViewModel->appendRow(crashReportsTypeItem);

    QStandardItem *pkgsTypeItem = new QStandardItem;
    pkgsTypeItem->setIcon(QIcon::fromTheme("ddc_cleaner_packages"));
    pkgsTypeItem->setText("Package Caches");
    typeListViewModel->appendRow(pkgsTypeItem);

    QStandardItem *trashTypeItem = new QStandardItem;
    trashTypeItem->setIcon(QIcon::fromTheme("ddc_cleaner_packages"));
    trashTypeItem->setText("Trash");
    typeListViewModel->appendRow(trashTypeItem);

    return layout;
}

QVBoxLayout *CleanerWidget::initScanResult()
{
    QVBoxLayout *layout = new QVBoxLayout;

    m_treeWidget = new QTreeWidget;
    m_treeWidget->setFrameShape(QFrame::NoFrame);
    m_treeWidget->setSelectionMode(QAbstractItemView::SelectionMode::NoSelection);
    // 设置复选框代理
    m_treeWidget->setItemDelegateForColumn(0, new CheckBoxDelegate);
    m_treeWidget->setColumnCount(2);
    m_treeWidget->setColumnWidth(0, 480);
    m_treeWidget->setHeaderLabels({"File Name", "File Size"});
    m_treeWidget->setContextMenuPolicy(Qt::CustomContextMenu);
    layout->addWidget(m_treeWidget);

    m_spinner = new DSpinner;
    m_spinner->setFixedSize(40, 40);
    layout->addWidget(m_spinner, 1, Qt::AlignmentFlag::AlignCenter);

    m_cleanBtn = new DPushButton("Clean");
    m_cleanBtn->setFixedWidth(100);
    layout->addWidget(m_cleanBtn, 0, Qt::AlignmentFlag::AlignRight);

    return layout;
}

void CleanerWidget::scan()
{
    m_totalSize = 0;
    m_treeWidget->clear();

    QThread::msleep(200);
    addRoot(CleanerWidget::Categories::APPLICATION_CACHES, "App Caches", CleanerWork::getAppCaches());
    QThread::msleep(200);
    addRoot(CleanerWidget::Categories::APPLICATION_LOGS, "App Logs", CleanerWork::getAppLogs());
    QThread::msleep(200);
    addRoot(CleanerWidget::Categories::CRASH_REPORTS, "Crash Reports", CleanerWork::getCrashReports());
    QThread::msleep(200);
    addRoot(CleanerWidget::Categories::PACKAGE_CACHE, "Dpkg Packages", CleanerWork::getDpkgPackages());
    QThread::msleep(200);
    addRoot(CleanerWidget::Categories::BASHSHELL_HISTORY, "Shell History", CleanerWork::getbashShellHistory());

    m_treeWidget->update();
    Q_EMIT sendScanFinished();
}

void CleanerWidget::addRoot(const CleanerWidget::Categories categories, const QString title,
                            const QFileInfoList infoList, bool noChild)
{
    QTreeWidgetItem *rootItem = new QTreeWidgetItem;
    // 设置根节点图标
    QString sThemeIconPath = "";
    switch (categories) {
    case CleanerWidget::Categories::APPLICATION_CACHES:
        sThemeIconPath = "ddc_cleaner_app_cache";
        m_appCacheRootItem = rootItem;
        break;
    case CleanerWidget::Categories::APPLICATION_LOGS:
        sThemeIconPath = "ddc_cleaner_app_logs";
        m_appLogsRootItem = rootItem;
        break;
    case CleanerWidget::Categories::CRASH_REPORTS:
        sThemeIconPath = "ddc_cleaner_crash";
        m_crashRepsRootItem = rootItem;
        break;
    case CleanerWidget::Categories::PACKAGE_CACHE:
        sThemeIconPath = "ddc_cleaner_packages";
        m_pkgsRootItem = rootItem;
        break;
    case CleanerWidget::Categories::BASHSHELL_HISTORY:
        sThemeIconPath = "ddc_cleaner_packages";
        m_shellCacheRootItem = rootItem;
        break;
    case CleanerWidget::Categories::TRASH:
        sThemeIconPath = "ddc_cleaner_packages";
        m_trashRootItem = rootItem;
        break;
    }
    rootItem->setData(0, Qt::ItemDataRole::DecorationRole, sThemeIconPath);

    m_treeWidget->addTopLevelItem(rootItem);

    quint64 totalSize = 0;

    if (!noChild) {
        int itemCount = 0;

        for (const QFileInfo &info : infoList) {
            const QString path = info.absoluteFilePath();
            quint64 fileSize = CleanerWork::getFileSize(path);

            if (fileSize == 0) {
                continue;
            }

            addChild(path, info.fileName(), fileSize, rootItem);

            totalSize += fileSize;
            ++itemCount;
        }

        rootItem->setText(0, QString("%1 (%2)").arg(title).arg(itemCount));
        QString sTotalSize = formatBytes(totalSize, 1);
        rootItem->setText(1, QString("%1").arg(sTotalSize));
    }

    m_totalSize += totalSize;
}

void CleanerWidget::addChild(const QString sFilePath, const QString text,
                             const quint64 size, QTreeWidgetItem *parent)
{
    QTreeWidgetItem *item = new QTreeWidgetItem(parent);
    // 设置图标
    item->setData(0, Qt::ItemDataRole::DecorationRole, text.split("_").first());
    // 设置文件名
    item->setText(0, text);
    // 设置文件大小
    item->setData(1, Qt::ItemDataRole::EditRole, size);
    QString sSize = formatBytes(size, 1);
    item->setText(1, sSize);
    // 设置文件路径
    item->setData(2, Qt::ItemDataRole::EditRole, sFilePath);
}

void CleanerWidget::handleItemClicked(QTreeWidgetItem *item, const int column)
{
    if (column != 0) {
        return;
    }

    qDebug() << "checkbox clicked";
    bool isbChecked = item->data(0, Qt::ItemDataRole::CheckStateRole).toBool();

    for (int i = 0; i < item->childCount(); ++i) {
        item->child(i)->setData(0, Qt::ItemDataRole::CheckStateRole, isbChecked);
    }
}

void CleanerWidget::prepareMenu(const QPoint &pos)
{
    QTreeWidgetItem *nd = m_treeWidget->itemAt(pos);

    // is top level item
    if (nd->parent() == nullptr) {
        return;
    }

    // 获取路径
    QString path = nd->data(2, Qt::ItemDataRole::EditRole).toString();
    QAction *act = new QAction("Display in file manager", this);
    connect(act, &QAction::triggered, this, [=] {
        if (path.isEmpty() || !QFile::exists(path)) {
            return;
        }

        QUrl url = QUrl::fromLocalFile(QFileInfo(path).absoluteFilePath());
        Dtk::Widget::DDesktopServices::showFileItem(url);
    });

    QMenu menu(this);
    menu.addAction(act);
    menu.exec(m_treeWidget->mapToGlobal(pos));
}

void CleanerWidget::clean()
{
    m_totalCleanedSize = 0;
    QStringList fileToDeleteList = {};
    QList<QTreeWidgetItem *> toDeleteChildItems = {};

    for (int i = 0; i < m_treeWidget->topLevelItemCount(); ++i) {
        QTreeWidgetItem *topItem = m_treeWidget->topLevelItem(i);

        for (int j = 0; j < topItem->childCount(); j++) {
             QTreeWidgetItem *childItem = topItem->child(j);
             bool isbChecked = childItem->data(0, Qt::ItemDataRole::CheckStateRole).toBool();
             if (isbChecked) {
                 QString sPath = childItem->data(2, Qt::ItemDataRole::EditRole).toString();
                 fileToDeleteList.append(sPath);

                 quint64 fileSize = childItem->data(1, Qt::ItemDataRole::EditRole).toUInt();
                 m_totalCleanedSize += fileSize;

                 toDeleteChildItems.append(childItem);
             }
        }
    }

    // remove selected files.
    if (!fileToDeleteList.isEmpty()) {
        QProcess *process = new QProcess;
        bool isbExced = false;
        int iExeRet = process->execute("pkexec", QStringList() << "rm" << "-rf" << fileToDeleteList);

        if (0 == iExeRet) {
            isbExced = true;
        }

        process->deleteLater();

        if (isbExced) {
            qDebug() << "exe sus";
            for (QTreeWidgetItem *item : toDeleteChildItems) {
                delete(item);
            }
        } else {
            qDebug() << "failed";
        }
    }

//    cleanAppLogs();
}

QFileInfoList getDirFileInfoList(QString sDir)
{
    QDir dir(sDir);
    QFileInfoList fileList = dir.entryInfoList(QDir::Files | QDir::NoSymLinks);
    QFileInfoList folderList = dir.entryInfoList(QDir::Dirs | QDir::NoDotAndDotDot);

    for(int i = 0; i != folderList.size(); i++)
    {
         QString sPath = folderList.at(i).absoluteFilePath();
         QFileInfoList childFileList = getDirFileInfoList(sPath);
         fileList.append(childFileList);
    }

    return fileList;
}

void CleanerWidget::cleanAppLogs()
{
    if(nullptr == m_appLogsRootItem) {
        return;
    }

    QTreeWidgetItem *topItem = m_appLogsRootItem;

    QStringList fileToDeleteList = {};
    QList<QTreeWidgetItem *> toDeleteChildItems = {};

    for (int j = 0; j < topItem->childCount(); j++) {
         QTreeWidgetItem *childItem = topItem->child(j);
         bool isbChecked = childItem->data(0, Qt::ItemDataRole::CheckStateRole).toBool();
         if (isbChecked) {
             QString sPath = childItem->data(2, Qt::ItemDataRole::EditRole).toString();
             QFileInfo fileInfo(sPath);
             if (fileInfo.isFile()) {
                 fileToDeleteList.append(sPath);
             } else if (fileInfo.isDir()) {
                 QStringList childFilePathList = {};
                 for (QFileInfo info : getDirFileInfoList(sPath)) {
                    childFilePathList.append(info.filePath());
                 }
                 fileToDeleteList.append(childFilePathList);
             }


             quint64 fileSize = childItem->data(1, Qt::ItemDataRole::EditRole).toUInt();
             m_totalCleanedSize += fileSize;

             toDeleteChildItems.append(childItem);
         }
    }

    // debug
    for (QString file : fileToDeleteList) {
        qDebug() << file;
    }


    // remove selected files.
    if (!fileToDeleteList.isEmpty()) {
        QProcess *process = new QProcess;
        bool isbExced = false;
//        process->execute("pkexec", QStringList() << "sudo" << "su");
        int iExeRet = process->execute("pkexec", QStringList() << "sudo" << "cat" << "/dev/null" << ">" << fileToDeleteList);

        if (0 == iExeRet) {
            isbExced = true;
        }

        process->deleteLater();

        if (isbExced) {
            qDebug() << "exe sus";
            for (QTreeWidgetItem *item : toDeleteChildItems) {
                delete(item);
            }
        } else {
            qDebug() << "failed";
        }
    }
}
