#include "checkboxdelegate.h"

#include <QPainter>
#include <QIcon>

/*
 * CheckBoxDelegate
 */
static QRect CheckBoxRect(const QStyleOptionViewItem &viewItemStyleOptions)/*const*/
{
    //绘制按钮所需要的参数
    QStyleOptionButton checkBoxStyleOption;
    //按照给定的风格参数 返回元素子区域
    QRect checkBoxRect = QApplication::style()->subElementRect( QStyle::SE_CheckBoxIndicator, &checkBoxStyleOption);
    //返回QCheckBox坐标
//    QPoint checkBoxPoint(viewItemStyleOptions.rect.x() + viewItemStyleOptions.rect.width() / 2 - checkBoxRect.width() / 2,
//                         viewItemStyleOptions.rect.y() + viewItemStyleOptions.rect.height() / 2 - checkBoxRect.height() / 2);
    QPoint checkBoxPoint(viewItemStyleOptions.rect.x(),
                         viewItemStyleOptions.rect.y() + viewItemStyleOptions.rect.height() / 2 - checkBoxRect.height() / 2);
    //返回QCheckBox几何形状
    return QRect(checkBoxPoint, checkBoxRect.size());
}

CheckBoxDelegate::CheckBoxDelegate(QObject *parent)
    : QStyledItemDelegate(parent)
    , m_iCheckboxCol(0)
{

}

void CheckBoxDelegate::setCheckboxCol(int col)
{
    m_iCheckboxCol = col;
}

// 绘制复选框
void CheckBoxDelegate::paint(QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index)const
{
    bool checked = index.model()->data(index, Qt::CheckStateRole).toBool();
    QString sText = index.model()->data(index, Qt::DisplayRole).toString();
    QString sThemeIcon = index.model()->data(index, Qt::DecorationRole).toString();

    if(index.column() == m_iCheckboxCol){
        QStyleOptionButton checkBoxStyleOption;
        checkBoxStyleOption.state |= QStyle::State_Enabled;
        checkBoxStyleOption.state |= checked? QStyle::State_On : QStyle::State_Off;
        checkBoxStyleOption.rect = CheckBoxRect(option);

        QApplication::style()->drawControl(QStyle::CE_CheckBox, &checkBoxStyleOption, painter);

        // 绘制图标
        int iIconX = checkBoxStyleOption.rect.right() + 10;
        int iIconY = option.rect.top();
        int iIconW = option.rect.height();
        QIcon icon = QIcon::fromTheme(sThemeIcon);
        if (icon.isNull()) {
            icon = QIcon::fromTheme("application-x-executable");
        }
        QPixmap pixmap = icon.pixmap(iIconW, iIconW);
        painter->drawPixmap(iIconX, iIconY, pixmap);

        // 绘制文字
        int iTextX = iIconX + 5 + iIconW;
        int iTextY = checkBoxStyleOption.rect.bottom();

        painter->drawText(iTextX, iTextY, sText);
    } else {
        QStyledItemDelegate::paint(painter, option, index);
    }
}

// 响应鼠标事件，更新数据
bool CheckBoxDelegate::editorEvent(QEvent *event, QAbstractItemModel *model,
                                   const QStyleOptionViewItem &option,
                                   const QModelIndex &index) {
    if(index.column() == m_iCheckboxCol){
        if((event->type() == QEvent::MouseButtonRelease) ||
                (event->type() == QEvent::MouseButtonDblClick)) {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent*>(event);
            if(mouseEvent->button() != Qt::LeftButton ||
                    !CheckBoxRect(option).contains(mouseEvent->pos())) {
                return true;
            }
            if(event->type() == QEvent::MouseButtonDblClick) {
                return true;
            }
        } else if(event->type() == QEvent::KeyPress) {
            if(static_cast<QKeyEvent*>(event)->key() != Qt::Key_Space &&
                    static_cast<QKeyEvent*>(event)->key() != Qt::Key_Select){
                return false;
            }
        }else{
            return false;
        }

        bool checked = index.model()->data(index, Qt::CheckStateRole).toBool();
        int data = checked ? 0 : 1;     //互逆
        return model->setData(index, data, Qt::CheckStateRole);
    } else {
        return QStyledItemDelegate::editorEvent(event, model, option, index);
    }
}
