#include "common.h"

#include <QFontMetrics>
#include <QDebug>

const long KBCount = 1 << 10;
const long MBCount = 1 << 20;
const long GBCount = 1 << 30;
const long TBCount = long(1) << 40;
/*!
 * \brief wordwrapText，根据指定换行长度向字符串中添加换行符
 * \param font 当前使用的字体
 * \param text　带换行的字符串
 * \param nLabelLenth　指定换行长度
 */
QString wordwrapText(const QFontMetrics &font, const QString &text, int nLabelLenth)
{
    int textLenth = font.width(text);
    if (textLenth <= nLabelLenth) return text;

    QString retStr;
    QString lineString;
    long lenthTmp = 0; //积累的字符串长度，到达标签长度时重置，并向返回的字符串retStr后添加"\n"
    for (int i = 0; i < text.size(); i++) {
        lineString.append(text.at(i));
        lenthTmp = font.width(lineString);
        if (nLabelLenth < lenthTmp) {
            lineString = lineString.left(lineString.count() - 1);
            i--;
            lineString.append("\n");
            retStr.append(lineString);
            lineString.clear();
        }

//        QString strTmp = text.at(i);
//        lenthTmp += font.width(strTmp) + 1;
//        qDebug() << "strTmp lenth" << font.width(strTmp);
//        if (nLabelLenth < lenthTmp) {
//            qDebug() << "nLabelLenth" << nLabelLenth;
//            qDebug() << "lenthTmp" << lenthTmp;
//            retStr.append("\n");
//            lenthTmp = font.width(strTmp);
//            qDebug() << retStr;
//        }
//        retStr.append(strTmp);
    }
    retStr.append(lineString);

    return retStr;


//    int nTextSize = font.width(text);
//    if (nTextSize > nLabelLenth) {
//        int nPos = 0;
//        long nOffset = 0;
//        for (int i = 0; i < text.size(); i++) {
//            nOffset += font.width(text.at(i));
//            if (nOffset >= nLabelLenth) {
//                nPos = i;
//                break;
//            }
//        }

//        nPos = (nPos - 1 < 0) ? 0 : nPos - 1; //zr.主要是为了防止截取字符串的位置是-1.回退一个字节的作用是换行,防止字符有一半显示一半未显示

//        QString qstrLeftData = text.left(nPos);
//        QString qstrMidData = text.mid(nPos);
//        return qstrLeftData + "\n" + wordwrapText(font, qstrMidData, nLabelLenth);
//    }
//    return text;
}

// 根据流量数据大小改变单位
QString changeFlowValueUnit(double input, int prec)
{
    QString flowValueStr;
    //根据输入值设定单位
    int unit = 1;
    if (1 << 20 < input) {
        //单位为GB
        unit = 1 << 20;
        flowValueStr = QString::number(input / unit, 'd', prec) + "GB";
    } else if (1 << 10 < input) {
        //单位为MB
        unit = 1 << 10;
        flowValueStr = QString::number(input / unit, 'd', prec) + "MB";
    } else {
        flowValueStr = QString::number(input / unit, 'd', prec) + "KB";
    }
    return flowValueStr;
}

// 根据网速大小改变单位
QString changeFlowSpeedValueUnit(double input, int prec)
{
    QString flowSpeedValueStr;
    //根据输入值设定单位
    int unit = 1;
    if (1 << 20 < input) {
        //单位为GB
        unit = 1 << 20;
        flowSpeedValueStr = QString::number(input / unit, 'd', prec) + "GB/s";
    } else if (1 << 10 < input) {
        //单位为MB
        unit = 1 << 10;
        flowSpeedValueStr = QString::number(input / unit, 'd', prec) + "MB/s";
    } else {
        flowSpeedValueStr = QString::number(input / unit, 'd', prec) + "KB/s";
    }
    return flowSpeedValueStr;
}

QString formatBytes(double input, int prec)
    {
        QString flowValueStr;
        if (KBCount > input) {
            flowValueStr = QString::number(input / 1, 'd', prec) + "B";
        } else if (MBCount > input) {
            flowValueStr = QString::number(input / KBCount, 'd', prec) + "KB";
        } else if (GBCount > input) {
            flowValueStr = QString::number(input / MBCount, 'd', prec) + "MB";
        } else if (TBCount > input) {
            flowValueStr = QString::number(input / GBCount, 'd', prec) + "GB";
        } else if (TBCount >= input) {
            // 大于TB单位
            flowValueStr = QString::number(input / TBCount, 'd', prec) + "TB";
        }
        return flowValueStr;
}
