/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "examdetailpage.h"
#include "widgets/scoreprogressbar.h"
#include "widgets/fixitemwindow.h"

#include <DFontSizeManager>
#include <DApplicationHelper>
#include <DSuggestButton>
#include <DCommandLinkButton>
#include <DProgressBar>

#include <QDebug>
#include <QTime>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QRadioButton>
#include <QButtonGroup>
#include <QProgressBar>
#include <QScrollArea>
#include <QGSettings>
#include <QThread>
#include <QStackedLayout>
#include <QGraphicsDropShadowEffect>
#include <QGraphicsOpacityEffect>

DWIDGET_USE_NAMESPACE

const QColor ScoreVerySafeColor = QColor(35, 196, 0, 255);
const QColor ScoreSafeColor = QColor(158, 202, 0, 255);
const QColor ScoreHaveDangersColor = QColor(255, 93, 0, 255);
const QColor ScoreUnderIllnessColor = QColor(216, 49, 108, 255);

//shadow color
const QColor ScoreVerySafeShadowColor = QColor(35, 196, 0, 128);
const QColor ScoreSafeShadowColor = QColor(158, 202, 0, 128);
const QColor ScoreHaveDangersShadowColor = QColor(255, 93, 0, 128);
const QColor ScoreUnderIllnessShadowColor = QColor(216, 49, 108, 128);

// 进度条背景色
const QColor ScoreBackgroundLight = QColor(0, 0, 0, 25);
const QColor ScoreBackgroundDark = QColor(255, 255, 255, 25);

#define HONE_PAGE_ITEM_WIDTH            390                 // 首页体检项显示宽度

ExamDetailPage::ExamDetailPage(HomePageModel *mode, HomePageWork *work, QWidget *parent)
    : QWidget(parent)
    , m_mode(mode)
    , m_work(work)
    , m_leftLayout(nullptr)
    , m_scoreLabel(nullptr)
    , m_currentValue(0)
    , m_value(0)
    , m_moveDir(1)
    , m_scoreTimer(nullptr)
    , m_centLabel(nullptr)
    , m_scoreProg(nullptr)
    , m_progressBarShadow(nullptr)
    , m_securLeftLabel(nullptr)
    , m_securRightLabel(nullptr)
    , m_itemsArea(nullptr)
    , m_noNeedFixItemImg(nullptr)
{
    QHBoxLayout *mainLayout = new QHBoxLayout;
    mainLayout->setMargin(0);
    setLayout(mainLayout);

    m_leftLayout = new QVBoxLayout;
    m_leftLayout->setContentsMargins(20, 50, 0, 0);
    m_leftLayout->setAlignment(Qt::AlignLeft);
    mainLayout->addLayout(m_leftLayout);

    //safety score
    m_scoreTimer = new QTimer;
    m_scoreTimer->setInterval(10);
    m_moveDir = 1;
    m_currentValue = 0;

    QHBoxLayout *scoreLayout = new QHBoxLayout; //系统评分行，水平布局
    scoreLayout->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
    m_leftLayout->addLayout(scoreLayout);

    m_scoreLabel = new QLabel;
    //m_scoreLabel->setFixedSize(80, 72);
    m_scoreLabel->setAlignment(Qt::AlignRight | Qt::AlignBottom);
    QFont scoreFont = m_scoreLabel->font();
    scoreFont.setBold(true);
    m_scoreLabel->setFont(scoreFont);
    m_scoreLabel->setContentsMargins(0, 0, 0, 0);
    DFontSizeManager::instance()->bind(m_scoreLabel, DFontSizeManager::T1);
    scoreLayout->addWidget(m_scoreLabel, 0, Qt::AlignRight | Qt::AlignBottom);

    m_centLabel = new QLabel(tr("points"));
    QString systemLanguageStr = QLocale::system().name();
    if ('z' != systemLanguageStr[0] || 'h' != systemLanguageStr[1])
        m_centLabel->setText(""); //如果语言不是中文就隐藏
    m_centLabel->setContentsMargins(0, 0, 0, 10);
    DFontSizeManager::instance()->bind(m_centLabel, DFontSizeManager::T5);
    scoreLayout->addWidget(m_centLabel, 0, Qt::AlignLeft | Qt::AlignBottom);

    //safety score right part
    scoreLayout->addSpacing(13);
    QVBoxLayout *scoreRightLayout = new QVBoxLayout;
    scoreRightLayout->setContentsMargins(0, 0, 0, 0);
    scoreLayout->addLayout(scoreRightLayout);

    //system security
    QHBoxLayout *securityLayout = new QHBoxLayout;
    securityLayout->setContentsMargins(0, 0, 0, 0);
    securityLayout->setSpacing(0);
    securityLayout->setMargin(0);
    securityLayout->setAlignment(Qt::AlignLeft);
    scoreRightLayout->addLayout(securityLayout);

    m_securLeftLabel = new QLabel(tr("Overview:")  + QString(" "));
    QFont securLabelFont = m_securLeftLabel->font();
    securLabelFont.setBold(true);
    m_securLeftLabel->setFont(securLabelFont);
    m_securLeftLabel->setContentsMargins(0, 0, 0, 0);
    DFontSizeManager::instance()->bind(m_securLeftLabel, DFontSizeManager::T5);
    securityLayout->addWidget(m_securLeftLabel, 0, Qt::AlignLeft | Qt::AlignBottom);

    m_securRightLabel = new QLabel;
    m_securRightLabel->setFont(securLabelFont);
    m_securRightLabel->setContentsMargins(0, 0, 0, 0);
    DFontSizeManager::instance()->bind(m_securRightLabel, DFontSizeManager::T5);
    securityLayout->addWidget(m_securRightLabel, 0, Qt::AlignLeft | Qt::AlignBottom);

    //score progress bar
    scoreRightLayout->addSpacing(9);
    QStackedLayout *scoreProgressLayout = new QStackedLayout;
    scoreProgressLayout->setStackingMode(QStackedLayout::StackAll);
    scoreProgressLayout->setContentsMargins(0, 0, 0, 0);
    scoreProgressLayout->setMargin(0);
    scoreProgressLayout->setSpacing(0);
    scoreRightLayout->addLayout(scoreProgressLayout);

    QSize scoreBarSize = QSize(220, 12);
    // 分数进度条背景
    m_scoreBg = new ScoreProgressBar;
    m_scoreBg->setFixedSize(scoreBarSize);
    m_scoreBg->setCurrentValue(100);
    scoreProgressLayout->addWidget(m_scoreBg);
    m_scoreBg->lower();

    m_scoreProg = new ScoreProgressBar;
    m_scoreProg->setFixedSize(scoreBarSize);
    m_scoreProg->setColor(QColor(0, 255, 0, 255));
    m_scoreProg->setCurrentValue(0);
    m_scoreProg->setValue(0);
    scoreProgressLayout->addWidget(m_scoreProg);
    m_scoreProg->raise();

    m_progressBarShadow = new QGraphicsDropShadowEffect(this); //阴影
    m_progressBarShadow->setOffset(0, 3);
    m_progressBarShadow->setBlurRadius(20);
    m_progressBarShadow->setColor(QColor(0, 255, 0, 128));
    m_scoreProg->setGraphicsEffect(m_progressBarShadow);
    scoreRightLayout->addSpacing(14);

    //need fix items
    m_leftLayout->addSpacing(40);
    if (DSysInfo::uosEditionType() == DSysInfo::UosEdition::UosCommunity) {
        m_fixItemInfoLst = {
            {"1", safetylevel, "dcc_home_restore", tr("Applications connect to the network directly"), "please use with caution", tr("Ask me always")},
            {"2", safetylevel, "dcc_home_restore", tr("Applications start remote connection automatically"), "please use with caution", tr("Ask me always")},
            {"3", safetylevel, "dcc_home_restore", tr("There are more than %d startup programs"), "please use with caution", tr("Go to Settings")},
            {"4", safetylevel, "dcc_home_restore", tr("Your password policy is low"), "please use with caution", tr("Go to Settings")},
            {"5", safetylevel, "dcc_home_restore", tr("User password has not been changed for more than 90 days"), "please use with caution", tr("Go to Settings")},
            {"6", safetylevel, "dcc_home_restore", tr("The lockscreen time is set to never"), "please use with caution", tr("Go to Settings")},
            {"7", safetylevel, "dcc_home_restore", tr("Your system is not up to date"), "please use with caution", tr("Go to Settings")},
            {"8", safetylevel, "dcc_home_restore", tr("Updates notification is turned off"), "please use with caution", tr("Turn on")},
        };
    }
    else {
        m_fixItemInfoLst = {
            {"1", safetylevel, "dcc_home_restore", tr("Applications connect to the network directly"), "please use with caution", tr("Ask me always")},
            {"2", safetylevel, "dcc_home_restore", tr("Applications start remote connection automatically"), "please use with caution", tr("Ask me always")},
            {"3", safetylevel, "dcc_home_restore", tr("There are more than %d startup programs"), "please use with caution", tr("Go to Settings")},
            {"4", safetylevel, "dcc_home_restore", tr("Your password policy is low"), "please use with caution", tr("Go to Settings")},
            {"5", safetylevel, "dcc_home_restore", tr("User password has not been changed for more than 90 days"), "please use with caution", tr("Go to Settings")},
            {"6", safetylevel, "dcc_home_restore", tr("The lockscreen time is set to never"), "please use with caution", tr("Go to Settings")},
            {"7", safetylevel, "dcc_home_restore", tr("Your system is not up to date"), "please use with caution", tr("Go to Settings")},
            {"8", safetylevel, "dcc_home_restore", tr("Updates notification is turned off"), "please use with caution", tr("Turn on")},
            {"9", safetylevel, "dcc_home_restore", tr("There are unsolved threats"), "please use with caution", tr("Scan Now")},
            {"10", safetylevel, "dcc_home_restore", tr("The virus database is not up to date"), "please use with caution", tr("Update Now")},
            {"11", safetylevel, "dcc_home_restore", tr("No virus scan > 30 days"), "please use with caution", tr("Scan Now")},
            {"12", safetylevel, "dcc_home_restore", tr("No virus scan > 90 days"), "please use with caution", tr("Scan Now")},
            {"13", safetylevel, "dcc_home_restore", tr("Root access allowed, which is risky"), "please use with caution", " "},
        };
    }

    initFixItems(m_fixItemInfoLst);

    QGraphicsOpacityEffect *opacity = new QGraphicsOpacityEffect;
    opacity->setOpacity(0.8);
    m_fixItemLst.at(0).btn->setGraphicsEffect(opacity);

    //no need fix items background
    m_noNeedFixItemImg = new QLabel;
    m_noNeedFixItemImg->setFixedHeight(283);
    m_noNeedFixItemImg->setPixmap(QIcon::fromTheme("dcc_full_marks_empty").pixmap(145, 106));
    m_leftLayout->addWidget(m_noNeedFixItemImg, 0, Qt::AlignCenter);

    QHBoxLayout *hlayout = new QHBoxLayout;
    m_leftLayout->addStretch(1);

    reExamineBtn = new DPushButton(tr("Check Again"));
    reExamineBtn->hide();
    hlayout->addWidget(reExamineBtn/*, 0, Qt::AlignLeft*/);
//    hlayout->margin(20);
    m_fixedBtn = new DSuggestButton(tr(""));
    hlayout->addWidget(m_fixedBtn);
    m_leftLayout->addLayout(hlayout);

    m_leftLayout->addStretch(1);

    //right picture
    mainLayout->addStretch(1);
    m_rightPic = new QLabel;
    m_rightPic->setMinimumWidth(258);
    m_rightPic->setMaximumWidth(258);

    QIcon icon = QIcon::fromTheme("dcc_home_detection_image");
    m_rightPic->setPixmap(icon.pixmap(258, 277));
    // 进度条 背景色
    m_scoreBg->setColor(ScoreBackgroundLight);
    mainLayout->addWidget(m_rightPic, 0, Qt::AlignRight);


    //connect
    connect(m_scoreTimer, &QTimer::timeout, this, &ExamDetailPage::updateScoreValue);

    //connect fix items' button
    connect(m_fixItemLst[0].btn, &DCommandLinkButton::clicked, this, &ExamDetailPage::requestSetNetworing);   //应用可以直接连接网络
    connect(m_fixItemLst[1].btn, &DCommandLinkButton::clicked, this, &ExamDetailPage::requestSetRemoteAccess); //应用可自行启远程访问
    connect(m_fixItemLst[2].btn, &DCommandLinkButton::clicked, this, &ExamDetailPage::requestShowStart);  //5个开机自启动
    connect(m_fixItemLst[3].btn, &DCommandLinkButton::clicked, this, &ExamDetailPage::requestShowLogin);
    connect(m_fixItemLst[4].btn, &DCommandLinkButton::clicked, this, &ExamDetailPage::requestShowPassWordLogin);
    connect(m_fixItemLst[5].btn, &DCommandLinkButton::clicked, this, &ExamDetailPage::requestShowScreen);
    connect(m_fixItemLst[6].btn, &DCommandLinkButton::clicked, this, &ExamDetailPage::requsetShowUpdate);
    connect(m_fixItemLst[7].btn, &DCommandLinkButton::clicked, this, &ExamDetailPage::requestSetOnOff);

    if (DSysInfo::uosEditionType() != DSysInfo::UosEdition::UosCommunity) {
        // 病毒查杀的按钮点击
        connect(m_fixItemLst[8].btn, &DCommandLinkButton::clicked, this, &ExamDetailPage::requstCustomScanPage);  //存在为处理的风险异常
        connect(m_fixItemLst[9].btn, &DCommandLinkButton::clicked, this, &ExamDetailPage::requestUpdateVDB);  //病毒库有更新
        connect(m_fixItemLst[10].btn, &DCommandLinkButton::clicked, this, &ExamDetailPage::requstScanPage);  //超过30天没有查杀病毒库
        connect(m_fixItemLst[11].btn, &DCommandLinkButton::clicked, this, &ExamDetailPage::requstScanPage);   //超过90天没有全盘查杀
    }

    //connect examine again button
    connect(reExamineBtn, &QPushButton::clicked, this, &ExamDetailPage::requestReEaxamine);
    connect(m_fixedBtn, &DSuggestButton::clicked, this, [ = ]() {
        Q_EMIT requestFixall(m_doneflag);
    });

    // 主题变换
    //connect(DGuiApplicationHelper::instance(), SIGNAL(themeTypeChanged(ColorType)), this, SLOT(themeTypeChange(ColorType)));
}

void ExamDetailPage::initFixItems(QList<FixItemInfo> InfoLst)
{
    QVBoxLayout *itemsLayout = new QVBoxLayout;
    itemsLayout->setAlignment(Qt::AlignTop);
//    itemsLayout->setSizeConstraint(QLayout::SetMinimumSize);
    itemsLayout->setSpacing(10);
    itemsLayout->setMargin(0);
    itemsLayout->setContentsMargins(0, 0, 0, 0);

    QWidget *itemsWidgt = new QWidget(this);
    itemsWidgt->setLayout(itemsLayout);

    m_itemsArea = new QScrollArea;
    m_itemsArea->setFixedHeight(350);
    m_itemsArea->setFixedWidth(HONE_PAGE_ITEM_WIDTH + 25);
    m_itemsArea->setAlignment(Qt::AlignTop);
    m_itemsArea->setWidgetResizable(true);
    m_itemsArea->setWidget(itemsWidgt);
    m_itemsArea->setFrameShape(QFrame::NoFrame);
    m_leftLayout->addWidget(m_itemsArea); //向左布局中加入滑动区域
    for (auto info : InfoLst) {
        QHBoxLayout *itemsHlayout = new QHBoxLayout(this);
        itemsLayout->addLayout(itemsHlayout);
        itemsHlayout->setContentsMargins(0,0,0,0);
        itemsHlayout->setSpacing(5);
        FixItem fixItemTmp;
        fixItemTmp.id = info.id;

        FixItemWindow *itemWindowTmp = new FixItemWindow(true);
        itemWindowTmp->setContentsMargins(0, 0, 0, 0);
        itemWindowTmp->setFixedWidth(HONE_PAGE_ITEM_WIDTH);
        itemWindowTmp->setIcon(QIcon::fromTheme(info.iconPath));
        if (fixItemTmp.id == "3") {
            itemWindowTmp->setText(tr("There are more than %1 startup programs").arg(/*m_work->GetEnabledata()*/5));
        } else {
            itemWindowTmp->setText(info.title);
        }
        itemWindowTmp->setButtonText(info.btnStr, false);
        itemsHlayout->addWidget(itemWindowTmp, 0, Qt::AlignLeft);

        // 加载的spinner
        DSpinner *spiner = new DSpinner;
        spiner->setFixedSize(QSize(15, 15));
        spiner->setObjectName("spinner");
        itemsHlayout->addWidget(spiner, 0, Qt::AlignLeft);
        itemsHlayout->addStretch(1);

        fixItemTmp.widget = itemWindowTmp;
        fixItemTmp.btn = itemWindowTmp->getButton();
        fixItemTmp.spinner = spiner;

        m_fixItemLst.append(fixItemTmp);
    }
}

// 展现修复结果
void ExamDetailPage::showRepairResult(QString result)
{
    m_loadtimer = new QTimer;

    for (auto at : m_fixItemLst) {
        if (at.id == result) {
            FixItemWindow *itemWindowTmp = static_cast<FixItemWindow *>(at.widget);
            // 判断当前是否修复  修复就不用管
            if (itemWindowTmp->getButtonText() == tr("Fixed")) {
                return;
            }
            at.widget->show();
            at.spinner->show();
            QTimer::singleShot(500, this, [=]{
                at.spinner->hide();
                onTimerout(at.spinner);
            });
            itemWindowTmp->setIcon(QIcon::fromTheme("dcc_home_normal"));
            itemWindowTmp->setButtonText(tr("Fixed"), true);
        }
    }
}

// 设置按钮名称
void ExamDetailPage::setButtontext()
{

    m_fixedBtn->setText(tr("Done"));
    m_doneflag = true;
}

void ExamDetailPage::setModelAndWork(HomePageModel *mode, HomePageWork *work)
{
    m_mode = mode;
    m_work = work;
}

// 设置得分
void ExamDetailPage::setScoreValue(int value)
{
    m_value = value;

    if (m_value == m_currentValue) {
        m_scoreLabel->setNum(m_value);
        return;
    } else if (m_value > m_currentValue) m_moveDir = 1;
    else if (m_value < m_currentValue) m_moveDir = -1;

    m_scoreTimer->start();
}

// 更新得分
void ExamDetailPage::updateScoreValue()
{
    m_currentValue += m_moveDir;
    m_scoreLabel->setNum(m_currentValue);
    if (m_value == m_currentValue) m_scoreTimer->stop();

    update();
}

// 展现结果
void ExamDetailPage::showResult(QList<int> fixItemIdLst)
{
    int score = m_mode->getSafetyScore();
    qDebug() << "score = " << score;
    setScoreValue(score);
    if (94 < score && score <= 100) {
        QPalette paTmp = m_scoreLabel->palette();
        paTmp.setColor(QPalette::Text, ScoreVerySafeColor);
        m_scoreLabel->setPalette(paTmp);

        m_securRightLabel->setText(tr("Very Safe"));
        m_scoreProg->setValue(score);
        m_scoreProg->setColor(ScoreVerySafeColor);
        m_progressBarShadow->setColor(ScoreVerySafeShadowColor);
        reExamineBtn->show();
    } else if (79 < score) {
        QPalette paTmp = m_scoreLabel->palette();
        paTmp.setColor(QPalette::Text, ScoreSafeColor);
        m_scoreLabel->setPalette(paTmp);

        m_securRightLabel->setText(tr("Safe"));
        m_scoreProg->setValue(score);
        m_scoreProg->setColor(ScoreSafeColor);
        m_progressBarShadow->setColor(ScoreSafeShadowColor);
        reExamineBtn->show();
    } else if (59 < score) {
        QPalette paTmp = m_scoreLabel->palette();
        paTmp.setColor(QPalette::Text, ScoreHaveDangersColor);
        m_scoreLabel->setPalette(paTmp);

        m_securRightLabel->setText(tr("Risky"));
        m_scoreProg->setValue(score);
        m_scoreProg->setColor(ScoreHaveDangersColor);
        m_progressBarShadow->setColor(ScoreHaveDangersShadowColor);
        reExamineBtn->show();
    } else {
        QPalette paTmp = m_scoreLabel->palette();
        paTmp.setColor(QPalette::Text, ScoreUnderIllnessColor);
        m_scoreLabel->setPalette(paTmp);

        m_securRightLabel->setText(tr("At High Risk"));
        m_securRightLabel->setPalette(paTmp);
        m_scoreProg->setValue(score);
        m_scoreProg->setColor(ScoreUnderIllnessColor);
        m_progressBarShadow->setColor(ScoreUnderIllnessShadowColor);
        reExamineBtn->show();
    }

    //show fix items
    if (0 == fixItemIdLst.size()) { //如果没有待修复项
        m_itemsArea->hide();
        m_noNeedFixItemImg->show();
    } else {
        m_itemsArea->show();
        m_noNeedFixItemImg->hide();
    }

    for (FixItem fixItem : m_fixItemLst) {
        fixItem.widget->hide();
        fixItem.spinner->hide();
    }
    bool flag = false;
    for (int i : fixItemIdLst) {
        m_fixItemLst[i - 1].widget->show();
        if (i == 1 || i == 2 || i == 8) {
            flag = true;
        }
    }
    if (score != 100 && flag) {
        m_fixedBtn->setText(tr("Fix All"));
    } else {
        m_fixedBtn->setText(tr("Done"));
        m_doneflag = true;
    }
}

//更新展现的页面
void ExamDetailPage::updateWidget()
{
    Q_EMIT requestShowupdateWidget();
}

void ExamDetailPage::onTimerout(DSpinner *spinner)
{
    spinner->hide();
    spinner->stop();
    m_loadtimer->stop();
}

// 更新病毒库
void ExamDetailPage::updateVdb()
{
    // 更新病毒库
    for (auto at : m_fixItemLst) {
        if (at.id == "10") {
            //at.widget->show();
            at.spinner->raise();
            at.spinner->show();
            at.spinner->start();
            qDebug()<<"point"<<at.spinner->pos();
        }
    }
}

void ExamDetailPage::setThemeType(int ntype)
{
    QIcon icon;
    // 只有是社区版才是社区  其他的都是专业
    if (DSysInfo::uosEditionType() == DSysInfo::UosEdition::UosCommunity) {
        // 社区版本
        if(ntype == DGuiApplicationHelper::ColorType::LightType){
            icon = QIcon::fromTheme("dcc_Home_detection-deepin_image");
            // 进度条 背景色
            m_scoreBg->setColor(ScoreBackgroundLight);
        }else{
            icon = QIcon::fromTheme("dcc_Home_detection_deepin_image_dark");
            // 进度条 背景色
            m_scoreBg->setColor(ScoreBackgroundDark);
        }
    } else {
        // 专业版本
        if(ntype == DGuiApplicationHelper::ColorType::LightType){
            icon = QIcon::fromTheme("dcc_home_detection_image");
            // 进度条 背景色
            m_scoreBg->setColor(ScoreBackgroundLight);
        }else{
            icon = QIcon::fromTheme("dcc_home_detection_image_dark");
            // 进度条 背景色
            m_scoreBg->setColor(ScoreBackgroundDark);
        }
    }
    // 展现图片
    m_rightPic->setPixmap(icon.pixmap(258, 277));
    qDebug()<<"THEME CHANGE";
}

// 病毒库更新结果
void ExamDetailPage::VdbUpdateResult(bool flag)
{
    for (auto at : m_fixItemLst) {
        if (at.id == "10") {
            at.spinner->hide();
            at.spinner->stop();
            if(flag){
                showRepairResult("10");
            }

        }
    }

}

/*void ExamDetailPage::restartTimer(QLabel *label)
{
    m_isbTimerStart = true;
    m_loadtimer->start(100);
    i_Angle = 0;
    onTimerout(label);
}

void ExamDetailPage::onTimerout(QLabel *label)
{
    static int timer = 0;
    timer++;
    ++i_Angle;
    i_Angle = i_Angle % 36;
    QImage image;
    QMatrix matrix;
    image.load(":/icons/deepin/builtin/icons/dcc_spinner_32px.svg");
    matrix.rotate(360 - 10.0 * i_Angle);//页面旋转
    image = image.transformed(matrix, Qt::SmoothTransformation);
    label->setPixmap(QPixmap::fromImage(image));
    if (timer == 5) {
        timer = 0;
        if (m_isbTimerStart) {
            m_isbTimerStart = false;
            m_loadtimer->stop();
            m_loadtimer->deleteLater();
        }
        label->hide();
    }
}*/
