/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "window/namespace.h"
#include "dtkwidget_global.h"
#include "widgets/scoreprogressbar.h"
#include "window/modules/homepage/homepagework.h"
#include "window/modules/homepage/examworker.h"
#include "src/window/modules/common/common.h"

#include <DCommandLinkButton>
#include <DProgressBar>
#include <DPushButton>
#include <DSpinner>
#include <DGuiApplicationHelper>
#include <DSysInfo>

#include <QWidget>
#include <QLabel>
#include <QProgressBar>
#include <QScrollArea>
#include <QGSettings>
#include <QGraphicsDropShadowEffect>
#include <QDebug>
#include <QTimer>

DWIDGET_BEGIN_NAMESPACE
class DSuggestButton;
class DCommandLinkButton;
DWIDGET_END_NAMESPACE

QT_BEGIN_NAMESPACE
class QHBoxLayout;
class QVBoxLayout;
class QLabel;
class QPushButton;
QT_END_NAMESPACE

DWIDGET_USE_NAMESPACE

DEF_NAMESPACE_BEGIN
DEF_ANTIVIRUS_NAMESPACE_BEGIN

DCORE_USE_NAMESPACE

class ExamDetailPage : public QWidget
{
    Q_OBJECT
public:
    enum FixItemStyle {safetylevel, virus, firewalls};
    struct FixItemInfo {
        QString id; //修复项标示
        FixItemStyle style;
        QString iconPath;
        QString title;
        QString tip;
        QString btnStr;
    };
    struct FixItem {
        QString id; //修复项标示
        QWidget *widget;
        //DPushButton *btn;
        DCommandLinkButton *btn;
        //QLabel *lable;
        DSpinner *spinner;
    };

public:
    explicit ExamDetailPage(HomePageModel *mode, HomePageWork *work, QWidget *parent = nullptr);

    void setModelAndWork(HomePageModel *mode, HomePageWork *work);
    void setScoreValue(int value);
    void showResult(QList<int> fixItemIdLst);
    void updateWidget();
    //void restartTimer(QLabel *label);
    //void onTimerout(QLabel *label);
    void onTimerout(DSpinner *spinner);

    // 更新病毒库
    void updateVdb();

    // 设置主题类型
    void setThemeType(int ntype);

private:
    void initFixItems(QList<FixItemInfo> InfoLst);

Q_SIGNALS:
    void requestBack();
    void requestShowSafetyLevelWidgt();
    void requestShowAntivirusWidgt();
    void requestReEaxamine();
    void requestSetNetworing();
    void requestSetRemoteAccess();
    void requestShowStart();
    void requestShowLogin();
    void requestShowPassWordLogin();
    void requestShowScreen();
    void requsetShowUpdate();
    void requestSetOnOff();
    void requestShowupdateWidget();
    void requestFixall(bool doneflag);

    // 切换到查杀页面
    void requstScanPage();
    // 后台更新病毒库
    void requestUpdateVDB();
    // 自定义查杀
    void requstCustomScanPage();

public Q_SLOT:
    void updateScoreValue();
    void showRepairResult(QString result);//修复后更改图标和按钮
    void setButtontext();
    void VdbUpdateResult(bool flag);

private:
    HomePageModel *m_mode;
    HomePageWork *m_work;
    QVBoxLayout *m_leftLayout;
    QLabel *m_scoreLabel;
    DSuggestButton *m_fixedBtn;
    DPushButton *reExamineBtn;
    int m_currentValue;
    int m_value;
    int m_moveDir;
    QTimer *m_scoreTimer;
    QLabel *m_centLabel;
    ScoreProgressBar *m_scoreProg;
    QGraphicsDropShadowEffect *m_progressBarShadow;
    QLabel *m_securLeftLabel;
    QLabel *m_securRightLabel;
    QLabel *m_scoreTip;
    QList<FixItemInfo> m_fixItemInfoLst;
    QList<FixItem> m_fixItemLst;
    QScrollArea *m_itemsArea;
    QLabel *m_noNeedFixItemImg;
    bool m_isbExamining;
    bool m_doneflag = false;
    int i_Angle;
    QTimer *m_loadtimer;
    bool m_isbTimerStart = false;
    QLabel                  *m_rightPic = nullptr ;                  // 右侧图片
    ScoreProgressBar        *m_scoreBg = nullptr ;                   // 进度条背景

};

DEF_ANTIVIRUS_NAMESPACE_END
DEF_NAMESPACE_END
