/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "examworker.h"
#include <pwd.h>
#include <unistd.h>

#include <QDebug>
#include <QThread>
#include <QDateTime>

//体检各阶段权重
const int AppNetworkingScoreCoe = 6;
const int RemoteAccessScoreCoe = 6;
const int StartScoreCoe = 2;
const int PasswordLevelScoreCoe = 6;
const int ChangePasswordScoreCoe = 5;
const int LockScreenScoreCoe = 6;
const int LatestEditionScoreCoe = 6;
const int SystemUpdateScroeCoe = 2;
const int UnsolvedThreatScoreCoe = 10;           //没有处理的异常
const int VirusDBUpdateScoreCoe = 2;             //病毒库没更新
const int ThitryNoScanScoreCoe = 6;              //30天没有扫描
const int NinetyNoScabScroeCoe = 10;            //90天没有扫描
const int RootTypeCoe = 2;                    //开发者模式

ExamWorker::ExamWorker(QObject *parent)
    : QObject(parent)
    , m_mode(nullptr)
    , m_work(nullptr)
    , m_updaterInter(new UpdaterInter("com.deepin.lastore", "/com/deepin/lastore",
                                      QDBusConnection::systemBus(), this))
    , m_managerInter(new ManagerInter("com.deepin.lastore", "/com/deepin/lastore",
                                      QDBusConnection::systemBus(), this))
    , m_status(running)
{
    // 首页体检系统更新默认设置为最新状态
    m_updatestatus = SYS_UPDATE_STATUS_NO;

//    connect(m_work, &HomePageWork::sendThreatsCount, this, [ = ](int nCount) {
//        m_threatsCount = nCount ;
//        m_isbRecedThreatsCount = true;
//    });

}

ExamWorker::~ExamWorker()
{
    qDebug() << "examine worker quit";
}

void ExamWorker::setModelAndWork(HomePageModel *mode, HomePageWork *work)
{
    m_mode = mode;
    m_work = work;
    connect(this, &ExamWorker::startCheckUpdate, m_work, &HomePageWork::checkForUpdates);
}

void ExamWorker::pause()
{
    m_status = paused;
}

void ExamWorker::run()
{
    m_status = running;
}

void ExamWorker::resume()
{
    m_status = running;
}

void ExamWorker::quit()
{
    m_status = quitted;
    qDebug() << "quit";
}

void ExamWorker::judgeStatus()
{
    if (quitted == m_status) return;
    while (paused == m_status) {
        qDebug() << "examine worker is paused";
        thread()->msleep(100);
    }
}

void ExamWorker::waitWithStatus(int mSec)
{
    if (quitted == m_status) return;
    int time = 0;
    while (time < mSec) {
        judgeStatus();
        thread()->msleep(100);
        time += 100;
    }
}


//进行体检
//每个阶段调用sendExamineProcess，发送待修复项id和当前体检进度
//待修复项id对应于ExamDetailPage里的m_fixItemLst
void ExamWorker::examining()
{
    if (nullptr == m_work || nullptr == m_mode) {
        qDebug() << "ExamWorker do not setWorkAndMode!";
        return;
    }

    // modified by caimc 20200909
    // 解决首页体检后，中途临时改变仓库地址导致更新失败， 再次体检后上一次保存的系统更新值没有改变的问题，
    // 此修改保证每次加载系统更新值之前都讲更新值还原
    // 首页体检系统更新默认设置为最新状态
    m_updatestatus = SYS_UPDATE_STATUS_NO;

    int scoreTotal = 0;
    m_scorePart = 100;
//    selectUnsolvedThreats();
//    // 如果没有接受到返回的异常数据　就一直等待
//    while(!m_isbRecedThreatsCount) {
//        waitWithStatus(100);
//    }

    if (DSysInfo::uosEditionType() != DSysInfo::UosEdition::UosCommunity) {
        //m_isbRecedThreatsCount = false ;
        // 总分减去 有没有异常 病毒库版本 上次扫描时间
        m_scorePart -= examiningUnsolvedThreats();
        scoreTotal = qRound(m_scorePart);
        m_work->setSafetyScore(scoreTotal); //检测结束储存分数
        if (ExamWorker::Status::quitted == m_status) {
            return;
        }

        //　ｍｘｚ由于现在没有病毒有版本更新了　以及会自动扫描快速查杀　目前这两个去掉　０８/２７
        m_scorePart -= examiningVDB();
    }

    // 防火墙
    m_scorePart -= examiningAppNetworking();
    m_scorePart -= examiningRemoteAccess();
    scoreTotal = qRound(m_scorePart);
    m_work->setSafetyScore(scoreTotal); //检测结束储存分数
    if (ExamWorker::Status::quitted == m_status) {
        return;
    }
    // 自启动
    m_scorePart -= examiningStart();
    scoreTotal = qRound(m_scorePart);
    m_work->setSafetyScore(scoreTotal); //检测结束储存分数
    if (ExamWorker::Status::quitted == m_status) {
        return;
    }
    // 系统安全
    m_scorePart -= examiningPasswordLevel();
    m_scorePart -= examiningChangePassword();
    m_scorePart -= examiningLockScreen();
    m_scorePart -= examiningSystemUpdate(); 
    m_scorePart -= examiningLatestEdition();
    // 判断是不是社区版本
    if(DSysInfo::uosEditionType() != DSysInfo::UosEdition::UosCommunity){
        m_scorePart -= examiningRootType();
    }
    scoreTotal = qRound(m_scorePart);
    m_work->setSafetyScore(scoreTotal); //检测结束储存分数
    if (ExamWorker::Status::quitted == m_status) {
        return;
    }

    Q_EMIT sendExamineProcess("finished", 0); //发送检测结束标志
    scoreTotal = qRound(m_scorePart);
    m_work->setSafetyScore(scoreTotal); //检测结束储存分数
}

void ExamWorker::getUpdate(int updatestatus)
{
    m_updatestatus = updatestatus;
}

int ExamWorker::examiningAppNetworking()
{
    int safetyLevelScore = AppNetworkingScoreCoe;
    int fixItemId = 0;
    int weightValue;
    //1.password level score
    int safetyLev = m_work->GetNetControlServer()->getNetControlDefaultStatus(); //默认查询-0，允许-2，禁止-1

    switch (safetyLev) {
    case 3:
        weightValue = 0;
        break;
    case 5:
        weightValue = 0;
        fixItemId = 0;
        break;
    case 4:
        weightValue = AppNetworkingScoreCoe;
        fixItemId = 1;
        break;
    default:
        weightValue = 0;
    }
    Q_EMIT sendExamineProcess(tr("Checking firewall..."), fixItemId);
    waitWithStatus(1000);
    qDebug() << "Examining -- examiningAppNetworking finished!";
    safetyLevelScore = weightValue;
    return safetyLevelScore;
}

int ExamWorker::examiningRemoteAccess()
{
    int safetyLevelScore = RemoteAccessScoreCoe;
    int fixItemId = 0;
    int weightValue;
    //1.password level score
    int safetyLev = m_work->GetNetControlServer()->getRemControlDefaultStatus(); //默认查询-3，允许-4，禁止-5

    switch (safetyLev) {
    case 3:
        weightValue = 0;
        break;
    case 5:
        weightValue = 0;
        fixItemId = 0;
        break;
    case 4:
        weightValue = RemoteAccessScoreCoe;
        fixItemId = 2;
        break;
    default:
        weightValue = 0;
    }
    Q_EMIT sendExamineProcess(tr("Checking firewall..."), fixItemId);
    waitWithStatus(1000);
    qDebug() << "Examining -- examiningRemoteAccess finished!";
    safetyLevelScore = weightValue;
    return safetyLevelScore;
}

int ExamWorker::examiningStart()
{
    int safetyLevelScore = StartScoreCoe;
    int fixItemId = 0;
    int weightValue;
    if (m_work->GetStartup()->startLauncherManage()) {  // 启动开机自启动数据管理
        QString EnableInfo = m_work->GetStartup()->getAppsInfoEnable(); // 得到所有开启状态的APP数据信息
        QString DisableInfo = m_work->GetStartup()->getAppsInfoDisable();   // 得到所有关闭状态的APP数据信息
        m_sEnableInfo = EnableInfo.split("|").count();
        Q_EMIT sedEnableInfo(m_sEnableInfo);
    }

    if (m_sEnableInfo > 5) {
        weightValue = StartScoreCoe;
        fixItemId = 3;
    } else {
        weightValue = 0;
    }

    Q_EMIT sendExamineProcess(tr("Checking startup programs..."), fixItemId);
    waitWithStatus(1000);
    qDebug() << "Examining -- examiningStart finished!";
    safetyLevelScore = weightValue;
    return safetyLevelScore;
}

int ExamWorker::examiningPasswordLevel()
{
    int safetyLevelScore = PasswordLevelScoreCoe;
    int fixItemId = 0;
    int weightValue;
    //1.password level score
    int safetyLev = m_work->GetDaemonService()->safetygrade(); //高-0，中-1，低-2
    switch (safetyLev) {
    case 0:
        weightValue = 0;
        break;
    case 1:
        weightValue = 0;
        break;
    case 2:
        weightValue = PasswordLevelScoreCoe;
        fixItemId = 4;
        break;
    default:
        weightValue = 0;
    }
    Q_EMIT sendExamineProcess(tr("Checking system security..."), fixItemId);
    waitWithStatus(1000);
    qDebug() << "Examining -- examiningPasswordLevel finished!";
    safetyLevelScore = weightValue;
    return safetyLevelScore;
}

int ExamWorker::examiningChangePassword()
{
    int safetyLevelScore = ChangePasswordScoreCoe;
    int fixItemId = 0;
    int weightValue;
    struct passwd *pws;
    pws = getpwuid(getuid());
    QString m_currentUserName = QString(pws->pw_name);
    QString user = m_work->GetAccounts()->FindUserByName(m_currentUserName);//获取当前用户名称
    qDebug() << "user = " << user;
    m_usermanage = new DbususerManager(user);
    int date =  m_usermanage->passwordLastChange();//获取上次设置密码距1970年1月1日的天数
    const QDateTime datetime(QDateTime::currentDateTime());
    QDateTime dt = QDateTime(QDate(1970, 1, 1));//获取当天距1970年1月1日的天数
    int m_day = dt.daysTo(datetime) - date;
    //1.ChangePassword score
    if (m_day >= 90) {
        weightValue = ChangePasswordScoreCoe;
        fixItemId = 5;
    } else {
        weightValue = 0;
    }
    Q_EMIT sendExamineProcess(tr("Checking system security..."), fixItemId);
    waitWithStatus(1000);
    qDebug() << "Examining -- examiningPasswordLevel finished!";
    safetyLevelScore = weightValue;
    return safetyLevelScore;
}

int ExamWorker::examiningLockScreen()
{
    int safetyLevelScore = LockScreenScoreCoe;
    int fixItemId = 0;
    int weightValue;
    //有电池
    if (m_work->haveBattery()) {
        if (m_work->getBatteryLockDelay() == 0 || m_work->getLinePowerLockDelay() == 0) {
            weightValue = LockScreenScoreCoe;
            fixItemId = 6;
        } else {
            weightValue = 0;
        }
    } else {
        if (m_work->getLinePowerLockDelay() == 0) {
            weightValue = LockScreenScoreCoe;
            fixItemId = 6;
        } else {
            weightValue = 0;
        }
    }
    Q_EMIT sendExamineProcess(tr("Checking system security..."), fixItemId);
    waitWithStatus(1000);
    qDebug() << "Examining -- examiningLockScreen finished!";
    safetyLevelScore = weightValue;
    return safetyLevelScore;
}

int ExamWorker::examiningLatestEdition()
{
    int safetyLevelScore = LatestEditionScoreCoe;
    int fixItemId = 0;
    int weightValue;
    Q_EMIT startCheckUpdate();
    waitWithStatus(15000);
    qDebug() << "m_updatestatus = " << m_updatestatus;
    if (m_updatestatus == 0) {
        if (m_work->GetDaemonService()->GetSystemUpdate() == 1) {
            weightValue = LatestEditionScoreCoe;
            fixItemId = 7;
        } else {
            weightValue = 0;
        }

    } else if (m_updatestatus == SYS_UPDATE_STATUS_YES) {
        // 有更新状态
        weightValue = LatestEditionScoreCoe;
        fixItemId = 7;
    } else if (m_updatestatus == SYS_UPDATE_STATUS_NO) {
        // 最新状态
        weightValue = 0;
    } else {
        weightValue = 0;
    }

    Q_EMIT sendExamineProcess(tr("Checking system security..."), fixItemId);
    qDebug() << "Examining -- examiningLatestEdition finished!";
    safetyLevelScore = weightValue;
    return safetyLevelScore;
}

int ExamWorker::examiningSystemUpdate()
{
    int safetyLevelScore = SystemUpdateScroeCoe;
    int fixItemId = 0;
    int weightValue;
    if (m_work->getUpdateNotify() == false) {
        weightValue = SystemUpdateScroeCoe;
        fixItemId = 8;
    } else {
        weightValue = 0;
    }

    Q_EMIT sendExamineProcess(tr("Checking system security..."), fixItemId);
    waitWithStatus(1000);
    qDebug() << "Examining -- examiningSystemUpdate finished!";
    safetyLevelScore = weightValue;
    return safetyLevelScore;
}

//　检测开发者模式
int ExamWorker::examiningRootType()
{
    int safetyLevelScore = RootTypeCoe;
    int fixItemId = 0;
    int weightValue;
    bool result = m_work->isDeveloperMode();
    if(result == false){
        weightValue = 0;
    }else {
        weightValue = RootTypeCoe;
        fixItemId = 13;
    }
    Q_EMIT sendExamineProcess(tr("Checking system security..."), fixItemId);
    waitWithStatus(1000);
    qDebug() << "Examining -- examiningPasswordLevel finished!";
    safetyLevelScore = weightValue;
    return safetyLevelScore;
}

void ExamWorker::selectUnsolvedThreats()
{
    m_work->selectAllThreatsData();
}


// 检查是有没有异常没有处理
int ExamWorker::examiningUnsolvedThreats()
{
    // 修复的下标
    int fixItemId = 0;
    // 权重比分
    int weightValue = 0;
//    QString spath = m_work->selectAllThreatsData();
//    QStringList spathList = spath.split("[*]");
//    int size = spathList.size() - 1;
    int size = m_work->selectAllThreatsData();
    // 更具长度判断比分
    if (size == 0) {
        weightValue = 0;
    } else {
        weightValue = UnsolvedThreatScoreCoe ;
        fixItemId = 9;
    }
    Q_EMIT sendExamineProcess(tr("Reading virus scan results..."), fixItemId);
    waitWithStatus(1000);
    return weightValue;
}

// 检测病毒库版本
int ExamWorker::examiningVDB()
{
    // 修复的下标
    int fixItemId = 10;
    // 权重比分
    int weightValue = 0;
    QString sSrvVdbVersion = m_work->getSrvVdbVersion();
    QString sLocalVdbVersion = m_work->getLocalVdbVersion();
    qDebug() << "服务器病毒库版本==" << sSrvVdbVersion << "，本地病毒库版本===" << sLocalVdbVersion;
    if (sSrvVdbVersion != sLocalVdbVersion) {
        weightValue = VirusDBUpdateScoreCoe;
        Q_EMIT sendExamineProcess(tr("Reading virus scan results..."), fixItemId);
    } else {
        weightValue = 0;
    }
    waitWithStatus(1000);
    return weightValue;
}

// 检查上次查杀的时间
int ExamWorker::examiningLastScan()
{
//    int Score = ThitryNoScanScoreCoe;
    // 修复的下标
    int fixItemId = 0;
    // 权重比分
    int weightValue = 0;
    QString lastScan = m_work->getLastScanTime();  //上次扫描的时间
    QString installTime = m_work->getInstallTime();
    if(lastScan == "-1"){
        // 如果是默认时间  那就跟对比
        lastScan = installTime;
    }
    QDateTime datetime = QDateTime::currentDateTime();  //当前时间
    QString str1 = datetime.toString("yyyy-MM-dd hh:mm:ss");

    QDateTime date1 = QDateTime::fromString(lastScan, "yyyy-MM-dd hh:mm:ss");
    QDateTime date2 = QDateTime::fromString(str1, "yyyy-MM-dd hh:mm:ss");
    uint stime = date1.toTime_t();
    uint etime = date2.toTime_t();
    int daysec = 24 * 60 * 60; //一天的秒数
    int  day = 0 ;
    if (stime > etime) {
        day = 0 ;
    } else {
        day = (etime - stime) / (daysec) + ((etime - stime) % (daysec) + (daysec - 1)) / (daysec) - 1;
    }
    qDebug() << "=======day" << day;
    if (day >= 90) {
        fixItemId = 12;
        Q_EMIT sendExamineProcess(tr("Reading virus scan results..."), fixItemId);
        weightValue = NinetyNoScabScroeCoe;
    } else if ((day < 90) && (day >= 30)) {
        fixItemId = 11;
        Q_EMIT sendExamineProcess(tr("Reading virus scan results..."), fixItemId);
        weightValue = ThitryNoScanScoreCoe;
    }
    waitWithStatus(1000);
    return weightValue;
}





