/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef EXAMWORKER_H
#define EXAMWORKER_H

#include "window/modules/homepage/homepagework.h"
#include "../systemsafety/loginsecurity/dbususermanager.h"

#include <QObject>
#include <DSysInfo>
#define SYS_UPDATE_STATUS_YES       1              // 有更新
#define SYS_UPDATE_STATUS_NO        2              // 最新


DCORE_USE_NAMESPACE

// 体检线程
class ExamWorker : public QObject
{
    Q_OBJECT
    enum Status {running, paused, quitted};
public:
    explicit ExamWorker(QObject *parent = nullptr);
    ~ExamWorker();

    void setModelAndWork(HomePageModel *mode, HomePageWork *work);
    void run();
    void pause();
    void resume();
    void quit();
    void waitWithStatus(int mSec); //随运行状态对线程进行等待
    void getUpdate(int updatestatus);

private:
    void judgeStatus();
    int examiningAppNetworking();
    int examiningRemoteAccess();
    int examiningStart();
    int examiningPasswordLevel();
    int examiningChangePassword();
    int examiningLockScreen();
    int examiningLatestEdition();
    int examiningSystemUpdate();
    // 开发者模式
    int examiningRootType();

    // 检测病毒库版本
    int examiningVDB();
    // 检查是有没有异常没有处理
    int examiningUnsolvedThreats();
    // 检查上次查杀的时间
    int examiningLastScan();

    void selectUnsolvedThreats();
Q_SIGNALS:
    void sendExamineProcess(QString place, int fixItemId); //发送当前扫描的位置
    void sedEnableInfo(int EnableInfo);
    void startCheckUpdate();

public Q_SLOTS:
    void examining();

private:
    HomePageModel *m_mode;
    HomePageWork *m_work;
    UpdaterInter *m_updaterInter = nullptr;
    ManagerInter *m_managerInter;
    Status m_status;
    int m_score;
    DbususerManager *m_usermanage;
    int m_sEnableInfo = 0;
    int m_updatestatus = SYS_UPDATE_STATUS_NO;
    double m_scorePart;
//    bool    m_isbRecedThreatsCount = false ;
//    int     m_threatsCount = 0;                 //未处理的异常数量
};

#endif // EXAMWORKER_H
