/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "homepagemodel.h"

#include <QGSettings>
#include <QDateTime>

#include <QDebug>

const QByteArray DeepinDefenderGSettings = "com.deepin.dde.deepin-defender";
const QString GSettingsSafetyScore = "safety-score";
const QString GSettingsSafetyLevel = "safety-level";
const QString GSettingsLocalVer = "local-ver";
const QString GSettingsLatestVer = "latest-ver";
const QString GSettingsLastKillStyle = "last-kill-style";
const QString GSettingsLastKillTime = "last-kill-time";
const QString GSettingsLastCheckTime = "last-check-time";
//上次检测时间
#define LAST_CHECK_TIME             "last-scan-time"             //上次检测时间
#define INSTALL_TIME                "defender-install-time"      // 安装时间

HomePageModel::HomePageModel(QObject *parent)
    : QObject(parent)
    , m_gs(nullptr)
{
    //使用GSetting读取参数
    m_gs = new QGSettings(DeepinDefenderGSettings, QByteArray(), this);
}

int HomePageModel::getSafetyScore()
{
    return m_gs->get(GSettingsSafetyScore).toInt();
}

int HomePageModel::getSafetyLevel()
{
    return m_gs->get(GSettingsSafetyLevel).toInt();
}

QString HomePageModel::getLocalVer()
{
    return m_gs->get(GSettingsLocalVer).toString();
}

QString HomePageModel::getLatestVer()
{
    return m_gs->get(GSettingsLatestVer).toString();
}

int HomePageModel::getLastKillStyle()
{
    return m_gs->get(GSettingsLastKillStyle).toInt();
}

QDateTime HomePageModel::getLastKillTime()
{
    QString timeStr = m_gs->get(GSettingsLastKillTime).toString();
    QDateTime time = QDateTime::fromString(timeStr, "yyyy-MM-dd-hh-mm-ss");;
    return time;
}

QDateTime HomePageModel::getLastCheckTime()
{
    QString timeStr = m_gs->get(GSettingsLastCheckTime).toString();
    QDateTime time = QDateTime::currentDateTime();
    if (!timeStr.contains("2019-11-30-17-30-35")) {
        time = QDateTime::fromString(timeStr, "yyyy-MM-dd-hh-mm-ss");
    }

    return time;
}

//测试临时加分，测试用
int HomePageModel::getTestScore()
{
    return m_gs->get("test-score").toInt();
}

// 上次病毒查杀时间
QString HomePageModel::getLastScanTime()
{
    QString timeStr = m_gs->get(LAST_CHECK_TIME).toString();
    qDebug()<<"上次体检timeStr==="<<timeStr;
    return timeStr;
}

// 获得安装时间
QString HomePageModel::getInstallTime()
{
    QString timeStr = m_gs->get(INSTALL_TIME).toString();
    qDebug()<<"安装时间==="<<timeStr;
    return timeStr;
}

