/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "homepagemodule.h"
#include "window/modules/common/common.h"
#include "window/interface/moduleinterface.h"
#include "window/interface/frameproxyinterface.h"
#include "window/modules/homepage/homepagewidget.h"
//include "../networkprotection/networkprotectionwidget.h"
#include <pwd.h>
#include <unistd.h>

#include <QTimer>
#include <QModelIndex>
#include <QStandardItemModel>

HomePageModule::HomePageModule(FrameProxyInterface *frame, QObject *parent)
    : QObject(parent)
    , ModuleInterface(frame)
    , m_homePageModel(nullptr)
    , m_homePageWork(nullptr)
    , m_homePageWidget(nullptr)
    , m_examDetailPage(nullptr)
    , m_animationthread(nullptr)
    , m_animationshow(nullptr)
    , m_hmiScreen(new HmiScreen("com.deepin.defender.hmiscreen", "/com/deepin/defender/hmiscreen", QDBusConnection::sessionBus(), this))
{

}


void HomePageModule::initialize()
{

}

void HomePageModule::preInitialize()
{
    m_homePageModel = new HomePageModel;
    m_homePageWork = new HomePageWork(m_homePageModel);
    m_guiHelper = DGuiApplicationHelper::instance();
    // 主题变换
    connect(m_guiHelper, SIGNAL(themeTypeChanged(ColorType)), this, SLOT(themeTypeChange(ColorType)));
}

const QString HomePageModule::name() const
{
    return "homepage";
}

void HomePageModule::active(int index)
{
    Q_UNUSED(index);
    // 首页是不可删除窗口, 所以始终保留
    if (nullptr == m_homePageWidget) {
        m_homePageWidget = new HomePageWidget(m_homePageModel, m_homePageWork);
        m_homePageWidget->setModelAndWork(m_homePageModel, m_homePageWork);
        m_homePageWidget->updateUI();
        connect(m_homePageWidget, &HomePageWidget::requestShowScoreDetailPage, this, &HomePageModule::showScoreDetailPage);
    }

    // 如果处于结果页,优先显示
    if (nullptr != m_examDetailPage) {
        m_frameProxy->pushWidget(this, m_homePageWidget);
        m_homePageWidget->hide();
        m_examDetailPage->updateWidget();
        m_frameProxy->pushWidget(this, m_examDetailPage);
    } else {
        m_homePageWidget->show();
        m_frameProxy->pushWidget(this, m_homePageWidget);
    }
}

void HomePageModule::deactive()
{

}

void HomePageModule::fixAll(bool doneflag)
{
    if (!doneflag) {
        m_animationthread->start();
    } else {
        // 先从主窗口显示栈中移除最后一层窗口, 当前情况最后一层窗口为体检结果页
        m_frameProxy->popWidget(this);
        // 删除结果页
        deleteExamDetailWidget();
        // 显示首页, 并更新数据
        m_homePageWidget->show();
        m_homePageWidget->updateUI();
    }
}

void HomePageModule::deletefixlist(int id)
{
    int flag  = true;
    for (int i = 0; i < m_fixItemIdLst.size(); i++) {
        if (id == m_fixItemIdLst[i]) {
            m_fixItemIdLst.takeAt(i);
        }
    }
    for (auto fixid : m_fixItemIdLst) {
        if (fixid == 1 || fixid == 2 || fixid == 8) {
            flag = false;
        }
    }
    if (flag)
        Q_EMIT reqeuesSetButton();
}

void HomePageModule::deleteExamDetailWidget()
{
    if (nullptr == m_examDetailPage) {
        return;
    }
    m_examDetailPage->setParent(nullptr);
    m_examDetailPage->hide();
    m_examDetailPage->deleteLater();
    // 删除后指向空指针
    m_examDetailPage = nullptr;
}

//展现分数详情页面
void HomePageModule::showScoreDetailPage(QList<int> fixItemIdLst)
{
    m_fixItemIdLst = fixItemIdLst;
    m_examDetailPage = new ExamDetailPage(m_homePageModel, m_homePageWork);
    m_examDetailPage->setModelAndWork(m_homePageModel, m_homePageWork);
    m_examDetailPage->showResult(fixItemIdLst);

    // 设置主题
    int type = m_guiHelper->themeType();
    m_examDetailPage->setThemeType(type);

    m_animationthread = new QThread;
    m_animationshow = new AnimationShow(m_homePageModel, m_homePageWork);
    m_animationshow->moveToThread(m_animationthread);
    connect(m_animationshow, &AnimationShow::reqeuesSetButton, m_examDetailPage, &ExamDetailPage::setButtontext);
    connect(this, &HomePageModule::reqeuesSetButton, m_examDetailPage, &ExamDetailPage::setButtontext);
    connect(m_animationshow, &AnimationShow::showRepairLable, m_examDetailPage, &ExamDetailPage::showRepairResult);

    // 更新的结果
    connect(m_homePageWork, &HomePageWork::sendVdbUpdateResult, m_examDetailPage, &ExamDetailPage::VdbUpdateResult);

    connect(m_animationthread, &QThread::started, [ = ]() {
        m_animationshow->fixAll(m_fixItemIdLst);
        m_animationthread->quit();

        // caimengci Modified  2020/05/23
        if (m_animationshow != nullptr) {
            m_animationshow->deleteLater();
        }
        if (m_animationthread != nullptr) {
            m_animationthread->deleteLater();
        }
    });

    connect(m_examDetailPage, &ExamDetailPage::requestShowupdateWidget, this, &HomePageModule::updateRepairResult);
    connect(this, &HomePageModule::showRepairLable, m_examDetailPage, &ExamDetailPage::showRepairResult);
    connect(m_examDetailPage, &ExamDetailPage::requestReEaxamine, this, [ = ]() {
        //回到主页之前, 先将结果页从主窗口中移除
        m_frameProxy->popWidget(this);
        // 删除结果页
        deleteExamDetailWidget();
        // 然后将主页显示
        m_homePageWidget->show();
        // 主页执行重新体检操作
        m_homePageWidget->reExamine();
    });
    connect(m_examDetailPage, &ExamDetailPage::requestFixall, this, [ = ](bool doneflag) {
        fixAll(doneflag);
    });

    //NetworkProtectionWidget *widget = new NetworkProtectionWidget(nullptr, nullptr);
    connect(m_examDetailPage, &ExamDetailPage::requestSetNetworing, this, [ = ]() {
        //设置联网默认询问
        m_homePageWork->GetNetControlServer()->setNetControlDefaultStatus(3);
        Q_EMIT showRepairLable("1");
        deletefixlist(1);
    });
    connect(m_examDetailPage, &ExamDetailPage::requestSetRemoteAccess, this, [ = ]() {
        //设置远程访问默认询问
        m_homePageWork->GetNetControlServer()->setRemControlDefaultStatus(3);
        Q_EMIT showRepairLable("2");
        deletefixlist(2);
    });
    // caitao/2020.6.19 use widget index global property replace number
    connect(m_examDetailPage, &ExamDetailPage::requestShowStart, this, [ = ]() {
        //跳转开机自启界面
        m_frameProxy->setCurrentModule(MODULE_STARTUP_INDEX, 0);
    });
    connect(m_examDetailPage, &ExamDetailPage::requestShowLogin, this, [ = ]() {
        //跳转登录安全界面
        m_frameProxy->setCurrentModule(MODULE_SYSTEM_SAFETY_INDEX, LOGIN_SECURITY_INDEX);
    });
    connect(m_examDetailPage, &ExamDetailPage::requestShowPassWordLogin, this, [ = ]() {
        //跳转登录安全界面
        m_frameProxy->setCurrentModule(MODULE_SYSTEM_SAFETY_INDEX, LOGIN_SECURITY_INDEX);
    });
    connect(m_examDetailPage, &ExamDetailPage::requestShowScreen, this, [ = ]() {
        //跳转屏幕安全界面
        //        m_homePageWork->GetDaemonService()->JumpScreenSecurity();
        m_frameProxy->setCurrentModule(MODULE_SYSTEM_SAFETY_INDEX, SCREEN_SECURITY_INDEX);
    });
    connect(m_examDetailPage, &ExamDetailPage::requsetShowUpdate, this, [ = ]() {
        //跳转升级策略界面
        //        m_homePageWork->GetDaemonService()->JumpUpdatePolicy();
        m_frameProxy->setCurrentModule(MODULE_SYSTEM_SAFETY_INDEX, UPDATE_POLICY_INDEX);
    });
    connect(m_examDetailPage, &ExamDetailPage::requestSetOnOff, this, [ = ]() {
        //打开更新提醒开关
        m_homePageWork->setAutoCheckUpdates(true);
        Q_EMIT showRepairLable("8");
        deletefixlist(8);
    });

    connect(m_examDetailPage, &ExamDetailPage::requstCustomScanPage, this, [ = ]() {
        // 病毒查杀自定义 展现没有处理的异常项目
        //m_frameProxy->setCurrentModule(1, 0);
        qDebug() << "没有处理的异常项 点击";
        m_homePageWork->scanThreats();
    });

    connect(m_examDetailPage, &ExamDetailPage::requestUpdateVDB, this, [ = ]() {
        // 病毒查杀后台更新
        //m_frameProxy->setCurrentModule(1, 0);
        qDebug() << "后台更新 点击";
        // 1.判断是不是在扫描中
        bool flag =m_homePageWork->isScanning();
        if(flag){
            // 在更新 弹出提示框
            m_homePageWork->showUpdateTip();
        }else {
            // 不在扫描中 后台更新  显示出spinner
            m_homePageWork->updateVDBVersion();
            m_examDetailPage->updateVdb();
        }
    });

    connect(m_examDetailPage, &ExamDetailPage::requstScanPage, this, [ = ]() {
        // 病毒查杀后台更新
        m_frameProxy->setCurrentModule(1, 0);
    });

    // 先将主页隐藏
    m_homePageWidget->hide();
    // 如果主框架显示的不是当前模块, 则不进行推栈
    if (this != m_frameProxy->getCurrentModule()) {
        qDebug() << "主框架显示的不是当前模块, 不进行推栈";
        return;
    }
    // 再显示结果页
    m_frameProxy->pushWidget(this, m_examDetailPage);
}

//　更新修复的结果
void HomePageModule::updateRepairResult()
{
    m_detectallitemthread = new QThread;
    m_detectallitem = new DetectAllItem(m_homePageModel, m_homePageWork);
    m_detectallitem->moveToThread(m_detectallitemthread);
    connect(m_detectallitem, &DetectAllItem::showRepairLable, m_examDetailPage, &ExamDetailPage::showRepairResult);
    connect(m_detectallitemthread, &QThread::started, [ = ]() {
        //　更新修复页面
        m_detectallitem->fixAll(m_fixItemIdLst);
        m_detectallitemthread->quit();
    });
    connect(m_detectallitem, &DetectAllItem::reqeuesRemoveList, [ = ](QList<int> removelist) {
        for (auto list : removelist) {
            deletefixlist(list);
        }
    });
    m_detectallitemthread->start();

}

void HomePageModule::themeTypeChange(ColorType themeType)
{
    int type;
    switch (themeType) {
    case UnknownType:
        type = 0;
        break;
    case LightType:
        type = 1;
        break;
    case DarkType:
        type = 2 ;
        break;
    }
    // 判断结果页是否为空
    if (m_examDetailPage) {
        m_examDetailPage->setThemeType(type);
    }
}

