/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "homepagework.h"
#include "src/window/modules/antivirus/antiviruswork.h"

#include <DDialog>
#include <QDateTime>

#define DIALOG_WIDTH                380                     //dialog  宽度
#define DIALOG_HEIGHT               145                     //dialog  高度
#define NORMAL_TYPE                 0                       // 正常的类型

HomePageWork::HomePageWork(HomePageModel *model, QObject *parent)
    : QObject(parent)
    , m_model(model)
    , m_daemonservice(new DaemonService("com.deepin.defender.daemonservice", "/com/deepin/defender/daemonservice",
                                        QDBusConnection::sessionBus(), this))
    , m_accounts(new accounts("com.deepin.daemon.Accounts", "/com/deepin/daemon/Accounts", QDBusConnection::systemBus(), this))
    , m_DataInterFaceServer(new DataInterFaceServer("com.deepin.defender.datainterface",
                                                    "/com/deepin/defender/datainterface",
                                                    QDBusConnection::sessionBus(), this))
    , m_powerInter(new PowerInter("com.deepin.daemon.Power", "/com/deepin/daemon/Power",
                                  QDBusConnection::sessionBus(), this))
    , m_updaterInter(new UpdaterInter("com.deepin.lastore", "/com/deepin/lastore",
                                      QDBusConnection::systemBus(), this))
    , m_managerInter(new ManagerInter("com.deepin.lastore", "/com/deepin/lastore",
                                      QDBusConnection::systemBus(), this))
    , m_antivirusServer(new AntivirusServer("com.deepin.defender.antiav", "/com/deepin/defender/antiav",
                                            QDBusConnection::systemBus(), this))
{
    m_gs = m_model->m_gs;

    m_helperInter = new QDBusInterface("com.deepin.sync.Helper",
                                       "/com/deepin/sync/Helper",
                                       "com.deepin.sync.Helper",
                                       QDBusConnection::systemBus(), this);

    //病毒库更新的结果
    connect(m_antivirusServer, &AntivirusServer::signal_UpdateVersionResult, this, &HomePageWork::doVdbUpdateResult);

    // 连接扫描结果信号
    //connect(m_antivirusServer, &AntivirusServer::onDissectionResult, this, &HomePageWork::onQuickScanResult);

    // 返回的数据
//    connect(m_DataInterFaceServer, &DataInterFaceServer::sendThreadsCount, this, [=](int ncount) {
//        emit sendThreatsCount(ncount);
//    });

}

void HomePageWork::getEnableInfo(int EnableInfo)
{
    m_Enabledata = EnableInfo;
}

bool HomePageWork::haveBattery()
{
    return  m_powerInter->batteryIsPresent().size() > 0;
}

int HomePageWork::getLinePowerLockDelay()
{
    return m_powerInter->linePowerLockDelay();
}

int HomePageWork::getBatteryLockDelay()
{
    return m_powerInter->batteryLockDelay();
}

void HomePageWork::setAutoCheckUpdates(bool isbAuto)
{
    m_updaterInter->SetUpdateNotify(isbAuto);
}

bool HomePageWork::getUpdateNotify()
{
    qDebug() << m_updaterInter->updateNotify();
    return m_updaterInter->updateNotify();
}

void HomePageWork::setSafetyScore(int score)
{
    m_gs->set(GSettingsSafetyScore, score);
}

void HomePageWork::setSafetyLevel(int level)
{
    m_gs->set(GSettingsSafetyLevel, level);
}

void HomePageWork::setLocalVer(QString ver)
{
    m_gs->set(GSettingsLocalVer, ver);
}

void HomePageWork::setLatestVer(QString ver)
{
    m_gs->set(GSettingsLatestVer, ver);
}

void HomePageWork::setLastCheckTime(QDateTime time)
{
    QString timeStr = time.toString("yyyy-MM-dd-hh-mm-ss");
    m_gs->set(GSettingsLastCheckTime, timeStr);
}

void HomePageWork::checkForUpdates()
{
    if (checkDbusIsValid()) {
        qDebug() << " checkDbusIsValid . do nothing";
        return;
    }

    QDBusPendingCall call = m_managerInter->UpdateSource();
    QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(call, this);
    connect(watcher, &QDBusPendingCallWatcher::finished, [this, call, watcher] {
        if (!call.isError())
        {
            QDBusReply<QDBusObjectPath> reply = call.reply();
            const QString jobPath = reply.value().path();
            setCheckUpdatesJob(jobPath);
        } else
        {
            if (!m_checkUpdateJob.isNull()) {
                m_managerInter->CleanJob(m_checkUpdateJob->id());
            }
            qWarning() << "UpdateFailed, check for updates error: " << call.error().message();
        }
        watcher->deleteLater();
    });
}

// 病毒库的更新结果
void HomePageWork::doVdbUpdateResult(bool isbFlag)
{
    // 发送病毒库更新结果
    emit sendVdbUpdateResult(isbFlag);
}

// 快速扫描的结果
void HomePageWork::onQuickScanResult(QString sPath, double dProgress, int iType, QString sDetailInfo)
{
    Q_UNUSED(dProgress);
    Q_UNUSED(sDetailInfo);
    if (iType != NORMAL_TYPE) {
        //　异常的数据个数
        m_quickThreats ++;
        // 得到数据库中所有的异常数据数据
//        QString pathlist = m_DataInterFaceServer->notifySelectThreatCount();
        QString pathlist = "";
        QStringList threatslist = pathlist.split("[*]");
        int threatsSize = threatslist.size() - 1;
        if (threatsSize == 0) {
            // 没有异常数据　那就直接写入到数据库中
            m_DataInterFaceServer->notifyAppendThreatDataByPath(sPath);
        } else {
            //　该异常是否存在
            bool threatFlag = false;
            for (int j = 0; j < threatsSize; j++) {
                if (sPath == threatslist.at(j)) {
                    // 存在改变标志状态
                    threatFlag = true;
                }
            }
            //没有就添加
            if (threatFlag == false) {
                m_DataInterFaceServer->notifyAppendThreatDataByPath(sPath);
            }
        }
    }
}

bool HomePageWork::checkDbusIsValid()
{
    bool ret = true;

    //如果检查任务已经存在，说明dbus有任务在执行
    if (!m_checkUpdateJob.isNull()) {
        if (m_checkUpdateJob->isValid() && ret) {
            return true;
        } else {
            //执行结束，删除任务
            m_checkUpdateJob->deleteLater();
        }
    }

    return false;
}

void HomePageWork::setCheckUpdatesJob(const QString &jobPath)
{
    if (!m_checkUpdateJob.isNull())
        return;

    m_checkUpdateJob = new JobInter("com.deepin.lastore", jobPath, QDBusConnection::systemBus(), this);
    connect(m_checkUpdateJob, &__Job::StatusChanged, [this](const QString & status) {
        qDebug() << "[setCheckUpdatesJob]status is: " << status;
        if (status == "failed") {
            qWarning() << "check for updates job failed";
            m_managerInter->CleanJob(m_checkUpdateJob->id());
            m_checkUpdateJob->deleteLater();
        } else if (status == "success" || status == "succeed") {
            m_checkUpdateJob->deleteLater();

            //apt update之后可以查看最新更新信息
            QDBusPendingCallWatcher *w = new QDBusPendingCallWatcher(m_updaterInter->ApplicationUpdateInfos(QLocale::system().name()), this);
            connect(w, &QDBusPendingCallWatcher::finished, this, &HomePageWork::onAppUpdateInfoFinished);
        }
    });
}

void HomePageWork::onAppUpdateInfoFinished(QDBusPendingCallWatcher *w)
{
    qDebug() << "onAppUpdateInfoFinished";
    QDBusPendingReply<AppUpdateInfoList> reply = *w;

    if (reply.isError()) {
        const QJsonObject &obj = QJsonDocument::fromJson(reply.
                                                         error().
                                                         message().
                                                         toUtf8())
                                 .object();

        if (obj["Type"].toString().contains("dependenciesBroken")) {
        } else {
            qWarning() << Q_FUNC_INFO << "UpdateFailed, error msg : " << obj["Type"].toString();
        }

        w->deleteLater();
        //        return;
    }

    //检查更新结束
    if (m_updaterInter->updatableApps().size() > 0
            || m_updaterInter->updatablePackages().size() > 0) {
        Q_EMIT sendHaveUpdates(1);

    } else {
        Q_EMIT sendHaveUpdates(2);
    }
    w->deleteLater();
}

// 上次病毒查杀时间
QString HomePageWork::getLastScanTime()
{
    return m_model->getLastScanTime();
}

// 获得安装时间
QString HomePageWork::getInstallTime()
{
    return m_model->getInstallTime();
}

// 获得病毒库版本
QString HomePageWork::getSrvVdbVersion()
{
    QString version = m_antivirusServer->getVdbVersion();
    return version;
}

// 获得本地病毒库版本
QString HomePageWork::getLocalVdbVersion()
{
    QString sVersion = SLocalVdbDefaultVer;
    //文件名
    QString m_fileName = SConfigDir + SLocalVdbVerCfgFileName;
    QFile f(m_fileName);
    if (!f.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug() << SConfigDir + SLocalVdbVerCfgFileName << "open failed!";
        sVersion = SLocalVdbDefaultVer;
        return sVersion;
    }
    //输入流
    QTextStream txtInput(&f);
    QString sLocalVdbVer = txtInput.readLine(0);

    // 格式校验
    if (4 == sLocalVdbVer.split(".").count()) {
        sVersion = sLocalVdbVer;
    } else {
        sVersion = SLocalVdbDefaultVer;
    }

    qDebug() << "local virus databas version:" << sVersion;
    f.close();
    return sVersion;
}

// 查询所有的异常数据
int HomePageWork::selectAllThreatsData()
{
    int size = m_DataInterFaceServer->getSelectThreatsCount();;
    bool flag = m_antivirusServer->isScanning();
    if (!flag) {
        // 不在扫描中 快速查杀
        quickScan();
        size = size + m_quickThreats ;
    }
    return size;
}

// 后台更新
void HomePageWork::updateVDBVersion()
{
    m_antivirusServer->updateVersion();
    m_antivirusServer->backgroundUpdate();

}

// 查杀异常项目
void HomePageWork::scanThreats()
{
    m_antivirusServer->scanThreatsFile();
}

// 数据库是否更新成功
bool HomePageWork::vdbUpdateSuccess()
{
    QString sSrvVdbVer = getSrvVdbVersion();
    QString sLocalVdbVer = getLocalVdbVersion();
    // 对比本地病毒库版本和服务病毒库版本
    if (sSrvVdbVer == sLocalVdbVer) {
        return true;
    } else {
        return false;
    }
}

// 异常的数据库是否为空
bool HomePageWork::threatCountIsEmpty()
{
    //QString result = m_DataInterFaceServer->notifySelectThreatCount();
    int count = m_DataInterFaceServer->getSelectThreatsCount();
    qDebug() << "result====" << count;
    return (0 == count) ;
}

// 查杀时间的间隔
int HomePageWork::scanTimeInterval()
{
    int result;
    QString lastScan = getLastScanTime();  //上次扫描的时间
    QDateTime datetime = QDateTime::currentDateTime();  //当前时间
    QString str1 = datetime.toString("yyyy-MM-dd hh:mm:ss");

    QDateTime date1 = QDateTime::fromString(lastScan, "yyyy-MM-dd hh:mm:ss");
    QDateTime date2 = QDateTime::fromString(str1, "yyyy-MM-dd hh:mm:ss");
    uint stime = date1.toTime_t();
    uint etime = date2.toTime_t();
    // 一天的秒数
    int daysec = 24 * 60 * 60;
    int  day = 0 ;
    // 计算间隔天数
    if (stime > etime) {
        day = 0 ;
    } else {
        day = (etime - stime) / (daysec) + ((etime - stime) % (daysec) + (daysec - 1)) / (daysec) - 1;
    }

    // 根据天数得到对应的值
    if (day >= 90) {
        result = 2;
    } else if ((day < 90) && (day >= 30)) {
        result = 1;
    }  else {
        result = 0;
    }

    return result;
}

// 病毒库是不是在扫描中
bool HomePageWork::isScanning()
{
    return m_antivirusServer->isScanning();
}

void HomePageWork::showUpdateTip()
{
    DDialog *dialog = new DDialog;
    dialog->setMinimumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    dialog->setMaximumSize(QSize(DIALOG_WIDTH, DIALOG_HEIGHT));
    dialog->setIcon(QIcon::fromTheme("dcc_login_tip"));
    // 当前正在查杀中，请结束查杀后再更新
    dialog->setMessage(tr("A task is in process, please update later"));
    dialog->setWordWrapMessage(true);
    // 确定   推荐
    dialog->addButton(tr("OK"), true, DDialog::ButtonType::ButtonRecommend);
    connect(dialog, &DDialog::buttonClicked, [ = ](int index, QString) {
        switch (index) {
        case 0:
            dialog->close();
            break;
        default:
            break;
        }
    });
    dialog->exec();
}

// 是否进入开发者模式
bool HomePageWork::isDeveloperMode()
{
    QDBusReply<bool> reply = m_helperInter->call("IsDeveloperMode");
    bool flag = reply.value();
    return flag;
}

// 快速查杀
void HomePageWork::quickScan()
{
    //快速查询路径
    QString quick = "/bin*/sbin*/usr/bin*/usr/sbin*/usr/local*/tmp";
    m_quickThreats = 0;
    m_antivirusServer->doDissection(false, quick);
}
