/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "window/namespace.h"
#include "dtkwidget_global.h"
#include "window/modules/homepage/homepagemodel.h"
#include "com_deepin_defender_daemonservice.h"
#include "com_deepin_defender_datainterface.h"
#include "com_deepin_daemon_accounts.h"
#include "com_deepin_daemon_power.h"
#include "com_deepin_lastore_updater.h"
#include "com_deepin_lastore_jobmanager.h"
#include "com_deepin_lastore_job.h"
#include "com_deepin_daemon_systeminfo.h"
#include "com_deepin_defender_antiav.h"
#include <QGSettings>
#include <QDBusInterface>
#include <QDBusConnection>
#include <QDBusReply>

using DaemonService = com::deepin::defender::daemonservice;
using accounts = com::deepin::daemon::Accounts;
using PowerInter = com::deepin::daemon::Power;
using UpdaterInter = com::deepin::lastore::Updater;
using ManagerInter = com::deepin::lastore::Manager;
using JobInter = com::deepin::lastore::Job;
using SystemInfoInter = com::deepin::daemon::SystemInfo;

using DataInterFaceServer = com::deepin::defender::datainterface;

//病毒库dbus接口 服务名称
using AntivirusServer = com::deepin::defender::antiav;

DEF_NAMESPACE_BEGIN
DEF_ANTIVIRUS_NAMESPACE_BEGIN

QT_BEGIN_NAMESPACE
class QVBoxLayout;
class QHBoxLayout;
class QLabel;
class QPushButton;
QT_END_NAMESPACE
DWIDGET_USE_NAMESPACE

class HomePageWork : public QObject
{
    Q_OBJECT
public:
    explicit HomePageWork(HomePageModel *model, QObject *parent = nullptr);

    inline DaemonService *GetDaemonService()
    {
        return m_daemonservice;
    }
    inline accounts *GetAccounts()
    {
        return m_accounts;
    }
    inline DataInterFaceServer *GetNetControlServer()
    {
        return m_DataInterFaceServer;
    }
    inline DataInterFaceServer *GetStartup()
    {
        return m_DataInterFaceServer;
    }
    inline int GetEnabledata()
    {
        return m_Enabledata;
    }

//    inline AntivirusServer *AntivirusServer()
//    {
//        return m_antivirusServer;
//    }

Q_SIGNALS:
    void sendHaveUpdates(int updatestatus);//updatestatus = 0 检测未完成 1有更新  2无更新

    // 病毒库更新的结果
    void sendVdbUpdateResult(bool result);
    //void updateVdbSuccess();

    void sendThreatsCount(int nCount);
public Q_SLOTS:
    void getEnableInfo(int EnableInfo);
    void checkForUpdates();

    //获得病毒库更新结果
    void doVdbUpdateResult(bool isbFlag);
    // 快速扫描结果
    void onQuickScanResult(QString sPath, double dProgress, int iType, QString sDetailInfo);
public:
    void setSafetyScore(int score); //设置当前系统安全评分
    void setSafetyLevel(int level);
    void setLocalVer(QString ver);
    void setLatestVer(QString ver);
    void setLastCheckTime(QDateTime time);
    bool haveBattery();
    //获取使用电源时的自动锁屏时间
    int getLinePowerLockDelay();
    //获取使用电池时的自动锁屏时间
    int getBatteryLockDelay();
    //设置自动检查更新
    void setAutoCheckUpdates(bool isbAuto);
    // 获得更新提示
    bool getUpdateNotify();
    bool checkDbusIsValid();
    void setCheckUpdatesJob(const QString &jobPath);
    void onAppUpdateInfoFinished(QDBusPendingCallWatcher *w);

    // 上次病毒查杀时间
    QString getLastScanTime();

    // 获得安装时间
    QString getInstallTime();
    // 获得服务器病毒库版本
    QString getSrvVdbVersion();
    // 获得本地病毒库版本
    QString getLocalVdbVersion();

    // 查询所有的异常数据
    int selectAllThreatsData();

    void updateVDBVersion();

    // 查杀异常项目
    void scanThreats();

    // 数据库是否更新成功
    bool vdbUpdateSuccess();
    // 异常的数据库是否为空
    bool threatCountIsEmpty();
    // 查杀时间的间隔
    int scanTimeInterval();

    // 是不是在扫描中
    bool isScanning();

    // 展现更新提示
    void showUpdateTip();

    // 是否进入开发者模式
    bool isDeveloperMode();

    // 快速查杀
    void quickScan();
private:
    HomePageModel       *m_model;
    QGSettings          *m_gs;
    DaemonService       *m_daemonservice = nullptr;
    accounts            *m_accounts = nullptr;
    DataInterFaceServer *m_DataInterFaceServer = nullptr;
    PowerInter          *m_powerInter = nullptr;
    UpdaterInter        *m_updaterInter = nullptr;
    ManagerInter        *m_managerInter;
    QPointer<JobInter>  m_checkUpdateJob;
    int                 m_Enabledata;

    AntivirusServer     *m_antivirusServer = nullptr;
    QDBusInterface      *m_helperInter = nullptr;
    int                 m_quickThreats ;                         // 快速查杀异常个数
};

DEF_ANTIVIRUS_NAMESPACE_END
DEF_NAMESPACE_END
