#include "netcontrolwidget.h"

NetControlWidget::NetControlWidget(const QString &sPkgName, DWidget *parent)
    : DDialog(parent)
    , m_tableDelegate(new NetTableViewItemDelegate(this))
    , m_sLocatPkgName(sPkgName)
    , m_DataInterFaceServer(new DataInterFaceServer("com.deepin.defender.datainterface", "/com/deepin/defender/datainterface", QDBusConnection::sessionBus(), this))
{
    // 初始化
    initUi();
    initData();

}

NetControlWidget::~ NetControlWidget()
{
    Q_EMIT onDialogClose();
}

// 初始化界面
void NetControlWidget::initUi()
{
    // 弹框样式
    setBackgroundRole(QPalette::Window);
    setFixedWidth(520);
    setFixedHeight(458);

    // 添加信息栏
    DLabel *title = new DLabel(tr("Internet Control"));
    QFont ft = title->font();
    // 字体剧中
    title->setAlignment(Qt::AlignCenter);
    ft.setBold(true);
    title->setFont(ft);
    DFontSizeManager::instance()->bind(title, DFontSizeManager::T6);

    // 表格 - 模型 连接
    m_tableView = new DTableView;
    m_tableModel = new QStandardItemModel;
    m_tableView->setModel(m_tableModel);
    m_tableView->setItemDelegate(m_tableDelegate);

    // 设置表格样式 (表头 网格线 排序)
    m_tableView->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_tableView->setSelectionMode(QAbstractItemView::NoSelection);
    m_tableView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_tableView->horizontalHeader()->setStretchLastSection(true);   //自适应调整宽度
    m_tableView->verticalHeader()->setHidden(true);
    m_tableView->setSortingEnabled(true);
    m_tableView->verticalHeader()->setDefaultSectionSize(48);
    m_tableView->horizontalHeader()->setVisible(true);
    m_tableView->setShowGrid(false);
    m_tableView->setFrameShape(QTableView::NoFrame);

    // 表格
    m_netTableWidget = new DefenderTable;
    m_netTableWidget->setTableAndModel(m_tableView, m_tableModel);

    // 添加信息栏
    m_tipwid =new DWidget;
    m_tipwid->setContentsMargins(0,0,0,0);
    m_tipwidget = new TipWidget;
    m_tipwidget->setIcon(QIcon::fromTheme("dcc_warning"));
    m_tipwidget->setTitle(tr("Note: Disabling Internet access may cause unknown exception of applications"));
    QHBoxLayout *layout1 = new QHBoxLayout;
    layout1->setContentsMargins(30,0,30,0);
    layout1->addWidget(m_tipwidget,0,Qt::AlignCenter);
    m_tipwid->setLayout(layout1);
    m_tipwid->setVisible(false);


    // 最后将所有控件放入布局
    addContent(title);
    addContent(m_netTableWidget);
    addContent(m_tipwid);
    setContentLayoutContentsMargins(QMargins(10, 0, 10, 5));
    setSpacing(10);
}

// 初始化数据
void NetControlWidget::initData()
{
    // 添加表格列
    QStandardItem *item_0 = new QStandardItem(tr("Name"));
    item_0->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *item_1 = new QStandardItem(tr("Status"));
    item_1->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    m_tableModel->setHorizontalHeaderItem(0, item_0);
    m_tableModel->setHorizontalHeaderItem(1, item_1);
    m_tableModel->setHorizontalHeaderItem(2, new QStandardItem(QString("Icon")));
    m_tableModel->setHorizontalHeaderItem(3, new QStandardItem(QString("TypeName")));
    m_tableModel->setHorizontalHeaderItem(4, new QStandardItem(QString("RealNmae")));
    m_tableModel->setHorizontalHeaderItem(5, new QStandardItem(QString("PkgName")));

    // 注册
    registerDefenderProcInfoMetaType();
    registerDefenderProcInfoListMetaType();

    // 连接信号槽
    connect(m_DataInterFaceServer, &DataInterFaceServer::requestNetAppsChange, this, [ = ](DefenderProcInfoList sNetAppsInfo) {
        doNetAppsInfo(sNetAppsInfo);
    });

    // 获取联网管控所有APP信息
    m_DataInterFaceServer->getNetAppsInfo();

}

// 接收所有APPS的详细信息
void NetControlWidget::doNetAppsInfo(QList<DefenderProcInfo> sNetAppsInfos)
{
    // 表格初始化排序
    int nSortIndicatorSection = 0;
    Qt::SortOrder sSortIndicatorSort = Qt::AscendingOrder;
    if (!m_bIniFlag) {
        nSortIndicatorSection = m_tableView->horizontalHeader()->sortIndicatorSection();
        sSortIndicatorSort = m_tableView->horizontalHeader()->sortIndicatorOrder();
    }

    // 表格加载数据
    int nRow = 0;
    m_comboToAppName.clear();
    for (DefenderProcInfo defenderprocinfo : sNetAppsInfos) {
        // 获取联网管控默认状态
        m_sDefaultStatus = defenderprocinfo.sDefaultStatus;

        // 加载表格数据
        QString sName = DTK_CORE_NAMESPACE::Chinese2Pinyin(defenderprocinfo.sAppName);
        m_tableModel->setItem(nRow, 0, new QStandardItem(sName));
        m_tableModel->setItem(nRow, 1, new QStandardItem(defenderprocinfo.sStatus));
        m_tableModel->setItem(nRow, 2, new QStandardItem(defenderprocinfo.sThemeIcon));
        m_tableModel->setItem(nRow, 3, new QStandardItem(defenderprocinfo.isbSysApp ? tr("System") : tr("Third-party")));
        m_tableModel->setItem(nRow, 4, new QStandardItem(defenderprocinfo.sAppName));
        m_tableModel->setItem(nRow, 5, new QStandardItem(defenderprocinfo.sPkgName));

        // 表格内加载下拉框
        DComboBox *box = new DComboBox;
        box->addItem(tr("Ask"), NET_STATUS_ITEMASK);
        box->addItem(tr("Allow"), NET_STATUS_ItemAllow);
        box->addItem(tr("Disable"), NET_STATUS_ItemDisallow);

        // 下拉框内添加默认值
        switch (defenderprocinfo.sDefaultStatus.toInt()) {
        case NET_STATUS_ItemDefaultAsk:
            box->addItem(QString(tr("Default (%1)").arg(tr("Ask"))), NET_STATUS_ItemDefaultAsk);
            break;
        case NET_STATUS_ItemDefaultAllow:
            box->addItem(QString(tr("Default (%1)").arg(tr("Allow"))), NET_STATUS_ItemDefaultAllow);
            break;
        case NET_STATUS_ItemDefaultDisallow:
            box->addItem(QString(tr("Default (%1)").arg(tr("Disable"))), NET_STATUS_ItemDefaultDisallow);
            break;
        default:
            box->addItem(QString(tr("Default (%1)").arg(tr("Ask"))), NET_STATUS_ItemDefaultAsk);
            break;
        }
        qDebug() << "index value: " << defenderprocinfo.sStatus.toInt();
        int nStatus = defenderprocinfo.sStatus.toInt();
        box->setCurrentIndex(nStatus > 2 ? 3 : nStatus);
        connect(box, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged), this, &NetControlWidget::onAppOptChanged);

        // 添加到布局 并将box插入容器保存
        DWidget *boxw = new DWidget;
        QHBoxLayout *boxLayout = new QHBoxLayout;
        boxLayout->setContentsMargins(6, 6, 6, 6);
        boxLayout->addWidget(box);
        boxw->setLayout(boxLayout);
        m_comboToAppName.insert(box, defenderprocinfo.sPkgName);
        m_tableView->setIndexWidget(m_tableModel->index(nRow, 1), boxw);
        ++nRow;
    }

    // 设置前两列列的宽度
    m_tableView->setColumnWidth(0, 300);
    m_tableView->setColumnWidth(1, 180);

    // 隐藏多余的列 及设置初始化 排序
    m_tableView->setColumnHidden(2, true);
    m_tableView->setColumnHidden(3, true);
    m_tableView->setColumnHidden(4, true);
    m_tableView->setColumnHidden(5, true);

    m_tableView->horizontalHeader()->setHighlightSections(false);
    if (!m_bIniFlag) {
        if (nSortIndicatorSection == 1) {
            m_tableView->horizontalHeader()->setSortIndicator(0, Qt::AscendingOrder);
        }

        //　再次改变表格数据后，还原改变之前的排序规则
        m_tableView->horizontalHeader()->setSortIndicator(nSortIndicatorSection, sSortIndicatorSort);
        m_tableModel->sort(nSortIndicatorSection, sSortIndicatorSort);
    } else {
        // 初始化排序　默认采用第一列升序
        m_tableView->horizontalHeader()->setSortIndicator(0, Qt::AscendingOrder);
        m_tableModel->sort(nSortIndicatorSection, sSortIndicatorSort);
        m_bIniFlag = false;
    }

    // 如果定位参数有值， 则直接定位到那行
    if (!m_sLocatPkgName.isEmpty()) {
        setNetControlIndex(m_sLocatPkgName);

        // 设置完包名定位，清除某个应用程序改变的包名
        m_sLocatPkgName.clear();
    }
}

// 表格内下拉框选项改变
void NetControlWidget::onAppOptChanged(int index)
{
    // 获取改变的状态
    int nType;
    if (index == 3) {
        nType = m_sDefaultStatus.toInt();
        qDebug() << "DefaultStatus value: " << nType;
    } else {
        nType = index;
    }

    // 联网状态选择禁止，弹警告框
    if (NET_STATUS_ItemDisallow == nType || NET_STATUS_ItemDefaultDisallow == nType) {
        m_tipwid->setVisible(true);
        QTimer::singleShot(3000, this, [=]{
            m_tipwid->setVisible(false);
        });
    }

    // 获取改变的包名
    QComboBox *box = qobject_cast<QComboBox *>(sender());
    QString sPkgName = m_comboToAppName.value(box);

    qDebug() << "AppOptChange value: " << sPkgName << "and  " << nType;

    // 某个应用程序状态改变，需要将包名保存，以防止排序后能够快速定位该应用程序
    m_sLocatPkgName = sPkgName;
    // 将包名以及对应的类型发送给服务
    m_DataInterFaceServer->setNetAppStatusChange(sPkgName, nType);
}

// 设置表格选中行
void NetControlWidget::setNetControlIndex(const QString &sPkgName)
{
    for (int i = 0; i < m_tableModel->rowCount(); ++i) {
        QModelIndex index = m_tableModel->index(i,  1);
        QStandardItem *item = m_tableModel->item(i, 5);
        // 获取表格中每行的 包名, 如果校验成功则切换到这行
        if (item->data(Qt::DisplayRole).toString() == sPkgName) {
            m_tableView->setCurrentIndex(index);
            break;
        }
    }
}

QPixmap NetControlWidget::getPixmap(const QString &appIcon, const QSize &size)
{
    QIcon icon = QIcon::fromTheme(appIcon, QIcon::fromTheme("application-x-desktop"));
    const qreal ratio = devicePixelRatioF();
    QPixmap pixmap = icon.pixmap(size * ratio).scaled(size * ratio, Qt::KeepAspectRatio, Qt::SmoothTransformation);
    pixmap.setDevicePixelRatio(ratio);
    return pixmap;
}

// 表格代理内， 实现表格重绘
NetTableViewItemDelegate::NetTableViewItemDelegate(NetControlWidget *widget, QObject *parent)
    : QItemDelegate(parent)
    , m_widget(widget)
{
}

void NetTableViewItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
//    if (index.row() % 2  == 0) {
//        QBrush brush(QColor(0, 0, 0, 20));
//        painter->fillRect(option.rect, brush);
//    }

    int Pix_Inteval = 2;
    QRect rect = option.rect;
    // 第一列
    if (index.column() == 0) {
        // 设置图片
        int pix_x = rect.x() + 5;
        int pix_y = rect.y() + rect.height() / 2 - 14;
        int height = rect.height() - Pix_Inteval * 2;

        QString iconName = index.model()->data(index.model()->index(index.row(), 2), Qt::DisplayRole).toString();
        QPixmap pixmap = m_widget->getPixmap(iconName, QSize(28, 28));

        painter->drawPixmap(pix_x, pix_y, pixmap);

        // 绘画文字的坐标
        painter->save();
        QFont ft = painter->font();
        int textWidth = rect.width() - height - Pix_Inteval * 2;
        int x = pix_x + 38;
        int y = pix_y + height / 4;
        QString sName = index.model()->data(index.model()->index(index.row(), 4), Qt::DisplayRole).toString();
        QString sElidedName = painter->fontMetrics().elidedText(sName, Qt::ElideRight, textWidth, Qt::TextShowMnemonic);
        painter->drawText(x, y, sElidedName);
        y += rect.height() / 2;
        ft.setPixelSize(12);
        painter->setFont(ft);
        painter->setPen("#526A7F");

        // 来源
        QString source = index.model()->data(index.model()->index(index.row(), 3), Qt::DisplayRole).toString();;
        QString sElidedName1 = painter->fontMetrics().elidedText(source, Qt::ElideRight, textWidth, Qt::TextShowMnemonic);
        painter->drawText(x, y, sElidedName1);
//        painter->drawText(x, y, index.model()->data(index.model()->index(index.row(), 3), Qt::DisplayRole).toString());
        painter->restore();
        } else {
    }
}
