/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     luokai <luokai_cm@deepin.com>
 *
 * Maintainer: luokai <luokai_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once

#include "window/interface/moduleinterface.h"
#include "window/interface/frameproxyinterface.h"
#include "networkprotectionworker.h"
#include "appdatadetaildialog.h"
#include "src/widgets/defendertable.h"

#include <DButtonBox>
#include <DTipLabel>
#include <DCommandLinkButton>
#include <DBackgroundGroup>
#include <DFrame>
#include <DSpinner>

#include <QItemDelegate>
#include <QStandardItemModel>
#include <QTableView>

DWIDGET_USE_NAMESPACE
using namespace def::widgets;

#define FlowWidget_Debug 1

class FlowWidget;
class FlowItemDelegate: public QItemDelegate
{
    Q_OBJECT
public:
    FlowItemDelegate(FlowWidget *widget = nullptr, QObject *parent = nullptr);
    virtual ~ FlowItemDelegate() {}

    void paint(QPainter *painter,
               const QStyleOptionViewItem &option,
               const QModelIndex &index) const;

private:
    FlowWidget *m_widget;
};

class NetworkProtectionModel;
class NetworkProtectionWorker;
class FlowWidget : public QWidget
{
    Q_OBJECT
public:
    explicit FlowWidget(NetworkProtectionModel *model = nullptr,
                        NetworkProtectionWorker *worker = nullptr, QWidget *parent = nullptr);
    void setWaitingPage(bool isWaiting);
Q_SIGNALS:

public Q_SLOTS:
    void onRecAllAppFlowList(DefenderProcInfoList allAppFlowList,  const QString &sTimeRange);

private:
    void SetFlowAttributes();
    void updateTable();
    void showAppFlowDetailDialog(DefenderProcInfo procInfo);

private:
    NetworkProtectionModel *m_model;
    NetworkProtectionWorker *m_worker;
    QVBoxLayout *m_layout;
    DTipLabel *m_totalUsageLabel;
    DPushButton *m_timeRangeSelectBtn;
    QTableView *m_flowtableview;
    QStandardItemModel *m_flowitemmodel;
    // 流量表格模块
    DefenderTable *m_flowTableWidget;
    FlowItemDelegate *m_flowitem;
    // 接收到的流量时间跨度类型
    FlowRangeStyle m_recedFlowRangeStyle;
    // 等待界面
    DSpinner *m_waitSpinner;
    // 所有应用流量列表
    DefenderProcInfoList m_allAppFlowList;
};
DEF_NAMESPACE_END
DEF_NETPROTECTION_NAMESPACE_END
