/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     luokai <luokai_cm@deepin.com>
 *
 * Maintainer: luokai <luokai_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "flowmonitorcontrol.h"
#include "window/modules/networkprotection/networkprotectionmodel.h"
#include "window/modules/networkprotection/networkprotectionworker.h"
#include "window/utils.h"

#include <DStandardItem>
#include <DStyle>
#include <DPalette>
#include <DFontSizeManager>
#include <DApplicationHelper>

#include <QVBoxLayout>
#include <QStandardItemModel>
#include <QComboBox>
#include <QTableView>
#include <QHeaderView>
#include <QLabel>
#include <QColor>
#include <QVariant>
#include <QApplication>
#include <QDebug>

DWIDGET_USE_NAMESPACE
DEF_USING_NAMESPACE
DEF_USING_ANTIVIRUS_NAMESPACE

FlowMonitorControl::FlowMonitorControl(NetworkProtectionModel *model,
                                       NetworkProtectionWorker *worker, QWidget *parent)
    : QDialog(parent)
    , m_model(new QStandardItemModel(this))
    , m_delegate(new FlowMonitorControlItemDelegate())
    , m_view(new QTableView)
    , m_npModel(model)
    , m_worker(worker)
{
    setMinimumWidth(587);
    setMinimumHeight(458);

    QList<QString> titles;
    titles << tr("Name") << tr("Status");
    m_model->setColumnCount(2);
    m_model->setHorizontalHeaderItem(0, new QStandardItem(tr("Name")));
    m_model->setHorizontalHeaderItem(1, new QStandardItem(tr("Status")));

    m_view->setModel(m_model);
    m_view->setItemDelegate(m_delegate);
    m_view->horizontalHeader()->setVisible(true);
    m_view->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
    m_view->setShowGrid(false);
    m_view->setSortingEnabled(true);
    m_view->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_view->verticalHeader()->hide();
    m_view->verticalHeader()->setDefaultSectionSize(48);
    onAppsAccessOptionsChanged(m_worker->getRunningProcessOption());
    connect(m_npModel, &NetworkProtectionModel::appAccessOptionsChanged, this,
            &FlowMonitorControl::onAppsAccessOptionsChanged);

    QVBoxLayout *layout = new QVBoxLayout;
    layout->setContentsMargins(44, 0, 43, 50);
    QLabel *title = new QLabel(tr("Settings"));
    QFont ft = title->font();
    ft.setBold(true);
    title->setFont(ft);
    DFontSizeManager::instance()->bind(title, DFontSizeManager::T5);
    layout->addWidget(title);
    layout->addSpacing(10);
    layout->addWidget(m_view);
    setLayout(layout);
    setWindowTitle("");
}

void FlowMonitorControl::onAppsAccessOptionsChanged(QList<NetworkProtectionModel::AppNetworkAccessStruct> list)
{
    int i = 0;
    m_model->clear();
    DPalette pe = palette();

    m_comboToAppName.clear();
    for (NetworkProtectionModel::AppNetworkAccessStruct appOpt : list) {
        QStandardItem *item0 = new QStandardItem;
        item0->setData(Qt::BackgroundColorRole);

        item0->setText(appOpt.appName);
        item0->setData(appOpt.icon, Qt::WhatsThisPropertyRole);
        item0->setData(QString("Third app"), Qt::StatusTipPropertyRole);
        m_model->setItem(i, 0, item0);

        QStandardItem *item1 = new QStandardItem;
        m_model->setItem(i, 1, item1);

        QModelIndex index = m_model->index(i, 1);

        QComboBox *box = new QComboBox;
        box->addItem(tr("Ask"));
        box->addItem(tr("Allow"));
        box->addItem(tr("Disable"));
        QString defaultTxt;
        m_npModel->getNetworkAccessOption();
        box->addItem(QString(tr("Default (%1)")).arg(m_npModel->getNetworkAccessOptionName()));
        box->setCurrentIndex(appOpt.option);
        m_comboToAppName.insert(box, i);

        connect(box, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged), this, &FlowMonitorControl::onAppOptChanged);

        m_view->setIndexWidget(index, box);
        ++i;
    }
}

void FlowMonitorControl::onAppOptChanged(int index)
{
    int accessOpt = index;
    QComboBox *box = qobject_cast<QComboBox *>(sender());
    qDebug() << "m_comboToAppName.value(box) = " << m_comboToAppName.value(box) << " index = " << index;
    QList<NetworkProtectionModel::AppNetworkAccessStruct> apps = m_npModel->getRunningProcessOption();
    NetworkProtectionModel::AppNetworkAccessStruct item = apps[m_comboToAppName.value(box)];
    item.option = static_cast<NetworkProtectionModel::AppsAccessOption>(accessOpt);
    apps.replace(m_comboToAppName.value(box), item);

    m_worker->setRunningProcessOption(apps);
}

QIcon FlowMonitorControl::getAppIcon(const QString &appIcon)
{
    return QIcon::fromTheme(appIcon, QIcon::fromTheme("application-x-desktop"));
}

FlowMonitorControlItemDelegate::FlowMonitorControlItemDelegate(QObject *parent)
    : QItemDelegate(parent)
{
}

void FlowMonitorControlItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    int Pix_Inteval = 2;
    QRect rect = option.rect;
    // 第一列
    if (index.column() == 0) {
        // 画背景
        if (index.row() % 2 == 0) {
            QPainterPath path;
            path.addRoundRect(rect.adjusted(10, 0, 0, 0), 8, 8);
            const DPalette &pal = qApp->palette();
            painter->fillPath(path, pal.background());
        }

        // 设置图片
        int pix_x = rect.x() + 20;
        int pix_y = rect.y() + rect.height() / 2 - 14;
        int height = rect.height() - Pix_Inteval * 2;

        const QStandardItemModel *model = dynamic_cast<const QStandardItemModel *>(index.model());
        QStandardItem *item = model->itemFromIndex(index);
        QString iconName = item->data(Qt::WhatsThisPropertyRole).toString();
        QIcon icon = FlowMonitorControl::getAppIcon(iconName);

        painter->drawPixmap(pix_x, pix_y, 28, 28, icon.pixmap(QSize(28, 28)));

        // 绘画文字的坐标
        painter->save();
        QFont ft = painter->font();
        int textWidth = rect.width() - height - Pix_Inteval * 2;
        int x = pix_x + 38;
        int y = pix_y + height / 4;
        QString sName = index.model()->data(index, Qt::DisplayRole).toString();
        QString sElidedName = painter->fontMetrics().elidedText(sName, Qt::ElideRight, textWidth, Qt::TextShowMnemonic);
        painter->drawText(x, y, sElidedName);
        y += rect.height() / 2;
        ft.setPixelSize(12);
        painter->setFont(ft);
        painter->setPen("#526A7F");
        painter->drawText(x, y, index.model()->data(index, Qt::StatusTipPropertyRole).toString());
        painter->restore();
    } else {
        // 画背景
        if (index.row() % 2 == 0) {
            QPainterPath path;
            path.addRoundRect(rect.adjusted(-10, 0, -10, 0), 8, 8);
            const DPalette &pal = qApp->palette();
            painter->fillPath(path, pal.background());
        }
        QItemDelegate::paint(painter, option, index);
        return;
    }
}
