/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci_cm@deepin.com>
 *
 * Maintainer: caimengci <caimengci_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once

#include "window/namespace.h"

#include <QObject>

QT_BEGIN_NAMESPACE
class QGSettings;
QT_END_NAMESPACE

DEF_NAMESPACE_BEGIN
class NetworkProtectionModel : public QObject
{
    Q_OBJECT
public:
    explicit NetworkProtectionModel(QObject *parent = nullptr);
    void reset();

    enum NetworkShareOption {
        INVISIBLE = 0,
        VISIBLE,
    };

    enum NetworkAccessOption {
        DEFAULT_ASK = 0,
        DEFAULT_DISALLOW,
        DEFAULT_ALLOW,
    };

    enum AppsAccessOption {
        ItemAsk = 0,
        ItemAllow,
        ItemDisallow,
        ItemDefault,

        ColumnButt,
    };

    enum AppAskOption {
        AllowOnce, //pop always
        AllowAlways, // change to allow
        Diallow, //disallow
    };

    enum AppType {
        SystemApp = 0,
        ThirdPartyApp,
    };

    struct AppNetworkAccessStruct {
        QString appName;
        QString icon;
        QString path;
        AppType type;
        NetworkProtectionModel::AppsAccessOption option;
        inline bool operator==(const AppNetworkAccessStruct &other) {
            return ((appName == other.appName) && option == other.option);
        }
    };

    struct AppAskStruct {
        QString appName;
        AppAskOption askOpt;
    };

    NetworkShareOption getShareOption();
    void setShareOption(NetworkShareOption opt);
    NetworkAccessOption getNetworkAccessOption();
    QString getNetworkAccessOptionName();
    void setNetworkAccessOption(NetworkAccessOption opt);
    NetworkAccessOption getRemoteAccessedOption();
    void setRemoteAccessedOption(NetworkAccessOption opt);
    QList<AppNetworkAccessStruct> getRunningProcessOption();
    void setRunningProcessOption(QList<AppNetworkAccessStruct> list);
    QList<AppNetworkAccessStruct> getSortRunningProcessOption(int order = 0, int column = 0);
    void setAppAskOption(const QStringList &appNamePath, AppAskOption askOpt);
    bool getAppAskOption(const QString &appName);
    NetworkProtectionModel::AppsAccessOption getAppNetworkAccessOption(const QString &appName);
    int getAppNetworkIndexbyName(const QStringList &appNamePath);
    void setAppNetworkOption(int index, AppsAccessOption option);
    QString foundAppPath(const QString &path);
    void setAllowApp(const QString &appName, bool allow);
    void initAskApp();
    void initAllowApp();

Q_SIGNALS:
    void shareOptionChanged(int);
    void networkAccessOptionChanged(int);
    void remoteAccessedOptionChanged(int);
    void appAccessOptionsChanged(QList<AppNetworkAccessStruct> list);

private:
    QGSettings *m_gs;
    NetworkShareOption m_shareValue;
    NetworkAccessOption m_accessNetwork;
    NetworkAccessOption m_remoteAccessed;
    QList<AppNetworkAccessStruct> m_appNetworkAccessOptions;
    QStringList m_appAskOptions;
    QStringList m_appAllowApps;
};
Q_DECLARE_METATYPE(NetworkProtectionModel::AppNetworkAccessStruct)
DEF_NAMESPACE_END
