/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     wubowen <wubowen_cm@deepin.com>
 *
 * Maintainer: wubowen <wubowen_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "networkprotectionmodule.h"
#include "window/interface/frameproxyinterface.h"
#include "networkprotectionmodel.h"
#include "networkprotectionworker.h"
#include "networkprotectionwidget.h"
#include "trafficdetailwidget.h"

DEF_USING_NAMESPACE
DEF_USING_ANTIVIRUS_NAMESPACE

NetworkProtectionModule::NetworkProtectionModule(FrameProxyInterface *frameProxy, QObject *parent)
    : QObject(parent)
    , ModuleInterface(frameProxy)
    , m_model(nullptr)
    , m_worker(nullptr)
{

}

void NetworkProtectionModule::preInitialize()
{
    m_model = new NetworkProtectionModel;
    m_worker = new NetworkProtectionWorker(m_model);

    connect(m_worker, &NetworkProtectionWorker::requestShowNetworkModuleSettings, this, [this](const QStringList & appNamePath) {
        m_frameProxy->setCurrentModule(m_frameProxy->getModuleIndex(name()), 0);
        m_worker->showNetworkAccessSettings(appNamePath);
    });
}

void NetworkProtectionModule::initialize()
{

}

const QString NetworkProtectionModule::name() const
{
    return "firewall";
}

void NetworkProtectionModule::active(int index)
{
    Q_UNUSED(index);
    m_widget = new NetworkProtectionWidget(m_model, m_worker);
    m_widget->show();

    connect(m_widget, &NetworkProtectionWidget::notifyShowNetFlowMonitor, [ = ] {
        m_widget->hide();
        TrafficDetailWidget *wid = m_widget->gettrafficdetailwidget();
        m_frameProxy->pushWidget(this, wid);
    });

    m_frameProxy->pushWidget(this, m_widget);
}

void NetworkProtectionModule::deactive()
{

}
