﻿/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     wubowen <wubowen_cm@deepin.com>
 *
 * Maintainer: wubowen <wubowen_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "networkprotectionwidget.h"
#include "netprowidget/menuwidget.h"
#include "networkprotectionmodel.h"
#include "networkprotectionworker.h"
#include "trafficdetailwidget.h"
#include "window/utils.h"
#include "widgets/switchwidget.h"

#include <DFontSizeManager>
#include <DApplicationHelper>
#include <DListView>

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QListView>
#include <QStandardItemModel>
#include <QPushButton>
#include <QDebug>
#include <QGSettings>

static const QString FIRE_WALL_ON_OFF = "fire-wall-on-off";



DWIDGET_USE_NAMESPACE
DEF_USING_ANTIVIRUS_NAMESPACE

#define NET_REM_DEFAULT_ASK             0
NetworkProtectionWidget::NetworkProtectionWidget(NetworkProtectionModel *model,
                                                 NetworkProtectionWorker *worker, QWidget *parent)
    : QWidget(parent)
    , m_model(model)
    , m_worker(worker)
    , m_DataInterFaceServer(new DataInterFaceServer("com.deepin.defender.datainterface", "/com/deepin/defender/datainterface", QDBusConnection::sessionBus(), this))
    , m_hmiScreen(new HmiScreen("com.deepin.defender.hmiscreen", "/com/deepin/defender/hmiscreen", QDBusConnection::sessionBus(), this))
{
    installEventFilter(this);

    // 设置主布局
    mainLayout = new QHBoxLayout;
    setLayout(mainLayout);
    m_guiHelper = DGuiApplicationHelper::instance();
    // 初始化界面
    initUI();
    // 主题变换
    connect(m_guiHelper, SIGNAL(themeTypeChanged(ColorType)), this, SLOT(themeTypeChange(ColorType)));
}

// 布局
void NetworkProtectionWidget::initUI()
{
    // 主控件
    mainWgt = new QWidget;
    mainLayout->addWidget(mainWgt);

    // 垂直布局
    m_layout = new QVBoxLayout();
    mainWgt->setLayout(m_layout);

    // 主布局
    m_layout->setSpacing(0);
    m_layout->setContentsMargins(leftSpace, 30, 0, 0);

    // 中间水平布局
    QHBoxLayout *m_mainLayout = new QHBoxLayout();
    m_mainLayout->setMargin(5);
    m_mainLayout->setSpacing(0);


    // 标题
    QLabel *title = new QLabel(tr("Firewall helps protect your network security"));
    QFont ft = title->font();
    ft.setBold(true);
    title->setFont(ft);
    DFontSizeManager::instance()->bind(title, DFontSizeManager::T5);
    m_layout->addWidget(title);

    // 重置按钮
    m_resetBtn = new QPushButton(tr("Restore Defaults"));
    m_resetBtn->setMinimumSize(141, 36);
    connect(m_resetBtn, &QPushButton::clicked, [this] {
        qDebug() << " click restore defaults";
        m_worker->reset();
    });

    // 左侧布局
    m_layout->addWidget(title, 0, Qt::AlignLeft);
    m_layout->addSpacing(15);
    m_layout->addLayout(m_mainLayout);
    m_layout->addSpacing(15);
    m_layout->addWidget(m_resetBtn, 0, Qt::AlignLeft);
    m_layout->addSpacing(10);

    //  中间水平布局
    m_mainLayout->addLayout(initMenu());

    // 下面布局
    QWidget *bottomWidget = new QWidget;
    bottomWidget->setFixedHeight(100);
    bottomWidget->setMinimumWidth(400);
    bottomWidget->setLayout(initBottomButton());
    m_layout->addWidget(bottomWidget, 0, Qt::AlignBottom | Qt::AlignLeft);

    // 右侧的logo
    QLabel *logo = new QLabel(this);
    logo->setMaximumWidth(307);
    logo->setMinimumWidth(307);
    //logo->setMinimumSize(QSize(307, 300));
    logo->setPixmap(QIcon::fromTheme("dcc_network_protection_detection_image").pixmap(QSize(307, 300)));
    m_mainLayout->addWidget(logo, 0, Qt::AlignVCenter | Qt::AlignLeft);

    // 重置按钮注册事件
    m_resetBtn->installEventFilter(this);
}

// 左侧中间的布局
QVBoxLayout *NetworkProtectionWidget::initMenu()
{
    // 垂直布局
    QVBoxLayout *VLayout = new QVBoxLayout;
    VLayout->setContentsMargins(0, 0, 0, 0);
    VLayout->setSpacing(0);

    QVBoxLayout *menuLayout = new QVBoxLayout;
    menuLayout->setContentsMargins(0, 0, 0, 0);
    menuLayout->setSpacing(0);

    // 获取防火墙总开关设置
    bool optFirewall = m_DataInterFaceServer->getFireWallSwitchStatus();

    QHBoxLayout *hlayout = new QHBoxLayout;
    hlayout->setContentsMargins(5,0,5,0);
    DLabel *switchTitle = new DLabel;
    switchTitle->setText(tr("Firewall"));
    DSwitchButton *switchButton = new DSwitchButton;
    switchButton->setChecked(optFirewall);
    hlayout->addWidget(switchTitle,0,Qt::AlignLeft);
    hlayout->addWidget(switchButton,0,Qt::AlignRight);


    // 联网管控/远程访问总开关
    /*SwitchWidget *fireWallSwitch = new SwitchWidget;
    fireWallSwitch->addBackground();
    fireWallSwitch->setTitle(tr("Firewall"));
    // 开关设置初始状态
    fireWallSwitch->switchButton()->setChecked(optFirewall);*/

    // 联网管控/远程访问总开关提示
    DTipLabel *fireWallTxt = new DTipLabel(tr("The startup time of apps will be extended if firewall enabled"));
    fireWallTxt->setWordWrap(true);
    fireWallTxt->setAlignment(Qt::AlignLeft);

    QVBoxLayout *lyFirewall = new QVBoxLayout;
    lyFirewall->setSpacing(5);
    lyFirewall->addLayout(hlayout);
    //lyFirewall->addWidget(fireWallSwitch);
    fireWallTxt->setMargin(5);
    lyFirewall->addWidget(fireWallTxt);
    menuLayout->addLayout(lyFirewall);
    //menuLayout->addSpacing(10);

    // 防火墙总开关信号槽
    connect(switchButton, &DSwitchButton::checkedChanged, this, &NetworkProtectionWidget::fireWallSwitchChange);
    fireWallSwitchChange(optFirewall);

    MenuWidget *canFindWidget = new MenuWidget;
    DTipLabel *shareTxt = new DTipLabel(tr("If visible, your shared folders and files are viewable to computers in LAN"));
    shareTxt->setWordWrap(true);
    shareTxt->setAlignment(Qt::AlignLeft);
    canFindWidget->setVisible(false);
    shareTxt->setVisible(false);

    MenuWidget *funConn = new MenuWidget;
    DTipLabel *npTxt = new DTipLabel(tr("Allow or disable Internet access of applications and services"));
    npTxt->setWordWrap(true);
    npTxt->setAlignment(Qt::AlignLeft);

    MenuWidget *remoteConn = new MenuWidget;
    DTipLabel *remoteTxt = new DTipLabel(tr("Specify whether remote access to your system through applications and services is permitted or not"));
    remoteTxt->setWordWrap(true);
    remoteTxt->setAlignment(Qt::AlignLeft);

    QList<MenuWidget *> menus;
    menus << canFindWidget << funConn << remoteConn;
    QList<DTipLabel *> titles;

    titles << shareTxt << npTxt << remoteTxt;
    for (int i = 0; i < menus.length(); ++i) {
        QVBoxLayout *layout = new QVBoxLayout;
        layout->setSpacing(5);
        layout->addWidget(menus[i]);
        titles[i]->setMargin(5);
        layout->addWidget(titles[i]);
        menuLayout->addLayout(layout);
        //menuLayout->addSpacing(10);
    }
    menuLayout->addStretch(1);
    connect(canFindWidget, &MenuWidget::notifyClicked, [ = ](int index) {
        qDebug() << " canFindWidget index : " << index;
        m_model->setShareOption(static_cast<NetworkProtectionModel::NetworkShareOption>(index));
    });

    connect(funConn, &MenuWidget::notifyClicked, [ = ](int index) {
        qDebug() << " funConn index : " << index;

        int nDefaultStatus = swapIndexAndStatus(index);
        m_DataInterFaceServer->setNetControlDefaultStatus(nDefaultStatus);
    });

    connect(remoteConn, &MenuWidget::notifyClicked, [ = ](int index) {
        int nRemDefaultStatus = swapIndexAndStatus(index);
        m_DataInterFaceServer->setRemControlDefaultStatus(nRemDefaultStatus);
    });

    QStringList list;

    canFindWidget->setTitle(tr("Network Sharing"));
    list.clear();
    list << tr("Hide") << tr("Visible");
    canFindWidget->setMenuData(list);

    funConn->setTitle(tr("Internet Access"));
    list.clear();
    list << tr("Ask me always") << tr("Disable always") << tr("Allow always");
    funConn->setMenuData(list);

    remoteConn->setTitle(tr("Remote Connection"));
    list.clear();
    list << tr("Ask me always") << tr("Disable always") << tr("Allow always");
    remoteConn->setMenuData(list);

    int shareOpt = m_model->getShareOption();
    int networkAccessOpt = m_DataInterFaceServer->getNetControlDefaultStatus();
    int remoteAccessedOpt = m_DataInterFaceServer->getRemControlDefaultStatus();

    int nNetStatus = swapIndexAndStatus(networkAccessOpt);
    int nRemStatus = swapIndexAndStatus(remoteAccessedOpt);
    canFindWidget->setMenuIndex(shareOpt);
    funConn->setMenuIndex(nNetStatus);
    remoteConn->setMenuIndex(nRemStatus);

    connect(m_DataInterFaceServer, &DataInterFaceServer::requestRemDefaultAllowStatus, [ = ]() {
        remoteConn->setMenuIndex(2);
    });

    connect(m_model, &NetworkProtectionModel::shareOptionChanged, canFindWidget, &MenuWidget::setMenuIndex);
    connect(m_model, &NetworkProtectionModel::networkAccessOptionChanged, funConn, &MenuWidget::setMenuIndex);
    connect(m_model, &NetworkProtectionModel::remoteAccessedOptionChanged, remoteConn, &MenuWidget::setMenuIndex);

    DWidget *widget1 = new DWidget;
    widget1->setLayout(menuLayout);

    // 采用滑调区域控件
    QScrollArea *menuArea = new QScrollArea;
    //menuArea->setMinimumWidth(10);
    menuArea->setFixedHeight(310);
    menuArea->setFixedWidth(380);
    menuArea->setAlignment(Qt::AlignTop);
    menuArea->setWidgetResizable(true);
    menuArea->setContentsMargins(5, 5, 5, 5);
    menuArea->setWidget(widget1);
    // 不显示边框
    menuArea->setFrameShape(QFrame::NoFrame);
    VLayout->addWidget(menuArea);

    return VLayout;
}

// 默认状态转换
int NetworkProtectionWidget::swapIndexAndStatus(int swap1)
{
    int nSwap2 = 0;
    switch (swap1) {
    case 0:
        nSwap2 = 3;
        break;
    case 1:
        nSwap2 = 5;
        break;
    case 2:
        nSwap2 = 4;
        break;
    case 3:
        nSwap2 = 0;
        break;
    case 4:
        nSwap2 = 2;
        break;
    case 5:
        nSwap2 = 1;
        break;
    default:
        nSwap2 = 0;
        break;
    }
    return nSwap2;
}

QHBoxLayout *NetworkProtectionWidget::initBottomButton()
{
    QHBoxLayout *layout = new QHBoxLayout;
    layout->setMargin(0);
    layout->setSpacing(0);

    buttonView = new DListView;
    buttonView->setItemMargins(QMargins(0, 0, 0, 0));
    //Initialize top page view and model
    buttonModel = new QStandardItemModel(buttonView);
    buttonView->setModel(buttonModel);

    //buttonView->setFrameShape(QFrame::Shape::NoFrame);
    buttonView->setEditTriggers(QListView::NoEditTriggers);
    buttonView->setResizeMode(QListView::Adjust);
    buttonView->setViewMode(QListView::IconMode);
    buttonView->setDragEnabled(false);
    buttonView->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    buttonView->setIconSize(QSize(38, 38));
    buttonView->setSpacing(5);
//    buttonView->setItemSize(QSize(100, 80));
    buttonView->clearSelection();
    buttonView->clearFocus();
    buttonView->clearMask();
    buttonView->setSelectionMode(QAbstractItemView::NoSelection);
    buttonView->setTextElideMode(Qt::ElideRight);
    buttonView->setBackgroundType(DStyledItemDelegate::BackgroundType(DStyledItemDelegate::RoundedBackground | DStyledItemDelegate::NoNormalState));

    //退出模态页面(子页面)后，清除选中效果
    connect(buttonView, &QAbstractItemView::clicked, [ = ] {
        buttonView->clearFocus();
    });

    QStringList titles;
    titles << tr("Internet Control") << tr("Data Usage");

    QStringList icons;
    // 获得主题类型
    int type = m_guiHelper->themeType();
    if(type == 1){
        icons << "Internet_Control" << "Data_Usage";
    }else {
        icons << "Internet_Control_dark" << "Data_Usage_dark";
    }

    for (int i =  0; i < titles.size(); ++i) {
        QStandardItem *item = new QStandardItem;
        item->setIcon(QIcon::fromTheme(icons[i]));
        item->setText(titles[i]);
        item->setToolTip(titles[i]);
        item->setSizeHint(QSize(110, 80));
        buttonModel->appendRow(item);
    }

    layout->addWidget(buttonView);
    connect(buttonView, &QListView::clicked, this, &NetworkProtectionWidget::displayWidget);

    return layout;
}

void NetworkProtectionWidget::displayWidget(QModelIndex index)
{
    switch (static_cast<WidgetType>(index.row())) {
    case FuncConnectNetControl:
        showFuncConnectNetControl(QString(""));
        break;
    case NetworkLog:
        showNetworkLog();
        break;
    case ProfessionMode:
        showFuncConnectRemControl(QString(""));
        break;
    default:
        break;
    }
}

void NetworkProtectionWidget::showFuncConnectNetControl(const QString &sPkgName)
{
    m_hmiScreen->showFuncConnectNetControl(sPkgName);
}

void NetworkProtectionWidget::showNetworkLog()
{
    widget = new TrafficDetailWidget(m_model, m_worker);
    Q_EMIT notifyShowNetFlowMonitor();
}

// 远程访问弹框
void NetworkProtectionWidget::showFuncConnectRemControl(const QString &sPkgName)
{
    m_hmiScreen->showFuncConnectRemControl(sPkgName);
}

// 事件过滤器
bool NetworkProtectionWidget::eventFilter(QObject *obj, QEvent *event)
{
    if (this == obj) {
        if (QEvent::PaletteChange == event->type()) {
            int sAppColor = DGuiApplicationHelper::instance()->paletteType();
            qDebug() << " APP BackGround Color : " << sAppColor;
        }
    } else if (obj == m_resetBtn) {
        // 如果是按钮的字体发生变化
        if (QEvent::FontChange == event->type()) {
            // 先把主控件删除
            mainWgt->hide();
            mainWgt->deleteLater();
            // 初始化界面 把所有的界面构建出来
            initUI();
        }
    }
    return QWidget::eventFilter(obj, event);
}

// 防火墙总开关状态改变槽
void NetworkProtectionWidget::fireWallSwitchChange(bool isOnOff)
{
    if (isOnOff) {
        m_DataInterFaceServer->setFireWallSwitchStatus(isOnOff);
    } else {
        m_DataInterFaceServer->setFireWallSwitchStatus(isOnOff);
    }
}

void NetworkProtectionWidget::themeTypeChange(ColorType themeType)
{
    int type;
    switch (themeType) {
    case UnknownType:
        type = 0;
        break;
    case LightType:
        type = 1;
        break;
    case DarkType:
        type = 2 ;
        break;
    }
    QStringList icons;
    // 浅色背景
    if(type == 1){
        icons << "Internet_Control" << "Data_Usage";
    }else {
        icons << "Internet_Control_dark" << "Data_Usage_dark";
    }
    for (int i = 0; i < buttonModel->rowCount(); i++) {
        QStandardItem *item = buttonModel->item(i);
        item->setIcon(QIcon::fromTheme(icons[i]));
    }
}
