/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci_cm@deepin.com>
 *
 * Maintainer: caimengci <caimengci_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once

#include "window/namespace.h"
#include "networkprotectionmodel.h"
#include "com_deepin_dde_daemon_launcherd.h"
#include "com_deepin_defender_monitornetflow.h"
#include "com_deepin_defender_datainterface.h"
#include "csingleteon.h"
#include "../common/common.h"

#include <QObject>
#include <QMap>

#define Work_Speed_Debug 0
#define Work_Flow_Debug 0

//struct AppParameter {
//    QString name;
//    double recv;
//    double send;
//    int pid;
//    QString device;
//    QString type;
//    QString pidname;//desktop
//    double apptotal = 0;//单个APP总流量
//    QString sName;//显示的名字
//};

enum FlowRangeStyle {None, Today, Yesterday, ThisMonth, LastMonth};

struct allApp {
    QString Path;
    QString Name;
    QString ID;
    QString Icon;
};
Q_DECLARE_METATYPE(allApp);

//Q_DECLARE_METATYPE(AppParameter);

using Launcherd = com::deepin::dde::daemon::Launcher;
using MonitorNetFlow = com::deepin::defender::MonitorNetFlow;
using DataInterFaceServer = com::deepin::defender::datainterface;

DEF_NAMESPACE_BEGIN
class NetworkProtectionWorker : public QObject
{
    Q_OBJECT
public:
    explicit NetworkProtectionWorker(NetworkProtectionModel *model, QObject *parent = nullptr);
    void reset();
    QList<NetworkProtectionModel::AppNetworkAccessStruct> getRunningProcessOption();
    void setRunningProcessOption(QList<NetworkProtectionModel::AppNetworkAccessStruct> list);

    inline Launcherd *getlauncherd()
    {
        return m_launcherInter;
    }
    inline QList<allApp> GetAllapp()
    {
        return m_allapp;
    }
    inline DataInterFaceServer *GetTrafficdetails()
    {
        return m_DataInterFaceServer;
    }
    // 获取流量排名当前显示的时间跨度
    inline FlowRangeStyle GetOnDispFlowRange() {return m_onDispFlowRange;}
    void setRunningProcessOption(int index, NetworkProtectionModel::AppsAccessOption option);
    static NetworkProtectionModel::AppType getAppType(const QString &path);
    static QString getAppTypeName(NetworkProtectionModel::AppType type);

    void showNetworkAccessSettings(const QStringList &appNamePath);

    struct AppIcon {
        QString appName;
        QString icon;
        QString path;
        NetworkProtectionModel::AppType type;
    };


    ///////-=======
    DefenderProcInfoList getProcInfoList();
    void notifyGetYesterdayFlowData();
    void notifyGetTodayFlowData();
    void notifyGetLastMonthFlowData();
    void notifyGetThisMonthFlowData();
    // 通知停止获取所有应用流量数据
    void notifyDontGetFlowData();
    //请求获取某个app流量数据
    void notifyGetAppYesterdayFlowData(QString sExecPath);
    void notifyGetAppTodayFlowData(QString sExecPath);
    void notifyGetAppLastMonthFlowData(QString sExecPath);
    void notifyGetAppThisMonthFlowData(QString sExecPath);

private:
    QList<AppIcon> getAllApps();
    QDBusPendingReply<LauncherItemInfoList> getLauncherList();

Q_SIGNALS:
    void requestShowNetworkSettings(const QStringList &appName);
    void requestShowNetworkModuleSettings(const QStringList &appNamePath);
    void notifyUpdateLauncherList();
    void notifyUpdate();


    // 发送来自服务的流量排名应用数据
    void sendAllAppFlowList(DefenderProcInfoList in0, const QString &sTimeRange);
    //向应用流量详情对话框实例发送显示的数据列表
    void sendAppFlowList(DefenderProcInfoList flowLst);

private Q_SLOTS:
    void onUpdateLauncherList();

private:
    NetworkProtectionModel *m_model;
    Launcherd *m_launcherInter;
    MonitorNetFlow *m_monitornetflow;
    DataInterFaceServer *m_DataInterFaceServer;//流量监控dbus类
    QList<allApp> m_allapp;
    // 当前选择显示的流量时间范围
    FlowRangeStyle m_onDispFlowRange;
};
DEF_NAMESPACE_END
