/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     luokai <luokai_cm@deepin.com>
 *
 * Maintainer: luokai <luokai_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "speedwidget.h"

#include <DPinyin>

#include <QVBoxLayout>
#include <QHeaderView>
#include <QApplication>

#include <DPushButton>

// 表格中每列索引
const int AppNameDispCol = 0;
const int DownloadSpeedDispCol = 1;
const int UploadSpeedDispCol = 2;
const int NetContrDispCol = 3;
const int ProcIsRunningCol = 4;
const int PidCol = 5;
const int IsbSysAppCol = 6;
const int ProcNameCol = 7;
const int DesktopCol = 8;
const int AppNameCol = 9;
const int IconCol = 10;
const int DownloadSpeedCol = 11;
const int UploadSpeedCol = 12;

SpeedWidget::SpeedWidget(NetworkProtectionModel *model, NetworkProtectionWorker *worker, QWidget *parent)
    : QWidget(parent)
    , m_model(model)
    , m_worker(worker)
    , m_layout(new QVBoxLayout)
    , m_totalSpeedLabel(nullptr)
    , m_speedtableview(new QTableView)
    , m_speeditemmodel(new QStandardItemModel)
    , m_speedTableWidget(nullptr)
    , m_speeditem(new SpeedItemDelegate(this, this))
    , m_DataInterFaceServer(new DataInterFaceServer("com.deepin.defender.datainterface", "/com/deepin/defender/datainterface", QDBusConnection::sessionBus(), this))
    , m_hmiScreen(new HmiScreen("com.deepin.defender.hmiscreen", "/com/deepin/defender/hmiscreen", QDBusConnection::sessionBus(), this))
    , m_waitSpinner(new DSpinner)
{
    m_layout->setContentsMargins(0, 0, 0, 0);
    m_layout->setSpacing(0);
    setLayout(m_layout);

    // 总速度标签
    m_totalSpeedLabel = new DTipLabel(tr("Total Download") + ": 0.00KB/s" + "  " + tr("Total Upload") + ": 0.00KB/s");
    m_totalSpeedLabel->setAlignment(Qt::AlignVCenter);
    m_totalSpeedLabel->setMinimumHeight(40);
    m_layout->addWidget(m_totalSpeedLabel, 0, Qt::AlignLeft);

    // 表格
    m_speedTableWidget = new DefenderTable;
    m_speedTableWidget->setTableAndModel(m_speedtableview, m_speeditemmodel);
    m_layout->addWidget(m_speedTableWidget, 1/*, Qt::AlignCenter*/);

    // 等待界面
    m_waitSpinner->setFixedSize(30, 30);
    m_waitSpinner->setVisible(false);
    m_layout->addWidget(m_waitSpinner, 0, Qt::AlignCenter);

    SetSpeedAttributes();
    updateTable();
}

//更新表格数据
void SpeedWidget::updateTable()
{
    // 总网速
    double dTotalDownloadSpeed = 0.0;
    double dTotalUploadSpeed = 0.0;

    // 获取进程网速信息列表
    DefenderProcInfoList procFlowInfoList = m_worker->getProcInfoList();
    if (procFlowInfoList.size() == 0) {
        //设置鼠标等待界面
        m_waitSpinner->setVisible(true);
        m_waitSpinner->start();
        m_speedTableWidget->setVisible(false);
    } else {
        //恢复显示界面
        m_waitSpinner->setVisible(false);
        m_waitSpinner->stop();
        m_speedTableWidget->setVisible(true);
    }

    // 设置当前列表中所有行的 应用是否在运行项 为 "false"
    for (int i = 0; i < m_speeditemmodel->rowCount(); i++) {
        m_speeditemmodel->setData(m_speeditemmodel->index(i, ProcIsRunningCol), "false");
    }

    // 更新已存在进程数据 或 添加新进程数据
    int currentRowCount = m_speeditemmodel->rowCount();
    for (int i = 0; i < currentRowCount; i++) {
        // 获取当前行的进程号
        int nProcPid = m_speeditemmodel->data(m_speeditemmodel->index(i, PidCol), Qt::ItemDataRole::DisplayRole).toInt();
        for (int j = 0; j < procFlowInfoList.size(); j++) {
            if (nProcPid == procFlowInfoList.at(j).nPid) {
                DefenderProcInfo procInfoTmp = procFlowInfoList.at(j);

                // 设置列表中当前行的 应用是否在运行项 为 "true"
                m_speeditemmodel->setData(m_speeditemmodel->index(i, ProcIsRunningCol), "true");
                // 更新是否是系统应用
                m_speeditemmodel->setData(m_speeditemmodel->index(i, IsbSysAppCol), procInfoTmp.isbSysApp);
                // 更新进程名
                m_speeditemmodel->setData(m_speeditemmodel->index(i, ProcNameCol), procInfoTmp.sProcName);
                // 更新desktop
                m_speeditemmodel->setData(m_speeditemmodel->index(i, DesktopCol), procInfoTmp.sPkgName);
                // 更新应用名
                QString appName = procInfoTmp.sAppName.isEmpty() ? procInfoTmp.sProcName : procInfoTmp.sAppName;
                m_speeditemmodel->setData(m_speeditemmodel->index(i, AppNameCol), appName);
                // 更新当前行图标数据
                m_speeditemmodel->setData(m_speeditemmodel->index(i, IconCol), procInfoTmp.sThemeIcon);
                // 更新当前行网速数据
                QString sDownloadSpeed = changeFlowValueUnit(procInfoTmp.dDownloadSpeed, 2).append("/s");
                QString sUploadSpeed = changeFlowValueUnit(procInfoTmp.dUploadSpeed, 2).append("/s");
                m_speeditemmodel->setData(m_speeditemmodel->index(i, DownloadSpeedCol), sDownloadSpeed);
                m_speeditemmodel->setData(m_speeditemmodel->index(i, UploadSpeedCol), sUploadSpeed);

                // 设置名称显示列，用于排序
                m_speeditemmodel->setData(m_speeditemmodel->index(i, AppNameDispCol), DTK_CORE_NAMESPACE::Chinese2Pinyin(appName), Qt::ItemDataRole::EditRole);

                // 原始单位为KB/s的网速大小，用于排序
                m_speeditemmodel->item(i, DownloadSpeedDispCol)->setData(procInfoTmp.dDownloadSpeed, Qt::ItemDataRole::EditRole);
                m_speeditemmodel->item(i, UploadSpeedDispCol)->setData(procInfoTmp.dUploadSpeed, Qt::ItemDataRole::EditRole);

                // 找到已存在于表格中的进程项，提取数据后，从列表中移除该项
                procFlowInfoList.removeAt(j);
                // 统计总网速
                dTotalDownloadSpeed += procInfoTmp.dDownloadSpeed;
                dTotalUploadSpeed += procInfoTmp.dUploadSpeed;
                break;
            }
        }

    }

    // 移除表格中没有运行的进程对应的行
    for (int i = 0; i < m_speeditemmodel->rowCount(); ) {
        if ("false" ==m_speeditemmodel->index(i, ProcIsRunningCol).data().toString()) {
            m_speeditemmodel->removeRow(i);
        } else {
            i++;
        }
    }

    // 添加新进程数据
    for (DefenderProcInfo procFlowInfo : procFlowInfoList) {
        QStandardItem *nameItem = new QStandardItem;
        QStandardItem *downloadSpeedItem = new QStandardItem;
        QStandardItem *uploadSpeedItem = new QStandardItem;
        QStandardItem *settingItem = new QStandardItem;

        QList<QStandardItem *> itemList;
        itemList << nameItem << downloadSpeedItem << uploadSpeedItem << settingItem;
        m_speeditemmodel->insertRow(0, itemList);

        // ==========设置储存数据========
        // 设置进程号
        m_speeditemmodel->setData(m_speeditemmodel->index(0, PidCol), procFlowInfo.nPid);
        // 设置列表中当前行的 应用是否在运行项 为 "true"
        m_speeditemmodel->setData(m_speeditemmodel->index(0, ProcIsRunningCol), "true");
        // 更新是否是系统应用
        m_speeditemmodel->setData(m_speeditemmodel->index(0, IsbSysAppCol), procFlowInfo.isbSysApp);
        // 更新进程名
        m_speeditemmodel->setData(m_speeditemmodel->index(0, ProcNameCol), procFlowInfo.sProcName);
        // 更新desktop
        m_speeditemmodel->setData(m_speeditemmodel->index(0, DesktopCol), procFlowInfo.sPkgName);
        // 更新应用名
        m_speeditemmodel->setData(m_speeditemmodel->index(0, AppNameCol), procFlowInfo.sAppName);
        // 更新当前行图标数据
        m_speeditemmodel->setData(m_speeditemmodel->index(0, IconCol), procFlowInfo.sThemeIcon);
        // 更新当前行网速数据
        QString sDownloadSpeed = changeFlowValueUnit(procFlowInfo.dDownloadSpeed, 2).append("/s");
        QString sUploadSpeed = changeFlowValueUnit(procFlowInfo.dUploadSpeed, 2).append("/s");
        m_speeditemmodel->setData(m_speeditemmodel->index(0, DownloadSpeedCol), sDownloadSpeed);
        m_speeditemmodel->setData(m_speeditemmodel->index(0, UploadSpeedCol), sUploadSpeed);
        // ==========设置储存数据-end========

        // ==========设置显示数据=========
        // 设置应用名称，用于排序
        // 如果能够获取到应用名，就取应用名；否则，取进程名
        QString appName = procFlowInfo.sAppName.isEmpty() ? procFlowInfo.sProcName : procFlowInfo.sAppName;
        m_speeditemmodel->setData(m_speeditemmodel->index(0, AppNameDispCol), DTK_CORE_NAMESPACE::Chinese2Pinyin(appName), Qt::ItemDataRole::EditRole);
        // 原始单位为KB/s的网速大小，用于排序
        m_speeditemmodel->item(0, DownloadSpeedDispCol)->setData(procFlowInfo.dDownloadSpeed, Qt::ItemDataRole::EditRole);
        m_speeditemmodel->item(0, UploadSpeedDispCol)->setData(procFlowInfo.dUploadSpeed, Qt::ItemDataRole::EditRole);

        // 设置 联网管控 按钮
        QWidget *btnWgt = new QWidget;
        QHBoxLayout *btnLayout = new QHBoxLayout;
        btnLayout->setAlignment(Qt::AlignmentFlag::AlignLeft);
        btnWgt->setLayout(btnLayout);

        DPushButton *settingBtn = new DPushButton;
        settingBtn->setContentsMargins(0 ,15, 0, 15);
        settingBtn->setFixedSize(20 ,20);
        settingBtn->setFlat(true);
        settingBtn->setIcon(QIcon::fromTheme("dcc_control_button"));
        settingBtn->setDisabled(false);
        btnLayout->addWidget(settingBtn);
        m_speedtableview->setIndexWidget(m_speeditemmodel->index(0, 3), btnWgt);
        // 若不存在包名, 则设置控制按钮不可用
        if (procFlowInfo.sDesktopPath.isEmpty()) {
            settingBtn->setDisabled(true);
        }
        // 连接按钮信号
        connect(settingBtn, &DPushButton::clicked, this, [ = ] {
            m_hmiScreen->showFuncConnectNetControl(procFlowInfo.sPkgName);
        });
        // ==========设置显示数据-end=========

        // 统计总网速
        dTotalDownloadSpeed += procFlowInfo.dDownloadSpeed;
        dTotalUploadSpeed += procFlowInfo.dUploadSpeed;
    }

    // 根据当前排序状态排序
    Qt::SortOrder sortOrder = m_speedtableview->horizontalHeader()->sortIndicatorOrder();
    int sortCol = m_speedtableview->horizontalHeader()->sortIndicatorSection();
    m_speedtableview->sortByColumn(sortCol, sortOrder);

    // 更新总网速标签
    QString sTotalDownloadSpeed = changeFlowValueUnit(dTotalDownloadSpeed, 2).append("/s");
    QString sTotalUploadSpeed = changeFlowValueUnit(dTotalUploadSpeed, 2).append("/s");
    m_totalSpeedLabel->setText(tr("Total Download") + ": " + sTotalDownloadSpeed + "  " +
                               tr("Total Upload") + ": " + sTotalUploadSpeed);
}

//设置网速view属性
void SpeedWidget::SetSpeedAttributes()
{
    m_speedtableview->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_speedtableview->setSelectionMode(QAbstractItemView::NoSelection);
    m_speedtableview->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_speedtableview->setFrameShape(QTableView::NoFrame);
    //自动调整最后一列的宽度使它和表格的右边界对齐
    m_speedtableview->horizontalHeader()->setStretchLastSection(true);
    m_speedtableview->verticalHeader()->setHidden(true);
    m_speedtableview->setShowGrid(false);
    m_speedtableview->setSortingEnabled(true);
    m_speedtableview->verticalHeader()->setDefaultSectionSize(48);

    // 连接表格与模型
    m_speedtableview->setModel(m_speeditemmodel);
    //设置代理
    m_speedtableview->setItemDelegate(m_speeditem);
    // 初始化表格设置
    // 设置表头
    QStandardItem *standaritem0 = new QStandardItem(tr("Name"));
    standaritem0->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *standaritem1 = new QStandardItem(tr("Download"));
    standaritem1->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *standaritem2 = new QStandardItem(tr("Upload"));
    standaritem2->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *standaritem3 = new QStandardItem(tr("Settings"));
    standaritem3->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    m_speeditemmodel->setHorizontalHeaderItem(AppNameDispCol, standaritem0);
    m_speeditemmodel->setHorizontalHeaderItem(DownloadSpeedDispCol, standaritem1);
    m_speeditemmodel->setHorizontalHeaderItem(UploadSpeedDispCol, standaritem2);
    m_speeditemmodel->setHorizontalHeaderItem(NetContrDispCol, standaritem3);
    m_speeditemmodel->setHorizontalHeaderItem(ProcIsRunningCol, new QStandardItem(QString("ProcIsRunning")));
    m_speeditemmodel->setHorizontalHeaderItem(PidCol, new QStandardItem(QString("Pid")));
    m_speeditemmodel->setHorizontalHeaderItem(IsbSysAppCol, new QStandardItem(QString("IsbSysApp")));
    m_speeditemmodel->setHorizontalHeaderItem(ProcNameCol, new QStandardItem(QString("ProcName")));
    m_speeditemmodel->setHorizontalHeaderItem(DesktopCol, new QStandardItem(QString("Desktop")));
    m_speeditemmodel->setHorizontalHeaderItem(AppNameCol, new QStandardItem(QString("AppName")));
    m_speeditemmodel->setHorizontalHeaderItem(IconCol, new QStandardItem(QString("Icon")));
    m_speeditemmodel->setHorizontalHeaderItem(DownloadSpeedCol, new QStandardItem(QString("DownloadSpeed")));
    m_speeditemmodel->setHorizontalHeaderItem(UploadSpeedCol, new QStandardItem(QString("UploadSpeed")));

    m_speedtableview->setColumnHidden(ProcIsRunningCol, true);
    m_speedtableview->setColumnHidden(PidCol, true);
    m_speedtableview->setColumnHidden(IsbSysAppCol, true);
    m_speedtableview->setColumnHidden(ProcNameCol, true);
    m_speedtableview->setColumnHidden(DesktopCol, true);
    m_speedtableview->setColumnHidden(AppNameCol, true);
    m_speedtableview->setColumnHidden(IconCol, true);
    m_speedtableview->setColumnHidden(DownloadSpeedCol, true);
    m_speedtableview->setColumnHidden(UploadSpeedCol, true);

    m_speedtableview->setColumnWidth(AppNameDispCol, 220);
    m_speedtableview->setColumnWidth(DownloadSpeedDispCol, 215);
    m_speedtableview->setColumnWidth(UploadSpeedDispCol, 215);
    m_speedtableview->setColumnWidth(NetContrDispCol, 64);
    m_speedtableview->horizontalHeader()->setHighlightSections(false);

    // 默认下载速度排序
    m_speedtableview->sortByColumn(DownloadSpeedDispCol, Qt::SortOrder::DescendingOrder);
}

// 网速代理函数
SpeedItemDelegate::SpeedItemDelegate(SpeedWidget *widget, QObject *parent)
    : QItemDelegate(parent)
{
    m_widget = widget;
}

// 网速重绘操作
void SpeedItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    // 第一列
    if (index.column() == AppNameDispCol) {
        // 绘制图标
        QRect rect = option.rect;
        int pix_x = rect.x() + 4;
        int pix_y = rect.y() + 4;
        int height = rect.height() - 4 * 2;

        // 如果没有图标，则使用默认图标
        QString sThemeIcon = index.model()->index(index.row(), IconCol).data(Qt::DisplayRole).toString();
        if (sThemeIcon.isEmpty()) {
            sThemeIcon = "dcc_application_executable";
        }
        painter->drawPixmap(pix_x, pix_y, height, height,
                            QIcon::fromTheme(sThemeIcon).pixmap(height, height));

        // 绘画应用名
        int textWidth = rect.width() - height - 4 * 8;
        int x = pix_x + height + 4 * 2;
        int y = pix_y + rect.height() / 4 + height / 8;
        int y1 = pix_y + rect.height() / 2 + height / 4 + 4;
        QString sName = index.model()->data(index.model()->index(index.row(), AppNameCol), Qt::DisplayRole).toString();
        // 翻译 (其他应用)
        if (sName == "other apps") {
            sName = tr("Other apps");
        }

        QString sElidedName = painter->fontMetrics().elidedText(sName, Qt::ElideRight, textWidth, Qt::TextShowMnemonic);
        painter->drawText(x, y, sElidedName);

        // 绘画应用来源
        // 设置字体
        QFont sourceTypeFont;
        sourceTypeFont.setFamily("SourceHanSansSC-Normal");//字体
        sourceTypeFont.setPixelSize(12);//文字像素大小
        QPen sourceTypePen;
        sourceTypePen.setColor(QColor("#52607f"));
        // 判断是否是系统应用
        bool isbSysApp = index.model()->data(index.model()->index(index.row(), IsbSysAppCol), Qt::DisplayRole).toBool();
        QString sAppSourceType;
        if (isbSysApp)
            sAppSourceType = tr("System");
        else {
            sAppSourceType = tr("Third-party");
        }
        QString sElidedSource = painter->fontMetrics().elidedText(sAppSourceType, Qt::ElideRight, textWidth, Qt::TextShowMnemonic);
        painter->save();
        painter->setFont(sourceTypeFont);
        painter->setPen(sourceTypePen);
        painter->drawText(x, y1, sElidedSource);
        // 还原绘画器
        painter->restore();
    } else if (index.column() == DownloadSpeedDispCol) {
        // 绘制下载速度
        QString download = index.model()->data(index.model()->index(index.row(), DownloadSpeedCol), Qt::DisplayRole).toString();

        // 绘画图片和文字的坐标
        QRect rect = option.rect;
        int x = rect.x()+ 6;
        int y = rect.y() + rect.height() / 2 + 5;
        painter->drawText(x, y, download);
    } else if (index.column() == UploadSpeedDispCol) {
        // 绘制上传速度
        QString upload = index.model()->data(index.model()->index(index.row(), UploadSpeedCol), Qt::DisplayRole).toString();

        // 绘画文字的坐标
        QRect rect = option.rect;
        int x = rect.x()+ 6;
        int y = rect.y() + rect.height() / 2 + 5;
        painter->drawText(x, y, upload);
    }
}
