/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     luokai <luokai_cm@deepin.com>
 *
 * Maintainer: luokai <luokai_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "trafficdetailwidget.h"
#include "netprowidget/menuwidget.h"
#include "networkprotectionmodel.h"
#include "networkprotectionworker.h"
#include "netprowidget/flowmonitorcontrol.h"
#include "speedwidget.h"
#include "window/modules/networkprotection/appdatadetaildialog.h"

#include <DFontSizeManager>
#include <DApplicationHelper>
#include <DListView>
#include <DSimpleListView>
#include <DTipLabel>

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QDebug>
#include <QComboBox>
#include <QTableView>
#include <QHeaderView>
#include <functional>
#include <QToolButton>
#include <QMenu>

DWIDGET_USE_NAMESPACE
DEF_USING_ANTIVIRUS_NAMESPACE

TrafficDetailWidget::TrafficDetailWidget(NetworkProtectionModel *model, NetworkProtectionWorker *worker, QWidget *parent)
    : DFrame(parent)
    , m_model(model)
    , m_worker(worker)
    , m_speedwidget(nullptr)
    , m_flowwidget(nullptr)
    , m_layout(new QVBoxLayout)
    , m_widget(new QWidget)
    , m_vlayout(new QVBoxLayout)
    , m_topSwitchWidgetBtn(new Dtk::Widget::DButtonBox)
    , m_tablelayout(new QVBoxLayout)
    , m_updateTimer(nullptr)
{
    initUI();
}

TrafficDetailWidget::~TrafficDetailWidget()
{
    //退出流量详情页时，通知服务停止读取并发送流量数据
    m_worker->notifyDontGetFlowData();
}

//初始化ui界面
void TrafficDetailWidget::initUI()
{
    //将　widget　加入到一个新建的　DBackgroundGroup中
    QVBoxLayout *bglayout = new QVBoxLayout;
    bglayout->setContentsMargins(0, 0, 0, 0);
    m_bgGroup = new DBackgroundGroup(bglayout, this);
    m_bgGroup->setBackgroundRole(QPalette::Window);
    bglayout->addWidget(m_widget);

    //将 m_bgGroup 沉底
    m_bgGroup->lower();

    //设置m_bgGroup 的大小
    m_bgGroup->setFixedSize(size());

    // 标题栏布局
    QVBoxLayout *tileLayout = new QVBoxLayout;
    tileLayout->setContentsMargins(0, 0, 0, 0);
    QLabel *title = new QLabel(tr("Data Usage"));
    DFontSizeManager::instance()->bind(title, DFontSizeManager::T5);
    tileLayout->addWidget(title, 0, Qt::AlignTop);

    //按钮item
    DButtonBoxButton *btnUpdate = new DButtonBoxButton(tr("Apps and Services"));
    DButtonBoxButton *btnSetting = new DButtonBoxButton(tr("Rankings"));
    m_btnlist.append(btnUpdate);
    m_btnlist.append(btnSetting);
    m_topSwitchWidgetBtn->setButtonList(m_btnlist, true);
    m_topSwitchWidgetBtn->setMaximumSize(240, 36);
    m_btnlist.first()->setChecked(true);
    m_topSwitchWidgetBtn->setId(btnUpdate, 0);
    m_topSwitchWidgetBtn->setId(btnSetting, 1);
    m_topSwitchWidgetBtn->setMinimumSize(240, 36);
    m_btnlist.at(0)->setChecked(true);
    connect(m_topSwitchWidgetBtn, &DButtonBox::buttonClicked, [this](QAbstractButton * value) {
        settype(static_cast<TabelType>(m_topSwitchWidgetBtn->id(value)));
    });

    m_layout->setMargin(0);
    m_layout->addLayout(tileLayout);
    m_layout->setAlignment(Qt::AlignTop);
    m_layout->addWidget(m_topSwitchWidgetBtn, 0, Qt::AlignHCenter);

    setLayout(m_layout);

//    m_speedwidget = new SpeedWidget(m_model, m_worker);
//    m_speedwidget->setContentsMargins(0, 0, 0, 0);
//    m_flowwidget = new FlowWidget(m_model, m_worker);
//    m_speedwidget->setContentsMargins(0, 0, 0, 0);
//    m_tablelayout->addWidget(m_speedwidget);
//    m_tablelayout->addWidget(m_flowwidget, 1);
    m_layout->addLayout(m_tablelayout);

    m_updateTimer = new QTimer(this);
    m_updateTimer->setInterval(1000 * 1);

    showAppsServices();
//    connect(m_updateTimer, &QTimer::timeout, m_speedwidget, &SpeedWidget::updateTable);
//    m_updateTimer->start();

//    connect(m_worker, &NetworkProtectionWorker::sendAllAppFlowList, m_flowwidget, &FlowWidget::onRecAllAppFlowList);
}

//根据按钮显示对应界面
void TrafficDetailWidget::settype(TabelType type)
{
    qDebug() << "----- click index : " << type;
    m_tabeltype = type;
    switch (type) {
    case AppsServices:
        showAppsServices();
        break;
    case Trafficranking:
        showTrafficranking();
        break;
    }
}
//网速界面
void TrafficDetailWidget::showAppsServices()
{
    if (m_flowwidget != nullptr) {
        m_flowwidget->hide();
        m_tablelayout->removeWidget(m_flowwidget);
        disconnect(m_flowwidget);
        m_flowwidget->deleteLater();
        m_flowwidget = nullptr;
        //退出流量详情页时，通知服务停止读取并发送流量数据
        m_worker->notifyDontGetFlowData();
    }

    if (nullptr == m_speedwidget) {
        m_speedwidget = new SpeedWidget(m_model, m_worker);
        m_speedwidget->setContentsMargins(0, 0, 0, 0);
        m_tablelayout->addWidget(m_speedwidget);
        m_speedwidget->show();

        connect(m_updateTimer, &QTimer::timeout, m_speedwidget, &SpeedWidget::updateTable);
        m_updateTimer->start();
    }
}

//流量界面
void TrafficDetailWidget::showTrafficranking()
{
    if (m_speedwidget != nullptr) {
        m_speedwidget->hide();
        m_tablelayout->removeWidget(m_speedwidget);
        disconnect(m_speedwidget);
        m_speedwidget->deleteLater();
        m_speedwidget = nullptr;
        m_updateTimer->stop();
    }

    if (nullptr == m_flowwidget) {
        m_flowwidget = new FlowWidget(m_model, m_worker);
        m_flowwidget->setContentsMargins(0, 0, 0, 0);
        m_tablelayout->addWidget(m_flowwidget);
        m_flowwidget->show();

        connect(m_worker, &NetworkProtectionWorker::sendAllAppFlowList, m_flowwidget, &FlowWidget::onRecAllAppFlowList);
    }
}

//设置背景大小
void TrafficDetailWidget::resizeEvent(QResizeEvent *event)
{
    QFrame::resizeEvent(event);

    //设置m_bgGroup 的大小
    if (m_bgGroup)
        m_bgGroup->setFixedSize(size());
}
