/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "safetylevelwidget.h"
#include "widgets/scorebar.h"
#include "widgets/securitylevelitem.h"

#include <DFontSizeManager>
#include <DApplicationHelper>
#include <DSuggestButton>

#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QRadioButton>
#include <QButtonGroup>
#include <QPalette>
#include <QGraphicsOpacityEffect>

DWIDGET_USE_NAMESPACE

const double SafetyLevelItemNormalOpacity = 1;
const double SafetyLevelItemDisableOpacity = 0.5;

SafetyLevelWidget::SafetyLevelWidget(QWidget *parent)
    : QWidget(parent)
    , m_mode(nullptr)
    , m_work(nullptr)
    , m_safetyLevelBtnGroup(nullptr)
    , m_highLevelItem(nullptr)
    , m_midLevelItem(nullptr)
    , m_lowLevelItem(nullptr)
    , m_rightImg(nullptr)
    , m_bottomTipIcon(nullptr)
    , m_bottomIncreasingScore(nullptr)
    , m_bottomCurrentScoreLabel(nullptr)
    , m_bottomCurrentScore(nullptr)
    , m_closeBottomTimer(nullptr)
{
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->setContentsMargins(0, 0, 0, 0);
    setLayout(mainLayout);

    //Safety level setting
    QHBoxLayout *safetyLevelLayout = new QHBoxLayout;
    safetyLevelLayout->setContentsMargins(0, 0, 0, 0);
    mainLayout->addLayout(safetyLevelLayout);

    QWidget *safetyLevelLeftWidgt = new QWidget;
    safetyLevelLayout->addWidget(safetyLevelLeftWidgt, 1);

    QVBoxLayout *safetyLevelLeftWidgtLayout = new QVBoxLayout; //安全等级设置左布局，垂直布局
    safetyLevelLeftWidgtLayout->setContentsMargins(0, 0, 0, 0);
    safetyLevelLeftWidgtLayout->setSpacing(0);
    safetyLevelLeftWidgtLayout->setMargin(0);
    safetyLevelLeftWidgt->setLayout(safetyLevelLeftWidgtLayout);

    safetyLevelLeftWidgtLayout->addSpacing(100);
    QLabel *safetyLevelLabel = new QLabel(tr("Set Up Security Level"));
    safetyLevelLabel->setWordWrap(true);
    safetyLevelLabel->setContentsMargins(35, 0, 0, 0);
    QFont safetyLevelLabelFont = safetyLevelLabel->font();
    safetyLevelLabelFont.setBold(true);
    safetyLevelLabel->setFont(safetyLevelLabelFont);
    DFontSizeManager::instance()->bind(safetyLevelLabel, DFontSizeManager::T4);
    safetyLevelLabel->setAlignment(Qt::AlignLeft);
    safetyLevelLeftWidgtLayout->addWidget(safetyLevelLabel);

    QLabel *safetyLevelTip = new QLabel(tr("Each security level corresponds to a set of security policies, please choose carefully based on your situations"));
    safetyLevelTip->setWordWrap(true);
    safetyLevelTip->setContentsMargins(35, 0, 0, 0);
    DFontSizeManager::instance()->bind(safetyLevelTip, DFontSizeManager::T6);
    safetyLevelTip->setAlignment(Qt::AlignLeft);
    safetyLevelLeftWidgtLayout->addWidget(safetyLevelTip);

    safetyLevelLeftWidgtLayout->addSpacing(40);

    m_safetyLevelBtnGroup = new QButtonGroup; //安全等级设置按钮组
    m_safetyLevelBtnGroup->setExclusive(true);

    //high level setting
    QHBoxLayout *highLevelLayout = new QHBoxLayout;
    highLevelLayout->setContentsMargins(35, 0, 0, 0);
    highLevelLayout->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    safetyLevelLeftWidgtLayout->addLayout(highLevelLayout);

    QRadioButton *highLevelBtn = new QRadioButton;
    highLevelLayout->addWidget(highLevelBtn, 0, Qt::AlignCenter);
    m_safetyLevelBtnGroup->addButton(highLevelBtn, SafetyLevel::high);

    m_highLevelItem = new SecurityLevelItem;
    m_highLevelItem->setIcon(QIcon::fromTheme("dcc_security_level_high"));
    m_highLevelItem->setLevelText(tr("High"));
    m_highLevelItem->setTipText(tr("Do not allow to level down security settings"));
    highLevelLayout->addWidget(m_highLevelItem);

    //middle level setting
    safetyLevelLeftWidgtLayout->addSpacing(20);
    QHBoxLayout *midLevelLayout = new QHBoxLayout;
    midLevelLayout->setContentsMargins(35, 0, 0, 0);
    midLevelLayout->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    safetyLevelLeftWidgtLayout->addLayout(midLevelLayout);

    QRadioButton *midLevelBtn = new QRadioButton;
    midLevelLayout->addWidget(midLevelBtn, 0, Qt::AlignCenter);
    m_safetyLevelBtnGroup->addButton(midLevelBtn, SafetyLevel::middle);

    m_midLevelItem = new SecurityLevelItem;
    m_midLevelItem->setIcon(QIcon::fromTheme("dcc_security_level_medium"));
    m_midLevelItem->setLevelText(tr("Medium"));
    m_midLevelItem->setTipText(tr("Allow to level down security settings"));
    midLevelLayout->addWidget(m_midLevelItem);

    //low level setting
    safetyLevelLeftWidgtLayout->addSpacing(20);
    QHBoxLayout *lowLevelLayout = new QHBoxLayout;
    lowLevelLayout->setContentsMargins(35, 0, 0, 0);
    lowLevelLayout->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    safetyLevelLeftWidgtLayout->addLayout(lowLevelLayout);

    QRadioButton *lowLevelBtn = new QRadioButton;
    lowLevelLayout->addWidget(lowLevelBtn, 0, Qt::AlignCenter);
    m_safetyLevelBtnGroup->addButton(lowLevelBtn, SafetyLevel::low);

    m_lowLevelItem = new SecurityLevelItem;
    m_lowLevelItem->setIcon(QIcon::fromTheme("dcc_security_level_low"));
    m_lowLevelItem->setLevelText(tr("Low"));
    m_lowLevelItem->setTipText(tr("Your system has been rooted, please use it carefully due to its high risk"));
    lowLevelLayout->addWidget(m_lowLevelItem);
    safetyLevelLeftWidgtLayout->addStretch(1);

    //safety layout right part
    m_rightImg = new QLabel;
    m_rightImg->setAlignment(Qt::AlignVCenter);
    safetyLevelLayout->addWidget(m_rightImg, 1, Qt::AlignRight);

    //mainLayout->addStretch(1);

    //bottom score tip area
    QHBoxLayout *bottomAreaLayout = new QHBoxLayout;
    bottomAreaLayout->setAlignment(Qt::AlignCenter);
    bottomAreaLayout->setContentsMargins(0, 0, 0, 0);

    QWidget *bottomArea = new QWidget;
    bottomArea->setFixedHeight(56);
    bottomArea->setLayout(bottomAreaLayout);
    //mainLayout->addWidget(bottomArea);

    m_bottomTipIcon = new QLabel;
    m_bottomTipIcon->setPixmap(QPixmap(":/icons/def_safetylevel_high.svg"));
    m_bottomTipIcon->setFixedSize(15, 20);
    m_bottomTipIcon->setAlignment(Qt::AlignCenter);
    bottomAreaLayout->addWidget(m_bottomTipIcon);

    m_bottomIncreasingScore = new QLabel("+10cent");
    m_bottomIncreasingScore->setContentsMargins(0, 0, 0, 0);
    bottomAreaLayout->addWidget(m_bottomIncreasingScore);

    m_bottomCurrentScoreLabel = new QLabel("Current safety score: ");
    m_bottomCurrentScoreLabel->setContentsMargins(0, 0, 0, 0);
    bottomAreaLayout->addWidget(m_bottomCurrentScoreLabel);

    m_bottomCurrentScore = new QLabel( QString("0").append(" cent"));
    m_bottomCurrentScore->setContentsMargins(0, 0, 0, 0);
    bottomAreaLayout->addWidget(m_bottomCurrentScore);

    m_closeBottomTimer = new QTimer(this);

    //connect
    connect(m_safetyLevelBtnGroup, QOverload<int>::of(&QButtonGroup::buttonPressed), this, &SafetyLevelWidget::safetyLevelSetting);
    connect(m_closeBottomTimer, &QTimer::timeout, this, &SafetyLevelWidget::closeBottomTip);

    //level item connect
    connect(m_highLevelItem, &SecurityLevelItem::clicked, highLevelBtn, &QRadioButton::click);
    connect(m_midLevelItem, &SecurityLevelItem::clicked, midLevelBtn, &QRadioButton::click);
}

void SafetyLevelWidget::setModelAndWork(SafetyLevelModel *mode, SafetyLevelWork *work)
{
    m_mode = mode;
    m_work = work;
    updateUI();
}

void SafetyLevelWidget::updateUI()
{
    //safety level page UI control
    //level setting
    int safetyLevel = m_mode->getSafetyLevel();
    if (SafetyLevel::low == safetyLevel) {
        m_safetyLevelBtnGroup->button(SafetyLevel::high)->show();
        m_safetyLevelBtnGroup->button(SafetyLevel::high)->setCheckable(false);
        QGraphicsOpacityEffect *highLevelOpacityEffect = new QGraphicsOpacityEffect;
        highLevelOpacityEffect->setOpacity(SafetyLevelItemDisableOpacity);
        m_safetyLevelBtnGroup->button(SafetyLevel::high)->setGraphicsEffect(highLevelOpacityEffect);

        m_highLevelItem->show();
        m_highLevelItem->setDisableStyle(true);

        m_safetyLevelBtnGroup->button(SafetyLevel::middle)->show();
        m_safetyLevelBtnGroup->button(SafetyLevel::middle)->setCheckable(false);
        QGraphicsOpacityEffect *midLevelOpacityEffect = new QGraphicsOpacityEffect;
        midLevelOpacityEffect->setOpacity(SafetyLevelItemDisableOpacity);
        m_safetyLevelBtnGroup->button(SafetyLevel::middle)->setGraphicsEffect(midLevelOpacityEffect);

        m_midLevelItem->show();
        m_midLevelItem->setDisableStyle(true);

        m_safetyLevelBtnGroup->button(SafetyLevel::low)->show();
        m_safetyLevelBtnGroup->button(SafetyLevel::low)->setCheckable(true);
        QGraphicsOpacityEffect *lowLevelOpacityEffect = new QGraphicsOpacityEffect;
        lowLevelOpacityEffect->setOpacity(SafetyLevelItemNormalOpacity);
        m_safetyLevelBtnGroup->button(SafetyLevel::low)->setGraphicsEffect(lowLevelOpacityEffect);

        m_lowLevelItem->show();
        m_lowLevelItem->setDisableStyle(false);

        m_rightImg->setPixmap(QIcon::fromTheme("dcc_security_level_low_image").pixmap(304, 300));
    }else {
        m_safetyLevelBtnGroup->button(SafetyLevel::high)->show();
        m_safetyLevelBtnGroup->button(SafetyLevel::high)->setCheckable(true);
        QGraphicsOpacityEffect *highLevelOpacityEffect = new QGraphicsOpacityEffect;
        highLevelOpacityEffect->setOpacity(SafetyLevelItemNormalOpacity);
        m_safetyLevelBtnGroup->button(SafetyLevel::high)->setGraphicsEffect(highLevelOpacityEffect);

        m_highLevelItem->show();
        m_highLevelItem->setDisableStyle(false);

        m_safetyLevelBtnGroup->button(SafetyLevel::middle)->show();
        m_safetyLevelBtnGroup->button(SafetyLevel::middle)->setCheckable(true);
        QGraphicsOpacityEffect *midLevelOpacityEffect = new QGraphicsOpacityEffect;
        midLevelOpacityEffect->setOpacity(SafetyLevelItemNormalOpacity);
        m_safetyLevelBtnGroup->button(SafetyLevel::middle)->setGraphicsEffect(midLevelOpacityEffect);

        m_midLevelItem->show();
        m_midLevelItem->setDisableStyle(false);

        m_safetyLevelBtnGroup->button(SafetyLevel::low)->hide();
        m_safetyLevelBtnGroup->button(SafetyLevel::low)->setCheckable(false);
        QGraphicsOpacityEffect *lowLevelOpacityEffect = new QGraphicsOpacityEffect;
        lowLevelOpacityEffect->setOpacity(SafetyLevelItemDisableOpacity);
        m_safetyLevelBtnGroup->button(SafetyLevel::low)->setGraphicsEffect(lowLevelOpacityEffect);

        m_lowLevelItem->hide();
        m_lowLevelItem->setDisableStyle(true);
        if (SafetyLevel::high == safetyLevel) { //根据等级设置右边图片
            m_rightImg->setPixmap(QIcon::fromTheme("dcc_security_level_high_image").pixmap(304, 300));
        } else if (SafetyLevel::middle == safetyLevel) {
            m_rightImg->setPixmap(QIcon::fromTheme("dcc_security_level_medium_image").pixmap(304, 300));
        }
    }
    m_safetyLevelBtnGroup->button(safetyLevel)->setChecked(true); //设置按钮

    m_bottomTipIcon->hide();
    m_bottomIncreasingScore->hide();
    m_bottomCurrentScoreLabel->hide();
    m_bottomCurrentScore->hide();
}

void SafetyLevelWidget::safetyLevelSetting(int level)
{
    //只有当前处于高、中等级才能设置安全
    int lastLevel = m_mode->getSafetyLevel();
    if (SafetyLevel::low == lastLevel) return;
    m_work->setSafetyLevel(level);
    qDebug() << "set safety level: " << level;
    updateUI();
    showBottomTip(level, lastLevel);
}

void SafetyLevelWidget::showBottomTip(int level, int lastLevel)
{
    if (level == lastLevel) return;
    //int score = m_mode->getSafetyScore();
    if (level == SafetyLevel::high && lastLevel == SafetyLevel::middle){ //分数增加
        //m_work->setSafetyScore(score + 10); //设置分数
        m_bottomTipIcon->show();

        m_bottomIncreasingScore->show();
        m_bottomIncreasingScore->setText(QString("+10").append("cent"));
        QPalette pe = m_bottomIncreasingScore->palette();
        pe.setColor(QPalette::Text, Qt::green);
        m_bottomIncreasingScore->setPalette(pe);

        m_bottomCurrentScoreLabel->show();
        m_bottomCurrentScore->show();
        QString score;
        score.append(QString::number(m_mode->getSafetyScore()));
        score.append(" ");
        score.append("cent");
        m_bottomCurrentScore->setText(score);
    }else {
        //m_work->setSafetyScore(score - 10);
        m_bottomTipIcon->show();

        m_bottomIncreasingScore->show();
        m_bottomIncreasingScore->setText(QString("-10").append("cent"));
        QPalette pe = m_bottomIncreasingScore->palette();
        pe.setColor(QPalette::Text, Qt::red);
        m_bottomIncreasingScore->setPalette(pe);

        m_bottomCurrentScoreLabel->show();
        m_bottomCurrentScore->show();
        QString score;
        score.append(QString::number(m_mode->getSafetyScore()));
        score.append(" ");
        score.append("cent");
        m_bottomCurrentScore->setText(score);
    }

    m_closeBottomTimer->start(5000); //底部提示窗显示5s

}

void SafetyLevelWidget::closeBottomTip()
{
    m_bottomTipIcon->hide();
    m_bottomIncreasingScore->hide();
    m_bottomCurrentScoreLabel->hide();
    m_bottomCurrentScore->hide();
}
