/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci_cm@deepin.com>
 *
 * Maintainer: caimengci <caimengci_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "startupwidget.h"

#include <DFontSizeManager>
#include <QScrollBar>
#include <QFontMetrics>
#include <QFile>
#include <DPinyin>
#include "qpaintdevice.h"

#define All_Inteval             10
#define Pix_Inteval             4                   // 表格里图片距离表格边线 间隔
#define Action_Status_Disable   "Disable"           // 操作状态 - 不允许
#define Action_Status_Enable    "Enable"            // 操作状态 - 允许
#define Action_Flag_Disable     0                   // 状态标志 - 不允许
#define Action_Flag_Enable      1                   // 状态标志 - 允许

#define Startup_Disable         ":/icons/deepin/builtin/icons/dcc_startup_disable_29px.svg"  // 状态资源图片 - 不允许
#define Startup_Enable          ":/icons/deepin/builtin/icons/dcc_startup_enable_29px.svg"   // 状态资源图片 - 允许

DWIDGET_USE_NAMESPACE

StartupWidget::StartupWidget(DataInterFaceServer *datainterface, QWidget *parent)
    : DFrame(parent)
    , m_layout(new QVBoxLayout)
    , m_DataInterFaceServer(datainterface)
{
    initUI();
    //setFrameShape(QFrame::Shape::NoFrame);
    setLineWidth(0);
}

StartupWidget::~StartupWidget()
{
    Q_EMIT requstCloseInfo();
}

void StartupWidget::initUI()
{
    setBackgroundRole(QPalette::Window);
    QVBoxLayout *layout = new QVBoxLayout;
    layout->setContentsMargins(All_Inteval, All_Inteval, All_Inteval, All_Inteval);

    QLabel *title = new QLabel(tr("System Startup"));
    QFont font1 = title->font();
    font1.setBold(true);
    title->setFont(font1);
    DFontSizeManager::instance()->bind(title, DFontSizeManager::T6);

    m_detailInfo = new QLabel();
    DFontSizeManager::instance()->bind(m_detailInfo, DFontSizeManager::T8);

    layout->addWidget(title);
    layout->addWidget(m_detailInfo);
//    DFrame *tableFrame = new DFrame;
//    QVBoxLayout *widgetLayout = new QVBoxLayout;
//    widgetLayout->addWidget(initAppsTable());
//    tableFrame->setLayout(widgetLayout);

//    layout->addWidget(tableFrame);

    // table_view 属性
    m_table_view = new QTableView;
    m_table_view->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_table_view->setSelectionMode(QAbstractItemView::NoSelection);
    m_table_view->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_table_view->horizontalHeader()->setStretchLastSection(true);
    m_table_view->verticalHeader()->setHidden(true);
    m_table_view->setSortingEnabled(true);
    m_table_view->verticalHeader()->setDefaultSectionSize(48);
    m_table_view->horizontalHeader()->setVisible(true);
    m_table_view->setShowGrid(false);
    m_table_view->setFrameShape(QTableView::NoFrame);

    // 设置QTableView样式
    //setTableViewStyle();

    // 连接代理
    m_delegate = new MyItemDelegate(this, this);
    m_table_view->setItemDelegate(m_delegate);

    // 连接表格与模型
    m_item_model = new StartupModel;
    m_table_view->setModel(m_item_model);

    // 初始化表格设置
    // 设置表头
    QStandardItem *item_0 = new QStandardItem(tr("Name"));
    item_0->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *item_1 = new QStandardItem(tr("Auto Startup"));
    item_1->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *item_2 = new QStandardItem(tr("Action"));
    item_2->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);

    m_item_model->setHorizontalHeaderItem(0, item_0);
    m_item_model->setHorizontalHeaderItem(1, item_1);
    m_item_model->setHorizontalHeaderItem(2, item_2);
    m_item_model->setHorizontalHeaderItem(3, new QStandardItem("Id"));
    m_item_model->setHorizontalHeaderItem(4, new QStandardItem("Icon"));
    m_item_model->setHorizontalHeaderItem(5, new QStandardItem("Path"));
    m_item_model->setHorizontalHeaderItem(6, new QStandardItem("RealName"));


    // 表格
    m_tableWidget = new DefenderTable;
    m_tableWidget->setTableAndModel(m_table_view, m_item_model);
    layout->addWidget(m_tableWidget);
    this->setLayout(layout);

    // 取系统所有应用
    getAppsData();
    m_table_view->horizontalHeader()->setHighlightSections(false);
    m_table_view->horizontalHeader()->setSortIndicator(1, Qt::DescendingOrder);


}

// 添加应用表格
/*QTableView *StartupWidget::initAppsTable()
{
    // table_view 属性
    m_table_view = new QTableView(this);
    m_table_view->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_table_view->setSelectionMode(QAbstractItemView::NoSelection);
    m_table_view->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_table_view->horizontalHeader()->setStretchLastSection(true);
    m_table_view->verticalHeader()->setHidden(true);
    m_table_view->setSortingEnabled(true);
    m_table_view->verticalHeader()->setDefaultSectionSize(48);

    // 设置QTableView样式
    setTableViewStyle();

    // 连接代理
    m_delegate = new MyItemDelegate(this, this);
    m_table_view->setItemDelegate(m_delegate);

    // 连接表格与模型
    m_item_model = new StartupModel;
    m_table_view->setModel(m_item_model);

    // 初始化表格设置
    // 设置表头
    QStandardItem *item_0 = new QStandardItem(tr("Name"));
    item_0->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *item_1 = new QStandardItem(tr("Auto Startup"));
    item_1->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *item_2 = new QStandardItem(tr("Action"));
    item_2->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);

    m_item_model->setHorizontalHeaderItem(0, item_0);
    m_item_model->setHorizontalHeaderItem(1, item_1);
    m_item_model->setHorizontalHeaderItem(2, item_2);
    m_item_model->setHorizontalHeaderItem(3, new QStandardItem("Id"));
    m_item_model->setHorizontalHeaderItem(4, new QStandardItem("Icon"));
    m_item_model->setHorizontalHeaderItem(5, new QStandardItem("Path"));
    m_item_model->setHorizontalHeaderItem(6, new QStandardItem("RealName"));

    // 取系统所有应用
    getAppsData();
    m_table_view->horizontalHeader()->setHighlightSections(false);
    m_table_view->horizontalHeader()->setSortIndicator(1, Qt::DescendingOrder);

    // 表格
//    m_tableWidget = new DefenderTable;
//    m_tableWidget->setTableAndModel(m_table_view, m_item_model);

    return m_table_view;
}*/

// 设置QTableView样式
/*void StartupWidget::setTableViewStyle()
{
    // 隐藏表格网线
    m_table_view->setShowGrid(false);
    m_table_view->setFrameShape(QTableView::NoFrame);
}*/

// 取系统所有应用
void StartupWidget::getAppsData()
{
    m_mapEnable.clear();
    m_mapDisable.clear();

    // 取所有应用数据信息
    if (m_DataInterFaceServer->startLauncherManage()) {
        QString sEnableInfo = m_DataInterFaceServer->getAppsInfoEnable();
        QString sDisableInfo = m_DataInterFaceServer->getAppsInfoDisable();

        qDebug() << "getAppsData-Enable: " << sEnableInfo;
        qDebug() << "getAppsData-Disable: " << sDisableInfo;

        // 取开启状态下的应用信息
        QStringList sEnableInfoList = sEnableInfo.split("|");
        for (int i = 0; i < sEnableInfoList.count(); ++i) {
            QStringList sEnableDetailInfoList = sEnableInfoList.at(i).split(",");
            if (sEnableDetailInfoList.count() >= 5) {
                QList<QString> sList;
                sList.append(sEnableDetailInfoList.at(0));
                sList.append(sEnableDetailInfoList.at(1));
                sList.append(sEnableDetailInfoList.at(2));
                sList.append(sEnableDetailInfoList.at(3));
                m_mapEnable[sEnableDetailInfoList.at(4)] = sList;
            }
        }

        // 取关闭状态下的应用信息
        QStringList sDisableInfoList = sDisableInfo.split("|");
        for (int i = 0; i < sDisableInfoList.count(); ++i) {
            QStringList sDisableDetailInfoList = sDisableInfoList.at(i).split(",");
            if (sDisableDetailInfoList.count() >= 5) {
                QList<QString> sList;
                sList.append(sDisableDetailInfoList.at(0));
                sList.append(sDisableDetailInfoList.at(1));
                sList.append(sDisableDetailInfoList.at(2));
                sList.append(sDisableDetailInfoList.at(3));
                m_mapDisable[sDisableDetailInfoList.at(4)] = sList;
            }
        }

        // 先按状态降序 再按名称升序
        m_rowCount = 0;
        if (m_mapEnable.count() > 0)
            loadMap(m_mapEnable);

        if (m_mapDisable.count() > 0)
            loadMap(m_mapDisable);

        // 设置统计行数据
        showTotalInfo();
    }
}

// QMap容器 数据加载
void StartupWidget::loadMap(QMap<QString, QStringList> map)
{
    // 加载应用
    QMapIterator<QString, QStringList> i(map);
    while (i.hasNext()) {
        i.next();
        QStringList list = i.value();
        if (list.count() < 4)
            break;

        QString sName = i.key();
        QString sPath = list.at(0);
        QString sIcon = list.at(1);
        QString sId = list.at(2);
        QString sRealName = list.at(3);

        // 加载应用
        loadAppData(m_rowCount, sPath, sName, sIcon, sId, sRealName);
        m_rowCount++;
    }

    // 隐藏最后两列
    m_table_view->setColumnHidden(3, true);
    m_table_view->setColumnHidden(4, true);
    m_table_view->setColumnHidden(5, true);
    m_table_view->setColumnHidden(6, true);

    // 设置前三列的宽度
    m_table_view->setColumnWidth(0, 400);
    m_table_view->setColumnWidth(1, 150);
    m_table_view->setColumnWidth(2, 150);
}

// 加载每个应用的详细信息
void StartupWidget::loadAppData(int nRow, QString sPath, QString sName, QString sIcon, QString sId, QString sRealName)
{
    // 判断该应用状态（是否是自启动）
    bool bStatus = m_DataInterFaceServer->isAutostart(sPath);
    // 设置每一行数据
    m_item_model->setItem(nRow, 0, new QStandardItem(QString("%1").arg(sName)));
    m_item_model->setItem(nRow, 1, new QStandardItem(QString("%1").arg(bStatus ? Action_Flag_Enable : Action_Flag_Disable)));
    m_item_model->setItem(nRow, 2, new QStandardItem(""));
    m_item_model->setItem(nRow, 3, new QStandardItem(QString("%1").arg(sId)));
    m_item_model->setItem(nRow, 4, new QStandardItem(QString("%1").arg(sIcon)));
    m_item_model->setItem(nRow, 5, new QStandardItem(QString("%1").arg(sPath)));
    m_item_model->setItem(nRow, 6, new QStandardItem(QString("%1").arg(sRealName)));

    // 字体样式
    QPalette pal;
    if (bStatus)
        pal.setColor(QPalette::ButtonText, Qt::red);
    // 添加操作按钮 文本、颜色、属性等 （操作与应用状态正好相反）
    QString sDisableText = (bStatus) ? tr("Remove") : tr("Allow");
    QPushButton *button = new QPushButton();
    button->setObjectName(sId);
    button->setText(QString("%1").arg(sDisableText));
    button->setPalette(pal);
    button->setProperty("status", bStatus ? Action_Flag_Disable : Action_Flag_Enable);
    button->setProperty("path", sPath);
    button->setProperty("id", sId);
    connect(button, &QPushButton::clicked, this, &StartupWidget::onTableBtnClicked);

    DFrame *buttonFrame = new DFrame;
    QVBoxLayout *buttonLayout = new QVBoxLayout;
    buttonLayout->setContentsMargins(6, 6, 6, 6);
    buttonLayout->addWidget(button);
    buttonFrame->setLayout(buttonLayout);
    buttonFrame->setFrameStyle(QFrame::NoFrame);
    m_table_view->setIndexWidget(m_item_model->index(nRow, 2), buttonFrame);
}

// 表格按钮点击槽
void StartupWidget::onTableBtnClicked()
{
    QPushButton *button = qobject_cast<QPushButton *>(sender());
    int nStatus = button->property("status").toInt();
    QString sPath = button->property("path").toString();
    QString sID = button->property("id").toString();

    // 执行自启动开启/关闭操作
    if (m_DataInterFaceServer->exeAutostart(nStatus, sPath)) {
        if (Action_Flag_Disable == nStatus)
            changeItemStatus(false, sID);
        else
            changeItemStatus(true, sID);

        showTotalInfo();
    }
}

// 按钮点击后改变相应行的显示数据和状态
void StartupWidget::changeItemStatus(bool isStartup, QString sFlagData)
{
    QModelIndex index = m_table_view->currentIndex();
    int rows = m_item_model->rowCount();
    int column = m_item_model->columnCount();
    for (int i = 0; i < rows; ++i) {
        for (int j = 0; j < column; ++j) {
            QString sData =  m_item_model->data(m_item_model->index(i, j), Qt::DisplayRole).toString();
            if (j == 3 && sData == sFlagData) {
                qDebug() << "item data: " << sData;
                m_item_model->setItem(i, 1, new QStandardItem(QString("%1").arg(isStartup ? Action_Flag_Enable : Action_Flag_Disable)));
                QWidget *widget = m_table_view->indexWidget(m_item_model->index(i, 2));
                QPushButton *button = widget->findChild<QPushButton *>(sFlagData);

                // 字体样式
                QPalette pal;
                if (isStartup)
                    pal.setColor(QPalette::ButtonText, Qt::red);
                // 添加操作按钮 文本、颜色、属性等 （操作与应用状态正好相反）
                QString sDisableText = (isStartup) ? tr("Remove") : tr("Allow");
                button->setText(QString("%1").arg(sDisableText));
                button->setPalette(pal);
                button->setProperty("status", isStartup ? Action_Flag_Disable : Action_Flag_Enable);
            }
        }
    }

    // 设置前三列的宽度
    m_table_view->setColumnWidth(0, 400);
    m_table_view->setColumnWidth(1, 150);
    m_table_view->setColumnWidth(2, 150);

    m_table_view->selectRow(index.row());
}



// 外部调用刷新表格数据
void StartupWidget::refreshData(bool bAdd, QString sID)
{
    changeItemStatus(bAdd, sID);
    showTotalInfo();
}

// 设置统计行信息
void StartupWidget::showTotalInfo()
{
    int nAllCount = 0;
    int nEnableCount = 0;
    int nDisableCount = 0;
    if (m_DataInterFaceServer->startLauncherManage()) {
        QString sEnableInfo = m_DataInterFaceServer->getAppsInfoEnable();
        QString sDisableInfo = m_DataInterFaceServer->getAppsInfoDisable();

        if (!sEnableInfo.isEmpty())
            nEnableCount = sEnableInfo.split("|").count();

        if (!sDisableInfo.isEmpty())
            nDisableCount = sDisableInfo.split("|").count();
    }
    nAllCount = nEnableCount + nDisableCount;

    // 设置统计行数据
    m_detailInfo->setText(tr("Startup apps") + QString(": %1,   ").arg(nEnableCount) + tr("All apps") + QString(": %1").arg(nAllCount));
}

// 代理函数
MyItemDelegate::MyItemDelegate(StartupWidget *widget, QObject *parent)
    : QItemDelegate(parent)
{
    m_widget = widget;
}

// 重绘操作
void MyItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
//    if (index.row() % 2  == 0) {
//        QBrush brush(QColor(0, 0, 0, 8));
//        painter->fillRect(option.rect, brush);
//    }

    // 第一列
    if (index.column() == 0) {
        // 设置图片
        QRect rect = option.rect;
        int pix_x = rect.x() + Pix_Inteval;
        int pix_y = rect.y() + Pix_Inteval;
        int height = rect.height() - Pix_Inteval * 2;

        QFont font1;
        font1.setFamily("SourceHanSansSC-Medium");
        font1.setPixelSize(13);//文字像素大小
        QPen pen1;
        pen1.setColor(QColor("#001a2e"));

        QFont font2;
        font2.setFamily("SourceHanSansSC-Normal");//字体
        font2.setPixelSize(12);//文字像素大小
        QPen pen2;
        pen2.setColor(QColor("#52607f"));

        // 读取该应用执行信息
        QString sAppsSources;
        QString sPath = index.model()->data(index.model()->index(index.row(), 5), Qt::DisplayRole).toString();
        QFile fileInfo(sPath);
        if (QFileInfo(sPath).isSymLink()) {
            sAppsSources = tr("Third-party");
        } else if (fileInfo.open(QIODevice::ReadOnly | QIODevice::Text)) {
            QTextStream in(&fileInfo);
            in.setCodec("UTF-8");
            QString sAllInfo = in.readAll();
            int nIndex = sAllInfo.indexOf("Exec");
            QString sOptInfo = sAllInfo.mid(nIndex, 10);
            if (sOptInfo.contains("opt"))
                sAppsSources = tr("Third-party");
            else
                sAppsSources = tr("System");
        }

        // 图标路径
        QString sIconPath = index.model()->data(index.model()->index(index.row(), 4), Qt::DisplayRole).toString();
        qDebug() << "MyItemDelegate--paint:" << sIconPath;
        QFile file(sIconPath);
        if (file.open(QIODevice::ReadOnly) && sIconPath.contains("/")) {
            // 绘画图片坐标
            QPixmap pixmap = QPixmap(sIconPath);
            pixmap = m_widget->getPixmap(pixmap);
            painter->drawPixmap(pix_x, pix_y, height, height, pixmap);
            file.close();
        } else {
            // 绘画图片坐标
            QIcon icon = QIcon::fromTheme(sIconPath);
            QRect rectIcon(pix_x, pix_y, height, height);
            icon.paint(painter, rectIcon);
        }

        // 绘画文字的坐标
        painter->save();
        int textWidth = rect.width() - height - Pix_Inteval * 8;
        int x = pix_x + height + Pix_Inteval * 2;
        int y = pix_y + rect.height() / 4 + height / 8;
        int y1 = pix_y + rect.height() / 2 + height / 4 + Pix_Inteval;
        QString sName = index.model()->data(index.model()->index(index.row(), 6), Qt::DisplayRole).toString();
        QString sElidedName = painter->fontMetrics().elidedText(sName, Qt::ElideRight, textWidth, Qt::TextShowMnemonic);
        painter->drawText(x, y, sElidedName);

        // 绘画应用来源
        QString sElidedSource = painter->fontMetrics().elidedText(sAppsSources, Qt::ElideRight, textWidth, Qt::TextShowMnemonic);
        painter->setFont(font2);
        painter->setPen(pen2);
        painter->drawText(x, y1, sElidedSource);
        painter->restore();
    }
    // 第二列
    else if (index.column() == 1) {
        // 设置图片
        int nStatus = index.model()->data(index, Qt::DisplayRole).toInt();
        QPixmap pixmap = (nStatus == 0)
                         ? QPixmap(Startup_Disable)
                         : QPixmap(Startup_Enable);

        pixmap = m_widget->getPixmap(pixmap);
        const QPixmap &star = pixmap;
        // 绘画图片和文字的坐标
        int height = star.height();
        QRect rect = option.rect;
        int x = rect.x() + Pix_Inteval * 2;
        int y = rect.y() + rect.height() / 2 - height / 2;

        painter->drawPixmap(x, y, star);

    } else {
//        QItemDelegate::paint(painter, option, index);
//        return;
    }
}

QPixmap StartupWidget::getPixmap(QPixmap pixmap)
{
    const qreal ratio = devicePixelRatioF();
    pixmap.scaled(pixmap.size() *ratio, Qt::KeepAspectRatio, Qt::SmoothTransformation);
    pixmap.setDevicePixelRatio(ratio);

    return pixmap;
}

StartupModel::StartupModel()
{
}

QVariant StartupModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
        return QVariant();

    if (role == Qt::TextAlignmentRole) {
        return int(Qt::AlignLeft | Qt::AlignVCenter);
    } else if (role == Qt::DisplayRole) {
        return QStandardItemModel::data(index, role);
    } else if (role == Qt::BackgroundColorRole) {
        if (index.row() % 2 == 0) {
            return QColor(0, 0, 0, 8);
        } else if (index.row() % 5 == 1)
            return QColor("#FFFFFF");
        else
            return QVariant();
    }

    return QVariant();
}
