/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci_cm@deepin.com>
 *
 * Maintainer: caimengci <caimengci_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "window/interface/moduleinterface.h"
#include "window/interface/frameproxyinterface.h"
#include "com_deepin_defender_datainterface.h"
#include "../src/widgets/defendertable.h"

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>

#include <DDialog>
#include <DFrame>
#include <QWidget>
#include <QItemDelegate>
#include <QTableView>
#include <QStandardItemModel>
#include <QSortFilterProxyModel>
#include <QAbstractItemView>
#include <QHeaderView>
#include <QModelIndex>
#include <QPainter>
#include <QPixmap>
#include <QPushButton>
#include <QDBusPendingReply>
#include <QPair>

using DataInterFaceServer = com::deepin::defender::datainterface;
DWIDGET_USE_NAMESPACE
using namespace def::widgets;

/************************* StartupWidget实现类 表格代理 ***********************/
class StartupWidget;
class MyItemDelegate: public QItemDelegate
{
    Q_OBJECT
public:
    MyItemDelegate(StartupWidget *widget, QObject *parent = nullptr);
    virtual ~ MyItemDelegate() {}

    void paint(QPainter *painter,
               const QStyleOptionViewItem &option,
               const QModelIndex &index) const;

private:
    QPixmap m_pixmap;
    StartupWidget *m_widget;
};

/************************* StartupWidget实现类 表格颜色重载 ***********************/
class StartupModel : public QStandardItemModel
{
    Q_OBJECT
public:
    StartupModel();
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
};

/**************************** StartupWidget实现类 ********************************/
class StartupWidget : public DFrame
{
    Q_OBJECT
public:
    explicit StartupWidget(DataInterFaceServer *datainterface, QWidget *parent = nullptr);
    ~StartupWidget();

    QPixmap getPixmap(QPixmap);
Q_SIGNALS:
    void requstCloseInfo();

public Q_SLOTS:
    // 表格按钮点击槽
    void onTableBtnClicked();
    // 取所有应用详细信息
    void getAppsData();

    // 外部调用刷新表格数据
    void refreshData(bool bAdd, QString sID);

private:
    // 初始化界面
    void initUI();
    // 添加表格布局
   // QTableView *initAppsTable();
    // 设置QTableView样式
    //void setTableViewStyle();
    // 加再数据到Map容器 进行数据处理
    void loadMap(QMap<QString, QStringList>);
    // 将处理后的数据加载到QTableView表格
    void loadAppData(int nRow, QString sPath, QString sName, QString sIcon, QString sId, QString sRealName);

    // 设置统计行信息
    void showTotalInfo();

    // 按钮点击后改变相应行的显示数据和状态
    void changeItemStatus(bool isStartup, QString sFlagData);

private:
    QVBoxLayout        *m_layout            = nullptr;
    QLabel             *m_detailInfo        = nullptr;
    int                 m_rowCount          = 0;                // 表格加载数据总行数
    QMap<QString, QStringList> m_mapEnable;                     // 不允许自启动数据容器
    QMap<QString, QStringList> m_mapDisable;                    // 允许自启动数据容器

    QTableView         *m_table_view        = nullptr;          // 表格
    StartupModel       *m_item_model        = nullptr;          // QT表格模型
    MyItemDelegate     *m_delegate          = nullptr;          // 表格代理
    DataInterFaceServer *m_DataInterFaceServer    = nullptr;      // 取应用数据接口

    int                 m_nSaveSortColumn   = 1;
    Qt::SortOrder       m_nSaveSortOrder    = Qt::DescendingOrder;
    DefenderTable       *m_tableWidget = nullptr;
};
