#include "loginsecuritywidget.h"
#include <pwd.h>
#include <unistd.h>

#include <QLabel>
#include <QScrollArea>

#include <DCommandLinkButton>
#include <DFontSizeManager>

LoginSecurityWidget::LoginSecurityWidget(SystemSafetyModel *model, SystemSafetyWork *work, QWidget *parent)
    : QWidget(parent)
    , m_model(model)
    , m_work(work)
    , m_accounts(new accounts("com.deepin.daemon.Accounts", "/com/deepin/daemon/Accounts", QDBusConnection::systemBus(), this))
    , m_daemonservice(new DaemonService("com.deepin.defender.daemonservice", "/com/deepin/defender/daemonservice" ,
                                    QDBusConnection::sessionBus(), this))
    , m_layout(new QVBoxLayout)
    #ifdef user_passwor_tip
    , m_PasswodUpdateTips(new QLabel)
    #endif
    , m_pButtonGroup(new QButtonGroup(this))
{
    struct passwd *pws;
    pws = getpwuid(getuid());
    QString m_currentUserName = QString(pws->pw_name);
    qDebug() << "m_currentUserName = " << m_currentUserName;
    user = m_accounts->FindUserByName(m_currentUserName);//获取当前用户名称
    qDebug() << "user = " << user;
    m_usermanage = new DbususerManager(user);
    int date =  m_usermanage->passwordLastChange();//获取上次设置密码距1970年1月1日的天数
    const QDateTime datetime(QDateTime::currentDateTime());
    QDateTime dt = QDateTime(QDate(1970, 1, 1));//获取当天距1970年1月1日的天数
    m_day = dt.daysTo(datetime) - date;
    initui();
    qDebug()<<"date = " <<date;
    //修改密码刷新天数
    connect(m_daemonservice, &DaemonService::UpdateUI, [this] {
        qDebug() << "PasswordLastChangeChanged";
        Setlastchage();
    });
}

LoginSecurityWidget::~LoginSecurityWidget()
{

}

// 初始化界面
void LoginSecurityWidget::initui()
{
    // 添加滑动条区域
    QScrollArea *scrollArea = new QScrollArea(this);
    scrollArea->setFrameShape(QFrame::Shape::NoFrame);
    scrollArea->setWidgetResizable(true);
    scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);
    m_layout->addWidget(scrollArea);

    // 将mainWidget设置为滑动区域的主窗口
    QWidget *scrollAreaWidget = new QWidget(scrollArea);
    scrollArea->setWidget(scrollAreaWidget);

    // 设置滑动条区域布局器
    QVBoxLayout *scrollAreaLayout = new QVBoxLayout;
    scrollAreaLayout->setContentsMargins(0, 0, 20, 0);
    scrollAreaWidget->setLayout(scrollAreaLayout);

    QLabel *title = new QLabel(tr("Password Policy"));
    QFont ft = title->font();
    ft.setBold(true);
    title->setFont(ft);
    DFontSizeManager::instance()->bind(title, DFontSizeManager::T5);
    scrollAreaLayout->setSpacing(0);
    scrollAreaLayout->addWidget(title, 0, Qt::AlignLeft | Qt::AlignTop);
    scrollAreaLayout->setAlignment(Qt::AlignTop);
    scrollAreaLayout->addSpacing(15);
    QVBoxLayout *vlayout = new QVBoxLayout;

    // 设置互斥
    m_pButtonGroup->setExclusive(true);

    QHBoxLayout *highlayout = new QHBoxLayout();
    highlayout->setContentsMargins(0, 0, 0, 0);
    highlayout->setAlignment(Qt::AlignLeft);
    QRadioButton *highButton = new QRadioButton(this);
    m_highLevelItem = new SecurityLevelItem;
    m_highLevelItem->setIcon(QIcon::fromTheme("dcc_security_level_high"));
    m_highLevelItem->setStatusIcon(QIcon::fromTheme("dcc_select"));
    m_highLevelItem->setLevelText(tr("High"));
    m_highLevelItem->setTipText(tr("The password must have at least 8 characters, and contain at least 3 of the four available character types: lowercase letters, uppercase letters, numbers, and symbols"));
    highlayout->addWidget(highButton, 0, Qt::AlignCenter);
    highlayout->addWidget(m_highLevelItem);
    vlayout->addLayout(highlayout);
    m_pButtonGroup->addButton(highButton, LoginSecurity::high);
    vlayout->addSpacing(15);

    QHBoxLayout *mediumlayout = new QHBoxLayout();
    mediumlayout->setContentsMargins(0, 0, 0, 0);
    mediumlayout->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QRadioButton *mediumButton = new QRadioButton(this);
    m_mediumLevelItem = new SecurityLevelItem;
    m_mediumLevelItem->setIcon(QIcon::fromTheme("dcc_security_level_medium"));
    m_mediumLevelItem->setStatusIcon(QIcon::fromTheme("dcc_select"));
    m_mediumLevelItem->setLevelText(tr("Medium"));
    m_mediumLevelItem->setTipText(tr("The password must have at least 6 characters, and contain at least 2 of the four available character types: lowercase letters, uppercase letters, numbers, and symbols"));
    mediumlayout->addWidget(mediumButton, 0, Qt::AlignCenter);
    mediumlayout->addWidget(m_mediumLevelItem);
    vlayout->addLayout(mediumlayout);
    m_pButtonGroup->addButton(mediumButton, LoginSecurity::middle);
    vlayout->addSpacing(15);

    QHBoxLayout *lowlayout = new QHBoxLayout();
    lowlayout->setContentsMargins(0, 0, 0, 0);
    lowlayout->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QRadioButton *lowButton = new QRadioButton(this);
    m_lowLevelItem = new SecurityLevelItem;
    m_lowLevelItem->setIcon(QIcon::fromTheme("dcc_security_level_low"));
    m_lowLevelItem->setStatusIcon(QIcon::fromTheme("dcc_select"));
    m_lowLevelItem->setLevelText(tr("Low"));
    m_lowLevelItem->setTipText(tr("No restrictions"));
    lowlayout->addWidget(lowButton, 0, Qt::AlignCenter);
    lowlayout->addWidget(m_lowLevelItem, 1);
    vlayout->addLayout(lowlayout);
    m_pButtonGroup->addButton(lowButton, LoginSecurity::low);

    vlayout->addSpacing(15);
    scrollAreaLayout->addLayout(vlayout);
    scrollAreaLayout->addSpacing(15);

    m_menu = new PassWordMenu;
    m_menu->setTitle(tr("Change Password"));
    m_menu->setMenuData(tr("Go to Settings"));
    scrollAreaLayout->addWidget(m_menu, 0, Qt::AlignTop);
    //跳转到控制中心修改密码界面
    connect(m_menu, &PassWordMenu::notifyClicked, this, [ = ] {
        m_work->GetControlCenter()->ShowModule("accounts");
    });
    scrollAreaLayout->addSpacing(15);
#ifdef user_passwor_tip
    if (m_day >= 90) {
        m_PasswodUpdateTips->setText(tr("You have not changed the password for %1 days, please change it regularly to improve system security.").arg(m_day));
    } else {
        m_PasswodUpdateTips->hide();
    }
    m_PasswodUpdateTips->setWordWrap(true);
    m_PasswodUpdateTips->setAlignment(Qt::AlignLeft);
    QFont ft1 = m_PasswodUpdateTips->font();
    ft1.setBold(true);
    m_PasswodUpdateTips->setFont(ft1);
    DFontSizeManager::instance()->bind(m_PasswodUpdateTips, DFontSizeManager::T8);
    scrollAreaLayout->addWidget(m_PasswodUpdateTips);
#endif
    QStringList annos;
    annos << tr("%1 days").arg(30) << tr("%1 days").arg(60) << tr("%1 days").arg(90) << tr("%1 days").arg(120) << tr("%1 days").arg(150) << tr("%1 days").arg(180) << tr("Never");

    //使用电源时自动锁屏时间设置滚动条
    m_linepasswordSliderItem = new TitledSliderItem(tr("Notify me before password expired"));
    m_linepasswordSliderItem->setParent(this);
    m_linepasswordSliderItem->slider()->setType(DCCSlider::Vernier);
    m_linepasswordSliderItem->slider()->setRange(0, 6);
    m_linepasswordSliderItem->slider()->setTickPosition(QSlider::TicksBelow);
    m_linepasswordSliderItem->slider()->setTickInterval(0);
    m_linepasswordSliderItem->slider()->setPageStep(0);
    m_linepasswordSliderItem->setAnnotations(annos);
    scrollAreaLayout->addWidget(m_linepasswordSliderItem);
    scrollAreaLayout->addSpacing(15);
    int maturityday = m_daemonservice->maturityDay();
    //设置初始值
    m_linepasswordSliderItem->slider()->setValue(maturityday);
//    m_linepasswordSliderItem->setValueLiteral(sliderValueToLiteralString(maturityday));

    DTipLabel *tiplable = new DTipLabel(tr("The notification will pop up, asking you to change the password before expiration"));
    tiplable->setWordWrap(true);
    tiplable->setAlignment(Qt::AlignLeft);
    scrollAreaLayout->addWidget(tiplable);
    // 连接信号槽
    connect(m_pButtonGroup, QOverload<int>::of(&QButtonGroup::buttonPressed), this, &LoginSecurityWidget::onButtonClicked);
    connect(m_highLevelItem, &SecurityLevelItem::clicked, highButton, &QRadioButton::click);
    connect(m_mediumLevelItem, &SecurityLevelItem::clicked, mediumButton, &QRadioButton::click);
    connect(m_lowLevelItem, &SecurityLevelItem::clicked, lowButton, &QRadioButton::click);
    //链接，使用电源时的锁屏时间滚动条数值改变时
    connect(m_linepasswordSliderItem->slider(), &DCCSlider::valueChanged,this, [ = ](int value){
        qDebug() << "vaule = " << value;
        m_daemonservice->SetGesttingMaturityDay(value);
    });

    setLayout(m_layout);

    // 单选按钮隐藏
    highButton->hide();
    mediumButton->hide();
    lowButton->hide();

    //获取gsetting默认值
    int safetygrade = m_work->GetDaemonService()->safetygrade();
    //设置dbus属性
    setStyle(safetygrade);
}

//修改密码刷新天数
void LoginSecurityWidget::Setlastchage()
{
#ifdef user_passwor_tip
    if(m_usermanage) {
        int date =  m_usermanage->passwordLastChange();//获取上次设置密码距1970年1月1日的天数
        const QDateTime datetime(QDateTime::currentDateTime());
        QDateTime dt = QDateTime(QDate(1970, 1, 1));//获取当天距1970年1月1日的天数
        m_day = dt.daysTo(datetime) - date;
        qDebug()<<"m_day = " <<m_day;
        if (m_day >= 90) {
            m_PasswodUpdateTips->setText(tr("You have not changed the password for %1 days, please change it regularly to improve system security.").arg(m_day));
        } else {
            m_PasswodUpdateTips->hide();
        }
    }
#endif
}

//密码等级设置
void LoginSecurityWidget::onButtonClicked(int button)
{
    setStyle(button);
    int safetygrade = m_work->GetDaemonService()->safetygrade();
    m_work->GetDaemonService()->SetGsetting(button);//设置gsetting的值
    if (button < safetygrade) {//设置密码等级提高提示信息
        m_tip = new TipsWidget(m_model,m_work);
        m_tip->setIcon(QIcon::fromTheme("dcc_login_tip"));
        m_tip->show();
    }
}

//设置安全等级透明度
void LoginSecurityWidget::setStyle(int grade)
{
    switch (grade) {
    case LoginSecurity::high :
       m_pButtonGroup->button(LoginSecurity::high)->setChecked(true); //设置按钮
        m_highLevelItem->setDisableStyle(false);
        m_mediumLevelItem->setDisableStyle(true);
        m_lowLevelItem->setDisableStyle(true);
        break;
    case LoginSecurity::middle :
        m_pButtonGroup->button(LoginSecurity::middle)->setChecked(true); //设置按钮
        m_highLevelItem->setDisableStyle(true);
        m_mediumLevelItem->setDisableStyle(false);
        m_lowLevelItem->setDisableStyle(true);
        break;
    case LoginSecurity::low :
        m_pButtonGroup->button(LoginSecurity::low)->setChecked(true); //设置按钮
        m_highLevelItem->setDisableStyle(true);
        m_mediumLevelItem->setDisableStyle(true);
        m_lowLevelItem->setDisableStyle(false);
        break;
    }
}

QString LoginSecurityWidget::sliderValueToLiteralString(const int delay) const
{
    QString strData = "";

    switch (delay) {
    case 0:
        strData = tr("%d days").arg(30);
        break;
    case 1:
        strData = tr("%d days").arg(60);
        break;
    case 2:
        strData = tr("%d days").arg(90);
        break;
    case 3:
        strData = tr("%d days").arg(120);
        break;
    case 4:
        strData = tr("%d days").arg(150);
        break;
    case 5:
        strData = tr("%d days").arg(180);
        break;
    case 6:
        strData = tr("Never");
        break;
    default:
        strData = tr("Never");
        break;
    }

    return strData;
}
