/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     luokai <luokai_cm@deepin.com>
 *
 * Maintainer: luokai <luokai_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "systemsafetymodule.h"
#include "loginsecurity/loginsecuritywidget.h"
#include "screensecurity/screensecuritywidget.h"
#include "updatepolicy/updatepolicywidget.h"
#include "usbconnection/usbconnectionwidget.h"

#include <QModelIndex>
#include <QStandardItemModel>

SystemSafetyModule::SystemSafetyModule(FrameProxyInterface *frame, QObject *parent)
    : QObject(parent)
    , ModuleInterface(frame)
    , m_systemsafetymodel(nullptr)
    , m_systemsafetywork(nullptr)
{
}

void SystemSafetyModule::initialize()
{

}

void SystemSafetyModule::preInitialize()
{
    m_systemsafetymodel = new SystemSafetyModel();
    m_systemsafetywork = new SystemSafetyWork(m_systemsafetymodel);
}

const QString SystemSafetyModule::name() const
{
    return "systemsafety";
}

void SystemSafetyModule::active(int index)
{
    m_widget = new SystemSafetyWidget(m_systemsafetymodel, m_systemsafetywork);
    m_widget->initialize();
    m_frameProxy->pushWidget(this, m_widget);
    connect(m_widget, &SystemSafetyWidget::requestShowLoginSecurity, this, &SystemSafetyModule::showLoginSecurity);
    connect(m_widget, &SystemSafetyWidget::requestShowScreenSecurity, this, &SystemSafetyModule::showScreenSecurity);
    connect(m_widget, &SystemSafetyWidget::requestShowUpdatePolicy, this, &SystemSafetyModule::showUpdatePolicy);
    connect(m_widget, &SystemSafetyWidget::requestShowUsbConnection, this, &SystemSafetyModule::showUsbConnection);
    qDebug() << "index == " << index;
    // 初次显示加载一个临时的三级页面
    m_frameProxy->pushWidget(this, new QWidget);
    if (index == 1) {
        m_widget->setScreenSecurityWidget();
    } else if (index == 2) {
        m_widget->setUpdatePolicyWidget();
    } else if (index == 3) {
        m_widget->setUsbConnectionWidget();
    } else {
        m_widget->setDefaultWidget();
    }
}

void SystemSafetyModule::deactive()
{

}

void SystemSafetyModule::showLoginSecurity()
{
    LoginSecurityWidget *loginsecurity = new LoginSecurityWidget(m_systemsafetymodel, m_systemsafetywork);
    m_frameProxy->popAndDelWidget(this);
    m_frameProxy->pushWidget(this, loginsecurity);
}

void SystemSafetyModule::showScreenSecurity()
{
    ScreenSecurityWidget *screensecurity = new ScreenSecurityWidget(m_systemsafetymodel, m_systemsafetywork);
    m_frameProxy->popAndDelWidget(this);
    m_frameProxy->pushWidget(this, screensecurity);
}

void SystemSafetyModule::showUpdatePolicy()
{
    UpdatePolicyWidget *updatePolicy = new UpdatePolicyWidget(m_systemsafetymodel, m_systemsafetywork);
    m_frameProxy->popAndDelWidget(this);
    m_frameProxy->pushWidget(this, updatePolicy);
}

void SystemSafetyModule::showUsbConnection()
{
    UsbConnectionWidget *usbConnection = new UsbConnectionWidget(m_systemsafetymodel, m_systemsafetywork);
    m_frameProxy->popAndDelWidget(this);
    m_frameProxy->pushWidget(this, usbConnection);
}

void SystemSafetyModule::activeUsbConnection()
{
    m_frameProxy->setCurrentModule(3, 3);
}

