#include "systemsafetywidget.h"
#include "window/modules/common/common.h"

//此处为带边距的宽度
const int first_widget_min_width = 200;

const QMargins navItemMargin(10, 8, 10, 8);
const QVariant NavItemMargin = QVariant::fromValue(navItemMargin);

SystemSafetyWidget::SystemSafetyWidget(SystemSafetyModel *model, SystemSafetyWork *work, QWidget *parent)
    : QWidget(parent)
    , m_model(model)
    , m_work(work)
    , m_listview(new DListView)
    , m_listViewModel(nullptr)
{

}

SystemSafetyWidget::~SystemSafetyWidget()
{

}

void SystemSafetyWidget::initialize()
{
    m_menuIconText = {
        {
            QIcon::fromTheme("dcc_login"), QIcon::fromTheme("dcc_login_active"), tr("Login Safety"),
            QMetaMethod::fromSignal(&SystemSafetyWidget::requestShowLoginSecurity)
        },
        {
            QIcon::fromTheme("dcc_screen"), QIcon::fromTheme("dcc_screen_active"), tr("Screen Protection"),
            QMetaMethod::fromSignal(&SystemSafetyWidget::requestShowScreenSecurity)
        },
        {
            QIcon::fromTheme("dcc_update_policy"), QIcon::fromTheme("dcc_update_policy_active"),
            tr("Update Policy"), QMetaMethod::fromSignal(&SystemSafetyWidget::requestShowUpdatePolicy)
        },
        {
            QIcon::fromTheme("dcc_usbConnection"), QIcon::fromTheme("dcc_usbConnection_active"),
            tr("USB Connection"), QMetaMethod::fromSignal(&SystemSafetyWidget::requestShowUsbConnection)
        },
    };

    m_listViewModel = new QStandardItemModel(this);
    DStandardItem *item = nullptr;
    for (auto menu : m_menuIconText) {
        item = new DStandardItem(menu.menuIcon, menu.menuText);
        item->setData(VListViewItemMargin, Dtk::MarginsRole);
        m_listViewModel->appendRow(item);
    }
    m_listview->setItemSize(QSize(180, 48));
    m_listview->setItemSpacing(10);
    m_listview->setSpacing(20);
    m_listview->setIconSize(Q_IconSize);
    m_listview->setModel(m_listViewModel);
    m_listview->setFrameShape(QFrame::Shape::NoFrame);
    m_listview->setEditTriggers(QListView::NoEditTriggers);
    m_listview->setAutoScroll(true);//滚动条
    m_listview->setTabKeyNavigation(true);//tab按键是否有效
    m_listview->setViewportMargins(QMargins(10, 0, 10, 0));//滚动区域边框大小
    m_listview->setFixedWidth(first_widget_min_width);//固定窗口值
    m_listview->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    m_listview->setSpacing(0);
    m_listview->setSelectionMode(QAbstractItemView::SingleSelection);
    m_listview->setViewportMargins(0, 0, 15, 0);

    connect(m_listview, &DListView::clicked, this, &SystemSafetyWidget::onItemClieck);
    connect(m_listview, &DListView::activated, m_listview, &QListView::clicked);

    QVBoxLayout *layout = new QVBoxLayout(this);
    layout->setMargin(10);
    layout->setSpacing(10);
    layout->addWidget(m_listview);
    setLayout(layout);
}

DListView *SystemSafetyWidget::getListViewPointer()
{
    return m_listview;
}

void SystemSafetyWidget::setDefaultWidget()
{
    m_listview->setCurrentIndex(m_listview->model()->index(0, 0));
    m_listview->clicked(m_listview->model()->index(0, 0));
}

void SystemSafetyWidget::setScreenSecurityWidget()
{
    m_listview->setCurrentIndex(m_listview->model()->index(1, 0));
    m_listview->clicked(m_listview->model()->index(1, 0));
}

void SystemSafetyWidget::setUpdatePolicyWidget()
{
    m_listview->setCurrentIndex(m_listview->model()->index(2, 0));
    m_listview->clicked(m_listview->model()->index(2, 0));
}

void SystemSafetyWidget::setUsbConnectionWidget()
{
    m_listview->setCurrentIndex(m_listview->model()->index(3, 0));
    m_listview->clicked(m_listview->model()->index(3, 0));
}

void SystemSafetyWidget::onItemClieck(const QModelIndex &index)
{
    for (int i = 0; i < m_menuIconText.size(); i++) {
        QIcon iconTmp;
        if (index.row() == i) {
            iconTmp = m_menuIconText[i].menuActivedIcon;
        } else {
            iconTmp = m_menuIconText[i].menuIcon;
        }
        m_listViewModel->item(i)->setIcon(iconTmp);
    }

    m_listview->setCurrentIndex(index);
    m_menuIconText[index.row()].method.invoke(this);
}
