/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     luokai <luokai_cm@deepin.com>
 *
 * Maintainer: luokai <luokai_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "window/interface/moduleinterface.h"
#include "window/interface/frameproxyinterface.h"
#include "systemsafetymodel.h"
#include "systemsafetywork.h"
#include "src/window/utils.h"

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QStandardItemModel>
#include <QWidget>
#include <QMargins>

#include <DFrame>
#include <DListView>
#include <DBackgroundGroup>
#include <DIconButton>

DWIDGET_USE_NAMESPACE
class SystemSafetyWidget : public QWidget
{
    Q_OBJECT
public:
    explicit SystemSafetyWidget(SystemSafetyModel *model, SystemSafetyWork *work, QWidget *parent = nullptr);
    ~SystemSafetyWidget();
    void initialize();
    DTK_WIDGET_NAMESPACE::DListView *getListViewPointer();
    void setDefaultWidget();
    void setScreenSecurityWidget();
    void setUpdatePolicyWidget();
    void setUsbConnectionWidget();

Q_SIGNALS:
    void requestPushWidget(int index);
    void requestShowLoginSecurity() const;
    void requestShowScreenSecurity() const;
    void requestShowUpdatePolicy() const;
    void requestShowUsbConnection() const;

private:
    struct MenuMethod {
        QIcon menuIcon;
        QIcon menuActivedIcon;
        QString menuText;
        QMetaMethod method;
    };

public Q_SLOTS:
    void onItemClieck(const QModelIndex &index);

private:
    SystemSafetyModel *m_model;
    SystemSafetyWork *m_work;
    DTK_WIDGET_NAMESPACE::DListView *m_listview;
    QList<MenuMethod> m_menuIconText;
    QStandardItemModel *m_listViewModel;
};
