/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     luokai <luokai_cm@deepin.com>
 *
 * Maintainer: luokai <luokai_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "window/namespace.h"
#include "dtkwidget_global.h"
#include "window/modules/common/common.h"
#include "window/modules/systemsafety/systemsafetymodel.h"
#include "com_deepin_dde_controlcenter.h"
#include "com_deepin_daemon_accounts.h"
#include "com_deepin_daemon_power.h"
#include "com_deepin_dde_controlcenter.h"
#include "com_deepin_defender_daemonservice.h"
#include "com_deepin_lastore_jobmanager.h"
#include "com_deepin_lastore_updater.h"
#include "com_deepin_lastore_job.h"
#include "com_deepin_daemon_systeminfo.h"
#include "usbconnection/usbconnectionwork.h"

#include <QDBusInterface>

using controlcenter = com::deepin::dde::ControlCenter;
using accounts = com::deepin::daemon::Accounts;
using PowerInter = com::deepin::daemon::Power;
using DaemonService = com::deepin::defender::daemonservice;
using ManagerInter = com::deepin::lastore::Manager;
using UpdaterInter = com::deepin::lastore::Updater;
using JobInter = com::deepin::lastore::Job;
using SystemInfoInter=com::deepin::daemon::SystemInfo;

DEF_NAMESPACE_BEGIN
DEF_ANTIVIRUS_NAMESPACE_BEGIN

class SystemSafetyWork : public QObject
{
    Q_OBJECT
public:
    explicit SystemSafetyWork(SystemSafetyModel *model,QObject *parent = nullptr);
    ~SystemSafetyWork();

    inline controlcenter *GetControlCenter()
    {
        return m_contorl;
    }
    inline accounts *GetAccounts()
    {
        return m_accounts;
    }
    inline DaemonService *GetDaemonService()
    {
        return m_daemonservice;
    }
    inline UsbConnectionWork *usbConnectionWork() {return m_usbConnectionWork;}
    bool haveBattery();
    //获取使用电源时的自动锁屏时间
    int getLinePowerLockDelay();
    void setLinePowerLockDelay(int value);
    //获取使用电池时的自动锁屏时间
    int getBatteryLockDelay();
    void setBatteryLockDelay(int value);
    //打开设置锁屏背景界面
    void openLockBackgroundSetting();
    //打开设置屏保界面
    void openLockSaverSetting();
    //打开息屏设置界面
    void openCloseMonitorSetting();
    //检查是否有更新
    void checkForUpdates();
    //设置自动检查更新
    void setAutoCheckUpdates(bool isbAuto);
    //设置自动下载更新
    void setAutoDownloadUpdates(bool isbAuto);
    //打开控制中心更新界面
    void openUpdatePage();
private:
     bool checkDbusIsValid();
Q_SIGNALS:
    //锁屏时间改变信号
    void linePowerLockDelayChanged(int value);
    void batteryLockDelayChanged(int value);
    //自动检查更新信号改变
    void AutoCheckUpdatesChanged(bool isbAuto);
    void AutoDownloadUpdatesChanged(bool isbAuto);
    void onJumpScreenSecurtiy();
    void onJumpUpdatePolicy();
    //通知打开usb管控界面
    void notifyOpenUsbConnectionWindow();

public Q_SLOTS:

private Q_SLOTS:
    void setCheckUpdatesJob(const QString &jobPath);
    void onAppUpdateInfoFinished(QDBusPendingCallWatcher *w);

private:
    SystemSafetyModel *m_model;
    controlcenter *m_contorl;
    accounts *m_accounts;
    DaemonService *m_daemonservice;
    PowerInter *m_powerInter;
    QDBusInterface *m_desktopInter;
    ManagerInter *m_managerInter;
    UpdaterInter *m_updaterInter;
    QPointer<JobInter> m_checkUpdateJob;
    SystemInfoInter *m_systemInfoInter;
    //usb管控数据处理
    UsbConnectionWork *m_usbConnectionWork;
};

DEF_ANTIVIRUS_NAMESPACE_END
DEF_NAMESPACE_END
