/*
 * Copyright (C) 2019 ~ 2020 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "checkingButton.h"

#include <QPainter>
#include <QDebug>

DWIDGET_USE_NAMESPACE

namespace def {
namespace widgets {

CheckingButton::CheckingButton(QWidget *parent)
    : QWidget(parent)
    , m_rotateAngle(0)
    , m_rotatingTimer(nullptr)
    , m_btn(nullptr)
    , m_rotatingSvgRender()
{
    initUI();
}

CheckingButton::~CheckingButton()
{
    m_rotatingTimer->deleteLater();
}

void CheckingButton::initUI()
{
    setContentsMargins(0, 0, 0, 0);
    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->setContentsMargins(0, 0, 0, 0);
    mainLayout->setMargin(0);
    mainLayout->setSpacing(0);
    setLayout(mainLayout);

    m_btn = new DPushButton(this);
    m_btn->setFlat(true);
    mainLayout->addWidget(m_btn, 0, Qt::AlignLeft | Qt::AlignTop);

    m_rotatingTimer = new QTimer(this);
    m_rotatingTimer->setInterval(100);
    connect(m_rotatingTimer, &QTimer::timeout, this, &CheckingButton::rotating);
}

void CheckingButton::setRotatingSvgRender(QString svgPath)
{
    m_rotatingSvgRender.load(svgPath);
    m_rotatingTimer->start();
}

void CheckingButton::startRotating()
{
    m_rotatingTimer->start();
}

void CheckingButton::pauseRotating()
{
    m_rotatingTimer->stop();
}

void CheckingButton::resizeEvent(QResizeEvent *event)
{
    Q_UNUSED(event);
    m_btn->resize(size());
}

void CheckingButton::paintEvent(QPaintEvent *event)
{
    Q_UNUSED(event);

    QPainter painter(this);
    painter.setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing); //设置渲染特性：反锯齿
    /* 设定旋转中心点 */
    painter.translate(width() / 2, height() / 2);
    /* 旋转的角度 */
    painter.rotate(m_rotateAngle);
    /* 恢复中心点 */
//    painter.translate(- width() / 2, - height() / 2);
    /* 画图操作 */
    QRectF rotatingSvgRect(- width() / 2, - height() / 2, width(), height());
    m_rotatingSvgRender.render(&painter, rotatingSvgRect);
}

void CheckingButton::rotating()
{
    /* 碟机转动 */
    m_rotateAngle += 10;
    if (m_rotateAngle == 360)
        m_rotateAngle = 0;

    update();
}

}
}
