/*
 * Copyright (C) 2019 ~ 2020 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "updatepolicywidget.h"
#include "widgets/switchwidget.h"
#include "widgets/settingsgroup.h"
#include "window/modules/systemsafety/updatepolicy/checkingButton.h"

#include <DLabel>
#include <DCommandLinkButton>
#include <DTipLabel>

#include <QVBoxLayout>
#include <QApplication>

UpdatePolicyWidget::UpdatePolicyWidget(SystemSafetyModel *model, SystemSafetyWork *work
                                           , QWidget *parent)
    : QWidget(parent)
    , m_model(model)
    , m_work(work)
{
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->setAlignment(Qt::AlignLeft);
    mainLayout->setSpacing(10);
    setLayout(mainLayout);

    //检测更新部分
    SettingsItem *checkUpdateItem = new SettingsItem;
    checkUpdateItem->addBackground();
    checkUpdateItem->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
    mainLayout->addWidget(checkUpdateItem);

    //检测更新部分主layout
    QHBoxLayout *checkUpdateItemMainLayout = new QHBoxLayout;
    checkUpdateItemMainLayout->setAlignment(Qt::AlignLeft);
    checkUpdateItem->setLayout(checkUpdateItemMainLayout);

    //检测更新部分左侧layout
    QVBoxLayout *checkUpdateItemLeftLayout = new QVBoxLayout;
    checkUpdateItemMainLayout->addLayout(checkUpdateItemLeftLayout);

    //检测更新部分左侧第一行layout
    QHBoxLayout *checkUpdateItemLeftUpLayout = new QHBoxLayout;
    checkUpdateItemLeftLayout->addLayout(checkUpdateItemLeftUpLayout);

    DLabel *sysVersionLabel = new DLabel(tr("System Edition") + ": " + m_model->systemVersionInfo());
    checkUpdateItemLeftUpLayout->addWidget(sysVersionLabel);

    checkUpdateItemLeftUpLayout->addSpacing(10);
    CheckingButton *checkBtn = new CheckingButton;
    checkBtn->setFixedSize(24, 24);
    checkBtn->setRotatingSvgRender(":/icons/deepin/builtin/actions/dcc_refresh_24px.svg");
    checkUpdateItemLeftUpLayout->addWidget(checkBtn, 0, Qt::AlignLeft);

    //check Update item left down layer
    QHBoxLayout *checkUpdateItemLeftDownLayout = new QHBoxLayout;
    checkUpdateItemLeftLayout->addLayout(checkUpdateItemLeftDownLayout);

    DTipLabel *updateTipLabel = new DTipLabel(tr("Up to date"));
    checkUpdateItemLeftDownLayout->addWidget(updateTipLabel, 0, Qt::AlignLeft);

    //检测更新部分右侧更新按钮
    checkUpdateItemMainLayout->addStretch(1);
    DCommandLinkButton *updateBtn = new DCommandLinkButton(tr("Update Now"));
    updateBtn->setVisible(false);
    checkUpdateItemMainLayout->addWidget(updateBtn, 0, Qt::AlignLeft);

    //更新提醒开关
    SwitchWidget *autoCheckUpdatesSwitch = new SwitchWidget;
    autoCheckUpdatesSwitch->addBackground();
    autoCheckUpdatesSwitch->setTitle(tr("Updates Notification"));
    //开关设置初始状态
    autoCheckUpdatesSwitch->switchButton()->setChecked(m_model->isbAutoCheckUpdates());
    mainLayout->addWidget(autoCheckUpdatesSwitch);

    //自动下载更新开关
    SwitchWidget *autoDownloadUpdateSwitch = new SwitchWidget;
    autoDownloadUpdateSwitch->addBackground();
    autoDownloadUpdateSwitch->setTitle(tr("Auto Download Updates"));
    //开关设置初始状态
    autoDownloadUpdateSwitch->switchButton()->setChecked(m_model->isbAutoDownloadUpdates());
    mainLayout->addWidget(autoDownloadUpdateSwitch);

    //自动下载更新tip
    DTipLabel *autoDownloadUpdateTip = new DTipLabel(tr("Switch it on to automatically download the updates in wireless or wired network"));
    autoDownloadUpdateTip->setAlignment(Qt::AlignLeft);
    autoDownloadUpdateTip->setWordWrap(true);
    autoDownloadUpdateTip->setContentsMargins(10, 0, 10, 0);
    mainLayout->addWidget(autoDownloadUpdateTip);

    //初始化 自动下载更新开关 自动下载更新tip 的显示状态
    if (!m_model->isbAutoCheckUpdates()) {
        autoDownloadUpdateSwitch->setVisible(false);
        autoDownloadUpdateTip->setVisible(false);
    }

    //根据work运行的状态，设置鼠标是否忙碌
    if (UpdatesStatus::UpdateDefault != m_model->updateStatus())
        checkBtn->startRotating();

    mainLayout->addStretch(1);

    connect(checkBtn->button(), &QPushButton::clicked, this, [=] {
        sysVersionLabel->setText(tr("System Edition") + ": " + m_model->systemVersionInfo());
        m_work->checkForUpdates();
    });

    connect(m_model, &SystemSafetyModel::updateStatusChanged, this, [=](UpdatesStatus status) {
        switch (status) {
        case UpdatesStatus::Checking:
            qDebug() << "UpdatesStatus::Checking";
            //检查更新中，设置鼠标状态为忙碌
            checkBtn->startRotating();
            break;
        case UpdatesStatus::UpdateDefault:
            qDebug() << "UpdatesStatus::UpdateDefault";
            //检查更新结束，设置鼠标状态为正常
            checkBtn->pauseRotating();
            break;
        case UpdatesStatus::CheckUpdatesFailed:
            qDebug() << "UpdatesStatus::CheckUpdatesFailed";
            //检查更新失败，设置鼠标状态为正常
            checkBtn->pauseRotating();
            break;
        default:
            break;
        }
    });
    connect(m_model, &SystemSafetyModel::sendHaveUpdates, this, [=](bool have) {
        if (have) {
            updateTipLabel->setText(tr("Updates available"));
            updateBtn->setVisible(true);
        } else {
            updateTipLabel->setText(tr("Up to date"));
            updateBtn->setVisible(false);
        }
    });

    connect(m_work, &SystemSafetyWork::AutoCheckUpdatesChanged, this, [=](bool isbAuto) {
        autoCheckUpdatesSwitch->switchButton()->setChecked(isbAuto);
        if (isbAuto) {
            autoDownloadUpdateSwitch->setVisible(true);
            autoDownloadUpdateTip->setVisible(true);
        } else {
            m_work->setAutoDownloadUpdates(false);
            autoDownloadUpdateSwitch->setVisible(false);
            autoDownloadUpdateTip->setVisible(false);
        }
    });
    connect(autoCheckUpdatesSwitch->switchButton(), &DSwitchButton::checkedChanged, m_work,
            &SystemSafetyWork::setAutoCheckUpdates);

    connect(m_work, &SystemSafetyWork::AutoDownloadUpdatesChanged, this, [=](bool isbAuto) {
        autoDownloadUpdateSwitch->switchButton()->setChecked(isbAuto);
    });
    connect(autoDownloadUpdateSwitch->switchButton(), &DSwitchButton::checkedChanged, m_work,
            &SystemSafetyWork::setAutoDownloadUpdates);

    //updateBtn
    connect(updateBtn, &DCommandLinkButton::clicked, this, [=] {
        m_work->openUpdatePage();
    });

    //初始化时，检查更新
    Q_EMIT checkBtn->button()->clicked();

}

UpdatePolicyWidget::~UpdatePolicyWidget()
{
    //注销时，断开相关信号链接
    disconnect(m_work, nullptr, this, nullptr);
}
