﻿/*
 * Copyright (C) 2019 ~ 2020 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "usbconnectionwidget.h"
#include "widgets/settingsgroup.h"
#include "widgets/titlebuttonitem.h"
#include "widgets/defendertable.h"
#include "widgets/settingsheaderitem.h"
#include "window/modules/systemsafety/usbconnection/widgets/usbinfolabel.h"
#include "window/modules/systemsafety/usbconnection/widgets/usbinfodialog.h"

#include <DCommandLinkButton>
#include <DTipLabel>
#include <DTableView>
#include <DApplicationHelper>
#include <DSimpleListView>
#include <DDialog>
#include <DFontSizeManager>
#include <DButtonBox>
#include <DApplicationHelper>
#include <DSuggestButton>

#include <QVBoxLayout>
#include <QApplication>
#include <QMenu>
#include <QStandardItemModel>
#include <QTableView>
#include <QScrollBar>

const QColor WhiteListRemoveColor = QColor(255, 87, 54, 255);
const QColor WhiteListAddColor = QColor(0, 130, 250, 255);

UsbConnectionWidget::UsbConnectionWidget(SystemSafetyModel *model, SystemSafetyWork *work
                                         , QWidget *parent)
    : QWidget(parent)
    , m_model(model)
    , m_work(work)
    , m_usbConnectionWork(nullptr)
    , m_isbSaveRecord(false)
    , m_limitationModel("0")
    , m_connectionRecordControlItem(nullptr)
    , m_usbLimitModeItem(nullptr)
    , m_usbLimitModeItemButtonLabel(nullptr)
    , m_connectionRecordTableWgt(nullptr)
    , m_whiteLstTableWgt(nullptr)
    , m_connectionRecordTable(nullptr)
    , m_connectionRecordTableModel(nullptr)
    , m_clearLogBtn(nullptr)
    , m_whiteLstTable(nullptr)
    , m_whiteLstTableModel(nullptr)
{
    //init data
    m_usbConnectionWork = m_work->usbConnectionWork();
    m_isbSaveRecord = m_usbConnectionWork->isbSaveRecord();
    m_limitationModel = m_usbConnectionWork->limitationModel();
    m_usbConnectionInfoLst = m_usbConnectionWork->usbConnectionInfoLst();
    m_usbConnectionWhiteLst = m_usbConnectionWork->usbConnectionWhiteLst();

    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->setAlignment(Qt::AlignLeft);
    mainLayout->setSpacing(10);
    setLayout(mainLayout);

    m_connectionRecordControlItem = new SwitchWidget;
    m_connectionRecordControlItem->addBackground();
    m_connectionRecordControlItem->setTitle(tr("Save connection record of USB devices"));
    m_connectionRecordControlItem->switchButton()->setChecked(m_isbSaveRecord);
    mainLayout->addWidget(m_connectionRecordControlItem);

    m_usbLimitModeItem = new TitleButtonItem;
    m_usbLimitModeItem->setTitle(tr("Connection limitation"));
    mainLayout->addWidget(m_usbLimitModeItem);

    //usbLimitModeItemButton
    m_usbLimitModeItem->button()->setMinimumWidth(150);
    QHBoxLayout *usbLimitModeItemButtonLayout = new QHBoxLayout;
    usbLimitModeItemButtonLayout->setContentsMargins(10, 0, 10, 0);
    m_usbLimitModeItem->button()->setLayout(usbLimitModeItemButtonLayout);
    m_usbLimitModeItemButtonLabel = new DLabel;
    usbLimitModeItemButtonLayout->addWidget(m_usbLimitModeItemButtonLabel);
    m_usbLimitModeItemButtonLabel->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);

    QMenu *usbLimitModeMenu = new QMenu;
    m_usbLimitModeItem->button()->setMenu(usbLimitModeMenu);
    QAction *allowAllAction = new QAction(tr("Allow All"));
    allowAllAction->setObjectName("AllowAll");
    allowAllAction->setCheckable(true);
    usbLimitModeMenu->addAction(allowAllAction);
    QAction *onlyWhitelistAction = new QAction(tr("Only Whitelist"));
    onlyWhitelistAction->setObjectName("OnlyWhitelist");
    onlyWhitelistAction->setCheckable(true);
    usbLimitModeMenu->addAction(onlyWhitelistAction);
    QAction *blockAllAction = new QAction(tr("Block All"));
    blockAllAction->setObjectName("BlockAll");
    blockAllAction->setCheckable(true);
    usbLimitModeMenu->addAction(blockAllAction);

    if ("0" == m_limitationModel) {
        allowAllAction->setChecked(true);
        m_usbLimitModeItemButtonLabel->setText(allowAllAction->text());
    } else if ("1" == m_limitationModel) {
        onlyWhitelistAction->setChecked(true);
        m_usbLimitModeItemButtonLabel->setText(onlyWhitelistAction->text());
    } else if ("2" == m_limitationModel) {
        blockAllAction->setChecked(true);
        m_usbLimitModeItemButtonLabel->setText(blockAllAction->text());
    }

    //connectionRecordBtn and whiteListBtn
    mainLayout->addSpacing(20);
    DButtonBox *tableSwitchBtn = new DButtonBox;
    mainLayout->addWidget(tableSwitchBtn, 0, Qt::AlignHCenter);

    QList<DButtonBoxButton *> tableSwitchBtnLst;
    DButtonBoxButton *connectionRecordBtn = new DButtonBoxButton(tr("Connection Record"));     //链接记录
    connectionRecordBtn->setFixedWidth(150);
    tableSwitchBtnLst.append(connectionRecordBtn);

    DButtonBoxButton *whiteListBtn = new DButtonBoxButton(tr("Whitelist"));     //白名单
    whiteListBtn->setFixedWidth(150);
    tableSwitchBtnLst.append(whiteListBtn);

    tableSwitchBtn->setButtonList(tableSwitchBtnLst, true);
    connectionRecordBtn->setChecked(true);

    //添加两个表格模块
    mainLayout->addLayout(initTwoTable());
    m_connectionRecordTableWgt->hide();
    m_whiteLstTableWgt->hide();
    showConnectionRecordTableWgt();

    //connect
    connect(m_usbConnectionWork, &UsbConnectionWork::sendAppendUsbConnectionLogInfo
            , this, &UsbConnectionWidget::onReceiveUsbInfo);
    connect(m_connectionRecordControlItem->switchButton(), &DSwitchButton::clicked
            , this, &UsbConnectionWidget::changeIsbSaveRecord);
    connect(usbLimitModeMenu, &QMenu::triggered, this, &UsbConnectionWidget::selectLimitationModel);

    connect(connectionRecordBtn, &DButtonBoxButton::clicked, this, &UsbConnectionWidget::showConnectionRecordTableWgt);
    connect(whiteListBtn, &DButtonBoxButton::clicked, this, &UsbConnectionWidget::showWhiteLstTableWgt);
}

UsbConnectionWidget::~UsbConnectionWidget()
{
    //注销时，断开相关信号链接
    disconnect(m_work, nullptr, this, nullptr);
}

void UsbConnectionWidget::onReceiveUsbInfo(UsbConnectionInfo info)
{
    //清除超过30天记录项
    cleanUsbConnectionLog();

    appendUsbConnectionLog(info);

    updateNameinWhiteLst(info.sName, info.sSerial, info.sVendorID, info.sProductID);

    //不显示 serial--vendorID--productID 栏
    m_connectionRecordTable->setColumnHidden(4, true);
    m_connectionRecordTable->setColumnHidden(5, true);
    m_connectionRecordTable->setColumnHidden(6, true);
}

bool UsbConnectionWidget::isUsbDeviceInWhiteList(QString serial, QString vendor, QString product)
{
    bool ret = false;
    for (UsbConnectionInfo info : m_usbConnectionWhiteLst) {
        if (serial == info.sSerial && vendor == info.sVendorID && product == info.sProductID) {
            ret = true;
            break;
        }
    }
    return ret;
}

void UsbConnectionWidget::refreshUsbConnectionTable()
{
    for (int i = 0; i < m_connectionRecordTableModel->rowCount(); i++) {
        QString serial = m_connectionRecordTableModel->item(i, 4)->text();
        QString vendor = m_connectionRecordTableModel->item(i, 5)->text();
        QString product = m_connectionRecordTableModel->item(i, 6)->text();
        QWidget *wgt = m_connectionRecordTable->indexWidget(m_connectionRecordTableModel->index(i, 3));
        DCommandLinkButton *linkBtn = wgt->findChild<DCommandLinkButton *>("linkBtn");
        if (isUsbDeviceInWhiteList(serial, vendor, product)) {
            linkBtn->setText(tr("Remove"));
            linkBtn->setToolTip(tr("Remove"));
            DPalette pa = DApplicationHelper::instance()->palette(linkBtn);
            pa.setColor(DPalette::ColorRole::Highlight, WhiteListRemoveColor);
            DApplicationHelper::instance()->setPalette(linkBtn, pa);
        } else {
            linkBtn->setText(tr("Add"));
            linkBtn->setToolTip(tr("Add"));
            DPalette pa = DApplicationHelper::instance()->palette(linkBtn);
            pa.setColor(DPalette::ColorRole::Highlight, WhiteListAddColor);
            DApplicationHelper::instance()->setPalette(linkBtn, pa);
        }
    }
}

void UsbConnectionWidget::removeItemFromWhiteLstTable(UsbConnectionInfo info)
{
    //如果id为空，提示添加失败
    if ("" == info.sSerial || "" == info.sVendorID || "" == info.sProductID) {
        qDebug() << "add to white list failed" << info.sSerial << info.sVendorID << info.sProductID;
        return;
    }

    for (int i = 0; i < m_usbConnectionWhiteLst.count(); i++) {
        UsbConnectionInfo infoTmp = m_usbConnectionWhiteLst.at(i);
        if (info.sSerial == infoTmp.sSerial && info.sVendorID == infoTmp.sVendorID && info.sProductID == infoTmp.sProductID) {
            m_usbConnectionWhiteLst.removeAt(i);
            qDebug() << "remove from white list succeed" << info.sSerial << info.sVendorID << info.sProductID;
            break;
        }
    }

    // 清除白名单表格中该名单
    for (int i = 0; i < m_whiteLstTableModel->rowCount(); i++) {
        QString serial = m_whiteLstTableModel->item(i, 4)->text();
        QString vendor = m_whiteLstTableModel->item(i, 5)->text();
        QString product = m_whiteLstTableModel->item(i, 6)->text();
        // 如果不在白名单中就移除该行
        if (info.sSerial == serial && info.sVendorID == vendor && info.sProductID == product) {
            m_whiteLstTableModel->removeRow(i);
            break;
        }
    }
}

QHBoxLayout *UsbConnectionWidget::initTwoTable()
{
    QHBoxLayout *mainLayout = new QHBoxLayout;
    m_connectionRecordTableWgt = initConnectionRecordTableWgt();
    mainLayout->addWidget(m_connectionRecordTableWgt);

    m_whiteLstTableWgt = initWhiteLstTableWgt();
    mainLayout->addWidget(m_whiteLstTableWgt);

    return mainLayout;
}

QWidget *UsbConnectionWidget::initConnectionRecordTableWgt()
{
    QWidget *mainWgt = new QWidget;
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainWgt->setLayout(mainLayout);

    m_connectionRecordTable = new DTableView;
    //设置表格行标题
    m_connectionRecordTable->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_connectionRecordTable->setSelectionMode(QAbstractItemView::NoSelection);
    m_connectionRecordTable->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_connectionRecordTable->setFrameShape(QTableView::NoFrame);
    m_connectionRecordTable->setIconSize(QSize(25, 25));
    //自动调整最后一列的宽度使它和表格的右边界对齐
    m_connectionRecordTable->horizontalHeader()->setStretchLastSection(true);
    m_connectionRecordTable->verticalHeader()->setHidden(true);
    m_connectionRecordTable->setShowGrid(false);
    m_connectionRecordTable->setSortingEnabled(true);
    m_connectionRecordTable->verticalHeader()->setDefaultSectionSize(30);

    // 设置表头
    QStandardItem *standaritem0 = new QStandardItem(tr("Device"));
    standaritem0->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *standaritem1 = new QStandardItem(tr("Time Connected"));
    standaritem1->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *standaritem2 = new QStandardItem(tr("Status"));
    standaritem2->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *standaritem3 = new QStandardItem(tr("Whitelist"));
    standaritem3->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);

    m_connectionRecordTableModel = new QStandardItemModel;
    m_connectionRecordTable->setModel(m_connectionRecordTableModel);
    m_connectionRecordTableModel->clear();
    m_connectionRecordTableModel->setHorizontalHeaderItem(0, standaritem0);
    m_connectionRecordTableModel->setHorizontalHeaderItem(1, standaritem1);
    m_connectionRecordTableModel->setHorizontalHeaderItem(2, standaritem2);
    m_connectionRecordTableModel->setHorizontalHeaderItem(3, standaritem3);
    m_connectionRecordTableModel->horizontalHeaderItem(3)->setSelectable(false);
    m_connectionRecordTableModel->setRowCount(0);

    //创建usb连接记录表格
    DefenderTable *detailTable = new DefenderTable;
    detailTable->setTableAndModel(m_connectionRecordTable, m_connectionRecordTableModel);
    detailTable->setRowHeight(30);

    mainLayout->addWidget(detailTable);

    //bottom line
    QHBoxLayout *bottomLayout = new QHBoxLayout;

    m_clearLogBtn = new DCommandLinkButton(tr("Clear All Records"));
    bottomLayout->addWidget(m_clearLogBtn);

    // 根据表格内容是否为空，设置清除记录按钮是否可用
    if (0 == m_connectionRecordTableModel->rowCount()) {
        m_clearLogBtn->setDisabled(true);
    }

    bottomLayout->addStretch(1);
    DTipLabel *logTableTip = new DTipLabel(tr("Show record in 30 days"));
    bottomLayout->addWidget(logTableTip);

    mainLayout->addLayout(bottomLayout);

    //connect
    connect(m_clearLogBtn, &DCommandLinkButton::clicked, this, &UsbConnectionWidget::confirmClearUsbConnectionLog);

    //设置usb连接记录表格内容
    for (int i = 0; i < m_usbConnectionInfoLst.size(); i++) {
        qDebug() << "UsbConnectionInfo" << i;
        UsbConnectionInfo info = m_usbConnectionInfoLst.at(i);
        appendUsbConnectionLog(info);
    }

    //不显示 serial--vendorID--productID 栏
    m_connectionRecordTable->setColumnHidden(4, true);
    m_connectionRecordTable->setColumnHidden(5, true);
    m_connectionRecordTable->setColumnHidden(6, true);

    //设置各栏宽度
    m_connectionRecordTable->setColumnWidth(0, 130);
    m_connectionRecordTable->setColumnWidth(1, 160);
    m_connectionRecordTable->setColumnWidth(3, 70);

    //默认排序
    m_connectionRecordTable->sortByColumn(1, Qt::SortOrder::DescendingOrder);

    return mainWgt;
}

QWidget *UsbConnectionWidget::initWhiteLstTableWgt()
{
    QWidget *mainWgt = new QWidget;
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainWgt->setLayout(mainLayout);

    m_whiteLstTable = new DTableView;
    //设置表格行标题
    m_whiteLstTable->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_whiteLstTable->setSelectionMode(QAbstractItemView::NoSelection);
    m_whiteLstTable->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_whiteLstTable->setFrameShape(QTableView::NoFrame);
    //自动调整最后一列的宽度使它和表格的右边界对齐
    m_whiteLstTable->horizontalHeader()->setStretchLastSection(true);
    m_whiteLstTable->verticalHeader()->setHidden(true);
    m_whiteLstTable->setShowGrid(false);
    m_whiteLstTable->setSortingEnabled(true);
    m_whiteLstTable->verticalHeader()->setDefaultSectionSize(35);

    // 设置表头
    QStandardItem *standaritem0 = new QStandardItem(tr("Device"));
    standaritem0->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *standaritem1 = new QStandardItem(tr("Time Added"));
    standaritem1->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *standaritem2 = new QStandardItem(tr("Action"));
    standaritem2->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);

    m_whiteLstTableModel = new QStandardItemModel;
    m_whiteLstTable->setModel(m_whiteLstTableModel);
    m_whiteLstTableModel->clear();
    m_whiteLstTableModel->setHorizontalHeaderItem(0, standaritem0);
    m_whiteLstTableModel->setHorizontalHeaderItem(1, standaritem1);
    m_whiteLstTableModel->setHorizontalHeaderItem(2, standaritem2);
    m_whiteLstTableModel->setRowCount(0);

    //创建usb连接记录表格
    DefenderTable *detailTable = new DefenderTable;
    detailTable->setTableAndModel(m_whiteLstTable, m_whiteLstTableModel);
    detailTable->setRowHeight(30);

    //设置usb连接记录表格内容
    for (int i = 0; i < m_usbConnectionWhiteLst.size(); i++) {
        UsbConnectionInfo info = m_usbConnectionWhiteLst.at(i);
        appendUsbConnectionWhiteList(info);
    }

    //不显示 status--serial--vendorID--productID 栏
    m_whiteLstTable->setColumnHidden(3, true);
    m_whiteLstTable->setColumnHidden(4, true);
    m_whiteLstTable->setColumnHidden(5, true);
    m_whiteLstTable->setColumnHidden(6, true);

    //设置各栏宽度
    m_whiteLstTable->setColumnWidth(0, 160);
    m_whiteLstTable->setColumnWidth(1, 160);

    //默认排序
    m_whiteLstTable->sortByColumn(1, Qt::SortOrder::DescendingOrder);

    mainLayout->addWidget(detailTable);

    //white list table tip
    DTipLabel *whiteListTableTip = new DTipLabel(tr("You can add devices to the whitelist from the connection record"));
    whiteListTableTip->setWordWrap(true);
    whiteListTableTip->setAlignment(Qt::AlignmentFlag::AlignLeft);
    mainLayout->addWidget(whiteListTableTip);

    return mainWgt;
}

void UsbConnectionWidget::showConnectionRecordTableWgt()
{
    qDebug();

    m_connectionRecordTableWgt->show();    //usb链接页面
    m_whiteLstTableWgt->hide();
}

void UsbConnectionWidget::showWhiteLstTableWgt()
{
    m_connectionRecordTableWgt->hide();
    m_whiteLstTableWgt->show();
}

void UsbConnectionWidget::appendUsbConnectionLog(UsbConnectionInfo info)
{
    QList<QStandardItem *> itemLst;
    QStandardItem *nameItem = new QStandardItem(info.sName);
    if (info.sName.isEmpty())
        nameItem->setText(info.sSize.append(" ").append(tr("Volume")));
    QStandardItem *timeItem = new QStandardItem(info.sDatetime);
    QStandardItem *statusItem = new QStandardItem(translateUsbStatus(info.sStatus));
    //设置限制状态图标
    if ("Connected" == info.sStatus)
        statusItem->setIcon(QIcon::fromTheme("dcc_startup_enable"));
    else {
        statusItem->setIcon(QIcon::fromTheme("dcc_startup_disable"));
    }
    QStandardItem *linkBtnItem = new QStandardItem;
    QStandardItem *serialItem = new QStandardItem(info.sSerial);
    QStandardItem *vendorIDItem = new QStandardItem(info.sVendorID);
    QStandardItem *productIDItem = new QStandardItem(info.sProductID);
    itemLst << nameItem << timeItem << statusItem << linkBtnItem << serialItem
            << vendorIDItem << productIDItem;

    m_connectionRecordTableModel->insertRow(0, itemLst);

    //按钮 - 添加白名单
    QWidget *btnWgt = new QWidget;
    QVBoxLayout *btnLayout = new QVBoxLayout;
    btnLayout->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    btnLayout->setContentsMargins(0, 0, 0, 0);
    btnWgt->setLayout(btnLayout);
    //判断设备是否在白名单中
    if (isUsbDeviceInWhiteList(info.sSerial, info.sVendorID, info.sProductID)) {
        DCommandLinkButton *linkBtn = new DCommandLinkButton(tr("Remove"));
        linkBtn->setToolTip(tr("Remove"));
        linkBtn->setObjectName("linkBtn");
        DPalette pa = DApplicationHelper::instance()->palette(linkBtn);
        pa.setColor(DPalette::ColorRole::Highlight, WhiteListRemoveColor);
        DApplicationHelper::instance()->setPalette(linkBtn, pa);

        btnLayout->addWidget(linkBtn);
        m_connectionRecordTable->setIndexWidget(m_connectionRecordTableModel->index(0, 3), btnWgt);
        //connect
        connect(linkBtn, &DCommandLinkButton::clicked, this, [ = ] {
            this->confirmChangeUsbConnectionWhiteLst(linkBtn->text(), info);
        });
    } else {
        DCommandLinkButton *linkBtn = new DCommandLinkButton(tr("Add"));
        linkBtn->setToolTip(tr("Add"));
        linkBtn->setObjectName("linkBtn");
        DPalette pa = DApplicationHelper::instance()->palette(linkBtn);
        pa.setColor(DPalette::ColorRole::Highlight, WhiteListAddColor);
        DApplicationHelper::instance()->setPalette(linkBtn, pa);

        btnLayout->addWidget(linkBtn);
        m_connectionRecordTable->setIndexWidget(m_connectionRecordTableModel->index(0, 3), btnWgt);
        //connect
        connect(linkBtn, &DCommandLinkButton::clicked, this, [ = ] {
            this->confirmChangeUsbConnectionWhiteLst(linkBtn->text(), info);
        });
    }

    //为使上面加入的一行内的控件对齐
    m_connectionRecordTableModel->insertRow(0, new QStandardItem);
    m_connectionRecordTableModel->removeRow(0);

    // 设置清除记录按钮可用
    m_clearLogBtn->setDisabled(false);
}

void UsbConnectionWidget::confirmClearUsbConnectionLog()
{
    DDialog *dialog = new DDialog;
    dialog->setModal(true);
    dialog->setIcon(QIcon::fromTheme("dcc_login_tip"));
    dialog->setMessage(tr("Are you sure you want to clear all the records?"));
    dialog->setWordWrapMessage(true);
    dialog->addButton(tr("Cancel"), false);
    dialog->addButton(tr("Confirm"), true, DDialog::ButtonType::ButtonWarning);
    connect(dialog, &DDialog::buttonClicked, [ = ](int index, QString) {
        switch (index) {
        case 0:
            break;
        case 1:
            this->clearUsbConnectionLog();
            break;
        default:
            break;
        }
    });
    dialog->show();
}

void UsbConnectionWidget::clearUsbConnectionLog()
{
    if (!m_usbConnectionWork->checkAuthentication())
        return;

    //清除表格数据
    int count = m_connectionRecordTableModel->rowCount();
    for (int i = 0; i < count; i++) {
        m_connectionRecordTableModel->removeRow(0);
    }

    m_usbConnectionInfoLst.clear();

    // 设置清除记录按钮不可用
    m_clearLogBtn->setDisabled(true);

    //同步到work
    m_usbConnectionWork->clearUsbConnectionLog();
    m_usbConnectionWork->clearSrvUsbConnectionLog();
}

void UsbConnectionWidget::appendUsbConnectionWhiteList(UsbConnectionInfo info)
{
    QList<QStandardItem *> itemLst;
    QStandardItem *nameItem = new QStandardItem(info.sName);
    if (info.sName.isEmpty())
        nameItem->setText(info.sParentSize.append(" ").append(tr("Volume")));
    QStandardItem *timeItem = new QStandardItem(info.sDatetime);
    QStandardItem *btnItem = new QStandardItem; //白名单 操作栏
    QStandardItem *statusItem = new QStandardItem(translateUsbStatus(info.sStatus));
    QStandardItem *serialItem = new QStandardItem(info.sSerial);
    QStandardItem *vendorIDItem = new QStandardItem(info.sVendorID);
    QStandardItem *productIDItem = new QStandardItem(info.sProductID);
    itemLst << nameItem << timeItem << btnItem << statusItem
            << serialItem << vendorIDItem << productIDItem;

    m_whiteLstTableModel->insertRow(0, itemLst);

    //按钮 - “详情”&“移除”
    QWidget *btnWgt = new QWidget;
    QHBoxLayout *btnLayout = new QHBoxLayout;
    btnLayout->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    btnLayout->setContentsMargins(0, 0, 0, 0);
    btnWgt->setLayout(btnLayout);

    DPushButton *detailBtn = new DPushButton;
    detailBtn->setToolTip(tr("Details"));
    detailBtn->setObjectName("details");
    detailBtn->setFlat(true);
    detailBtn->setIcon(QIcon::fromTheme("dcc_usb_connection_detail"));
    detailBtn->setIconSize(QSize(17, 17));
    btnLayout->addWidget(detailBtn);

    btnLayout->addSpacing(20);
    DPushButton *removeBtn = new DPushButton;
    removeBtn->setToolTip(tr("Remove"));
    removeBtn->setObjectName("remove");
    removeBtn->setFlat(true);
    removeBtn->setIcon(QIcon::fromTheme("dcc_usb_connection_remove"));
    removeBtn->setIconSize(QSize(14, 19));
    btnLayout->addWidget(removeBtn);
    m_whiteLstTable->setIndexWidget(m_whiteLstTableModel->index(0, 2), btnWgt);
    //connect
    connect(detailBtn, &DPushButton::clicked, this, [ = ] {
        this->openUsbConnectionDetails(info);
    });
    connect(removeBtn, &DPushButton::clicked, this, [ = ] {
        //从白名单列表 m_usbConnectionWhiteLst 中移除该项usb设备信息
        this->confirmChangeUsbConnectionWhiteLst(removeBtn->toolTip(), info);
    });
}

void UsbConnectionWidget::updateUsbLimitModeItem()
{
    int checkedActionIndex = 0;
    if ("0" == m_limitationModel) {
        checkedActionIndex = 0;
    } else if ("1" == m_limitationModel) {
        checkedActionIndex = 1;
    } else if ("2" == m_limitationModel) {
        checkedActionIndex = 2;
    }
    for (QAction *ttaction : m_usbLimitModeItem->button()->menu()->actions()) {
        ttaction->setChecked(false);
    }
    QAction *checkedAction = m_usbLimitModeItem->button()->menu()->actions().at(checkedActionIndex);
    checkedAction->setChecked(true);
    m_usbLimitModeItemButtonLabel->setText(checkedAction->text());
}

QString UsbConnectionWidget::translateUsbStatus(QString status)
{
    if ("Connected" == status)
        return tr("Connected");
    else {
        return tr("Blocked");
    }
}

void UsbConnectionWidget::cleanUsbConnectionLog()
{
    QDateTime limitTime = QDateTime::currentDateTime().addDays(-30);
    for (int i = 0; i < m_connectionRecordTableModel->rowCount(); i++) {
        QString datetime = m_connectionRecordTableModel->item(i, 1)->text();
        QDateTime itemTime = QDateTime::fromString(datetime, "yyyy-MM-dd hh:mm:ss");
        //清除超过30天记录项
        if (limitTime > itemTime) {
            m_connectionRecordTableModel->removeRow(i);
            i--;
        }
    }
}

void UsbConnectionWidget::updateNameinWhiteLst(QString sName, QString sSerial, QString sVendorID, QString sProductID)
{
    if (sName.isEmpty()) {
        return;
    }

    for (int i = 0; i < m_whiteLstTableModel->rowCount(); i++) {
        QString serial = m_whiteLstTableModel->item(i, 4)->text();
        QString vendor = m_whiteLstTableModel->item(i, 5)->text();
        QString product = m_whiteLstTableModel->item(i, 6)->text();
        if (sSerial == serial && sVendorID == vendor && sProductID == product) {
            m_whiteLstTableModel->item(i, 0)->setText(sName);
        }
    }
}

void UsbConnectionWidget::changeIsbSaveRecord(bool clicked)
{
    Q_UNUSED(clicked)
    m_usbConnectionWork->changeIsbSaveRecord(!m_isbSaveRecord);
    m_isbSaveRecord = m_usbConnectionWork->isbSaveRecord();
    m_connectionRecordControlItem->switchButton()->setChecked(m_isbSaveRecord);
}

void UsbConnectionWidget::selectLimitationModel(QAction *action)
{
    action->setChecked(true);

    if ("AllowAll" == action->objectName()) {
        //如果再次选择相同项，则不做处理
        if ("0" == m_limitationModel) return;
        //身份验证
        if (!m_usbConnectionWork->checkAuthentication()) {
            updateUsbLimitModeItem();
            return;
        }
        m_limitationModel = "0";
        updateUsbLimitModeItem();
        //状态修改，同步到work
        m_usbConnectionWork->selectLimitationModel(m_limitationModel);
    } else if ("OnlyWhitelist" == action->objectName()) {
        //如果再次选择相同项，则不做处理
        if ("1" == m_limitationModel) return;
        //用户确认
//        DDialog *dialog = new DDialog;
//        dialog->setModal(true);
//        dialog->setOnButtonClickedClose(true);
//        dialog->setMessage(tr("Only USB storage devices with complete hardware info can be added to the whitelist"));
//        dialog->setWordWrapMessage(true);
//        dialog->addButton(tr("OK"), false, DDialog::ButtonType::ButtonRecommend);
//        dialog->getButton(0)->setFixedWidth(100);
//        dialog->exec();
        if ("2" == m_limitationModel) {
            if (!m_usbConnectionWork->checkAuthentication()) {
                updateUsbLimitModeItem();
                return;
            }
        }
        m_limitationModel = "1";
        updateUsbLimitModeItem();
        //状态修改，同步到work
        m_usbConnectionWork->selectLimitationModel(m_limitationModel);
    } else if ("BlockAll" == action->objectName()) {
        if ("2" == m_limitationModel) return;
        //用户确认
        DDialog *dialog = new DDialog;
        dialog->setModal(true);
        dialog->setOnButtonClickedClose(true);
        // 设置左上角图标
        dialog->setIcon(QIcon::fromTheme("dcc_login_tip"));
        dialog->setMessage(tr("Are you sure you want to block the connection of all USB storage devices?"));
        dialog->setWordWrapMessage(true);
        dialog->addButton(tr("Cancel"), false);
        dialog->addButton(tr("Confirm"), true);
        connect(dialog, &DDialog::buttonClicked, [ = ](int index, QString) {
            switch (index) {
            case 0:
                this->updateUsbLimitModeItem();
                break;
            case 1:
                m_limitationModel = "2";
                this->updateUsbLimitModeItem();
                //状态修改，同步到work
                m_usbConnectionWork->selectLimitationModel(m_limitationModel);
                break;
            default:
                this->updateUsbLimitModeItem();
                break;
            }
        });
        connect(dialog, &DDialog::closed, this, &UsbConnectionWidget::updateUsbLimitModeItem);
        dialog->show();
    }
}

void UsbConnectionWidget::confirmChangeUsbConnectionWhiteLst(QString op, UsbConnectionInfo info)
{
    //如果id为空，提示添加失败
    if (info.sSerial.isEmpty() || info.sVendorID.isEmpty() || info.sProductID.isEmpty()) {
        DDialog *dialog = new DDialog;
        dialog->setOnButtonClickedClose(true);
        dialog->setModal(false);
        // 设置弹窗左上角图标
        dialog->setIcon(QIcon::fromTheme("dcc_login_tip"));
        dialog->setMessage(tr("The hardware info of this device is incomplete, unable to add it to the whitelist"));
        dialog->setWordWrapMessage(true);
        dialog->addButton(tr("OK"), true, DDialog::ButtonType::ButtonRecommend);
        dialog->show();
        qDebug() << "add to white list failed";
        return;
    }

    //用户确认
    if (tr("Remove") == op) {
        DDialog *dialog = new DDialog;
        dialog->setModal(true);
        dialog->setMessage(tr("Are you sure you want to remove it from the whitelist?"));
        dialog->setWordWrapMessage(true);
        // 设置弹窗左上角图标
        dialog->setIcon(QIcon::fromTheme("dcc_login_tip"));
        dialog->addButton(tr("Cancel"), false);
        dialog->addButton(tr("Remove"), true, DDialog::ButtonType::ButtonWarning);
        connect(dialog, &DDialog::buttonClicked, [ = ](int index, QString) {
            switch (index) {
            case 0:
                break;
            case 1:
                this->changeUsbConnectionWhiteLst(op, info);
                break;
            default:
                break;
            }
        });
        dialog->show();
    } else if (tr("Add") == op) {
        if (!m_usbConnectionWork->checkAuthentication())
            return;
        DDialog *dialog = new DDialog;
        // 设置弹窗左上角图标
        dialog->setIcon(QIcon::fromTheme("deepin-defender"));
        dialog->setMessage(tr("Added to the whitelist successfully"));
        dialog->setWordWrapMessage(true);
        dialog->addButton(tr("OK"), true);
        dialog->exec();
        changeUsbConnectionWhiteLst(op, info);
    }
}

void UsbConnectionWidget::changeUsbConnectionWhiteLst(QString op, UsbConnectionInfo info)
{
    if (tr("Add") == op) {
        //白名单添加时间
        info.sDatetime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss");
        m_usbConnectionWhiteLst.append(info);
        appendUsbConnectionWhiteList(info);
        m_usbConnectionWork->appendToUsbConnectionWhiteLst(info);
        //不显示 status--serial--vendorID--productID 栏
        m_whiteLstTable->setColumnHidden(3, true);
        m_whiteLstTable->setColumnHidden(4, true);
        m_whiteLstTable->setColumnHidden(5, true);
        m_whiteLstTable->setColumnHidden(6, true);
        qDebug() << "add to white list succeed" << info.sSerial;
    } else if (tr("Remove") == op) {
        m_usbConnectionWork->removeFromUsbConnectionWhiteLst(info);
        //从白名单表格显示窗口中删除该行
        removeItemFromWhiteLstTable(info);
    }
    //刷新usb连接记录表格中各行的按钮栏
    refreshUsbConnectionTable();
}

void UsbConnectionWidget::openUsbConnectionDetails(UsbConnectionInfo info)
{
    UsbInfoDialog *dialog = new UsbInfoDialog(this);

    dialog->setFixedWidth(380);
//    dialog->setMinimumHeight(252);
    dialog->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

    // 添加标题
//    dialog->setWindowTitle(info.name);

    QGridLayout *infoLayout = new QGridLayout;
    infoLayout->setContentsMargins(0, 0, 0, 0);
    infoLayout->setAlignment(Qt::AlignTop | Qt::AlignLeft);
    infoLayout->setSpacing(0);
    QWidget *infoWgt = new QWidget;
    infoWgt->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    infoWgt->setLayout(infoLayout);
    //添加信息窗口到对话框中
    dialog->mainLayout()->addWidget(infoWgt);

    DLabel *deviceNameTitle = new DLabel(tr("Device").append(": "));
    deviceNameTitle->setContentsMargins(0, 0, 20, 0); //标题右侧预留空间
    infoLayout->addWidget(deviceNameTitle, 0, 0, Qt::AlignTop);
    UsbInfoLabel *deviceNameValue = new UsbInfoLabel(info.sName);
    deviceNameValue->setContentsMargins(0, 0, 0, 0);
    infoLayout->addWidget(deviceNameValue, 0, 1, Qt::AlignTop);

    DLabel *vendorIDTitle = new DLabel("VendorID: ");
    vendorIDTitle->setContentsMargins(0, 0, 20, 0); //标题右侧预留空间
    infoLayout->addWidget(vendorIDTitle, 1, 0, Qt::AlignTop);
    UsbInfoLabel *vendorIDValue = new UsbInfoLabel(info.sVendorID);
    vendorIDValue->setContentsMargins(0, 0, 0, 0);
    infoLayout->addWidget(vendorIDValue, 1, 1, Qt::AlignTop);

    DLabel *productIDTitle = new DLabel("ProductID: ");
    productIDTitle->setContentsMargins(0, 0, 20, 0); //标题右侧预留空间
    infoLayout->addWidget(productIDTitle, 2, 0, Qt::AlignTop);
    UsbInfoLabel *productIDValue = new UsbInfoLabel(info.sProductID);
    vendorIDValue->setContentsMargins(0, 0, 0, 0);
    infoLayout->addWidget(productIDValue, 2, 1, Qt::AlignTop);

    DLabel *serialTitle = new DLabel("SerialNumber: ");
    serialTitle->setContentsMargins(0, 0, 20, 0); //标题右侧预留空间
    infoLayout->addWidget(serialTitle, 3, 0, Qt::AlignTop);
    UsbInfoLabel *serialValue = new UsbInfoLabel(info.sSerial);
    serialValue->setContentsMargins(0, 0, 0, 0);
    infoLayout->addWidget(serialValue, 3, 1, Qt::AlignTop);

    dialog->mainLayout()->addSpacing(10);
    // 正常的按钮
    DPushButton *okBtn = new DPushButton(tr("OK"));
    dialog->mainLayout()->addWidget(okBtn);

    dialog->show();
    connect(okBtn, &DPushButton::clicked, dialog, &UsbInfoDialog::deleteLater);
}
