/*
 * Copyright (C) 2019 ~ 2020 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "../systemsafetymodel.h"
#include "../systemsafetywork.h"
#include "window/namespace.h"
#include "dtkwidget_global.h"
#include "src/widgets/titlebuttonitem.h"
#include "src/widgets/titledslideritem.h"
#include "src/widgets/dccslider.h"
#include "widgets/switchwidget.h"
#include "usbmanagemonitor.h"
#include "usbconnectionwork.h"

#include <DTableView>
#include <DCommandLinkButton>
#include <DLabel>

#include <QVBoxLayout>
#include <QStandardItemModel>
#include <QSettings>

DEF_NAMESPACE_BEGIN
DWIDGET_USE_NAMESPACE
using namespace def::widgets;


/*
USB链接页面
*/
class UsbConnectionWidget : public QWidget
{
    Q_OBJECT
public:
    explicit UsbConnectionWidget(SystemSafetyModel *model, SystemSafetyWork *work, QWidget *parent = nullptr);
    ~UsbConnectionWidget();

protected:

private:
    bool isUsbDeviceInWhiteList(QString serial, QString vendor, QString product);

    void refreshUsbConnectionTable();

    void removeItemFromWhiteLstTable(UsbConnectionInfo info);

    QHBoxLayout *initTwoTable();

    QWidget *initConnectionRecordTableWgt();
    QWidget *initWhiteLstTableWgt();
    void showConnectionRecordTableWgt();
    void showWhiteLstTableWgt();
    void appendUsbConnectionLog(UsbConnectionInfo info);
    void confirmClearUsbConnectionLog();
    void clearUsbConnectionLog();
    void appendUsbConnectionWhiteList(UsbConnectionInfo info);
    //根据状态更新usb限制模式项的按钮菜单
    void updateUsbLimitModeItem();
    //翻译记录中设备的限制状态文字
    QString translateUsbStatus(QString status);
    //cleanUsbConnectionLog:超过30天，界面
    void cleanUsbConnectionLog();
    // update Name in WhiteLst
    void updateNameinWhiteLst(QString sName, QString sSerial, QString sVendorID, QString sProductID);
Q_SIGNALS:

public Q_SLOTS:
    void changeIsbSaveRecord(bool clicked);
    void selectLimitationModel(QAction *action);
    void confirmChangeUsbConnectionWhiteLst(QString op, UsbConnectionInfo info);
    void changeUsbConnectionWhiteLst(QString op, UsbConnectionInfo info);
    void openUsbConnectionDetails(UsbConnectionInfo info);
    void onReceiveUsbInfo(UsbConnectionInfo info);

private:
    SystemSafetyModel *m_model;
    SystemSafetyWork *m_work;
    UsbConnectionWork *m_usbConnectionWork;

    //初始配置
    bool m_isbSaveRecord;
    QString m_limitationModel;
    QList<UsbConnectionInfo> m_usbConnectionInfoLst;
    QList<UsbConnectionInfo> m_usbConnectionWhiteLst;

    SwitchWidget *m_connectionRecordControlItem;
    TitleButtonItem *m_usbLimitModeItem;
    DLabel *m_usbLimitModeItemButtonLabel;

    QWidget *m_connectionRecordTableWgt;
    QWidget *m_whiteLstTableWgt;

    DTableView *m_connectionRecordTable;
    QStandardItemModel *m_connectionRecordTableModel;
    // 清除记录按钮
    DCommandLinkButton *m_clearLogBtn;
    DTableView *m_whiteLstTable;
    QStandardItemModel *m_whiteLstTableModel;
};

DEF_NAMESPACE_END
