/*
 * Copyright (C) 2019 ~ 2020 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "usbconnectionwork.h"
#include "policykithelper.h"

#include <DDialog>

#include <QDir>
#include <QDebug>

UsbConnectionWork::UsbConnectionWork(QObject *parent)
    : QObject(parent)
{
    m_monitorNetFlowInter = new MonitorNetFlowInter("com.deepin.defender.MonitorNetFlow",
                                                    "/com/deepin/defender/MonitorNetFlow", QDBusConnection::systemBus(), this);

    connect(m_monitorNetFlowInter, &MonitorNetFlowInter::SendUsbConnectionInfo, this, &UsbConnectionWork::onReceiveUsbInfo);

    loadConfig();

    //读取白名单
    readConnectionWhiteLst();
    //读取usb连接记录
    readConnectionRecord();
    //清除超过30天记录
    cleanUsbConnectionLog();
}

UsbConnectionWork::~UsbConnectionWork()
{
    //注销时，断开相关信号链接

//    m_usbManageMonitor->quit();
//    m_usbManageMonitor->deleteLater();

//    m_usbManageMonitorThread->quit();
//    connect(m_usbManageMonitorThread, &QThread::finished, m_usbManageMonitorThread, &QThread::deleteLater); //线程结束后销毁
}

void UsbConnectionWork::loadConfig()
{
    m_isbSaveRecord = m_monitorNetFlowInter->GetRecordSaveSwitch();;

    m_limitationModel = m_monitorNetFlowInter->GetLimitModel();
}

void UsbConnectionWork::readConnectionRecord()
{
    //name--time--isbBlocked--serial--vendorID--productID--sParentSize--sSize
    QString sRecords = m_monitorNetFlowInter->GetUsbConnectionRecords();

    QTextStream txtOutput(&sRecords);
    QString lineStr;
    while (!txtOutput.atEnd()) {
        lineStr = txtOutput.readLine();
        qDebug() << lineStr << endl;
        QStringList infoLst = lineStr.split(",");
        if (8 != infoLst.size()) continue;
        //usb连接日志信息结构
        UsbConnectionInfo info;
        info.sName = infoLst.at(0);
        info.sDatetime = infoLst.at(1);
        info.sStatus = infoLst.at(2);
        info.sSerial = infoLst.at(3);
        info.sVendorID = infoLst.at(4);
        info.sProductID = infoLst.at(5);
        info.sParentSize = infoLst.at(6);
        info.sSize = infoLst.at(7);

        m_usbConnectionInfoLst.append(info);
    }
}

void UsbConnectionWork::readConnectionWhiteLst()
{
    //name--time--isbBlocked--serial--vendorID--productID--sParentSize--sSize
    QString whiteListAllStr = m_monitorNetFlowInter->GetWhiteList();
    qDebug() << whiteListAllStr;

    QStringList whiteListStrLine = whiteListAllStr.split("\n");
    for (int i =0; i < whiteListStrLine.size(); i++) {
        QStringList infoLst = whiteListStrLine.at(i).split(",");
        if (8 != infoLst.size()) continue;
        //usb连接信息结构
        UsbConnectionInfo info;
        info.sName = infoLst.at(0);
        info.sDatetime = infoLst.at(1);
        info.sStatus = infoLst.at(2);
        info.sSerial = infoLst.at(3);
        info.sVendorID = infoLst.at(4);
        info.sProductID = infoLst.at(5);
        info.sParentSize = infoLst.at(6);
        info.sSize = infoLst.at(7);

        m_usbConnectionWhiteLst.append(info);
    }
}

void UsbConnectionWork::updateNameInWhiteLst(QString sName, QString sSerial, QString sVendorID, QString sProductID)
{
    if (sName.isEmpty()) {
        return;
    }

    QList<UsbConnectionInfo> lastUsbWhiteList = m_usbConnectionWhiteLst;
    m_usbConnectionWhiteLst.clear();

    for (UsbConnectionInfo info : lastUsbWhiteList) {
        if (sSerial == info.sSerial && sVendorID == info.sVendorID && sProductID == info.sProductID) {
            info.sName = sName;
        }
        m_usbConnectionWhiteLst.append(info);
    }
}

void UsbConnectionWork::onReceiveUsbInfo(QString sInfos)
{
    //解析信息字符串
    QStringList infoStrLst = sInfos.trimmed().split(",");
    if (8 != infoStrLst.size()) return;
    if (infoStrLst.at(1).isEmpty()) return;
    qDebug() << "receive usb device info ======:" << sInfos;

    UsbConnectionInfo info;
    info.sName = infoStrLst.at(0);
    info.sDatetime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss");

    info.sStatus = infoStrLst.at(2);
    info.sSerial = infoStrLst.at(3);
    info.sVendorID = infoStrLst.at(4);
    info.sProductID = infoStrLst.at(5);
    info.sParentSize = infoStrLst.at(6);
    info.sSize = infoStrLst.at(7);

    updateNameInWhiteLst(info.sName, info.sSerial, info.sVendorID, info.sProductID);

    //记录开关是否打开
    if (!m_isbSaveRecord) return;

    m_usbConnectionInfoLst.append(info);
    //根据当前时间清理连接记录
    cleanUsbConnectionLog();
    Q_EMIT sendAppendUsbConnectionLogInfo(info);
}

void UsbConnectionWork::clearUsbConnectionLog()
{
    m_usbConnectionInfoLst.clear();
}

void UsbConnectionWork::clearSrvUsbConnectionLog()
{
    // 使用服务清空日志
    m_monitorNetFlowInter->ClearUsbConnectionLog();
}

void UsbConnectionWork::cleanUsbConnectionLog()
{
    QList<UsbConnectionInfo> usbConnectionInfoLstTmp = m_usbConnectionInfoLst;
    clearUsbConnectionLog();
    QDateTime limitTime = QDateTime::currentDateTime().addDays(-30);
    for (UsbConnectionInfo info : usbConnectionInfoLstTmp) {
        QDateTime infoTime = QDateTime::fromString(info.sDatetime, "yyyy-MM-dd hh:mm:ss");
        //只记录30天以内的连接数据
        if (limitTime < infoTime) {
            m_usbConnectionInfoLst.append(info);
        }
    }
}

bool UsbConnectionWork::checkAuthentication()
{
    bool ret = false;
    ret = PolicyKitHelper::instance()->checkAuthorization(
              "com.deepin.deepin-defender.checkAuthentication", getpid());

    if (!ret) {
        qDebug() << "Authentication failed !!";
    }
    return ret;
}

bool UsbConnectionWork::isUsbDeviceInWhiteList(QString serial)
{
    bool ret = false;
    for (UsbConnectionInfo info : m_usbConnectionWhiteLst) {
        if (serial == info.sSerial) {
            ret = true;
            break;
        }
    }
    return ret;
}

void UsbConnectionWork::appendToUsbConnectionWhiteLst(UsbConnectionInfo info)
{
    //如果id为空，提示添加失败
    if ("" == info.sSerial) {
        qDebug() << "add to white list failed";
        return;
    }

    m_usbConnectionWhiteLst.append(info);
    qDebug() << "add to white list succeed" << info.sSerial;

    //修改后更新服务胡白名单
    QString sDatetime = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss");
    m_monitorNetFlowInter->WriteConnectionWhiteLst(info.sName, sDatetime, info.sStatus, info.sSerial,
                                                   info.sVendorID, info.sProductID, info.sParentSize, info.sSize);
}

void UsbConnectionWork::removeFromUsbConnectionWhiteLst(UsbConnectionInfo info)
{
    //如果id为空，提示添加失败
    if ("" == info.sSerial || "" == info.sVendorID || "" == info.sProductID) {
        qDebug() << "add to white list failed" << info.sSerial << info.sVendorID << info.sProductID;
        return;
    }

    for (int i = 0; i < m_usbConnectionWhiteLst.count(); i++) {
        // 修改问题: 在removeFromUsbConnectionWhiteLst中,将白名单列表写成连接记录列表,导致读取列表数据越界
        UsbConnectionInfo infoTmp = m_usbConnectionWhiteLst.at(i);
        if (info.sSerial == infoTmp.sSerial && info.sVendorID == infoTmp.sVendorID && info.sProductID == infoTmp.sProductID) {
            m_usbConnectionWhiteLst.removeAt(i);
            qDebug() << "remove from white list succeed" << info.sSerial << info.sVendorID << info.sProductID;
            break;
        }
    }

    //修改后更新服务的白名单
    m_monitorNetFlowInter->RemoveFromWhiteLst(info.sName, info.sSerial, info.sVendorID, info.sProductID);
}

void UsbConnectionWork::changeIsbSaveRecord(bool change)
{
    if (!change && !checkAuthentication()) return;

    m_isbSaveRecord = change;
    m_monitorNetFlowInter->ChangeIsbSaveRecord(change);
}

void UsbConnectionWork::selectLimitationModel(QString modelStr)
{
    m_limitationModel = modelStr;

    m_monitorNetFlowInter->SelectLimitationModel(modelStr);
}
