/*
 * Copyright (C) 2019 ~ 2020 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "window/namespace.h"
#include "dtkwidget_global.h"
#include "usbmanagemonitor.h"
#include "com_deepin_defender_monitornetflow.h"

#include <QWidget>
#include <QThread>
#include <QSettings>

DEF_NAMESPACE_BEGIN
DWIDGET_USE_NAMESPACE

using MonitorNetFlowInter = com::deepin::defender::MonitorNetFlow;

//usb连接日志信息结构
struct UsbConnectionInfo {
    QString sName;
    QString sDatetime;
    QString sStatus;
    QString sId;
    QString sSerial;
    QString sVendorID;
    QString sProductID;
    QString sParentSize;        // 父设备大小
    QString sSize;              // 分区大小
};

class UsbConnectionWork : public QObject
{
    Q_OBJECT
public:
    explicit UsbConnectionWork(QObject *parent = nullptr);
    ~UsbConnectionWork();
    inline bool isbSaveRecord() {return m_isbSaveRecord;}
    inline QString limitationModel() {return m_limitationModel;}
    inline QList<UsbConnectionInfo> usbConnectionInfoLst() {return m_usbConnectionInfoLst;}
    inline QList<UsbConnectionInfo> usbConnectionWhiteLst() {return m_usbConnectionWhiteLst;}

    void clearUsbConnectionLog();
    void clearSrvUsbConnectionLog();
    void appendToUsbConnectionWhiteLst(UsbConnectionInfo info);
    void removeFromUsbConnectionWhiteLst(UsbConnectionInfo info);
    bool checkAuthentication();
    bool isUsbDeviceInWhiteList(QString serial);

private:
    void loadConfig();
    //读取usb连接记录
    void readConnectionRecord();
    //读取usb连接白名单
    void readConnectionWhiteLst();
    void updateNameInWhiteLst(QString sName, QString sSerial, QString sVendorID, QString sProductID);
    //cleanUsbConnectionLog:超过30天
    void cleanUsbConnectionLog();

Q_SIGNALS:
    //发送需要添加到日志表格的信息
    void sendAppendUsbConnectionLogInfo(UsbConnectionInfo info);
    void notifyOpenUsbConnectionWindow();

public Q_SLOTS:
    void changeIsbSaveRecord(bool change);
    void selectLimitationModel(QString modelStr);
    // 当接收到设备接入的信息
    void onReceiveUsbInfo(QString sInfos);

private:
    //初始配置
    QSettings *m_config;
    bool m_isbSaveRecord;
    QString m_limitationModel;

    QList<UsbConnectionInfo> m_usbConnectionInfoLst;
    QList<UsbConnectionInfo> m_usbConnectionWhiteLst;

    UsbManageMonitor *m_usbManageMonitor;
    QThread *m_usbManageMonitorThread;

    MonitorNetFlowInter *m_monitorNetFlowInter;

};

DEF_NAMESPACE_END
