/*
 * Copyright (C) 2019 ~ 2020 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once

#include "com_deepin_defender_antiav.h"

#undef signals
extern "C" {
    #include <glib.h>
    #include <gio/gio.h>
}
#define signals public

#include <QObject>
#include <QGSettings>

//dbus接口 服务名称
using AntivirusServer = com::deepin::defender::antiav;

///
/// \brief The GvfsMountManager class
/// GIO虚拟文件系统挂载管理器
class GvfsMountManager : public QObject
{
    Q_OBJECT
public:
    explicit GvfsMountManager(QObject *parent = nullptr);
    // 使用实例
    static GvfsMountManager *instance();
    // 接受到卷监视器的挂载信号时的槽函数
    static void onMountAdded(GVolumeMonitor* monitor, GMount* mount, gpointer data);

signals:
    // 有新挂载的信号
    void mountAdded(QString sMountPoint);

private:
    static GvfsMountManager *m_gvfsMountManager;
};

// 扫描状态
enum ScanState {
    Pause,
    Continue,
    Stop,
};

enum ScanResultType {
    Normal,      //正常,
    Malware,     //恶意软件
    Trojan,      //木马
    Backdoor,    //后门远控
    Worm,        //蠕虫
    Rootkit,     //恶意驱动
    Exploit,     //漏洞利用
    HackTool,    //黑客工具
    Adware,      //恶意广告
    Stealer,     //盗号木马
    Spammer,     //垃圾邮件
    Spyware,     //间谍软件
    Virus,       //感染型病毒
    Joke,        //玩笑程序
    Junk,        //病毒僵尸
    PUA,         //可疑
    Downloader,  //下载器
    Dropper,     //释放器
    Ransom,      //勒索软件
    Hoax,        //诈骗软件
    Riskware,    //灰色软件
    Unwanted,    //可疑
    Monetizer,   //套现程序
    Payment,     //恶意扣费
    Privacy,     //隐私窃取
    Remote,      //远程控制
    Spread,      //恶意传播
    Expense,     //资费消耗
    System,      //系统破坏
    Fraud,       //诱骗欺诈
    Rogue,       //流氓行为
    Other,       //其他
//具体都是有什么类型需要商定
    ScanTaskError,// 扫描任务错误
    NetAnomaly,//网络异常,
    ServerAnomaly//服务器异常,
};

///
/// \brief The UsbScanWork class
/// USB病毒查杀数据处理类
class UsbScanWork: public QObject
{
    Q_OBJECT
public:
    explicit UsbScanWork(QObject *parent);
    virtual ~UsbScanWork();

    // 扫描 （全盘true/自定义false）
    void doDissection(bool isbAll, QString sPaths);
    // 设置扫描状态 （暂停/继续/终止）
    void requestSetDissectionStatus(ScanState status);
    // 是否在扫描中
    bool isScanning();

Q_SIGNALS:

public Q_SLOTS:
    void onRecDissectionResult(QString sPath, double dProgress, int iType, QString sDetailInfo);
    // 响应扫描结束信号
    void onScanningFinished();
    void onMountAdded(QString sMountPoint);

private:
    // 设置是否是usb储存设备查杀
    void setIsbUsbDissection(bool isbUsb);
    void showUsbScanningSysNotification();
    void showUsbScanWhenBusySysNotification();
    void showUsbHaveDangersSysNotification();

private:
    AntivirusServer         *m_AntivirusServer = nullptr;         // 病毒库dbus 接口
    QGSettings              *m_gsetting;
    ScanState               m_scanStatus;
    // 是否是usb储存设备查杀
    bool                    m_isbUsbDissection;
    QString                 m_sUsbScanPath;
};
